!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Addition of global parameters for the coupling in FORHYCS

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  All_par
!
!  Remarks:  contains the global parameters for TreeMig
!
!==============================================================================


MODULE All_par

!****************** CONTROL PARAMETERS ***************************************!
!----- Important values are read in ReadControlPars from file control.pars

  !----- Basic setup for simulation
  INTEGER  numyrs                                                 ! number of years to be simulated
  INTEGER  simustartyear                                          ! start year of simulation
  INTEGER, Parameter:: climatestartyear=1971           !1900 for observations     ! start year of bioclimate data
  INTEGER  rptint                                                 ! report interval (years)
  INTEGER  rptdist                                                ! report distance (cells)
  INTEGER, PARAMETER :: spi_sample_yrs         = 30
  INTEGER, PARAMETER :: maxArraylat            = 206!86!10 !1134! 1132 !2201        ! maximal number of rows, only for declarations
  INTEGER, PARAMETER :: maxArraylon            = 131!63 !10! 1840! 1834             ! maximal number of colums, only for declarations
  INTEGER, PARAMETER :: maxCCTimes             = 1              ! maximal number of timepoints for climate change scenario
  INTEGER, PARAMETER :: maxImmigrationsPerSpec =  1              ! maximal number of timepoints for climate change scenario
  INTEGER, PARAMETER :: maxKernelDimFine     = 150 ! 600 !200!  1601             	! maximal dimension of coarse kernel
  INTEGER, PARAMETER :: maxKernelDim           = 25  ! 600 !200!  1601             	! maximal dimension of coarse kernel
!  INTEGER, PARAMETER :: maxSeededCells         = (2*maxKernelDim+1)**2 		! max. number of cells which receive seeds
  INTEGER, PARAMETER :: maxSeededCells         = (2*maxKernelDim*maxArrayLon)		! max. number of cells which receive seeds
  INTEGER, PARAMETER :: nmeasuredtimes         = 200
  INTEGER, PARAMETER :: maxNumbProcs           = 64               ! maximal number of processes for parallel computing
  ! INTEGER            :: requests(maxArraylat*maxArraylon)
  INTEGER  maxlat                                                 ! chosen number of rows
  INTEGER  maxlon                                                 ! chosen number of colums
  REAL ::  latRealStart, latRealEnd, lonRealStart, lonRealEnd     ! maximum latitude  of rectangle in real coordinates
  INTEGER  nCCTimes                                               ! chosen number of timepoints for climate change scenario
  INTEGER  startlatStart, startlatEnd, startlonStart, startlonEnd ! position of initial cells
  INTEGER  unitOfSpatialData                                      ! unit of bioclim and stockability data in meters
  INTEGER  starttime                                              ! length of initial simulation
  INTEGER  latEvalCell, lonEvalCell                               ! coordinates of single cell to be evaluated for integrated results
  LOGICAL  readStateVector                                        ! read initial state from file statefile?
  REAL, PARAMETER         :: biothreshold=1.    !  0.001       ! ( t/ha) minimal total species biomass under which cell state is set to zero when reading in statefile
  LOGICAL  ienv                                                   ! include environmental info for calcs?
  LOGICAL  envFromFile                                            ! read environmental info from file?
  !----- Setup for plotting
  INTEGER  maxrowscols, rowscalefac, colscalefac             ! how many rows and columns are at maximum in plot
  REAL     maxDataInPlot                                     ! max. biomass in each stacked column when plotting
  INTEGER, PARAMETER :: edge = 0                             ! Security edge
  REAL     alpha                                             ! Alpha-value of kernel                         [=..] !DSP
  REAL     cellSideLength                                    ! Side-lenght of grid cell                      [=..] !DSP
  REAL     epsKernel                                         ! threshold for cut-off of kernel
  INTEGER  kernelType                                        ! 0: radius, 1: single exponential, 2: double exponential
  LOGICAL, PARAMETER :: idsp =.TRUE.                         ! Include seed dispersal between plots?
  LOGICAL  active                                            ! Active or passive seed dispersal (calculations)
  LOGICAL  doStochSeedDisp                                   ! Stochastic or deterministic seed dispersal (calculations)
  LOGICAL  seedsDifferent                                    ! seed production of all species different or equal
  LOGICAL  mathgtDifferent                                   ! maturation height of all species different or equal
  LOGICAL  dispersalDifferent                                ! do dispersal distances differ between species?
  LOGICAL  mastSeeding                                       ! periodic seed production  included?
  LOGICAL  seedBank                                          ! seed bank  included?
  LOGICAL  readDispersalKernel                               ! seed bank  included?
  LOGICAL  withSeedAntagonists                               ! seed pathogenes, parasites or predators included?
  REAL     seedAntaGraz                                      ! grazing rate of seed antagonists
  REAL     seedAntaEff                                       ! efficiency of grazing of seed antagonists
  REAL     seedAntaMort                                      ! mortality of seed antagonists
  REAL     seedAntaRain                                      ! constant rain of seed antagonists
  LOGICAL  withSeedCarrCap                                   ! seed  carryin gapacity included?
  REAL     seedCarrCap                                       ! carrying capacity for seeds
  LOGICAL  immigration                                       ! disturbances  included?
  INTEGER  immiPeriod                                        ! period around immigration times, when species immigrate
  LOGICAL  disturbances                                      ! disturbances  included?
  REAL     disturbProb                                       ! disturbance probability (1/year) per cell
  REAL     disturbIntensity                                  ! disturbance intensity (0.0..1.0)
  LOGICAL  tabsepOutput                                      ! tabulator (T) or comma separated output?
  LOGICAL :: estabEqHC0                                      ! light dependence of establishment  used by gemination or life of seedlings?
  LOGICAL :: contLightDepGerm                                ! light dependence of establishment continuous or step-function as in ForClim?
  CHARACTER(2):: pathletter1                                 ! './':UNIX, ' :':Mac, '.\':Windows
  CHARACTER(1):: pathletter2                                 ! boundary conditions: '/':UNIX,  ':':Mac, '.\':Windows

  CHARACTER*1 :: boundaries                                  ! boundary conditions: 'c':cyclic, i.e torus,  'a':absorbing
  INTEGER   :: OuterSeeds                                    ! no. of seeds coming from outside grid
  LOGICAL    :: cellNotYetAllocated
  !----- Basic model setup
  LOGICAL, PARAMETER :: slow   = .FALSE.                     ! slowing down initial growth (yes/no)
  INTEGER, PARAMETER :: maxspc =  31                         ! MAX # of species        [=nspecs]
  INTEGER, PARAMETER :: maxhc  = 15                          ! MAX # of height classes [=nhmax]
  INTEGER, PARAMETER :: maxlc  = 10                          ! MAX # of light  classes [=nLightClasses]
  CHARACTER(15):: specfilename                               ! name of file with species parameters
  CHARACTER(15):: immifilename                               ! name of file with immigration sites and times
  CHARACTER(15):: bioclimfilename1                            ! name of file with bioclimate , FORHYCS - 2 bioclimfiles, TreeMig BC and Prevah BC, important for spinup
  CHARACTER(15):: bioclimfilename2                            ! name of file with bioclimate
  CHARACTER(15):: stockAreafilename,landCoverFilterFilename    ! name of file with proportion of stockable area of cell
  CHARACTER(12) :: experimentID                              ! identifier for output of experiment

  !----- Basic parameters
  REAL, PARAMETER :: pi    = 3.141592654                     ! Pi
  REAL, PARAMETER :: birht = 1.37                            ! Birth height [=minHBirth]
  REAL, PARAMETER :: toplt = 1.0                             ! Light at the top of the canopy                   [=lightAtTop]
  REAL  facvar                                               ! Fac.variability for light freq.  dynamics        [=facVariability]
  REAL  fcvarInit                                            ! Fac.variability for light freq.  initialisation  [=facVariability]
  REAL  fcvarDyn                                             ! Fac.variability for light freq.  initialisation  [=facVariability]
  REAL            :: facvarh(1:maxhc)                        ! Fac.variability for light freq. per height class [=facVariability]             !TJL2
  REAL            :: facvars(1:maxspc)                       ! Fac.variability for light freq. per height class [=facVariability for species] !TJL3
  REAL            :: facvarsh(1:maxspc,1:maxhc)              ! Fac.variability for light freq. per height class [=facVariability for species] !TJL4
  REAL            :: pltsiz                                  ! Plot size [=patchSize]
  REAL, PARAMETER :: potbirth = 0.5                          ! constant birth rate parameter                    [=birth]
  REAL            :: seedlcrowndiamInCm                      ! diameter of average seedling crown (cm)

  !----- Basic variables
  REAL :: a1(5)                                              ! Parameter for LA calcs      [=a1(sTypeN)]
  REAL :: a2(5)                                              ! Parameter for LA calcs      [=a2(sTypeN)]
  REAL :: c1(2)                                              ! Parameter for biomass calcs [=c1(sTypeN)]
  REAL :: c2(2)                                              ! Parameter for LA calcs      [=c2(sTypeN)]
  REAL :: kn1(3)                                             ! Parameter for N-growth dep. [=kN1] !ENV
  REAL :: kn2(3)                                             ! Parameter for N-growth dep. [=kN2] !ENV

  !----- FORHYCS
!  INTEGER :: nspc_g							!nspc as global variable
!  INTEGER :: counter_g						!counter as global variable
!  !REAL :: fhla(1:maxArraylat,1:maxArraylon,maxspc)

!  MOVED TO forhycsglobals.f90


  TYPE wrOutp
    LOGICAL :: biomass, number, seeds, antagonists, hstruct, pollen, ingrowth, biodiv, plot
    LOGICAL :: leafarea !FORHYCS
  END TYPE wrOutp


  TYPE(wrOutp) :: writeOutput

  !----- Overall result variables
  REAL    :: sumDiversity
  REAL    :: avdiversity80, avdiversity160, avdiversity240, avdiversity500, avdiversityEq
  REAL    :: biocrit(10)
  !REAL    :: specAppearance(10, maxspc, maxArraylat, maxArraylon)
  !REAL    :: specMigSpeed(10, maxspc, maxArraylat, maxArraylon)
  real, allocatable :: specAppearance(:,:,:,:), specMigSpeed(:,:,:,:)
  INTEGER :: nonzeroSpecs80, nonzeroSpecs160, nonzeroSpecs240, nonzeroSpecs500, nonzeroSpecsEq
  INTEGER :: domspec80(3), domspec160(3), domspec240(3), domspec500(3), domspecEq(3)
  REAL        :: kernel(-maxKernelDim:maxKernelDim,-maxKernelDim:maxKernelDim)

  !natalie:
  REAL    :: stmvol

  ! DEBUG

  real::sumseeds3


!****************** SPECIES TYPES AND PARAMETERS  ***************************************!
!****************** User: DO NOT TOUCH!           ***************************************!
  TYPE IData
    INTEGER :: year
    REAL    :: lat
    REAL    :: lon
  END TYPE IData

  TYPE ImmigrationData
    INTEGER     :: n
    TYPE(IData) :: dat(maxImmigrationsPerSpec)
  END TYPE ImmigrationData

  TYPE DfSpc
    INTEGER               :: nb              ! Species number[=spec]
    INTEGER               :: stn             ! Species Type/N[=sType/N]
    INTEGER               :: kdmax           ! MAX Diameter (DBH)[=kDMax]
    INTEGER               :: khmax           ! MAX Height (in m)[maxHeight <- =kHMax]
    INTEGER               :: kamax           ! MAX Age (in years)[=kAMax]
    INTEGER               :: kg              ! MAX growth rate (cm/year)[=kG]
    INTEGER               :: kddmin          ! MIN DDEG limit[=kDDMin]
    INTEGER               :: kddmax          ! MAX DDEG limit[=kDDMax]
    INTEGER               :: kwit            ! MIN winter TEMP accepted[=kWiT]
    INTEGER               :: kntol           ! Nitrogen tolerance (value)[=kNTol]
    INTEGER               :: kbrow           ! Susceptibility to browsing[=kBrow]
    INTEGER               :: klighs          ! Light parameter Saplings[=kLighS]
    INTEGER               :: kligha          ! Light parameter Adults[=kLighA]
    INTEGER               :: klq             ! Leaf litter quality[=kLQ]
    INTEGER               :: kimm            ! Immigration parameter ???[=kImm]
    INTEGER               :: resld           ! ????[=reslD]
    INTEGER               :: ploto           ! plot or not (1/0)[=plotOrNot]
    INTEGER               :: par0            ! Parameter 0[=par0]
    INTEGER               :: par1            ! Parameter 1[=par1]
    INTEGER               :: par2            ! Parameter 2[=par2]
    INTEGER               :: par3            ! Parameter 3[=par3]
    INTEGER               :: par4            ! Parameter 4[=par4]
    INTEGER               :: par5            ! Parameter 5[=par5]
    INTEGER               :: red             ! colorschemes for plotting
    INTEGER               :: green           ! colorschemes for plotting
    INTEGER               :: blue            ! colorschemes for plotting
    INTEGER               :: par9            ! Parameter 9[=par9]
    INTEGER               :: tb              ! sTypeB for calculating LA/Biomass[itypeB -> iTreeType]
    INTEGER               :: maxhtc          ! MAX height class(ified)[=nh]
    INTEGER               :: mxseed          ! should be read from file later,individualized            [=..] !DSP
    INTEGER               :: rad
    REAL                  :: kdrt            ! Drought tolerance (rate)[=kDrT]
    REAL                  :: par6            ! Parameter 6[=par6]
    REAL                  :: par7            ! Parameter 7[=par7]
    REAL                  :: par8            ! Parameter 8[=par8]
    REAL                  :: maxdhm          ! Species MAX DHM (in m)[=kdmax -> dmax]
    REAL                  :: maxgro          ! Species MAX growth rate(cm/year?)               [=kg -> vmax]
    REAL                  :: cmort           ! Species specific constantmortality rate               [=mort]
    REAL                  :: seedGerm        ! Species specific seed germination rate                [=mort]
    REAL                  :: seedSurv        ! Species specific seed mortality rate                  [=mort]
    REAL                  :: seedLoss        ! Species specific seed loss rate                       [=mort]
    REAL                  :: lthres          ! light threshold calc. from klighs[=threshold]
    REAL                  :: envdep          ! environment (-based) growthfactor               [=allOtherDep] !ENV
    REAL                  :: enveta          ! environment (-based)establishment factor          [=otheretab] !ENV
    REAL                  :: minmat          ! minimum height of maturity[=..] !DSP
    REAL                  :: maxseed         ! should be read from file later,individualized            [=..] !DSP
    REAL                  :: dispFac         ! should be read from file later,individualized            [=..] !DSP
    REAL                  :: alfa1           ! should be read from file later,individualized            [=..] !DSP
    REAL                  :: alfa2           ! should be read from file later,individualized            [=..] !DSP
    REAL                  :: birth
    REAL                  :: period          ! period of mast seeding
    REAL                  :: mathcp          ! minimum height class proportionof maturity               [=..] !DSP
    REAL                  :: repFac          ! pollen representation factor
    REAL                  :: sumTimeBio      ! biomass summed over time (for dominance)
    REAL                  :: kernel(-maxKernelDim:maxKernelDim,-maxKernelDim:maxKernelDim)
    CHARACTER(25)         :: nl              ! Species name long[=species]
    CHARACTER(6)          :: ns              ! Species name short[=Abbr]
    CHARACTER(1)          :: stb             ! Species Type/B character[sType/B]
    TYPE(ImmigrationData) :: immi
END TYPE DfSpc


  TYPE SpecPot
    REAL :: htmin(0:maxhc+1)       ! Lower height (in m) for eachheight class                       [=trueHeights]
    REAL :: diain(0:maxhc)         ! Diameter (in m) for each heightclass                                [=diaInH]
    REAL :: lain(0:maxhc)          ! LA (in m^2) for each height class[=leaveAreaInH]
    REAL :: bioin(0:maxhc)         ! Biomass (in kg) for each heightclass                            [=biomassInH]
    REAL :: hlgro(0:maxhc,maxlc)   ! Height/Light dependent growth byspecies                       [=growthHL(..)]
    REAL :: envVitality(0:maxhc,maxlc)   ! Stress-related vitality factor                       [=arOtherDepMortHL(..)]
    REAL :: growthVitality(0:maxhc,maxlc)   ! Growth-related vitality factor                     [=arDiamIncFacMortHL(..)]
    real :: snowMortality(maxlc)    ! Mortality related to snow cover (only for lowest height class)
    REAL :: mxsdht(0:maxhc)        ! Max seeds per height class, isfraction of maxseed!                      [=..] !DSP
    REAL :: lightDepA(1:maxlc)     ! light dependence function
    REAL :: lightDepE(1:maxlc)     ! light dependence Etab
    real :: crArea(0:maxhc)         ! Crown area - Matthias
  END TYPE SpecPot

  TYPE SpecState
    REAL    :: sb             ! # of seeds in seedbed or derived values for reporting
    REAL    :: seedsum        ! # of seeds produced per species
    REAL    :: caches         ! cache site area (areaper grid-cell area)         [=..] !DSP
    REAL    :: htmps          ! Max. poss. height (inc.) perhtcl [=rMaxPossibleHeight]
    REAL    :: htdmp          ! Delta htmps[=rDevMaxPossibleHeight]
    REAL    :: antagonist     ! # of individuals/htcls[=numbInH]
    REAL    :: newSeeds       ! # new seeds reaching cell
    REAL    :: verynewSeeds   ! #  very new seeds reaching cell    before update        ! # of seeds in seedbed
    REAL    :: seedbed        ! # new seeds reaching cell
    REAL    :: ingrowth       ! # trees growing into first height class
    REAL    :: numin(0:maxhc )     ! # of individuals/htcls[=numbInH]
    REAL    :: seeds(0:maxhc )     ! # of produced seeds/htcl
    LOGICAL :: there          ! logical array -> seeds arehere?            [~immiYear]
  END TYPE SpecState

  TYPE SeedCSpec
    INTEGER :: lat
    INTEGER :: lon
    REAL    :: seeds
  END TYPE SeedCSpec



  TYPE SeedC
     TYPE(SeedCSpec) ::sp(maxspc)
  END TYPE SeedC

 TYPE SeedsSpec
      REAL    :: newseeds
      REAL    :: verynewseeds
  END TYPE SeedsSpec


 TYPE SeedSpecArr
     TYPE(SeedsSpec) ::sp(maxspc)
  END TYPE SeedSpecArr


  TYPE SpecsStateInCell
     Type(SpecState) :: sp(maxspc)
  END TYPE SpecsStateInCell


  TYPE BioClimInOneCell
    REAL :: muDD, stdDD, muWiT, stdWiT, prop0DrStr, muDrStr, stdDrStr, smday
  END TYPE BioClimInOneCell

  TYPE BIOCLIMDATA
    INTEGER                :: cctime
    !TYPE(BioClimInOneCell) :: bc(1:maxArraylat,1:maxArraylon)
    TYPE(BioClimInOneCell), allocatable, dimension(:,:) :: bc
  END TYPE BIOCLIMDATA

  TYPE CurrStateInCell
     TYPE(SpecState)       :: sp(maxspc)
     INTEGER               :: lat,lon
     INTEGER               :: maxCarpetLength
     REAL                  :: ddegs,wtemp,drstr,avnit, brwpr
     Logical               :: chestnutcult
   END TYPE CurrStateInCell

   TYPE CurrEnvInCell
     REAL                  :: ddegs,wtemp,drstr,avnit, brwpr
   END TYPE CurrEnvInCell

   TYPE NewSeedsInCell
     TYPE( SeedsSpec)                 :: sp(maxspc)
   END TYPE NewSeedsInCell

  TYPE YearCharacteristics
     INTEGER   :: year
     REAL      :: facvar
     Logical   :: idsp
  END TYPE YearCharacteristics

  TYPE OutPutLine
     INTEGER   :: iam,lat, lon, year
     REAL      :: num(maxspc)
     REAL      :: sum
  END TYPE OutPutLine

  TYPE(CurrStateInCell):: thiscell

 ! TYPE(CurrStateInCell):: cell(1:maxArraylat,1:maxArraylon)
  TYPE(CurrStateInCell), dimension (:,:) , allocatable :: cell


  !TYPE(NewSeedsInCell):: seedrain(1:maxArraylat,1:maxArraylon)
  type(NewSeedsInCell), allocatable :: seedrain(:,:)

  TYPE(DfSpc)    :: spec(maxspc)    ! Structured Type        [=treeSpec]
  TYPE(SpecPot)  :: spc(maxspc)     ! Structured Type   [part.=treeSpec]
  TYPE(YearCharacteristics):: thisyear

   TYPE(OutPutLine):: outputL


 ! MS, Feb 2019: Cannot have an array of TYPEs with an allocatable element.
 ! Therefore, replacing ThirtyYearBioClim with 30 BIOCLIMDATA objects
  TYPE(BIOCLIMDATA) oldBioClim, newBioClim!, ThirtyYearBioClim(1:spi_sample_yrs)
  TYPE(BIOCLIMDATA) :: BioClimSU1, BioClimSU2, BioClimSU3, BioClimSU4, BioClimSU5, &
                       BioClimSU6, BioClimSU7, BioClimSU8, BioClimSU9, BioClimSU10, &
                       BioClimSU11, BioClimSU12, BioClimSU13, BioClimSU14, BioClimSU15, &
                       BioClimSU16, BioClimSU17, BioClimSU18, BioClimSU19, BioClimSU20, &
                       BioClimSU21, BioClimSU22, BioClimSU23, BioClimSU24, BioClimSU25, &
                       BioClimSU26, BioClimSU27, BioClimSU28, BioClimSU29, BioClimSU30

  !REAL    :: stockability(1:maxArraylat,1:maxArraylon), landcoverfilter(1:maxArraylat,1:maxArraylon)
  real, allocatable :: stockability(:,:), landcoverfilter(:,:)
  !REAL    :: stocksum(1:maxArraylat ) , stocksumcum(1:maxArraylat ) ! summed over lons, used for load balancing
  real, allocatable :: stocksum(:), stocksumcum(:)
  REAL    :: hcwidth                    ! discrete height class width             [=dh]
  INTEGER :: htmax                      ! Maximum height over all species(in m)   [=hmax]
  !REAL    ::   avbio0(1:maxArraylat,1:maxArraylon,1:maxspc)! ,&
  real, allocatable :: avbio0(:,:,:)
!                     avbio1(1:maxArraylat,1:maxArraylon,1:maxspc),&
!                     avbio2(1:maxArraylat,1:maxArraylon,1:maxspc),&
!                     avbio3(1:maxArraylat,1:maxArraylon,1:maxspc)



  double precision, dimension(nmeasuredtimes)::  oldtime, sumtime
  double precision::  measuredStartTime
!****************** STAND TYPES AND PARAMETERS ***************************************!
!****************** User: DO NOT TOUCH!           ***************************************!

   REAL    :: ltfrq(maxlc)              ! Light frequency classes                  [=lightFrequency]
   REAL    :: ltcum(maxlc)              ! Cumulative lightfreq. classes              [=lightFreqCum]
   REAL    :: ltprf(0:maxhc,maxlc)      ! Light Frequency Profile                [=lightFreqProfile]
   REAL    :: ltval(maxlc)              ! Light Values                                   [=lightVal]
   REAL    :: la(maxlc)                 ! LA in ltcl                                   [=laiOfLight]

   INTEGER :: htcl                      ! Height Class (numbered)                          [=height]
   INTEGER :: ltcl                      ! Light Class (numbered)                           [=ilight]

END MODULE All_par
!****<Here the module ends>************************************************************************
