!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  DispersalKernel
!
!  Remarks:  contains the subroutines + functions
!
!            DispersalKernel (<GetSpc<TreeMig)
!            DoubleKernel (<DispersalKernel<GetSpc<TreeMig)
!            SingleKernel (<DispersalKernel<GetSpc<TreeMig AND/OR <DoubleKernel<
!                            DispersalKernel<GetSpc<TreeMig)
!            CenterToCenterValue (<SingleKernel<DispersalKernel<GetSpc<TreeMig
!                                   AND/OR <SingleKernel<DoubleKernel<
!                                   DispersalKernel<GetSpc<TreeMig)
!        (F) KernelValue (<CenterToCenterValue<SingleKernel<DispersalKernel<
!                           GetSpc<TreeMig AND/OR <CenterToCenterValue<SingleKernel<
!                           DoubleKernel<DispersalKernel<GetSpc<TreeMig)
!
!==============================================================================


SUBROUTINE DispersalKernel(i)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: dispersalDifferent, alpha, cellSideLength, epsKernel, kernelType, spec, maxKernelDim, kernel

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: i

!<Local variables>---------------------------------------------------------------------------------
INTEGER ::   radi
real :: alf

 !<Here the subroutine starts>**********************************************************************
  IF (.NOT. dispersalDifferent) THEN
    CALL SingleKernel (alpha, cellSideLength, epsKernel , radi )
  ELSEIF (kernelType == 1) THEN
    alf=spec(i)%alfa1
    CALL SingleKernel (alf, cellSideLength, epsKernel , radi )
  ELSEIF (kernelType == 2) THEN
    CALL DoubleKernel (spec(i)%dispFac, spec(i)%alfa1, spec(i)%alfa2, cellSideLength, epsKernel , radi )
  ELSE
  END IF
  spec(i)%kernel = kernel
  spec(i)%rad    = radi

END SUBROUTINE DispersalKernel
!****<Here the subroutine ends>********************************************************************
!==============================================================================
SUBROUTINE SingleKernel(alpha, cellSideLength, epsKernel , rad )
Use All_Par, Only: kernel, maxKernelDim,maxKernelDimFine
!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE
!<Passed variables>--------------------------------------------------------------------------------

REAL,    INTENT(IN)  :: alpha, cellSideLength, epsKernel
INTEGER, INTENT(OUT) :: rad
!<Local variables>---------------------------------------------------------------------------------
INTEGER :: latwin, latwingr, latwinstart, latwinend, &    ! index of relative window latitude
           lonwin, lonwingr,lonwinstart, lonwinend,  &    ! index of relative window longitude
           icellSideLengthFak, radMax,irad                               ! for switching between min. and actual discretization
REAL    :: krnlsm,                                                            & ! sum of dispersal kernel
           rmaxKernelDim, cellSideLengthMin, &
            CenterToCenterValue
REAL:: kernelFine(-maxKernelDimFine:maxKernelDimFine,-maxKernelDimFine:maxKernelDimFine) ! for kernel in finest discretization

!<Here the subroutine starts>**********************************************************************
!----- "Integrate" kernel over window-size to define general shape

  kernel        = 0.0
  kernelFine    = 0.0

  cellSideLengthMin       = 1000.0 / 32.0             ! side length of  finest discretization, about 0.0001 of kernel cut off
  icellSideLengthFak      = cellSideLength / cellSideLengthMin            ! actual to finest discretization
  radMax        = alpha * 20 / cellSideLengthMin + 1  ! radius of finest discretization (The 20 is sufficient at small gridsizes)
  irad           = INT((alpha * 20) / cellSideLength + 1.0 )    ! radius of actual discretization
  rad=irad
!  print *, ' in SingleKernel  alpha,  rad, radMax, maxKernelDimFine,maxKernelDim', &
 !         alpha,  rad, radMax, maxKernelDimFine,maxKernelDim
 !----- fine scale kernel, calculate one quadrant
  KernelLat: DO latwin = 0, radMax
    KernelLon: DO lonwin = 0, radMax
       kernelFine (latwin, lonwin) = CenterToCenterValue (alpha, cellSideLengthMin, latwin, lonwin)
    END DO KernelLon
  END DO KernelLat
!----- fine scale kernel,copy other quadrants
  KernelLat2: DO latwin = -radMax, radMax
    KernelLon2: DO lonwin = -radMax, radMax
      IF ((latwin .lt. 0) .or. (lonwin .lt. 0)) &
        kernelFine(latwin, lonwin) = kernelFine(ABS (latwin), ABS (lonwin))
    END DO KernelLon2
  END DO KernelLat2
!----- The kernel on the  coarse (i.e. actual) grid is calculated by
!----- summing over the kernel values in all fine-grid cells in each coarse-grid cell
  KernelLat1: DO latwingr = -rad, rad
    KernelLon1: DO lonwingr = -rad, rad
      latwinstart = Max (Min (radMax, latwingr * icellSideLengthFak - icellSideLengthFak / 2), -radMax)
      latwinend   = Max (Min (radMax, latwingr * icellSideLengthFak + icellSideLengthFak / 2), -radMax)
      lonwinstart = Max (Min (radMax, lonwingr * icellSideLengthFak - icellSideLengthFak / 2), -radMax)
      lonwinend   = Max (Min (radMax, lonwingr * icellSideLengthFak + icellSideLengthFak / 2), -radMax)
      DO latwin = latwinstart, latwinend
        DO lonwin = lonwinstart, lonwinend
          kernel(latwingr, lonwingr) = kernel(latwingr, lonwingr) + kernelFine(latwin, lonwin) / 2
        END DO !----- lonwin
      END DO !----- latwin
    END DO KernelLon1
  END DO KernelLat1
!----- Normalize kernel
  krnlsm = 0
  DO latwingr = -rad, rad
    DO lonwingr = -rad, rad
      krnlsm = krnlsm + kernel(latwingr, lonwingr)
    END DO
  END DO
  kernel = kernel / krnlsm

END SUBROUTINE SingleKernel
! ****<Here the subroutine ends>*******************************************************************!****<Here the subroutine ends>********************************************************************
!==============================================================================
SUBROUTINE DoubleKernel(dispFac, alfa1, alfa2, cellSideLength, epsKernel,  rad)
Use All_Par, Only: kernel, maxKernelDim
!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

REAL,    INTENT(IN)  :: alfa1, alfa2, dispFac, cellSideLength, epsKernel
INTEGER, INTENT(OUT) :: rad
! REAL,    INTENT(OUT) :: kernel (-maxKernelDim:maxKernelDim, -maxKernelDim:maxKernelDim)    ! value of dispersal kernel
!<Local variables>---------------------------------------------------------------------------------
INTEGER :: rad1, rad2
REAL    :: kernel1 (-maxKernelDim:maxKernelDim, -maxKernelDim:maxKernelDim), &    ! value of dispersal kernel
           kernel2 (-maxKernelDim:maxKernelDim, -maxKernelDim:maxKernelDim)       ! value of dispersal kernel
!<Here the subroutine starts>**********************************************************************

  IF ((1.0 - dispFac) .gt. 0.0)   THEN
    CALL SingleKernel (alfa1, cellSideLength, epsKernel,rad1 )
	kernel1=kernel
    CALL SingleKernel (alfa2, cellSideLength, epsKernel, rad2 )
	kernel2=kernel
    kernel = dispFac * kernel1 + (1. - dispFac) * kernel2
    rad    = Max (rad1, rad2)
  ELSE
    kernel = 0.0
    CALL SingleKernel (alfa1, cellSideLength, epsKernel, rad)
  END IF
END SUBROUTINE DoubleKernel
! ****<Here the Function CenterToCenterValue starts>***********************************************
! -------------------------------------------------------------------------------------------------
REAL FUNCTION CenterToCenterValue(alpha, cellSideLength, latwin, lonwin)
IMPLICIT NONE
INTEGER, INTENT(IN) :: latwin,lonwin
REAL,    INTENT(IN) :: alpha, cellSideLength
REAL :: dist, &                 ! distance from source to target
        KernelValue
!<Here the function starts>************************************************************************
  dist                = SQRT ((REAL (latwin) ** 2) + (REAL (lonwin) ** 2)) * cellSideLength
  CenterToCenterValue = KernelValue (dist, alpha, cellSideLength)
END FUNCTION CenterToCenterValue
! ****<Here the function ends>*********************************************************************
! -------------------------------------------------------------------------------------------------
REAL FUNCTION KernelValue(dist, alpha, cellSideLength)
IMPLICIT NONE
REAL, INTENT(IN) :: dist, alpha, cellSideLength
!-----> OLD Kernel  KernelValue = EXP (-dist / alpha) * cellSideLength ** 2
  KernelValue = EXP (-dist / alpha)  !/ (1.0 + dist)
END FUNCTION KernelValue
! ****<Here the function ends>*********************************************************************
