!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  DrawFromDistri
!
!  Purpose:  draws random value from  binomial, poisson or normal distribution
!
!  Remarks:  contains the subroutines
!
!            DrawFromDistri (<SeedsDispFromThisCellAndSpec<SeedsDispersedFromThisCell<
!                              DoLocalWork<DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!            DrawFromPoissonDist }   (<GetVariate<GetBioclim<GetEnvFactors<
!            DrawFromBinomDist   }      DistrLocalDynThisTime<TimeLoop<TreeMig AND/OR
!            DrawFromNormalDist  }      DrawFromDistri<SeedsDispFromThisCellAndSpec<
!                                       SeedsDispersedFromThisCell<DoLocalWork<
!                                       DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================

SUBROUTINE DrawFromDistri(n, p, ix)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)  :: n      ! parameter
REAL,    INTENT(IN)  :: p      ! parameter
INTEGER, INTENT(OUT) :: ix     ! drawn value

!<Local variables>---------------------------------------------------------------------------------
REAL :: randomvalue,                & ! [0,1] random value
        startdist, mu, sigma, rx

!<Here the subroutine starts>**********************************************************************
  IF (p .ge. 1.0) THEN
    ix = n
    RETURN
  END IF
  IF ((p .le. 0.0) .or. (n .le. 0)) THEN
    ix = 0.0
    RETURN
  END IF

  CALL RANDOM_NUMBER (randomvalue)
  startdist = (1.0 - p) ** n
  IF ((startdist .eq. 0.0) .or. (startdist .eq. 1.0) .or. (n .gt. 50)) THEN
    mu = p * n
    IF (mu .lt. 20) THEN
      CALL DrawFromPoissonDist (mu, ix, randomvalue)
    ELSE
      sigma = SQRT (mu)
      CALL DrawFromNormalDist (mu, sigma, rx, randomvalue)
      ix = rx
    END IF
  ELSE
    CALL DrawFromBinomDist (n, p, ix, randomvalue)
  END IF

  RETURN
END SUBROUTINE DrawFromDistri
! ****<Here the subroutine ends>*******************************************************************



! ****<Here the subroutine DrawFromPoissonDist starts>*********************************************
! -------------------------------------------------------------------------------------------------
SUBROUTINE DrawFromPoissonDist(mu, x, randomvalue)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
REAL,    INTENT(IN)  :: mu,         &   ! parameter                     [=..]
                        randomvalue     ! [0,1] random value            [=..]
INTEGER, INTENT(OUT) :: x               ! looping variable for latitude [=..]

!<Local variables>---------------------------------------------------------------------------------
DOUBLE PRECISION :: distrival, distrisum, ri, fak, mud
INTEGER          :: i

!<Here the subroutine starts>**********************************************************************
  x         = 0
  mud       = mu
  distrival = EXP (-mud)
  distrisum = distrival

  DO i=1,10000
    ri = i
    IF ((distrisum .ge. randomvalue) .or. (distrisum .ge. 1.0)) EXIT
    fak       = mu / ri
    distrival = distrival * fak
    distrisum = distrisum + distrival
  END DO

  x = ri - 1
  RETURN
END SUBROUTINE DrawFromPoissonDist
! ****<Here the subroutine ends>*******************************************************************



! ****<Here the subroutine DrawFromBinomDist starts>***********************************************
! -------------------------------------------------------------------------------------------------
SUBROUTINE DrawFromBinomDist(n, p, x, randomvalue)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)  :: n              ! number of species
REAL,    INTENT(IN)  :: p,          &  ! probability                      [=..]
                        randomvalue    ! [0,1] random value               [=..]
INTEGER, INTENT(OUT) :: x              ! looping variable for latitude    [=..]

!<Local variables>---------------------------------------------------------------------------------
INTEGER          :: i
REAL             :: pTq
DOUBLE PRECISION :: distrival, distrisum, rn, ri, fak, fak1, fak2

!<Here the subroutine starts>**********************************************************************
  x         = 0
  distrival = (1.0 - p) ** n
  distrisum = distrival
  rn        = n
  pTq       = p /( 1.0 - p)
  DO i=1,n
    IF (distrisum .ge. randomvalue) EXIT
    ri        = i
    fak1      = (rn - ri + 1.0)
    fak2      = fak1 / ri
    fak       = pTq * fak2
    distrival = distrival * fak
    distrisum = distrisum + distrival
    x         = i
  END DO
RETURN
END SUBROUTINE DrawFromBinomDist
!****<Here the subroutine ends>********************************************************************




! ****<Here the subroutine DrawFromNormalDist starts>**********************************************
! -------------------------------------------------------------------------------------------------
SUBROUTINE DrawFromNormalDist(mu, s, rx, randomvalue)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
REAL, INTENT(IN)  :: mu,         &  ! mean               [=..]
                     s,          &  ! standard deviation [=..]
                     randomvalue    ! [0,1] random value [=..]
REAL, INTENT(OUT) :: rx             ! real output value  [=..]

!<Local variables>---------------------------------------------------------------------------------
INTEGER          :: i, index, istart
REAL             :: xtransf,  &
                    v1, f1,   &
                    v2, f2,   &
                    nd(0:201)     !                                       [=NormDistFuncArr(index)]

! <Here the subroutine starts>*********************************************************************
! <First, initialize the Standard Normal Distribution array nd(..)>------------
  nd=(/0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0000106885,0.0000133457,      &
       0.0000166238,0.0000206575,0.0000256088,0.0000316712,0.0000390756,0.0000480963,0.0000590589, &
       0.000072348,0.0000884173,0.0001078,0.00013112,0.000159109,0.000192616,0.000232629,          &
       0.000280293,0.000336929,0.000404058,0.000483424,0.000577025,0.000687138,0.000816352,        &
       0.000967603,0.00114421,0.0013499,0.00158887,0.00186581,0.00218596,0.00255513,0.00297976,    &
       0.00346697,0.00402459,0.00466119,0.00538615,0.00620967,0.00714281,0.00819754,0.00938671,    &
       0.0107241,0.0122245,0.0139034,0.0157776,0.0178644,0.0201822,0.0227501,0.0255881,0.0287166,  &
       0.0321568,0.0359303,0.0400592,0.0445655,0.0494715,0.0547993,0.0605708,0.0668072,0.0735293,  &
       0.0807567,0.088508,0.0968005,0.10565,0.11507,0.125072,0.135666,0.146859,0.158655,0.171056,  &
       0.18406,0.197663,0.211855,0.226627,0.241964,0.257846,0.274253,0.29116,0.308538,0.326355,    &
       0.344578,0.363169,0.382089,0.401294,0.42074,0.440382,0.460172,0.480061,0.5,0.519939,        &
       0.539828,0.559618,0.57926,0.598706,0.617911,0.636831,0.655422,0.673645,0.691462,0.70884,    &
       0.725747,0.742154,0.758036,0.773373,0.788145,0.802337,0.81594,0.828944,0.841345,0.853141,   &
       0.864334,0.874928,0.88493,0.89435,0.9032,0.911492,0.919243,0.926471,0.933193,0.939429,      &
       0.945201,0.950529,0.955435,0.959941,0.96407,0.967843,0.971283,0.974412,0.97725,0.979818,    &
       0.982136,0.984222,0.986097,0.987776,0.989276,0.990613,0.991802,0.992857,0.99379,0.994614,   &
       0.995339,0.995975,0.996533,0.99702,0.997445,0.997814,0.998134,0.998411,0.99865,0.998856,    &
       0.999032,0.999184,0.999313,0.999423,0.999517,0.999596,0.999663,0.99972,0.999767,0.999807,   &
       0.999841,0.999869,0.999892,0.999912,0.999928,0.999941,0.999952,0.999961,0.999968,0.999974,  &
       0.999979,0.999983,0.999987,0.999989,0.999991,0.999993,0.999995,0.999996,0.999997,0.999997,  &
       0.999998,0.999998,0.999999,0.999999,0.999999,0.999999,1.0,1.0,1.0,1.0/)
  rx = 0
  IF (randomvalue .eq. 0.0) THEN
    rx = -5.0 * s + mu
    RETURN
  END IF

  IF (randomvalue .gt. 0.5) THEN
   istart = 100
  ELSE
   istart = 0
  END IF

  DO i=istart,istart+101
    IF (nd(i) .ge. randomvalue) THEN
       index = i
       EXIT
    END IF
  END DO

  f1      = nd(index - 1)
  f2      = nd(index)
  v1      = (index - 1) / 20.0 - 5.0
  v2      = (index)     / 20.0 - 5.0
  xtransf = v1 + (v2 - v1) * (randomvalue - f1) / (f1 - f2)
  rx      = xtransf * s + mu

  RETURN
END SUBROUTINE DrawFromNormalDist
!****<Here the subroutine ends>********************************************************************
