!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  GetBioClim
!
!  Purpose:  They calculate the spatial forest dynamics on a grid with or
!            without seed dispersal
!
!  Remarks:  contains the subroutines
!
!            GetBioClim (<GetEnvFactors<DistrLocalDynThisTime<TimeLoop<TreeMig)
!            Interpolate (<GetBioClim<GetEnvFactors<DistrLocalDynThisTime<
!                           TimeLoop<TreeMig)
!            GetVariate (<GetBioClim<GetEnvFactors<DistrLocalDynThisTime<
!                          TimeLoop<TreeMig)
!
!==============================================================================


SUBROUTINE GetBioClim(yr, lat, lon, ddegs, wtemp, drstr)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: newBioClim, oldBioClim, simustartyear

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: yr, lat, lon
REAL, INTENT(OUT)   :: ddegs, wtemp, drstr

!<Local variables>---------------------------------------------------------------------------------
REAL :: muDD, stdDD, muWiT, stdWiT, prop0DrStr, muDrStr, stdDrStr, oldT, newT, randomvalue, year

!<Here the subroutine starts>
  year = yr + simustartyear
  oldT = oldBioClim%cctime
  newT = newBioClim%cctime

  CALL Interpolate (year, oldT, newT, oldBioClim%bc(lat,lon)%muDD, &
                    newBioClim%bc(lat,lon)%muDD, muDD)
  CALL Interpolate (year, oldT, newT, oldBioClim%bc(lat,lon)%stdDD, &
                    newBioClim%bc(lat,lon)%stdDD, stdDD)
  CALL GetVariate (muDD, stdDD, ddegs)
  CALL Interpolate (year, oldT, newT, oldBioClim%bc(lat,lon)%muWiT, &
                    newBioClim%bc(lat,lon)%muWiT, muWiT)
  CALL Interpolate (year, oldT, newT, oldBioClim%bc(lat,lon)%stdWiT, &
                    newBioClim%bc(lat,lon)%stdWiT, stdWiT)
  CALL GetVariate (muWiT, stdWiT, wtemp)
  CALL Interpolate (year, oldT, newT, oldBioClim%bc(lat,lon)%prop0DrStr, &
                    newBioClim%bc(lat,lon)%prop0DrStr, prop0DrStr)
  CALL RANDOM_NUMBER (randomvalue)

  IF (randomvalue <= prop0DrStr) THEN
    drstr = 0.0
  ELSE
    CALL Interpolate (year, oldT, newT, oldBioClim%bc(lat,lon)%muDrStr, &
                      newBioClim%bc(lat,lon)%muDrStr, muDrStr)
    CALL Interpolate (year, oldT, newT, oldBioClim%bc(lat,lon)%stdDrStr, &
                      newBioClim%bc(lat,lon)%stdDrStr, stdDrStr)

    CALL GetVariate (muDrStr, stdDrStr, drstr)
    IF (drstr < 0.0) drstr = 0.0
    IF (drstr > 1.0) drstr = 1.0
  END IF

END SUBROUTINE GetBioClim
!****<Here the subroutine ends>*******************************************************************



SUBROUTINE GetBioClimSimple(yr, lat, lon, ddegs, wtemp, drstr, smday)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: newBioClim, oldBioClim, simustartyear

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: yr, lat, lon
REAL, INTENT(OUT)   :: ddegs, wtemp, drstr, smday

!<Local variables>---------------------------------------------------------------------------------
REAL :: muDD, stdDD, muWiT, stdWiT, prop0DrStr, muDrStr, stdDrStr, oldT, newT, randomvalue, year

!<Here the subroutine starts>
  year = yr + simustartyear
  oldT = oldBioClim%cctime
  newT = newBioClim%cctime
  ddegs=newBioClim%bc(lat,lon)%muDD
  wtemp=newBioClim%bc(lat,lon)%muWiT
  drstr=newBioClim%bc(lat,lon)%muDrStr
  smday=newBioClim%bc(lat,lon)%smday
    IF (drstr < 0.0) drstr = 0.0
    IF (drstr > 1.0) drstr = 1.0


END SUBROUTINE GetBioClimSimple
!****<Here the subroutine ends>*******************************************************************



SUBROUTINE Interpolate(x, x1, x2, y1, y2, y)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
REAL, INTENT(IN) :: x, x1, x2, y1, y2
REAL, INTENT(OUT):: y

!<Here the subroutine starts>
  IF (ABS (x1-x2) < 1.E-20) THEN
    Write(1070,*) 'x1 = x2  bei interpolation ', x1,x2
    STOP
  END IF

  y = y1 + (x - x1) * (y2 - y1) / (x2 - x1)
END SUBROUTINE Interpolate
!****<Here the subroutine ends>********************************************************************




SUBROUTINE  GetVariate(mu,stdev,x)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
REAL, INTENT(IN)  :: mu, stdev
REAL, INTENT(OUT) :: x

!<Local variables>---------------------------------------------------------------------------------
REAL :: randomvalue

!<Here the subroutine starts>
  CALL RANDOM_NUMBER (randomvalue)
  CALL DrawFromNormalDist (mu, stdev, x, randomvalue)

END SUBROUTINE GetVariate
!****<Here the subroutine ends>********************************************************************
