!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Addition to FORHYCS MPI structure

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  GetSpc
!
!  Purpose:  Reads species parameters and calculates constant species
!            specific values from them
!
!  Remarks:  contains the subroutine
!
!            GetSpc (<TreeMig)
!
!==============================================================================

SUBROUTINE GetSpc(nspc, errtyp)

!<CALL modules for typedefinitions>---------------------------------------------------------------
USE All_par, ONLY: maxhc, birht, maxspc, a1, a2, c1, c2, kn1, kn2,                  &
                   alpha, simustartyear, experimentID, toplt, maxlc,                &
                   spec, spc,  htmax, hcwidth, seedsDifferent, mathgtDifferent, &
                   dispersalDifferent, mastSeeding,                                 &
                   specfilename,                                                    &
                   facvarh, facvars, facvarsh,                                      &
                   tabsepOutput,kernelType, pathletter1,            &
                   pathletter2, readDispersalKernel, cellSideLength,maxkerneldim,   &
                    kernel

use prevahglobals, ONLY: diroutput
use MPIVars, only: rank, root
use forhycsglobals, only: tpswitch, TMdir

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(OUT) :: nspc,  &     ! number of species[=spec]
                        errtyp       ! number of species[=spec]

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: errtypimmi, &
           err,        & ! I/O error handling
           ht,         & ! height index for calculating seedsproduced per height class      ! DSP
           i,          & ! looping variable for species and immigration dates
           ii, jj, tmpos,     &
            ltcl,j1,j2
REAL    :: sumprop,    &     ! sum to calculate proportion of seedper height class (of max)    ! DSP
           swidth,     &     ! step width for scaling seedfraction along height classes        ! DSP
           seedMaxAge, &     ! maximum seed ager
           lih, lidep

CHARACTER*1 :: tab
CHARACTER*140 header
character*256 :: specPath
LOGICAL     :: finish, uniqueDispersalKernel

!<Here the subroutinestarts>**********************************************************************
  finish = .FALSE.

  !----- <First, open file for reading species parameter informations>
  errtyp = 0
  IF (readDispersalKernel) THEN
    CALL OpenExistingFile ('dispKern.txt', 1100, err)
    IF (err .ne. 0) THEN
       errtyp = 1
       RETURN
    END IF
  END IF
!print*,'errtyp1',errtyp

specPath = trim(TMdir)//'P/'//specfilename
!  CALL OpenExistingFile (pathletter1//'P'//pathletter2//specfilename , 1012, err)
  CALL OpenExistingFile (specPath , 1012, err)
  print*,specfilename
  IF (err .ne. 0) THEN
    errtyp = 2
    RETURN
  END IF
!print*,'errtyp2',errtyp

  !----- <Skip first 4 lines>
    READ(1012,"(A)") header

    !print*, header, rank
   nspc = 0

  Read_Spec: DO i=1,maxspc
    !----- Read data strings
    nspc = nspc + 1
    READ (1012,*,IOSTAT=err) &
         spec(i)%nl, spec(i)%ns, spec(i)%stb, spec(i)%stn, spec(i)%kdmax,                       &
         spec(i)%khmax, spec(i)%kamax, spec(i)%kg,spec(i)%kddmin, spec(i)%kddmax,spec(i)%kwit,  &
         spec(i)%kdrt, spec(i)%kntol, spec(i)%kbrow,spec(i)%klighs, spec(i)%kligha,spec(i)%klq, &
         spec(i)%kimm, spec(i)%resld, spec(i)%ploto,spec(i)%par0, spec(i)%par1,spec(i)%par2,    &
         spec(i)%minmat,                                                                        &
         spec(i)%seedGerm, spec(i)%seedLoss, seedMaxAge,                                        &
         spec(i)%period, spec(i)%maxseed, spec(i)%dispFac,spec(i)%alfa1,spec(i)%alfa2, spec(i)%repFac,&
         spec(i)%red, spec(i)%green, spec(i)%blue
!       print*, 'i, maxspc', i, maxspc, rank
!      print*, 'spec(i)%nl', spec(i)%nl
!      print*, 'spec(i)%ns', spec(i)%ns
!      print*, 'spec(i)%stb', spec(i)%stb
!      print*, 'spec(i)%stn', spec(i)%stn
!      print*, 'spec(i)%kdmax', spec(i)%kdmax
!      print*, 'spec(i)%khmax', spec(i)%khmax
!      print*, 'spec(i)%kamax', spec(i)%kamax
!      print*, 'spec(i)%kg', spec(i)%kg
!      print*, 'spec(i)%kddmin', spec(i)%kddmin
!      print*, 'spec(i)%kddmax', spec(i)%kddmax
!      print*, 'spec(i)%kwit', spec(i)%kwit
!      print*, 'spec(i)%kdrt', spec(i)%kdrt
!      print*, 'spec(i)%kntol', spec(i)%kntol
!      print*, 'spec(i)%kbrow', spec(i)%kbrow
!      print*, 'spec(i)%klighs', spec(i)%klighs
!      print*, 'spec(i)%kligha', spec(i)%kligha
!      print*, 'spec(i)%klq', spec(i)%klq
!      print*, 'spec(i)%kimm', spec(i)%kimm
!      print*, 'spec(i)%resld', spec(i)%resld
!      print*, 'spec(i)%ploto', spec(i)%ploto
!      print*, 'spec(i)%par0', spec(i)%par0
!      print*, 'spec(i)%par1', spec(i)%par1
!      print*, 'spec(i)%par2', spec(i)%par2
!      print*, 'spec(i)%minmat', spec(i)%minmat
!      print*, 'spec(i)%seedGerm', spec(i)%seedGerm
!      print*, 'spec(i)%seedLoss', spec(i)%seedLoss
!      print*, 'seedMaxAge', seedMaxAge

     !print*, "Species read in, err, rank:", i, err, rank
    spec(i)%seedSurv = 0.05 ** (1.0 / seedMaxAge)
    spec(i)%nb       = i
    !----- If EOF, then exit the read loop
    Exit_Loop: IF (err /= 0) THEN
                 nspc = nspc - 1
                 EXIT Read_Spec
    END IF Exit_Loop
  END DO Read_Spec

  !----- <Put species parameters that are used into new derived type"spc">
  spec%maxdhm = REAL (spec%kdmax) / 100.0
  spec%maxgro = REAL (spec%kg) / 100.0

  DefMort: DO i=1,nspc
    spec(i)%cmort = 4.605 / REAL (spec(i)%kamax)
  END DO DefMort

  spec%tb = 2.0
  WHERE (spec%stb == "C") spec%tb = 1

  spec%lthres = (0.1 * REAL (spec%klighs)) - 0.4
  WHERE (spec%klighs < 5)   spec%lthres = 0.025 * REAL (spec%klighs - 1.0)
  WHERE (spec%lthres < 0.0) spec%lthres = 0.0

  DO i = 1, nspc
    DO ltcl = 1,maxlc
      lih = REAL(ltcl * (ltcl + 1)) / REAL (maxlc * (maxlc + 1))
      CALL ContLightDep (lih, spec(i)%kligha, lidep)
      spc(i)%lightDepA(ltcl) = lidep
      CALL ContLightDep (lih, spec(i)%klighs, lidep)
      spc(i)%lightDepE(ltcl) = lidep
    END DO
  END DO

  !----- <Calculate absolute MAX height over all trees>
  htmax   = MAXVAL (spec%khmax)
  hcwidth = (REAL (htmax) - birht) / REAL (maxhc - 1)    !----- Heike's version

  !----- <Calculate absolute MAX height over all trees of one species>
  spec%maxhtc = MIN (FLOOR (REAL (maxhc - 1) * &
                (REAL (spec%khmax) - birht) / (REAL (htmax) - birht) + 1.0), maxhc) !----- Heike's version

  !----- Set values to the parameters for LA & BIOMASS calculations
  a1 = (/EXP (-2.5),EXP (-2.3),EXP (-2.9),EXP (-1.8),EXP (-1.5)/)
  a2 = (/1.4,1.4,1.7,1.4,1.6/)
  c1 = (/0.45,0.35/)
  c2 = (/6.0,12.0/)

  kn1 = (/-0.016,-0.022,-0.016/)
  kn2 = (/ 2.245,30.605,43.973/)

  !----- tools for seed dispersalmodeling
  !----- needs to be read from parameter file later
  !----- min. maturation height, is read from parameter file
  IF (.NOT. mathgtDifferent) &
    spec%minmat = birht + 0.8 * (spec%khmax - birht)

  !----- This is converting the minmat variable into a real number of classproportion
  spec%mathcp =((REAL (maxhc - 1) * (spec%minmat - birht) / (REAL (htmax) - birht)) + 1.0)

  !----- This is setting up the fraction of seeds produced by height class of the potential maximum
  Init_seed: DO i=1,nspc
  !----- Here max. seed number, dispersal distance and mast seeding period are set to standard values, if these scenarios are not used
    IF (.NOT. seedsDifferent) &
      spec(i)%maxseed = 10000.0 * spec(i)%khmax / htmax          !----- read from parameterfile
    IF (.NOT. dispersalDifferent) spec(i)%alfa1 = alpha
    IF (.NOT. mastSeeding) spec(i)%period = 1.0

    sumprop       = 0.0
    spc(i)%mxsdht = 0.0

    !----- Here the amount of seeds produced in each height class per tree is calculated (ask Nick for details...)
    IF (spec(i)%mathcp > REAL (spec(i)%maxhtc)) THEN
      spec(i)%mathcp = spec(i)%maxhtc
      swidth         = 11.0 / REAL (spec(i)%maxhtc - FLOOR (spec(i)%mathcp) + 1) !----- stepwidth to scale grad. of htcl
    ELSE
      swidth         = 11.0 / REAL (spec(i)%maxhtc - FLOOR (spec(i)%mathcp) + 1) !----- stepwidth to scale grad. of htcl
    END IF

    Init_sdht: DO ht=FLOOR(spec(i)%mathcp),spec(i)%maxhtc
      !----- range between 0 = 10 for x along height classes (ranging fromminmat to maxhtc); divided by
      !----- 100.0 to scale from 0.1 to 1.0; ((ht-1)*swidth)=x of y=(10.+(.9*x**2))/100; scaled by sum!!
      spc(i)%mxsdht(ht) = (10. + (.9 * ((REAL (ht - FLOOR (spec(i)%mathcp)) * swidth) ** 2))) / 100.0
      sumprop           = sumprop + spc(i)%mxsdht(ht)
    END DO Init_sdht

    spc(i)%mxsdht         = spc(i)%mxsdht / sumprop


! ----- Calculate dispersal kernels a priori.
     uniqueDispersalKernel = .TRUE.
!-----  If same parameters already in other species, take that kernel
    DO ii = 1, i-1
       IF ((spec(i)%alfa1   .eq. spec(ii)%alfa1) .AND. &
          (spec(i)%alfa2   .eq. spec(ii)%alfa2) .AND. &
          (spec(i)%dispFac .eq. spec(ii)%dispFac)) THEN
        spec(i)%rad           = spec(ii)%rad
 !--- store the kernel values of already calculated kernel to the current kernel;
        DO j1=-spec(i)%rad,spec(i)%rad
         DO j2=-spec(i)%rad,spec(i)%rad
           spec(i)%kernel(j1,j2)       = spec(ii)%kernel(j1,j2)
         END DO
        END DO
 !       spec(i)%kernel        = spec(ii)%kernel  ! HL:didnt work with my Absoft, has been replaced by loops above; d
         uniqueDispersalKernel = .FALSE.
        EXIT
      END IF
    END DO!ii

    IF (readDispersalKernel) THEN
      READ(1100,*) spec(i)%rad
      DO ii = -spec(i)%rad, spec(i)%rad
        READ(1100,*) (spec(i)%kernel(ii,jj),jj=-spec(i)%rad, spec(i)%rad)
      END DO
    ELSE
!-----  If these parameters appear first time, kernel has to be calculated
      IF (uniqueDispersalKernel) THEN
        kernelType = 1
        CALL DispersalKernel (i)
        IF (spec(ii)%dispFac .lt. 1.0) THEN
          kernelType = 2
          CALL DispersalKernel (i)
        END IF
      END IF
 ! Write the kernel to "newdispersalkernels.txt"
!    print *, 'art, rad in dispkern ', i, spec(i)%rad
      IF (i .eq. 1) THEN
        DO ii = -spec(i)%rad, spec(i)%rad
          DO jj = -spec(i)%rad, spec(i)%rad
            Write (1110,*) cellSideLength, i, spec(i)%rad,ii,jj, cellSideLength * ii,cellSideLength * jj, spec(i)%kernel(ii,jj)
          END DO
        END DO
      END IF
    END IF !----- (readDispersalKernel)
  END DO Init_seed

  facvars(1) = 1.0
  SHet: DO i=1,maxhc
       facvars(i) = 1.0
  END DO SHet

  !print*, "nspc in GetSpc", nspc, rank

  !----- output of species and immigration input

  if (rank==root) then

      IF ( tabsepOutput) THEN
      tab = CHAR(9)
      ELSE
      tab = ','
      END IF
      CALL POSNAM (diroutput,1,tmpos)
      OPEN(UNIT=1052, FILE=diroutput(1:tmpos)//'treemig'//pathletter2//'R'//pathletter2//experimentID//'_'//&
                tpswitch//'_species.pars', STATUS='UNKNOWN')
      WRITE (1052,*) 'Species',         tab, 'Abbrv',        tab, 'sType/B',      tab, 'sType/N',      tab, 'kDMax',           tab, &
               'kHMax',           tab, 'kAMax',        tab, 'kG',           tab, 'kDDMin',       tab, 'kDD75',          tab, &
               'kWiT',            tab, 'kDrT',         tab, 'kNTol',        tab, 'kbrow',        tab, 'klighs',          tab, &
               'kligha',          tab, 'klq',          tab, 'kimm',         tab, 'resID',        tab, 'ploto',           tab, &
               'par0',            tab, 'par1',         tab, 'par2',         tab, 'inmat',        tab, 'seedGerm',        tab, &
               'seedLoss',        tab, 'seedMaxAge',   tab, 'period',       tab, 'maxseed',      tab, 'dispFac',         tab, &
               'alfa1',           tab, 'alfa2',        tab, 'repFac', tab, 'red', tab, 'green', tab, 'blue'
      DO i=1,nspc
      WRITE (1052,*) spec(i)%nl,       tab, spec(i)%ns,    tab, spec(i)%stb,    tab, spec(i)%stn,     tab, spec(i)%kdmax,    tab  , &
                  spec(i)%khmax,    tab, spec(i)%kamax, tab, spec(i)%kg,     tab, spec(i)%kddmin,  tab, spec(i)%kddmax,   tab, &
                  spec(i)%kwit,     tab, spec(i)%kdrt,  tab, spec(i)%kntol,  tab, spec(i)%kbrow,   tab, spec(i)%klighs,   tab, &
                  spec(i)%kligha,   tab, spec(i)%klq,   tab, spec(i)%kimm,   tab, spec(i)%resld,   tab, spec(i)%ploto,    tab , &
                  spec(i)%par0,     tab, spec(i)%par1,  tab, spec(i)%par2,   tab, spec(i)%minmat,  tab, spec(i)%seedGerm, tab , &
                  spec(i)%seedLoss, tab, seedMaxAge,    tab, spec(i)%period, tab, spec(i)%maxseed, tab  ,                        &
                  spec(i)%dispFac,  tab, spec(i)%alfa1, tab, spec(i)%alfa2,  tab, spec(i)%repFac , tab, &
                  spec(i)%red,  tab, spec(i)%green,  tab, spec(i)%blue

      END DO


      CLOSE(1052)

  end if  ! Master

  CLOSE(1012)
  CLOSE(1013)

  !----- Immigration input

!  print *, 'vor call readimmigrationData', nspc
  CALL ReadImmigrationData (nspc, errtypimmi)
    !print *, 'nach call readimmigrationData', nspc , errtypimmi
  IF (errtypimmi .ne. 0) THEN
    errtyp = errtypimmi
    RETURN
  END IF

 END SUBROUTINE GetSpc
!****<Here the subroutine ends>*****************************
