!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  GetStd
!
!  Purpose:  Here, cell specific, but constant information is set
!
!  Remarks:  contains the subroutine
!
!            GetStd (<TreeMig)
!
!==============================================================================


SUBROUTINE GetStd(nspc)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: maxhc, maxlc, toplt, pltsiz, birht, a1, a2, c1, c2, slow, maxlat, maxlon, edge, &
                   ltfrq, ltcum, htcl, ltcl, ltval, la,                                            &
                   spc, cell, seedrain,spec, hcwidth, htmax
use MPIVars, only: rank, root, firstRow, lastRow
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: nspc    ! Passed number of species                                   [=spec]

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: latIni, &     ! tmp var to initialize  tree array          !DSP
           lonIni, &     ! tmp var to initialize  tree array          !DSP
           i             ! looping variable for species               !DSP
REAL    :: argmt,     &  ! tmp var to calculate nexdm                                       [=argu]
           truht,     &  ! tmp var to calculate heights                               [=trueHeight]
           nexht,     &  ! tmp var to calculate heights                               [=nextHeight]
           diamt,     &  ! tmp var to calculate diameters               [in DetermineTree..  =diam]
           nexdm,     &  ! tmp var to calculate diameters                               [=nextDiam]
           lndia,     &  !                                                               [=rLnDiam]
           bio           ! tmp var to calculate biomass                      [=BioMassAtHeight(..)]

real, dimension(31) :: cra, crb  ! Species-specific parameters for calculating crown area, taken from
                                 ! N. Zurbriggen's PhD thesis, p. 63.
real :: ca

!<Here the subroutine starts>**********************************************************************

! These parameters are hard-coded for now. Possibly include them in species parameter file later?
! Matthias

  print*, "Start GetStd", rank

  data cra /0.0161, 0.0077, 0.0067, -0.0073, -0.0054, -0.014, -0.014, & !Conifers
    -0.0177, -0.0177, 0.0225, 0.0061, 0.0061, 0.08, -0.0197, 0.0052, 0.0, & ! Last one is Castanea sativa - no value!
    -0.0197, 0.0052, -0.0067, 0.0, 0.0, 0.0532, 0.0532, -0.0214, -0.016, &
    -0.0611, -0.0611, 0.0674, 0.0674, -0.0185, 0.0/

  data crb /0.8038, 1.2468, 1.0325, 1.579, 0.8972, 1.6888, 1.6888, & ! Conifers
    1.8758, 1.8758, 0.8625, 0.8672, 0.8672, 0.8672, 1.6764, 2.0096, 0.0, & ! Last one is Castanea sativa - no value!
    1.6764, 2.0096, 1.9035, 2.432, 2.432, 0.8246, 0.8246, 2.5864, 2.5106, &
    2.7725, 2.7725, 0.8341, 0.8341, 1.7834, 0.0/


  !----- <The first part is what DISCFORM calls "DetermineLightInLightClasses">
  !----- <Represents "LightAboveCanopy">
  ltfrq        = 0.0
  ltcum        = 1.0
  ltfrq(maxlc) = 1.0

  !----- <Initialize LAI-array [=DetermineLightInLightClasses]>
  ByLtCls1: DO ltcl=1,maxlc
    ltval(ltcl) = REAL (ltcl * (ltcl + 1)) / REAL (maxlc * (maxlc + 1))
    la(ltcl)    = -LOG (MAX (ltval(ltcl) / toplt , 0.0000001)) / .25
  END DO ByLtCls1

  !----- <Initialize tree array [=DetermineTreeSizeVariables]>
  BySpcs1: DO i=1,nspc

      !print*, "Before cell loops_1", rank, i
      !print*, "maxlon", maxlon
      !print*, "shape cell", shape(cell)

    !----- <Set the species presence to TRUE: Later, this has to be changed>
    !----- <dynamically through seed dispersal based presence calculations.>
    !----- <The main part of this is in the subroutine INTERACT>
    LatLoop1: DO latIni=firstRow, lastRow   !1,maxlat
      LonLoop1: DO lonIni=1,maxlon
        cell(latIni,lonIni)%sp(i)%there  = .TRUE.
        IF (slow) THEN                 !----- For slowing down initial growth
          cell(latIni,lonIni)%sp(i)%htmps  = birht
          cell(latIni,lonIni)%sp(i)%htdmp  = 0.0
        END IF

        !----- Here the  state variable values for all height classes are initialized
        ByHtCls11: DO htcl=0,maxhc
          cell(latIni,lonIni)%sp(i)%numin(htcl ) = 0.0
        END DO ByHtCls11

        cell(latIni,lonIni)%sp(i)%antagonist  = 10.0
      END DO LonLoop1
    END DO LatLoop1

    !print*, "After cell loops_1", rank, i

    nexht = birht
    argmt = MIN (MAX (1.0 - ((nexht - birht) / (REAL (spec(i)%khmax) - birht)), 0.0), 1.0)
    nexdm = spec(i)%maxdhm * (1.0 - SQRT (argmt)) + 0.0127

    !----- Here height class dependent but constant values are calculated: biomass, diameter, LAI
    ! Matthias Jul 2015: Crown area
    ByHtCls1: DO htcl=1,spec(i)%maxhtc
      truht              = nexht
      nexht              = MIN ((REAL (htcl) * hcwidth) + birht, REAL (spec(i)%khmax))
      spc(i)%htmin(htcl) = truht
      diamt              = nexdm
      argmt              = MIN (MAX (1.0 - ((nexht - birht) / (REAL (spec(i)%khmax) - birht)), 0.0), 1.0)
      nexdm              = spec(i)%maxdhm * (1.0 - SQRT (argmt)) + 0.0127
      spc(i)%diain(htcl) = diamt
      ca = cra(i) * truht**2 + crb(i) * truht
      spc(i)%crArea(htcl) = ca
      IF (diamt .gt. 0.0) THEN
        lndia = LOG (diamt * 100.0)
      ELSE
        lndia = 0
      END IF

      spc(i)%lain(htcl) = c2(spec(i)%tb) * a1(spec(i)%stn) * EXP (lndia * a2(spec(i)%stn))
      IF (nexdm > diamt) THEN
        CALL Biomass (diamt, a1(spec(i)%stn), a2(spec(i)%stn), c1(spec(i)%tb), bio)
        spc(i)%bioin(htcl) = bio * 10 / pltsiz
        !NZ: use factor 10000 instead of 10 if biomass is stem volume
        ! factor 10 transforms kg/m2 to t/ha, if you use biomass (normal version)
        ! factor 10000 transforms into dm3 per ha (NOT m3!!) if stem volume is used instead of biomass
        ! (see subroutines Biomass, and Report)

      ELSE
        CALL Biomass (diamt, a1(spec(i)%stn), a2(spec(i)%stn), c1(spec(i)%tb), bio)
        spc(i)%bioin(htcl) = bio * 10 / pltsiz
        !NZ: use factor 10000 instead of 10 if biomass is stem volume

        CALL Biomass (nexdm, a1(spec(i)%stn), a2(spec(i)%stn), c1(spec(i)%tb), bio)
        spc(i)%bioin(htcl) = (spc(i)%bioin(htcl) + (bio * 10 / pltsiz)) / 2.0
        !NZ: use factor 10000 instead of 10 if biomass is stem volume

      END IF
    END DO ByHtCls1

    !----- Here the  values for the lowest height class (seedlings up to 1.37 m) is initialized
    spc(i)%htmin(0) = 0.0
    spc(i)%diain(0) = 0.0
    spc(i)%lain(0) = spc(i)%lain(1) / 10.0
    spc(i)%bioin(0) = spc(i)%bioin(1) / 10.0
    spc(i)%crArea(0) = spc(i)%crArea(1) / 10.0

    !print*, "Before cell loops_2", rank

    LatLoop2: DO latIni=firstRow,lastRow    !1,maxlat
      LonLoop2: DO lonIni=1,maxlon
        cell(latIni,lonIni)%sp(i)%numin(0 ) = 0.0
      END DO LonLoop2
    END DO LatLoop2

    !print*, "After cell loops_2", rank

    !----- Here the proportion of the seed production is determined proportional to LAI
    ByHtCls3: DO htcl=0,spec(i)%maxhtc
      IF (spc(i)%htmin(htcl)>=spec(i)%minmat) THEN
        spc(i)%mxsdht(htcl) = spc(i)%lain(htcl) / spc(i)%lain(spec(i)%maxhtc)
      ELSE
        spc(i)%mxsdht(htcl) = 0
      END IF
    END DO ByHtCls3

    !----- Here the  values for all  height classes higher than the max. one are initialized
    IF (spec(i)%maxhtc < maxhc) THEN
      RestHtCls2: DO htcl=spec(i)%maxhtc+1,maxhc
        spc(i)%htmin(htcl)  = REAL (spec(i)%khmax) + 1.0
        spc(i)%diain(htcl)  = spec(i)%maxdhm
        spc(i)%lain(htcl)   = 0.0
        spc(i)%bioin(htcl)  = 0.0
        spc(i)%mxsdht(htcl) = 0
        spc(i)%crArea(htcl) = 0.0

        !print*, "Before cell loops_3", rank

        LatLoop3: DO latIni=firstRow,lastRow     !1,maxlat
          LonLoop3: DO lonIni=1,maxlon
            cell(latIni,lonIni)%sp(i)%numin(htcl ) = 0.0
          END DO LonLoop3
        END DO LatLoop3

        !print*, "After cell loops_3", rank

      END DO RestHtCls2

      spc(i)%htmin(spec(i)%maxhtc+1) = spec(i)%khmax
      spc(i)%htmin(maxhc)            = htmax
    END IF

    !----- Currently, the caches are initialized here! Needs to be adjusted later
    !----- Initialize the study area

    !----- Initialize seed sources (matrix) and cache sites

    !print*, "Before cell loops_4", rank
    !print*, "shape seedrain", shape(seedrain)

    LoopLatInit: DO latIni=firstRow,lastRow     !1,maxlat
      LoopLonInit: DO lonIni=1,maxlon
        cell(latIni,lonIni)%sp(i)%caches  = 1.0
        cell(latIni,lonIni)%sp(i)%seedbed =0.0
         seedrain(latIni,lonIni)%sp(i)%newSeeds =0.0

      END DO LoopLonInit
    END DO LoopLatInit

    !print*, "After cell loops_4", rank

  END DO  BySpcs1

END SUBROUTINE GetStd
!****<Here the subroutine ends>********************************************************************
