!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation to the FORHYCS MPI structure

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------



!==============================================================================
!
!  Name   :  Interact
!
!  Remarks:  contains the subroutines
!
!            SeedsDispersedFromThisCell (<DoLocalWork<DistrLocalDynThisTime<
!                                          TimeLoop<TreeMig)
!            SeedsDispFromThisCellAndSpec (<SeedsDispersedFromThisCell<DoLocalWork<
!                                            DistrLocalDynThisTime<TimeLoop<TreeMig)
!            UpdateNewSeeds (<DistrLocalDynThisTime<TimeLoop<TreeMig)
!            SeedsFromOutsSimRegion (<DistrLocalDynThisTime<TimeLoop<TreeMig)
!            BoundaryConditions (<SeedsDispFromThisCellAndSpec<SeedsDispersedFromThisCell<
!                                  DoLocalWork<DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================


!****<Here the subroutine starts>********************************************************************
SUBROUTINE SeedsDispersedFromThisCell (nspc, lat, lon,year, Iam )
 IMPLICIT NONE
!<Passed variables>--------------------------------------------------------------------------------
INTEGER,         INTENT(IN) :: nspc, lat, lon ,year, Iam   ! number of species
INTEGER  ::isp

DO isp=1, nspc
  CALL SeedsDispFromThisCellAndSpec(isp, lat, lon,year, Iam )
END DO

RETURN
END Subroutine SeedsDispersedFromThisCell
!****<Here the subroutine ends>




 SUBROUTINE SeedsDispFromThisCellAndSpec(isp,lat,lon,year,Iam)
!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: CurrStateInCell, SeedC,SeedCSpec,&
                   maxlat, maxlon,maxspc, maxSeededCells, active, edge,                         &
                   cell,seedrain, spec,   doStochSeedDisp, seedsDifferent, &
                   boundaries,  &      ! Use control and species parameters
                   stockability
use MPIvars, only: firstRow, lastRow, phantomSeedsOut
IMPLICIT NONE
!<Passed variables>--------------------------------------------------------------------------------
INTEGER,         INTENT(IN) :: isp, lat, lon,year, Iam ! number of species
REAL       :: seedsum
! TYPE(SeedCSpec), INTENT(OUT) :: dispersedSeedsSpecCell(0:maxSeededCells)
 !<Local variables>---------------------------------------------------------------------------------
INTEGER :: seeds_inAir, seeds_onGround, &  ! number of seeds emitted and fallen to ground
           latwin,                      &  ! index of relative window latitude
           lonwin,                      &  ! index of relative window longitude
           wlat,                        &  ! index of absolute window latitude
           wlon,                        &  ! index of absolute window longitude
           wlatold,                        &  ! index of absolute window latitude
           wlonold,                        &  ! index of absolute window longitude
           ncells                          ! number of cells where seeds are distributed to by this cell
REAL    :: cachind,          &             ! help var. to speed computation; stores cache site area
           contrib, prob                   ! contribution of source cell to sink cell
LOGICAL :: dodisp

!<Here the subroutine starts>
  !----- <See also subroutine GETSTD for the initialization!>
   seedsum= cell(lat,lon)%sp(isp)%seedsum

   IF (stockability(lat,lon) == 0.0) RETURN

 !    IF_Active: IF (active) THEN
      cachind = 1.0
      !----- <Loop through species to perform seed dispersal calculations>
         ncells=0
         IF ( seedsum > 0.0) THEN
             !----- <Loop for general discrete convolution>
          !----- Go through all possible sink cells around source sell
          ! Cells must be within dispersal radius (rad)
          LoopLatWin: DO latwin=-spec(isp)%rad,+spec(isp)%rad
            LoopLonWin:  DO lonwin=-spec(isp)%rad,+spec(isp)%rad
              !----- Determine absolute coordinates from those of source cell
              wlat = lat - latwin
              wlon = lon - lonwin

              !print*, "lat,latwin,wlat",lat,latwin,wlat

              CALL BoundaryConditions( wlat, maxlat, wlon, maxlon, dodisp, boundaries)

              IF_inAllowedArea: IF (dodisp) THEN
                ifStock:IF (stockability(wlat,wlon) > 0.0) THEN
    !              cachind = cell(wlat,wlon)%sp(isp)%caches
                   cachind = 1
               prob    = spec(isp)%kernel(latwin,lonwin)
                  IF (doStochSeedDisp) THEN
                    seeds_inAir =  seedsum
                    CALL DrawFromDistri (seeds_inAir, prob, seeds_onGround)  !----- Stochastic
                    contrib = seeds_onGround
                  ELSE
                    contrib = prob *  seedsum                                 !----- Deterministic
                  END IF
                  ! Matthias: for parallel seed dispersal within FORHYCS:
                  ! if seeds land on a cell in the same stripe as cell of origin, proceed normally
                  ! Otherwise, add cells to the array PhantomSeeds. This will be added to the
                  ! veryNewSeeds in the subroutine MPISeedExchange (MPIHelp.f90), called by UpdateNewSeeds (below).
                  if((wlat .ge. firstRow) .and. (wlat .le. lastRow)) then
                  !print*, "same stripe: lat,wlat", lat, wlat
                    seedrain(wlat,wlon)%sp(isp)%verynewSeeds = seedrain(wlat,wlon)%sp(isp)%verynewSeeds + contrib * cachind
                  else  ! Seeds are transferred to another stripe
! Careful: grids in PREVAH are indexed with (lon,lat) and those in TM with (lat,lon)
                  !print*, "to phantom", lat, wlat
                    phantomSeedsOut(wlon,wlat,isp) = phantomSeedsOut(wlon,wlat,isp) + contrib * cachind
                  end if

                   END IF ifStock
              END IF IF_inAllowedArea
            END DO LoopLonWin
          END DO LoopLatWin
        END IF !-----  seedsum>0.0
  !       END IF IF_Active
 END SUBROUTINE SeedsDispFromThisCellAndSpec
!****<Here the subroutine ends>********************************************************************



   Subroutine UpdateNewSeeds (nspc,iam,idsp,year,lats,late,lons,lone )
!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: maxlat, maxlon, OuterSeeds,seedrain,spec      ! Use control and species parameters
use MPIVars, only: phantomSeedsSum, rank, root, phantomSeedsList
IMPLICIT NONE
!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: nspc, iam, lats,late,lons,lone , year    ! number of species
LOGICAL, INTENT(IN) :: idsp      ! dispersal on/off flag                                      [=..]
!<Local variables>---------------------------------------------------------------------------------
INTEGER ::  latwin,                      &  ! index of relative window latitude
            lonwin,                      &  ! index of relative window longitude
            isp                             ! index of species

real:: sumseeds1,sumseeds2  ! DEBUG

if (( (lats<1)  .or. (lats> maxlat ).or.(late<1).or.( late>maxlat) .or. &
      (lons<1) .or. (lons>maxlon).or.(lone<1).or.(lone>maxlon) ) .and. &
      (rank .ne. root)) THEN
  print *, 'stopped in updateNewseeds, because of wrong coordinates ',&
         'iam,lats, late,lons, lone, maxlat,maxlon', &
           iam, lats, late,lons, lone, maxlat,maxlon
   stop
end if

sumseeds1=0
sumseeds2=0

!call MPISeedExchange
if(idsp) call MPISeedExchangeList

if(rank .ne. root) then
  IF (idsp) THEN
    LoopSpc2: DO isp=1,nspc
      IF (year >= spec(isp)%immi%dat(1)%year) THEN
        LoopLatWin2: DO latwin=lats,late
          LoopLonWin2:  DO lonwin=lons,lone
! Careful: grids in PREVAH are indexed with (lon,lat) and those in TM with (lat,lon)
            sumseeds1 = sumseeds1 + seedrain(latwin,lonwin)%sp(isp)%verynewSeeds
            seedrain(latwin,lonwin)%sp(isp)%verynewSeeds = &
            seedrain(latwin,lonwin)%sp(isp)%verynewSeeds + phantomSeedsSum(lonwin,latwin,isp)
            seedrain(latwin,lonwin)%sp(isp)%newSeeds  = seedrain(latwin,lonwin)%sp(isp)%verynewSeeds
            sumseeds2 = sumseeds2 + seedrain(latwin,lonwin)%sp(isp)%newSeeds
            seedrain(latwin,lonwin)%sp(isp)%verynewSeeds  = 0.0
          END DO LoopLonWin2
        END DO LoopLatWin2
      END IF
    END DO LoopSpc2
  END IF ! idisp
            IF ((year.eq.2) .and. ((iam.eq.2).or.(iam.eq.9))) THEN
!             print *,    'ende updatenewseeds  ', iam
  !            stop
           end if
end if  ! Slaves

    print*, "sumseeds1", sumseeds1, rank
    print*, "sumseeds2", sumseeds2, rank
END SUBROUTINE UpdateNewSeeds





!==============================================================================
   Subroutine SeedsFromOutsSimRegion (nspc,  idsp,year,lats,late,lons,lone )
!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: maxlat, maxlon, OuterSeeds,seedrain,spec      ! Use control and species parameters
IMPLICIT NONE
!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: nspc, lats,late,lons,lone , year    ! number of species
LOGICAL, INTENT(IN) :: idsp      ! dispersal on/off flag                                      [=..]
!<Local variables>---------------------------------------------------------------------------------
INTEGER ::  latwin,                      &  ! index of relative window latitude
            lonwin,                      &  ! index of relative window longitude
            isp                             ! index of species
  IF (idsp) THEN
    IF (OuterSeeds > 0) THEN  !----- outerspace boundary conditions
      LoopSpc2: DO isp=1,nspc
        IF (year >= spec(isp)%immi%dat(1)%year) THEN
          LoopLatWin2: DO latwin=lats,late
            LoopLonWin2:  DO lonwin=lons,lone
              seedrain(latwin,lonwin)%sp(isp)%newSeeds  = seedrain(latwin,lonwin)%sp(isp)%newSeeds + OuterSeeds
            END DO LoopLonWin2
          END DO LoopLatWin2
        END IF
      END DO LoopSpc2
    END IF
   END IF ! idisp
END SUBROUTINE SeedsFromOutsSimRegion




!--------------------------------------------------------------------------------------------------
SUBROUTINE BoundaryConditions(wlat, maxlat, wlon, maxlon, dodisp, boundaries)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)      :: maxlat, maxlon  ! looping variable for longitude               [=..]
CHARACTER(1), INTENT(IN) :: boundaries      ! type of boundary conditions
INTEGER, INTENT(IN OUT)  :: wlat, &         ! variable for latitude                        [=..]
                            wlon            ! variable for longitude                       [=..]
LOGICAL, INTENT(OUT)     :: dodisp          ! dispersal into this cell?                    [=..]

!<LOCAL variables>--------------------------------------------------------------------------------

!<Here the subroutine starts>**********************************************************************
   dodisp  = .TRUE.

!TODO: through these preprocessor treatment the variable boundaries is not used anymore
! it should be removed from the control file and from all other files

!PREPROCESSOR statement
#define BOUNDS_C 1
#define BOUNDS_A 2
#define BOUNDS_S 3
#define BOUNDS_W 4

!If the option BOUNDS is not defined: take default a (absorbing boundaries)
#ifndef BOUNDS
#define BOUNDS BOUNDS_A
!TODO: (information) the following is only working with the gcc compiler hence there will
!not be any notation what has happened for other compilers....
#ifdef __GFORTRAN__
#warning "No option for BOUNDS took default option which are absorbing boundaries"
#endif
#endif



#if BOUNDS == BOUNDS_C
    !----- Cyclic boundary conditions
    IF (wlat .ge. 0) THEN
      wlat = MOD (wlat, maxlat)
    ELSE
      wlat = maxlat - MOD (ABS (wlat), maxlat)
    END IF
    IF (wlat .eq. 0) wlat = maxlat
    IF (wlon .ge. 0) THEN
      wlon = MOD (wlon,maxlon)
    ELSE
      wlon = maxlon - MOD (ABS (wlon), maxlon)
    END IF
    IF (wlon .eq. 0) wlon = maxlon
#elif BOUNDS == BOUNDS_A
    !----- absorbing boundary conditions
    IF ((wlat .lt. 1) .OR. (wlat .gt. maxlat) .OR. &
        (wlon .lt. 1) .OR. (wlon .gt. maxlon)) dodisp = .FALSE.
#elif BOUNDS == BOUNDS_S
    !----- absorbing boundary conditions in north and south, cyclic in east and west
    IF (wlon .ge. 0) THEN
      wlon = MOD (wlon, maxlon)
    ELSE
      wlon = maxlon - MOD (ABS (wlon), maxlon)
    END IF
    IF (wlon .eq. 0) wlon=maxlon
    IF ((wlat .lt. 1) .OR. (wlat .gt. maxlat)) dodisp = .FALSE.
#elif BOUNDS == BOUNDS_W
    !----- absorbing boundary conditions in west and eats, cyclic in south and north
    IF (wlat .ge. 0) THEN
      wlat = MOD (wlat, maxlat)
    ELSE
      wlat = maxlat - MOD (ABS (wlat), maxlat)
    END IF
    IF (wlat .eq. 0) wlat = maxlat
    IF ((wlon .lt. 1) .OR. (wlon .gt. maxlon)) dodisp = .FALSE.
#else
#error "Wrong option for BOUNDS: choose one of 1:'a', 2:'c', 3:'s', 4:'w'!"
#endif

!  SELECT CASE (boundaries)
!  CASE('c')                          !----- Cyclic boundary conditions
!    IF (wlat .ge. 0) THEN
!      wlat = MOD (wlat, maxlat)
!    ELSE
!      wlat = maxlat - MOD (ABS (wlat), maxlat)
!    END IF
!    IF (wlat .eq. 0) wlat = maxlat
!    IF (wlon .ge. 0) THEN
!      wlon = MOD (wlon,maxlon)
!    ELSE
!      wlon = maxlon - MOD (ABS (wlon), maxlon)
!    END IF
!    IF (wlon .eq. 0) wlon = maxlon
!  CASE('a')                          !----- absorbing boundary conditions
!    IF ((wlat .lt. 1) .OR. (wlat .gt. maxlat) .OR. &
!        (wlon .lt. 1) .OR. (wlon .gt. maxlon)) dodisp = .FALSE.
!  CASE('s')                          !----- absorbing boundary conditions in north and south, cyclic in east and west
!    IF (wlon .ge. 0) THEN
!      wlon = MOD (wlon, maxlon)
!    ELSE
!      wlon = maxlon - MOD (ABS (wlon), maxlon)
!    END IF
!    IF (wlon .eq. 0) wlon=maxlon
!    IF ((wlat .lt. 1) .OR. (wlat .gt. maxlat)) dodisp = .FALSE.
!  CASE('w')                          !----- absorbing boundary conditions in west and eats, cyclic in south and north
!    IF (wlat .ge. 0) THEN
!      wlat = MOD (wlat, maxlat)
!    ELSE
!      wlat = maxlat - MOD (ABS (wlat), maxlat)
!    END IF
!    IF (wlat .eq. 0) wlat = maxlat
!    IF ((wlon .lt. 1) .OR. (wlon .gt. maxlon)) dodisp = .FALSE.
!  CASE DEFAULT
!    WRITE(1070,*) 'wrong boundary conditions : ', boundaries
!  END SELECT

  RETURN
END SUBROUTINE BoundaryConditions
