!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Update the phenological status for each species in each cell. This status is
! used to reduce leaf area in spring and autumn, and depends on the temperature
! indices calculated in ChilldaysAndGDD5.
!-------------------------------------------------------------------------------

SUBROUTINE LeafPhenology(chd,gdd,tday,domspec,stock,leafarea,vFrac,gs)

!<CALL modules for type definitions>---------------------------------------------------------------
use prevahglobals
use forhycsglobals, only: sen_day_count,aphen,oldphen,spcgdd0,fhla, &
                          perCC, phenDoyBB,phenDoyMax,phenDoySen,phenDoyFall, &
                          ssen3d, doySen  ! Matthias

IMPLICIT NONE

!<Local variables>---------------------------------------------------------------------------------
integer, intent(in) :: domspec
REAL :: pgr, gdd, aph, leafarea, sen_days_left
real, intent(out) :: vFrac
logical, intent(in) :: stock
real, intent(in) :: tday      ! Mean daily temperature
INTEGER ::  chd, aphen_max, nspc, landtype
REAL ::  phen(1:30) 		! vegetation phenological state (fraction of annual maximal leaf cover) for that certain gridcell and species
real, dimension(30) :: active ! 0 for species that are currently dormant, >0 if they are active (growing season). For
                              ! summergreen species, this is equal to PHEN. For evergreens, PHEN is always 1, but ACTIVE may vary.
REAL ::  phengdd5ramp(30), k_chillb(30), k_chilla(30)	! Species-specific parameters for spring phenology
real, dimension(30) :: k_dmp, k_dmt, k_dmx, k_dmy     ! Species-specific parameters for autumn phenology
INTEGER ::  phen_type(30) ! plant specific parameter - 0 for evergreen conifers, 1 for deciduous broadleaves and 2 for Larix decidua
!real, dimension(30) :: clump  ! Clumping index
real, dimension(30) :: fproj  ! Projection factor
REAL ::  chilla, chillb, chillk ! Parameters for current species - spring model
real :: dm_ycrit, dm_pd, dm_tb, dm_x, dm_y
REAL :: TMLA(1:30),SUMTMLA, dummy
CHARACTER*800 :: dummytext
INTEGER :: errindex
real ::     laMax, &  ! Maximum possible ("potential") LAI in the current cell and year
            laRatio   !  Ratio of actual to potential LAI
logical, intent(out) :: gs  ! Indicates whether growing season is on or off in the current cell
real :: daylen, daylength
integer, parameter :: min_sen_doy = 250   ! Earliest day for onset of senescence (August 1st)
real :: rsen, ssen      ! Stored state of leaf senescence progress (Delpierre model)
real, parameter :: geolat=47.0
real :: sendoy




!number of species
 nspc=30

!Maximum number of equivalent days with full leaf cover per growing season for summergreen species
 !aphen_max=210
 !chilla=0
 chillk=0.05
 dm_ycrit = 100   ! Common parameter for autumn senescence model (arbitrary value - other parameters were fitted in function of this)
 data(k_chilla(j),j=1,30) /0.0,0.0,0.0,0.0,0.0,0.0,0.0, & ! Conifers
                           85.0, 85.0, 85.0, 0.0, 0.0, 0.0, 0.0, 0.0, 157.0, 0.0, 0.0, 0.0, 0.0, &
                           0.0, 85.0, 85.0, 85.0, 0.0, 0.0, 0.0, 48.0, 48.0, 0.0/
 !data (k_chillb(j),j=1,30) /100.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,400.0,600.0,100.0,400.0,600.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,600.0,100.0,100.0/
 data (k_chillb(j),j=1,30) /983.0, 967.0, 983.0,983.0,983.0,983.0,983.0,&
                            927.0, 927.0, 927.0, 745.0, 745.0, 745.0, 648.0, 730.0, 960.0, 798.0, 730.0, 730.0, 730.0, &
                            730.0, 927.0, 927.0, 927.0, 745.0, 745.0, 745.0, 993.0, 993.0, 730.0/

 data (phengdd5ramp(j),j=1,30) /331.0, 141.0, 331.0, 331.0, 331.0, 331.0, 331.0, & ! Conifers
                                103.0, 103.0, 103.0, 165.0, 165.0, 165.0, 185.0, 249.0, 144.0, 193.0, 249.0, 249.0, 249.0, &
                                249.0, 103.0, 103.0, 103.0, 165.0, 165.0, 165.0, 221.0, 221.0, 249.0/
 data (phen_type(j),j=1,30) /0,2,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1/
! data(clump(j),j=1,30) /0.56,0.56,0.56,0.56,0.56,0.56,0.56,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85/
 data (fproj(j),j=1,30) /0.8,0.8,0.8,0.8,0.8,0.8,0.8,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0/

 data(k_dmp(j),j=1,30) /13.0, 13.0, 13.0, 13.0, 13.0, 13.0, 13.0, & !Conifers
                        15.16, 15.16, 15.16, 14.73, 14.73, 14.73, 14.73, 12.1, 12.1, 14.73, 12.1, 12.1, 12.1, &
                        12.1, 15.16, 15.16, 15.16, 14.73, 14.73, 14.73, 12.93, 13.71, 12.1/
 data(k_dmt(j),j=1,30) /22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, & ! Conifers
                        23.1, 23.1, 23.1, 23.31, 23.31, 23.31, 23.31, 17.57, 17.57, 23.31, 17.57, 17.57, 17.57, &
                        17.57, 23.1, 23.1, 23.1, 23.31, 23.31, 23.31, 29.61, 18.36, 17.57/
 data(k_dmx(j),j=1,30) /0.75, 0.75, 0.75, 0.75, 0.75, 0.75, 0.75, & !Conifers
                        0.361, 0.361, 0.361, 0.291, 0.291, 0.291, 0.291, 1.824, 1.824, 0.291, 1.824, 1.824, 1.824, &
                        1.824, 0.361, 0.361, 0.361, 0.291, 0.291, 0.291, 0.446, 0.476, 1.824/
 data(k_dmy(j),j=1,30) /1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, & !Conifers
                        1.805, 1.805, 1.805, 0.469, 0.469, 0.469, 0.469, 1.65, 1.65, 0.469, 1.65, 1.65, 1.65, &
                        1.65, 1.805, 1.805, 1.805, 0.469, 0.469, 0.469, 0.411, 0.489, 1.824/

chd=min(chd,364)
leafarea=0.0
!laie = 0.0
landtype=0.0
laMax = 0.0


IF (land .eq. 3) landtype=1
IF (land .eq. 4) landtype=1
IF (land .eq. 5) landtype=1
if (land .eq. 8) landtype = 1  ! Matthias, July 2015
IF (land .eq. 21) landtype=1
IF (land .eq. 23) landtype=1
IF (land .eq. 24) landtype=1
if (land .eq. 25) landtype = 1  ! Matthias, July 2015
!IF (land .eq. 7) landtype=2 ! for testing
IF (land .eq. 26) landtype=1

if(stock==.false.) landtype=0

IF (landtype .eq. 0) then
leafarea=-1.0
laMax = -1.0
RETURN
END IF
IF (landtype.eq.1) THEN
BySpecies: DO i=1,nspc

! Reset senescence doy variable at the beginning of the year
  if(dkal==1) then
    doysen(xcoor,ycoor,i) = -1.0
    ssen3d(xcoor,ycoor,i) = 0.0
  end if

 !IF (LAmax(i).eq.0) CYCLE		!proceed with next species if it is not present

! Species-specific parameters for spring model
 chilla=k_chilla(i)
 chillb=k_chillb(i)
 pgr=phengdd5ramp(i)

! Species-specific parameters for spring model
 dm_pd = k_dmp(i)
 dm_tb = k_dmt(i)
 dm_x = k_dmx(i)
 dm_y = k_dmy(i)
 ssen = ssen3d(xcoor,ycoor,i)

 sendoy = doysen(xcoor,ycoor,i)

 phen(i)=0.0

    	 ! Check whether the conditions for onset of leaf senescence have been reached
    	 if(dkal.ge.min_sen_doy) then
    	   daylength = daylen(dkal,geolat)
    	   if((daylength < dm_pd).and.(tday < dm_tb)) then
    	     rsen = ((dm_tb-tday)**dm_x) * ((daylength/dm_pd)**dm_y)
    	     ssen = ssen + rsen
    	     ssen3d(xcoor, ycoor,i) = ssen
    	   end if
    	   if((sendoy.lt.0.0).and.(ssen >= dm_ycrit)) then
    	     doysen(xcoor,ycoor,i) = real(dkal)
    	     sendoy = real(dkal)
    	   end if
    	 end if

      ! Reset senescence day counter for Larch model (Scherstjanoi et al. 2014)
     IF ((TAG.eq.1).and.(MON.eq.1)) sen_day_count(xcoor,ycoor,i) = 0


      ! Check whether the required amount of degree-days (varying with chill days)
      ! has already been reached.
      IF((phen_type(i).eq.1).or.(phen_type(i).eq.2))  THEN !summergreen or summergreen with slowly increasing senescence (larch)
    	 IF (spcgdd0(i,chd+1).lt.0) THEN !look up table
    		spcgdd0(i,chd+1)=chilla + chillb*exp(-chillk*REAL(chd))
    	 END IF

    	 ! If growing season has started and senescence criterion has not been reached yet
    	 IF ((gdd.gt.spcgdd0(i,chd+1)).and.(ssen<dm_ycrit)) THEN
		phen(i)=min(1.0, (gdd - spcgdd0(i,chd+1))/pgr)
  	            if((xcoor==56) .and. (ycoor==97) .and. (i==2)) then
                        !print*, "phen larch normal", phen(i), gdd, chd, spcgdd0(i,chd+1)
                  end if  ! print
            active(i) = phen(i)
    	 ELSE
    	  IF  (phen_type(i).eq.2) THEN  	!summergreen with slowly increasing senescence (larch)
 	     IF ((oldphen(xcoor,ycoor,i).gt.0).and.(DKAL.lt.335))THEN
 	     !IF ((gdd.gt.spcgdd0(i,chd+1)).and.(DKAL.lt.335))THEN
    	  	 sen_days_left= MAX(0,335-DKAL+sen_day_count(xcoor,ycoor,i))   !sen_days_left for one species and one stand always the same in one year
    	  	 sen_day_count(xcoor,ycoor,i)=sen_day_count(xcoor,ycoor,i)+1
    	  	 phen(i)=1/(1+ 0.5 *EXP(13.5/REAL(sen_days_left)*REAL(sen_day_count(xcoor,ycoor,i))-6.75))
    		 IF (phen(i).lt.0.01) phen(i)=0
    		 active(i) = phen(i)
  	            if((xcoor==56) .and. (ycoor==97) .and. (i==2)) then
                        !print*, "phen larch ", phen(i), sen_day_count(xcoor,ycoor,i)
     	                  ! print*, "sen_days_left", sen_days_left
                  end if  ! print
    	     ELSE
    		 phen(i)=0
    		 active(i) = phen(i)
    	     END IF

        else if (phen_type(i)==1) then
        ! Deciduous broadleaves: linear reduction of foliage cover after onset of leaf senescence
          if((sendoy>0).and.((real(dkal)-sendoy) > 14)) phen(i) = 0.0
          if((oldphen(xcoor,ycoor,i).gt.0.0).and.(ssen >= dm_ycrit)) phen(i) = max(0.0,(1.0-(real(dkal)-sendoy)/14.0))


    	  END IF ! phen_type==1 or 2
         !IF (phen_type(i).eq.1) phen(i)=0 !summergreen
         active(i) = phen(i)

    	 END IF ! GDD > spcGDD AND aph<aphen_max

     else if (phen_type(i)==0) then ! Evergreens - leaf phenology is always 1, but dormancy/activity is simulated
                                    ! (start and end of activity are assumed to be analogous to bud burst and leaf fall)

      phen(i) = 1.0

    	 IF (spcgdd0(i,chd+1).lt.0) THEN !look up table
    		spcgdd0(i,chd+1)=chilla + chillb*exp(-chillk*REAL(chd))
    	 END IF

    	 IF ((gdd.gt.spcgdd0(i,chd+1)).and.(ssen<dm_ycrit)) THEN
		active(i)=min(1.0, (gdd - spcgdd0(i,chd+1))/pgr)
       ELSE
            IF ((oldphen(xcoor,ycoor,i).gt.0).and.(DKAL.lt.335))THEN
    	  	 sen_days_left= MAX(0,335-DKAL+sen_day_count(xcoor,ycoor,i))   !sen_days_left for one species and one stand always the same in one year
    	  	 sen_day_count(xcoor,ycoor,i)=sen_day_count(xcoor,ycoor,i)+1
    	  	 active(i)=1/(1+ 0.5 *EXP(13.5/REAL(sen_days_left)*REAL(sen_day_count(xcoor,ycoor,i))-6.75))
    	  	else
    	  	  active(i) = 0.0
    	  	end if
       END IF

     END IF  ! phen_type

     ! Set larch phenology to zero
     if((dkal .ge. 335).or.(dkal .le. 15)) then
       if(phen_type(i)==2) then
         phen(i) = 0.0
         active(i) = 0.0
       else if(phen_type(i)==0) then
         active(i) = 0.0
       end if
     end if

IF (MON .EQ. 1) THEN
IF(TAG.eq.15)  aph=0.0
END IF


! Get phenodates of dominant species for each cell

if((dkal .gt. 15).and.(domspec==i).and.(phen_type(i).gt.0)) then
! Date of bud burst
      if((phenDoyBB(xcoor,ycoor).lt.0).and.(phen(i).gt.0.0)) then
        phenDoyBB(xcoor,ycoor) = dkal
        !print*, "phendoybb, dkal", phendoybb(xcoor,ycoor), dkal, xcoor,ycoor
       ! print*, "species, phen", i, phen(i)
      end if
! Phenological maximum reached
      if((phenDoyMax(xcoor,ycoor).lt.0).and.(phen(i)==1.0)) then
        phenDoyMax(xcoor,ycoor) = dkal
      end if
! Date of onset of senescence
      if((phenDoyMax(xcoor,ycoor).gt.0).and.(phenDoySen(xcoor,ycoor).lt.0).and.(phen(i).lt.1.0)) then
        phenDoySen(xcoor,ycoor) = dkal
      end if
! Date of leaf fall
      if((phenDoyMax(xcoor,ycoor).gt.0).and.(phenDoyFall(xcoor,ycoor).lt.0).and.(phen(i)==0.0)) then
        phenDoyFall(xcoor,ycoor) = dkal
      end if
end if

!aphen(xcoor,ycoor,i) = aph + phen(i)

if((phen_type(i)==1).or.(phen_type(i)==2)) then
  oldphen(xcoor,ycoor,i)=phen(i)
else
  oldphen(xcoor,ycoor,i)=active(i)
end if


leafarea=leafarea+fhla(xcoor,ycoor,i)*phen(i) * fproj(i) !here the TreeMig input is taken
!laie = laie + fhla(xcoor,ycoor,i)*phen(i) * clump(i)
laMax = laMax + fhla(xcoor,ycoor,i)



END DO BySpecies

! Growing season if activity status of dominant species is at least 0.5
if(active(domspec).ge.0.5) then
  gs = .true.
else
  gs = .false.
end if





! Fractional canopy cover for forested areas is reduced by the ratio of
! current LAI to maximum possible LAI in this cell and year. Done in this way
! in the absence of a published method for varying fractional cover.
! Matthias, July 2015.

laRatio = leafarea/laMax

!leafarea=leafarea/nspc

ELSE IF (landtype.eq.2) THEN ! Grassland
     	! Summergreen grasses have no maximum number of leaf-on days per growing season, and no chilling requirement
	leafarea= 100000.0 * MIN(1.0,gdd/100.0)
	 !just for first compilation , replace 1000.0 by maximum leafarea this year

	 laRatio = 1.0
END IF


leafarea=leafarea/833.0 !leafarea to leaf area index (833 m2, artefect from gap models)

vFrac = perCC(xcoor,ycoor) * laRatio
if(laMax==0.0) vFrac = 0.0

END SUBROUTINE LeafPhenology

!=======================================================================


!=======================================================================

! Auxiliary function to compute day length
! (needed by Delpierre autumn phenology model)

real function daylen(doy, geolat)

  implicit none

! Passed variables
  integer :: doy
  real :: geolat
! Local variables
  real, parameter :: declin = -23.44      ! Declination at the winter solstice [°]
  real, parameter :: pi = 3.1416
  real :: curdec, coshz, tsr, tss, rdoy

!---------------------------------------

  rdoy = real(doy)
  curdec = declin * cos((360.0*(rdoy+10.0)/365.0)*pi/180.0) ! Current solar declination [°]
  coshz = sin(geolat*pi/180.0) * sin(curdec*pi/180.0) / (cos(geolat*pi/180.0) * cos(curdec*pi/180.0)) ! Cosine of hour angle at sunrise
  tsr = acos(coshz) * 12.0/pi
  tss = 24.0-tsr

  daylen = tss-tsr

end function daylen

!=======================================================================
