!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  Lfcalc
!
!  Purpose:
!
!  Remarks:  contains the subroutine
!
!            Lfcalc (<LocalForestDynamics<DoLocalWork<DistrLocalDynThisTime<
!                      TimeLoop<TreeMig)
!
!
!==============================================================================


SUBROUTINE Lfcalc(nspc, height, lasum, lasqsum, numin0)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: maxspc, maxhc, maxlc,   pltsiz, facvar, &
                   ltfrq, ltcum, ltprf, la,                     &
                   spc                                ! Use control, stand and species parameters
use forhycsglobals, only : laRed
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: height, &                 ! height class                            [=height]
                       nspc                      ! number of species                         [=spec]
REAL, INTENT(IN)    :: numin0(0:maxhc,1:maxspc)  ! temp var originating from numNInH           [=..]
REAL, INTENT(INOUT) :: lasqsum, &                ! sq. sum of LA                        [=LAIsqrSum]
                       lasum                     ! leave area (LA)                         [=LAISum]

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: light, &                              ! lightclass index variable                    [=l]
           i                                     ! looping variable  for species
REAL    :: ntinh,         &                      ! # tree in height class                    [=nInH]
           newla, newla2, &                      ! new lai  and squared lai          [=rNewLeafArea]
           treelai,       &                      ! lai  per tree                     [=rNewLeafArea]
           sigma,         &                      ! std?                                     [=sigma]
           sum,           &                      ! running sum                                [=sum]
           disbot(maxlc), &                      ! Distr. of light (bottom of class)?    [=distrBot]
           distop,        &                      ! Distr. of light (top of class)?       [=distrTop]
           value                                 ! Out-variable from NormDist
real :: laRedFac

!<Here the subroutine starts>**********************************************************************
  !----- By species, the number of trees is calculated and evaluated per height class
  BySpecies: DO i=1,nspc
    !----- The current height class is completely included, i.e. there is also competition between trees in this height class
    ntinh =  numin0(height,i)
    if(height == 0) then
       laRedFac = laRed(i,1)
    else
       laRedFac = laRed(i,height)
    end if

   !----- The leaf area and the squared leaf area are calculated
   IF (ntinh /= 0.0) THEN
     treelai = spc(i)%lain(height)*laRedFac / pltsiz
     newla   = ntinh   * treelai
     newla2  = ntinh   * treelai ** 2
     lasum   = lasum   + newla
     lasqsum = lasqsum + newla2
   END IF
  END DO BySpecies

  !----- lasum is the mean, sigma the standard deviation of the leaf area Normal distribution
  !----- facvar describes how narrow the Normal distributions is.
  !----- 0: uniform spatial distribution, 1: random spat. distribution, >1 clumped spatial distribution.
  !----- It is parameterized in All_PAR

  IF ((facvar <= 0.0) .or. (lasqsum .lt. 1.E-7)) THEN
    sigma = 0.001         !----- sigma has to be larger 0
  ELSE
    sigma = SQRT (lasqsum) * facvar
  END IF

  ltfrq = 0.0
  !----- IF there are (nearly) no leaves, then there is full light...
  IF (ABS(lasum) .lt. 1.E-10) THEN
    ltfrq(maxlc) = 1.0
    sum          = 1.0
  ELSE
    !----- ... otherwise for each light class from full light  (light=maxlc ) to complete darkness (light=1)
    !----- the (cumulative) normal distribution function with parameters lasum and sigma is evaluated
    !----- at the leaf area (la(light)) required to produce the light value. The difference between
    !----- two values of the distribution function is the frequency of this leaf areas, resp. light values.
    sum = 0.0
    CALL NormDist (value, lasum, sigma, la(1))
    ltfrq(1) = 1 - value
    sum      = sum + 1 - value
    CALL NormDist (value, lasum, sigma, la(maxlc - 1))
    ltfrq(maxlc) = value
    sum          = sum + value
    distop       = value

    ByLtCls1: DO light = maxlc-1, 2, -1
      CALL NormDist (value, lasum, sigma, la(light - 1))
      disbot(light) = value
      ltfrq(light)  = disbot(light) - distop
      sum           = sum + ltfrq(light)
      distop        = disbot(light)
    END DO ByLtCls1
  END IF

  !----- If the leaf area is so large that even the la value set for the lowest light class is much lower,
  !----- and all lftrq's are 0, then the lowest light class frequency is set to 1
  IF (ABS (sum) < 1.E-20) THEN
    ltfrq(1) = 1.0
    sum      = 1.0
  END IF

  !----- The light distribution is normalized, because it is truncated a 0
  ltfrq(maxlc)        = ltfrq(maxlc) / sum
  ltprf(height,maxlc) = ltfrq(maxlc)
  ltcum(maxlc)        = ltfrq(maxlc)

  ByLtCls2: DO light=maxlc-1,1,-1
    ltfrq(light)        = ltfrq(light) / sum
    ltprf(height,light) = ltfrq(light)
    ltcum(light)        = ltcum(light+1) + ltfrq(light)
  END DO ByLtCls2

END SUBROUTINE Lfcalc
!****<Here the subroutine ends>********************************************************************
!**************************************************************************************************
