!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Addition of new processes (reduction of leaf area and maximum
! tree height)

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  LocalForestDynamics
!
!  Purpose:   calculates the forest dynamics in one cell
!
!  Remarks:  contains the subroutine
!
!            LocalForestDynamics (<DoLocalWork<DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================


SUBROUTINE LocalForestDynamics( lat, lon, Iam, nspc,year,idsp,thiscell)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: maxspc, maxhc, maxlc, slow, &
                   ltfrq,  htcl, ltcl,   &
                   spec, stockability,    &
                   disturbances, disturbProb, disturbIntensity ,&
                   CurrStateInCell,&         ! Use control parameters
                   spc !FORHYCS
use forhycsglobals, ONLY: fhla, perCC, curhcmax, laRed


IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) ::  lat, lon, Iam, nspc,year     ! Passed number of species                   [=spec]

Logical, INTENT(IN) :: idsp



!<Local variables>---------------------------------------------------------------------------------
REAL :: lasum,                     & ! Sum of LA per height?                                    [=laisum]
        lasqsum,                   & ! Sum of squared LA per height?                         [=laisqrsum]
        srvinhl,                   & ! remaining ind. per spec/height/light condition in loop [=survNInH]
        disturbancemort,           & ! mortality by disturbance, species independent
        ntinh, ntinhl,             & ! # tree in height class               [=nInH  <- rNoTreesUnderCond]
        randomvalue,               & ! random value for disturbance calculation
        randomvalue2,              & ! random value for disturbance calculation
        numin0(0:maxhc, 1:maxspc), &      ! temporary variable originating from numNInH
        stock, &
        la(nspc), sumla, & !FORHYCS
        caSum, &  ! Sum of canopy area over species and height classes
        numinSqm
INTEGER :: minhc, &                  ! minimum height class
           i, &                         ! looping variable
           height, &
           chcmax       ! Maximum height for each species/cell, as reduced by drought stress
TYPE(CurrStateInCell):: thiscell

real, dimension(0:maxhc) :: nTrees, nTreesRel
real :: nTreesTot, nTreesCumu, zVeg

integer :: class90

real, dimension(0:maxhc) :: laByHt, laByHtRel
real :: laTot, laCumu

integer :: class95

real :: laRedFac

  !<Here the subroutine starts>**********************************************************************
  stock=stockability (thiscell%lat,thiscell%lon)
  IF (stock<= 1.E-20) RETURN            ! if cell cant be inhabited, no dynamics
  ! print *, "stockability", stock
  !----- First: Initialize the light distribution arrays, i.e. max light on top
  ltfrq        = 0.0
  ltfrq(maxlc) = 1.0
  !----- Initialize leaf areas for summing on leaf area
  lasum   = 0.0
  lasqsum = 0.0
  !----- Depending on kind of seed production (constant seed pool or by parent trees) the 0th height class is included
  if (idsp) THEN
    minhc = 0
    do i=1,nspc
      thiscell%sp(i)%ingrowth  = 0.0
    end do
  ELSE
    minhc = 1
    do i=1,nspc
      thiscell%sp(i)%numin(0) = 0.0
    end do
  END IF
  !----- For slowing down initial growth (normally not used)
  IF (slow) THEN
    DO i=1,nspc
      thiscell%sp(i)%htdmp = 0.0
    END DO
  END IF
  !----- Determine whether disturbance occurs in this year and cell (if disturbances are wished)
  disturbanceMort = 0.0
! Commented out for DEBUG only!
!  IF (disturbances .AND. idsp) THEN
!    CALL RANDOM_NUMBER (randomvalue)
!    IF (randomvalue .lt. disturbProb) THEN
!      CALL RANDOM_NUMBER (randomvalue2)
!      disturbanceMort = disturbIntensity * randomvalue2
!    END IF
!  END IF
! ------------------ New: Determine  chestnutcultivation
   CALL ChestnutCult( lat, lon, year, idsp,thiscell)


  DO htcl = maxhc, minhc,-1
    DO i=1,nspc
! To take into account also the height class above the highest one, which can be reached,
!  because the growth rate is calculated at the lower bound of the height classes, here I test,
!   here I test, to "activate" this next height class, to include it into the shading
!   and to apply mortality to it
!     IF (htcl <= spec(i)%maxhtc) THEN
     chcmax = curhcmax(i,lon,lat)  ! MS - March 2017: dynamic adaptation of Hmax due to drought
     !IF (htcl <= spec(i)%maxhtc+1) THEN
     IF (htcl <= (chcmax+1)) THEN
      numin0(htcl, i) = thiscell%sp(i)%numin (htcl )
     ELSE
     ! Trees higher than what is currently possible (due to water stress) are assigned to the
     ! highest possible height class for this species/cell. This may happen when conditions become
     ! drier and maximum height gets reduced.
     numin0((htcl-1), i) = numin0((htcl-1), i) + thiscell%sp(i)%numin (htcl )
       numin0(htcl, i) = 0.0
     END IF
    END DO
  END DO

  !----- Next, we loop through height classes, top-down
  ByHCls: DO htcl = maxhc, minhc,-1
    !----- Here the light distribution is calculated based on  leaves  of current and higher height classes
    CALL Lfcalc (nspc, htcl, lasum, lasqsum, numin0)   ! Light frequency & Leave area
     !----- By species, the number of trees is calculated and evaluated per height class


    BySpecies: DO i=1,nspc
      thiscell%sp(i)%seeds(htcl) = 0.0
      ntinh                      = numin0(htcl, i)
      !----- If the species reaches this height class and there are trees in this height class ...
      IfHCls: IF (ntinh > 0.0) THEN

       !----- ... determine seed production. Here, as long as it is not yet light dependent

        CALL Seedprod (i, htcl, ntinh,year,lat,lon,Iam , thiscell)    ! Seed production
        !----- ...loop through all light classes
        ByLCls: DO ltcl = 1, maxlc
          !----- ... if  this light  class occurs...
          IF (ltfrq(ltcl) > 0.0) THEN
            !----- ... count how many trees are in this height and light class
             ntinhl = ntinh * ltfrq(ltcl)
            !----- ... and let the trees live part of their live: produce seeds, die and grow
 !          CALL Seedprod(i,htcl,ntinhl,year, thiscell)                 ! Seed production (if light dependent)
            CALL Mort(i,htcl,ltcl,srvinhl,ntinhl,disturbanceMort, thiscell)  ! Mortality
            CALL Grow(i,htcl,ltcl,srvinhl,minhc, thiscell)                   ! Growth
          END IF !----- ltfrq(ltcl) > 0.0
        END DO ByLCls
       END IF IfHCls

    END DO BySpecies
  END DO ByHCls


  ! TEST: if trees are higher than their current maximum, assign them to the highest class possible
  ! (also *after* mortality and growth calculations)

  DO htcl = maxhc, minhc,-1
    DO i=1,nspc

! To take into account also the height class above the highest one, which can be reached,
!  because the growth rate is calculated at the lower bound of the height classes, here I test,
!   here I test, to "activate" this next height class, to include it into the shading
!   and to apply mortality to it
!     IF (htcl <= spec(i)%maxhtc) THEN
     chcmax = curhcmax(i,lon,lat)  ! MS - March 2017: dynamic adaptation of Hmax due to drought
     !IF (htcl <= spec(i)%maxhtc+1) THEN
     !IF (htcl <= (chcmax+1)) THEN
     ! numin0(htcl, i) = thiscell%sp(i)%numin (htcl )

    ! ELSE
     if(htcl > (chcmax)) then
     ! Trees higher than what is currently possible (due to water stress) are assigned to the
     ! highest possible height class for this species/cell. This may happen when conditions become
     ! drier and maximum height gets reduced.
     thiscell%sp(i)%numin (chcmax) = thiscell%sp(i)%numin (chcmax) + thiscell%sp(i)%numin (htcl )
       thiscell%sp(i)%numin (htcl ) = 0.0
     END IF
    END DO
  END DO

!----- Last, regeneration is calculated
!----- Here part three of regeneration takes place (after seed production and dispersal)
!----- the germination of seeds and the survival of seedlings

 CALL SumSpecsSeedprodsOverHeights (nspc,  thiscell)                    !  In SeedProd.f90

 CALL SelfRegulation (nspc, idsp, thiscell)

 CALL SpecImmigration (nspc,year,idsp, thiscell)
 CALL Regen (year,nspc, stock, idsp, thiscell)                          ! Regeneration of saplings

 if((lon==56) .and. (lat==97)) then
   print*, "numin 5697", year, thiscell%sp(2)%numin(0:maxhc)
 end if




!  ---------FORHYCS - leaf area and crown area
!            perCC(lon,lat) = 0.0
!            caSum = 0.0
!            nTrees = 0.0
!            laByHt = 0.0
!
!            fhla(lon,lat,32) = 0.0
!            BySpc: DO i=1,nspc
!            fhla(lon,lat,i)=0
!            DO height=0,spec(i)%maxhtc
!               fhla(lon,lat,i) =fhla(lon,lat,i)+ thiscell%sp(i)%numin(height )* spc(i)%lain(height)/2.0 !FORHYCS
!               if(i .le. maxspc-1) then  ! Exclude ground vegetation from total LAI calculations
!                 laByHt(height) = laByHt(height)+ thiscell%sp(i)%numin(height )* spc(i)%lain(height)/2.0
!               end if
!               numinSqm = thiscell%sp(i)%numin(height) / 833.0
!               caSum = caSum + numinSqm * spc(i)%crArea(height)  ! Number of trees per square meter
!
!            END DO
!
!               if(i .le. maxspc-1) then  ! Exclude ground vegetation from total LAI calculations
!                 fhla(lon,lat,32) = fhla(lon,lat,32) + fhla(lon,lat,i)  ! Temp Matthias
!               end if
!
!           END DO BySpc
!
! Percent crown coverage for this cell (Diss Zurbriggen p. 64)
!
!   perCC(lon,lat) = max(0.0, min(1.0, 1.0 - exp(-1.0*caSum)))
!
! ============================================
! Trying to determine canopy height through number of trees in each class
! => class that covers a cumulative 90% of trees determines canopy height.
! BUT: this returns very low values (1st or 2nd class for most cells) -> unrealistic
!
! Trying with cumulative leaf area instead of number of trees (below)
!
! Relative number of trees in each height class
!   nTreesTot = sum(nTrees)
!   nTreesRel = nTrees(0:maxhc)/nTreesTot
!   if(nTreesTot==0) nTreesRel(0:maxhc) = 0
!
!   nTreescumu=0.0
!
! ! Determine the height class containing a cumulative 90% of all trees in
! ! the current cell => determines canopy height
!   do i = 0,maxhc
!     nTreesCumu = nTreesCumu + nTreesRel(i)
!     print*, lon, lat, i, nTreesRel(i), nTreesCumu
!     if(nTreesCumu .ge. 0.9) then
!       class90 = i
!       print*, lon, lat, 'exit'
!       exit
!     end if
!   end do
!
!   if(nTreesTot==0) class90 = 0
!
!   print*, 'gaga', lat, lon
!   print*, 'class90', lat, lon, class90
!   zVeg = 1.37 + real(class90)*5.0
!
!   vHeight(lon,lat) = zVeg
!
! ============================================
!
!   laTot = sum(laByHt)
!   laByHtRel = laByHt(0:maxhc)/laTot
!   if(laTot==0.0) laByHtRel(0:maxhc) = 0.0
!
!   laCumu = 0.0
!
!   do i = 0,maxhc
!     laCumu = laCumu + laByHtRel(i)
!     if(laCumu .gt. 0.95) then
!       class95 = i
!       exit
!     end if
!   end do
!
!   if(laTot == 0.0) class95 = 0
!
!   zVeg = 1.37 + real(class95) * 5.0
!
!   vHeight(lon,lat) = zVeg

  call vegPars(lon,lat,thiscell,nspc)



END SUBROUTINE LocalForestDynamics
!****<Here the subroutine ends>********************************************************************
