!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Helper subroutines for parallel execution of code with MPI
!
! Contains:
! module MPIvars  ! Contains all variables necessary for MPI execution (MPI-specific or not)
! module outvars  ! Contains helper variables for writing output tables
! subroutine MPIInit  ! Initialization of MPI environment
! subroutine MPIStripes (Determines width and address of stripes to be read and processed by each node)
! subroutine MPIReadStripes (Reads input from a binary/ASCII-like file for each stripe and calculates statistics for each stripe)
!                            Implemented both for INTEGER and REAL arrays
!							 NOT USED - Incompatible with data format used
! subroutine MPIReduceStats  (Calculates statistics (count,max,min,sum,mean) for the whole
!							  grid from statistics for each stripe)
! subroutine MPIWriteTable  (Reduces statistics from all stripes and writes output tables)
! subroutine MPISeedExchangeList (for seed dispersal across stripes in TreeMig)
!-------------------------------------------------------------------------------

module MPIvars

!use fp_ops

use forhycsglobals, only: dblP

implicit none

logical:: doParallel   					! determines if PREVAH shall be run in parallel or not

integer:: numproc, &   					! number of processes
          rank, &      					! rank of each process
          ierr, &      					! error code
          root, &     					! rank of master/root node
          rc           					! used for MPI_ABORT

integer:: slaveRank						! used in 'slaves' communicator

character(len=256)::procName
integer:: procNameLen

integer,allocatable :: status(:)
integer:: mpiTag

integer:: slaves  						! Used as a communicator
integer:: color  						! Used to define Slaves communicator

integer:: numslaves    					! number of processes that do slave work

integer:: nrowLocal    					! number of rows read/processed by each process
integer:: nrowGlobal  					! number of rows of the whole grid
integer :: cntGlobal    ! number of non-No Data cells
integer:: firstRow, lastRow
integer, allocatable :: nrowLocalArray(:),firstRowArray(:),lastRowArray(:)
integer:: ycoorLocal
integer:: loopRowA, loopRowB						! Special variable used where certain outer

integer::numStatElem  ! Number of REDUCE or ALLREDUCE operations necessary for statistics
character(256),allocatable:: reduceStatOps(:)    	! Specifies which operations to perform while using MPI_REDUCE
								!or MPI_ALLREDUCE for statistics

integer:: sizes(2), subsizes(2), starts(2)  		! Dimensions of subarrays used for writing grids
integer:: filetype,fileinfo  				! Used for writing grids

character(1):: mess	! Dummy message for MPI_SEND and _RECV when writing grids

real,allocatable:: phantomInA(:),phantomOutA(:),&	! Arrays of "phantom" cells for lateral transfer (MTD) between processes
                   phantomInB(:),phantomOutB(:)
real, allocatable:: phantomMaskA(:),phantomMaskB(:)

real, allocatable:: phantomSeedsOut(:,:,:),phantomSeedsIn(:,:,:),phantomSeedsSum(:,:,:)   ! For seed dispersal in TreeMig
                                                                     ! Allocated at TM initialization, once number of species is known
real, allocatable:: phantomSeedsList(:,:)
integer, allocatable:: listLenArray(:)

integer:: tableStart   ! first row of each slave in output tables of TreeMig Report
integer,allocatable:: tableStartArray(:)

integer:: MPIdbltype ! Handle for data type twoDbl (containing two dblP numbers) for Kahan summation

type :: xyztype
  integer :: x, y
  real :: z
end type xyztype

integer :: MPIxyztype  ! MTD
integer :: MPIGlacType
!integer :: kahanOP  ! Handle for operation

!external :: MPIkahansum

!contains


end module MPIvars



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module outvars

implicit none

real, allocatable:: writeArgs(:)
integer,allocatable:: intArgs(:), writeArgOpts(:)
integer:: fh
character(len=256),dimension(18):: formats
data formats/&
"(I5,3I3,2I6,9F8.3,6F8.2,4F8.4)",&						! 1. Hourly output for every HRU (prevahcore)
"(I5,I3,I3,2I6,7F8.3,11F8.2)",&						! 2. Daily output for every HRU (prevahcore)
"(3(I4,1X),7(F8.3,1X),15(F8.2,1X))",&					! 3. Daily average for the whole catchment (prevahendday)
"(I5,3I3,I6,9F8.3,5F8.2,4F8.4)",&						! 4. Hourly output for the whole catchment (prevahendday)
"(I4,3I3,F9.5,F11.2,6F9.5,F11.2,i5)",&					! 5. Hourly runoff components for the whole catchemnt (prevahendday)
"(I4.4,2(A1,I2.2),1X,I2,A3,1X,F10.1,8(1X,F8.3),1X,F6.3)",&		! 6. Hourly runoff components for the whole catchemnt - bulletin (prevahendday)
"(I4,3I3,999F10.4)",&							! 7. Hourly sub-catchment output (prevahendday)
"(I4,I3,I4,I6,17F7.1)",&							! 8. Monthly sums (prevaho, format 140)
"(I3,I5,I4,I6,18F7.1)",&							! 9. Monthly sums (prevaho, format 141)
"(I4,1x,2(I4,1x),I2.2,1x,A22,1x,F5.2)",&					! 10. Forest - Land cover not changed yet (prevaho)
"(I4,1x,2(I4,1x),3(I2.2,1x),A19,1x,F5.2)",&				! 11. Forest - first forest update (prevaho)
"(I4,1x,2(I4,1x),3(I2.2,1x),A18,1x,F5.2)",&				! 12. Forest - Forest updated yet (prevaho)
"(I4,1x,2(I4,1x),3(I2.2,1x),A17,1x,F5.2)",&				! 13. Forest - New forest update (prevaho)
"(I2,I5,I4,I8,11F7.0,6F7.1)",&						! 14. Yearly sums (prevaho)
"(A8,999(1x,F10.3))",&							! 15. Statzones
"(I4,1x,2(I4,1x),I2.2,1x,I1,1x,F5.2)",&					! 16. Dummy format for forest log with only integers and floats
"(I4,3I3,F9.5,F11.2,6F9.5,F11.2,F1.0)",&					! 17. Modified version of format 5 with real instead of integer for last value
"(2(I2,1x),I4,1x,10(F9.2,1x))"&                                   ! 18. Temporary, for glacier mass balance
/

end module outvars
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine MPIInit

use mpi
use MPIvars
use forhycsglobals, only: dblPVal, dblPR
!use glacGlob, only: glacUpdate
implicit none

integer :: structCnt
integer, dimension(2) :: structBL, structTypes
integer(KIND=MPI_ADDRESS_KIND), dimension(2) :: structDisp
integer:: rExtent

!integer :: structCntG
!integer, dimension(4) :: structBLG, structTypesG
!integer(KIND=MPI_ADDRESS_KIND), dimension(4) :: structDispG
!integer(kind=MPI_Address_kind) :: startloc, endloc
!integer, dimension(6) :: structBLG, structTypesG
!integer(KIND=MPI_ADDRESS_KIND), dimension(6) :: structDispG
!integer:: rExtentG, iExtentG, dExtentG
!type(glacUpdate) :: dummyGlac

call MPI_INIT(ierr)
if (ierr .ne. MPI_SUCCESS) then
  print*, "Error starting MPI program. Terminating."
  call MPI_ABORT(MPI_COMM_WORLD, rc, ierr)
end if
call MPI_COMM_RANK(MPI_COMM_WORLD, rank, ierr)
call MPI_COMM_SIZE(MPI_COMM_WORLD, numproc,ierr)
print*, "Hello from process number ", rank
call MPI_GET_PROCESSOR_NAME(procName,procNameLen,ierr)

allocate(status(MPI_STATUS_SIZE))

numslaves = numproc-1

! Create a communicator containing only slaves
if(rank==root) then
  color=0
else
  color=1
end if
call MPI_COMM_SPLIT(MPI_COMM_WORLD,color,0,slaves,ierr)
call MPI_COMM_RANK(slaves, slaveRank, ierr)


! Specify the order of reduce operations
numStatElem = 4
allocate(reduceStatOps(4))
reduceStatOps(1)="MPI_SUM"
reduceStatOps(2)="MPI_MIN"
reduceStatOps(3)="MPI_MAX"
reduceStatOps(4)="MPI_SUM"

mess="a"


! Create a MPI datatype for extended precision floats to be used where
! necessary. The first argument of the MPI_TYPE_CREATE_F90_REAL subroutine
! below must match
call MPI_TYPE_CREATE_F90_REAL(dblPVal,dblPR,MPIdbltype,ierr)
call MPI_TYPE_COMMIT(MPIdbltype,ierr)

! Create a MPI datatype for (xcoor, ycoor, snow) structure, to be used in MTD routine.
! Structure: (integer, integer, real)
structCnt = 2
structBL(1) = 2
structBL(2) = 1
call MPI_TYPE_EXTENT(MPI_REAL,rExtent,ierr)

structDisp(1) = 0
structDisp(2) = 2 * rExtent
structTypes(1) = MPI_INTEGER
structTypes(2) = MPI_REAL

call MPI_TYPE_CREATE_STRUCT(structCnt,structBL,structDisp,structTypes,MPIxyztype,ierr)
call MPI_TYPE_COMMIT(MPIxyztype,ierr)

end subroutine

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine MPIStripes

use mpi
use MPIvars
use prevahglobals, only: nrow,ncol,gezg,rnodat
implicit none

integer:: i  ! Loop index
integer:: ctrl
integer, dimension(nrow):: cells,cellssum,cellsdist
integer:: cellsTot,optNum,lastRowP
integer,dimension(1):: optLoc

allocate(nrowLocalArray(numslaves))
allocate(firstRowArray(numslaves))
allocate(lastRowArray(numslaves))
allocate(tableStartArray(numslaves))


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! A more intelligent procedure for distributing the rows among processes, taking into account
! the number of No Data cells in each row.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Create a vector of non-NA count per row (cells)
! Create a vector with cumulative sum of non-NA cells for each row (cellssum)

do i=1,nrow
  cells(i)=count(gezg(:,i) .ne. rnodat)
  if(i==1) then
    cellssum(i)=cells(i)
  else
    cellssum(i)=cellssum(i-1)+cells(i)
  end if
end do

! Calculate total number of non-NA cells
cellsTot = sum(cells)

! Calculate the optimal number of non-NA cells per stripe
optNum=int(cellsTot/numslaves)

! First slave:
! FirstRow=1
! Calculates distance of each cumulative sum from the optimal number of non-NA cells
if(rank==1) then
! Finds index of row closest to the optimal one -> This is lastRow
  firstRow=1
  cellsDist=cellsSum-optNum
  optLoc=minloc(abs(cellsDist))
  lastRow=optLoc(1)

! Sends lastRow to the next slave.
  call MPI_SEND(lastRow,1,MPI_INTEGER,rank+1,mpiTag,MPI_COMM_WORLD,ierr)
  print*, "Process", rank, "covers rows", firstRow, "through", lastRow
! Sends cumulative sum at lastRow to next slave (necessary?!)

else if(rank .gt. 1) then
! Slaves > 1:
! Receive lastRow from previous slave; firstRow=that+1

  call MPI_RECV(lastRowP,1,MPI_INTEGER,rank-1,mpiTag,MPI_COMM_WORLD,status,ierr)
  firstRow=lastRowP+1

! Receive cumulative sum at lastRow from previous slave (necessary?!)

! Calculate distance from optimal number of cells; include sum of previous slave
  if(rank .lt. numslaves) then
    cellsDist=(cellsSum-cellssum(lastRowP))-optNum

! Set cellsdist(1:lastrowoflastslave) to an absurdly large number
    cellsDist(1:lastRowP)=999999

! Get lastRow
    optLoc=minloc(abs(cellsDist))
    lastRow=optLoc(1)
! Send lastRow to next slave
    call MPI_SEND(lastRow,1,MPI_INTEGER,rank+1,mpiTag,MPI_COMM_WORLD,ierr)
    print*, "Process", rank, "covers rows", firstRow, "through", lastRow

! Last slave:
  else
! Trivial
    firstRow=lastRowP+1
    lastRow=nrow
    print*, "Process", rank, "covers rows", firstRow, "through", lastRow
  end if
end if  ! end of cells distribution

nrowLocal=(lastRow-firstRow) + 1

! Each slave calculates the first line of its respective stripe in
! TreeMig's Report output tables

if(rank .gt. 0) then
  tableStart = ((firstRow-1)*ncol)+1
end if

print*, "tableStart", tableStart, rank

call MPI_BARRIER(slaves,ierr)

! First and last rows get gathered by one of the slaves and communicated to the master

if(rank==1) then
  firstRowArray(1)=1
  lastRowArray(1)=lastRow
  tableStartArray(1) = tableStart

  do i=2,numslaves
    call MPI_RECV(firstRowArray(i),1,MPI_INTEGER,i,22,MPI_COMM_WORLD,status,ierr)
    call MPI_RECV(lastRowArray(i),1,MPI_INTEGER,i,23,MPI_COMM_WORLD,status,ierr)
    call MPI_RECV(tableStartArray(i),1,MPI_INTEGER,i,24,MPI_COMM_WORLD,status,ierr)
  end do
!  call MPI_GATHER(firstRow,1,MPI_INTEGER,firstRowArray,1,MPI_INTEGER,rank,slaves,ierr)
  call MPI_SEND(firstRowArray,numslaves,MPI_INTEGER,root,mpiTag,MPI_COMM_WORLD,ierr)
!  call MPI_GATHER(lastRow,1,MPI_INTEGER,lastRowArray,1,MPI_INTEGER,rank,slaves,ierr)
  call MPI_SEND(lastRowArray,numslaves,MPI_INTEGER,root,mpiTag,MPI_COMM_WORLD,ierr)
  call MPI_SEND(tableStartArray,numslaves,MPI_INTEGER,root,mpiTag,MPI_COMM_WORLD,ierr)
else if(rank .gt. 1) then
  call MPI_SEND(firstRow,1,MPI_INTEGER,1,22,MPI_COMM_WORLD,ierr)
  call MPI_SEND(lastRow,1,MPI_INTEGER,1,23,MPI_COMM_WORLD,ierr)
  call MPI_SEND(tableStart,1,MPI_INTEGER,1,24,MPI_COMM_WORLD,ierr)
else if(rank==0) then
  call MPI_RECV(firstRowArray,numslaves,MPI_INTEGER,1,mpiTag,MPI_COMM_WORLD,status,ierr)
  call MPI_RECV(lastRowArray,numslaves,MPI_INTEGER,1,mpiTag,MPI_COMM_WORLD,status,ierr)
  call MPI_RECV(tableStartArray,numslaves,MPI_INTEGER,1,mpiTag,MPI_COMM_WORLD,status,ierr)
  nrowLocalArray=(lastRowArray-firstRowArray) + 1

end if

call MPI_BCAST(firstRowArray,numslaves,MPI_INTEGER,root,MPI_COMM_WORLD,ierr)
call MPI_BCAST(lastRowArray,numslaves,MPI_INTEGER,root,MPI_COMM_WORLD,ierr)
call MPI_BCAST(nrowLocalArray,numslaves,MPI_INTEGER,root,MPI_COMM_WORLD,ierr)

return
end subroutine

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine MPIReduceStats(local)

use MPIVars
use prevahglobals, only: wstats, rnodat

use mpi
!use fp_ops
implicit none
!include 'mpif.h'

real,intent(inout):: local(6)
real::global(6),current,curSums(numslaves), curSort(numslaves)
real:: curMin
real:: stdevs(numslaves),counts(numslaves),means(numslaves)
real, dimension(numslaves):: essg,gss
real:: ess,tgss,gv
integer:: i,j  ! loop index

! gstats(1-4): Count (non-NA), min, max, sum

do i=1,numStatElem
 ! print*, "i=",i,"wstats(i)=", wstats(i), rank
  select case (i)
    case(1)
      call MPI_ALLREDUCE(local(i),current,1,MPI_REAL,&
      MPI_SUM,slaves, ierr)
    case(2)
      call MPI_ALLREDUCE(local(i),current,1,MPI_REAL,&
      MPI_MIN,slaves, ierr)
    case(3)
      call MPI_ALLREDUCE(local(i),current,1,MPI_REAL,&
      MPI_MAX,slaves, ierr)
    case(4)
     call MPI_ALLREDUCE(local(i),current,1,MPI_REAL,&
      MPI_SUM,slaves, ierr)  ! DUMMY
  end select
  global(i) = current
end do



global(5) = global(4)/global(1)


global(6) = 0

! End
local=global

!print*, "End reached", rank
return

end subroutine


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine MPIWriteTable(fileID,fmtNumber)

use mpi
use outvars
use MPIvars
use prevahglobals, only:rnodat

implicit none

integer:: fileID  ! Identifier that was assigned to the file in OPEN statement
integer:: fmtNumber
integer:: year
real, allocatable:: globalArgs(:), localArgs(:)
integer:: i  ! loop index
real :: tempSum
real:: current
logical:: dibs
integer::firstProcess

dibs=.false.

! reduce all variables contained in writeArgs

allocate(globalArgs(size(writeArgs)))
allocate(localArgs(numslaves))

do i=1,size(writeArgs)
!print*, "i=", i, "of", size(writeArgs), rank
  select case(writeArgOpts(i))
    case(1)  ! Sum
      call MPI_ALLREDUCE(writeArgs(i),globalArgs(i),1,MPI_REAL,&
                         MPI_SUM,MPI_COMM_WORLD,ierr)
!      call MPI_ALLGATHER(writeArgs(i),1,MPI_REAL,localArgs,1,MPI_REAL,slaves,ierr)
!      globalArgs(i) = local_kahan_sum(localArgs,numslaves,rnodat)
    case(2)  ! Mean
      call MPI_ALLREDUCE(writeArgs(i),globalArgs(i),1,MPI_REAL,&
                         MPI_SUM,MPI_COMM_WORLD,ierr)
!      call MPI_ALLGATHER(writeArgs(i),1,MPI_REAL,localArgs,1,MPI_REAL,slaves,ierr)
!      tempSum = local_kahan_sum(localArgs,numslaves,rnodat)
      globalArgs(i)=globalArgs(i)/numslaves
  end select
end do

!print*, "loop done", rank

if(rank==root) then
!  print*, formats(fmtNumber)
!print*, "yoyo Master here"
  write(fileID,formats(fmtNumber),advance="yes")(intArgs(i),i=1,size(intArgs)),&
                                  (globalArgs(i),i=1,size(globalArgs))
!print*, "writing done"

end if

deallocate(globalArgs)
deallocate(localArgs)
return

end subroutine


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


subroutine MPISeedExchangeList

! This subroutine manages the transfer of dispersed seeds across stripes. It is possible to transfer the whole phantom seed arrays
! across processes, but this may require lots of memory. To reduce the volume of transferred data, each process creates a list of
! lat/lon/species combinations for which there are seeds to transfer, and sends this list to the master. The master merges the list
! and broadcasts it to all slaves. Finally, the slaves update their phantomSeeds arrays based on the information from master.
! The "phantom seeds" will be added to the corresponding cell's verynewseeds in the subroutine updateNewSeeds (Interact.f90).

use mpi
use MPIVars, only: rank, root, numslaves,status,mpiTag,ierr,phantomSeedsOut,phantomSeedsSum, phantomSeedsList,&
                   listLenArray, phantomSeedsIn, MPIdbltype
use prevahglobals, only: ncol,nrow
use forhycsglobals, only: nspc_g, dblP

implicit none

integer*4:: listLen     ! length of array to be transferred to Master
integer:: listLenTot  ! used by Master only
integer:: i, j, k  ! Loop indices
integer:: cnt        ! counter
integer:: listCount  ! helper variable for merging lists (used by master only)
integer:: aggrLen    ! length of aggregated array
!real, allocatable:: phantomSeedsAggr(:,:)
integer, allocatable :: phantomSeedsCoords(:,:), coordSlice(:,:)  ! Array containing the coordinates (x,y,species) of the cells for which there is a value
real(kind=dblP), allocatable :: phantomSeedsVect(:), vectSlice(:)  ! Contains the actual values to be transferred; indexed in the same way as the 2nd dimension of phantomSeedsCoords
integer:: lat, lon, spec
logical, allocatable:: mask(:)

if (rank .ne. root) then
! SLAVES: each slave counts the number of lat/lon/species combinations for which there are seeds to disperse.
! This will be the length of the array passed to the master


  listLen = count(phantomSeedsOut .ne. 0)
  call MPI_SEND(listLen,1,MPI_INTEGER,root,251,MPI_COMM_WORLD,ierr)

! Create a list of all array elements for which there are seeds to transfer.
  allocate(phantomSeedsCoords(3,listLen))
  allocate(phantomSeedsVect(listLen))
  cnt = 1

  do k = 1,nspc_g
  do j = 1,nrow
  do i = 1,ncol
    if(phantomSeedsOut(i,j,k) > 0) then
      phantomSeedsCoords(1,cnt) = i
      phantomSeedsCoords(2,cnt) = j
      phantomSeedsCoords(3,cnt) = k
      phantomSeedsVect(cnt) = phantomSeedsOut(i,j,k)
      cnt = cnt + 1
    end if
    if((cnt > listLen+1) .and. (listLen > 0)) print*, "cnt", cnt, "listLen", listLen, rank
  end do
  end do
  end do

  call MPI_SEND(phantomSeedsCoords,3*listLen,MPI_INTEGER,root,252,MPI_COMM_WORLD,ierr)
  call MPI_SEND(phantomSeedsVect,listLen,MPIdbltype,root,253,MPI_COMM_WORLD,ierr)

  aggrLen = 0  ! Initialization


else  ! MASTER
  allocate(listLenArray(numslaves))

  do i = 1,numslaves
    call MPI_RECV(listLen,1,MPI_INTEGER,i,251,MPI_COMM_WORLD,status,ierr)
    listLenArray(i) = listLen
  end do

  listLenTot = sum(listLenArray)
! Receive lists from the slaves. For now, all entries are merged into one array
  allocate(phantomSeedsCoords(3,listLenTot))
  allocate(phantomSeedsVect(listLenTot))


  listCount = 1
  do i = 1,numslaves

    allocate(coordSlice(3, listLenArray(i)))
    allocate(vectSlice(listLenArray(i)))
    call MPI_RECV(coordSlice,3*listLenArray(i),MPI_INTEGER,&
                  i, 252,MPI_COMM_WORLD,status,ierr)
    phantomSeedsCoords(:,listCount:listCount+listLenArray(i)-1) = coordSlice

    call MPI_RECV(vectSlice,listLenArray(i),MPIDbltype,&
                  i, 253,MPI_COMM_WORLD,status,ierr)
    phantomSeedsVect(listCount:listCount+listLenArray(i)-1) = vectSlice


    listCount = listCount + listLenArray(i)

    deallocate(coordSlice)
    deallocate(vectSlice)
  end do


! Some cells might receive phantom seeds from more that one stripe (i.e. the lon/lat/species combination
! appears more than once in the merged list). To correct this, sums are calculated for each l/l/s combination

  phantomSeedsIn = 0.0
  do i = 1,listLenTot
    lon = phantomSeedsCoords(1,i)
    lat = phantomSeedsCoords(2,i)
    spec = phantomSeedsCoords(3,i)
    phantomSeedsIn(lon,lat,spec) = phantomSeedsIn(lon,lat,spec)+phantomSeedsVect(i)
  end do

  print*, "PhantomSeedsIn master", sum(phantomSeedsIn)

! To reduce the volume of sent messages, the temporary phantom seeds array is "packed" again into a list
  listLen = count(phantomSeedsIn .ne. 0.0)

! Create a list of all array elements for which there are seeds to transfer.
  deallocate(phantomSeedsCoords)
  deallocate(phantomSeedsVect)
  allocate(phantomSeedsCoords(3,listLen))
  allocate(phantomSeedsVect(listLen))
  cnt = 1

  do k = 1,nspc_g
  do j = 1,nrow
  do i = 1,ncol
    if(phantomSeedsIn(i,j,k) > 0) then
      phantomSeedsCoords(1,cnt) = i                       ! longitude
      phantomSeedsCoords(2,cnt) = j                       ! latitude
      phantomSeedsCoords(3,cnt) = k                       ! species
      phantomSeedsVect(cnt) = phantomSeedsIn(i,j,k)        ! number of seeds
      cnt = cnt + 1
    end if
    if((cnt > listLen+1) .and. (listLen > 0)) print*, "cnt", cnt, "listLen", listLen, rank
  end do
  end do
  end do



end if  ! Slaves / Master

! Master broadcasts dimension of final array to all slaves

call MPI_BCAST(listLen,1,MPI_INTEGER,root,MPI_COMM_WORLD,ierr)


if (rank .ne. root) then
  deallocate(phantomSeedsCoords)
  deallocate(phantomSeedsVect)
  allocate(phantomSeedsCoords(3,listLen))
  allocate(phantomSeedsVect(listLen))
end if

! Master broadcasts final array to all slaves

call MPI_BCAST(phantomSeedsCoords,3*listLen,MPI_INTEGER,root,MPI_COMM_WORLD,ierr)
call MPI_BCAST(phantomSeedsVect,listLen,MPIDbltype,root,MPI_COMM_WORLD,ierr)

! Update the PhantomSeedsSum array

if (rank .ne. root) then
  phantomSeedsSum = 0
  do i = 1,listLen

    lon = phantomSeedsCoords(1,i)
    lat = phantomSeedsCoords(2,i)
    spec = phantomSeedsCoords(3,i)
    phantomSeedsSum(lon,lat,spec) = phantomSeedsVect(i)
  end do


end if

if(allocated(phantomSeedsCoords)) deallocate(phantomSeedsCoords)
if(allocated(phantomSeedsVect)) deallocate(phantomSeedsVect)
if(allocated(listLenArray)) deallocate(listLenArray)

! Reset
phantomSeedsOut = 0.0
end subroutine
