!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Writes tables of mean monthly temperature [°C] and monthly precipitation sums [cm]
! in the format required by BioClim
!-------------------------------------------------------------------------------

subroutine MPIbcOut(year)

  use mpi
  !use prevahglobals, only: meanTOut, sumPOut, nrow, ncol, rnodat, gezg, rxll, ryll, rgridsize, landin
  use prevahglobals, only: sumPOut, nrow, ncol, rnodat, gezg, rxll, ryll, rgridsize, landin
  use MPIvars, only: rank, root, numslaves, nrowLocal, nrowLocalArray, &
      status, ierr, firstRow, lastRow, firstRowArray, lastRowArray
  use forhycsglobals, only: TMDir, meant, meantC


  implicit none

  integer, intent(in) :: year
  integer :: locArraySize
  real, allocatable :: locArray(:,:,:), globArray(:,:,:)
  character*4 :: cyear
  character*100 :: outname, toutname
  integer :: xcoor, ycoor, actxcoor, actycoor
  real, dimension(12) :: curT, curTc, curP
  integer :: i

!===============================================

  if(rank .ne. root) then
! Set size of array for each stripe
    locArraySize = nrowLocal*ncol*36
    allocate(locArray(ncol,nrowLocal,36))

    ! transfer of temperature and precip tables
    locArray(:,:,1:12) = meant(:,firstRow:lastRow,:)
    locArray(:,:,13:24) = meantc(:,firstRow:lastRow,:)
    locArray(:,:,25:36) = sumPOut(:,firstRow:lastRow,:)
    call MPI_SEND(locArray,locArraySize,MPI_INTEGER,root,22,MPI_COMM_WORLD,ierr)

    deallocate(locArray)

  else   ! Master
    allocate(globArray(ncol,nrow,36))
    globArray = 0.0
    ! Receive sub-grids from slaves and stitch them together
    do i = 1,numslaves
      locArraySize = ncol*nrowLocalArray(i)*36
      allocate(locArray(ncol,nrowLocalArray(i),36))
      call MPI_RECV(locArray,locArraySize,MPI_INTEGER,i,22,MPI_COMM_WORLD,status,ierr)
      globArray(:,firstRowArray(i):lastRowArray(i),:) = locArray
      deallocate(locArray)
    end do
    ! Prepare to write
    write(cyear,"(I4.4)") year  ! Year as char
    open(unit=555, file=trim(TMDir)//"pre_clim_GoT_"//cyear//".txt")
      do xcoor = 1,ncol
      do ycoor = 1,nrow
        actxcoor = rxll + (xcoor-1)*rgridsize
        actycoor = ryll + (ycoor-1)*rgridsize
        curT = globArray(xcoor,ycoor,1:12)
        curTc = globArray(xcoor,ycoor,13:24)
        curP = globArray(xcoor,ycoor,25:36)

        write(555,"(I8,X,I8,X,I4,X,36(F7.2,X))") int(actxcoor), int(actycoor), year, curT(1:12),curTc(1:12),curP(1:12) 
      end do
      end do

    close(555)
    deallocate(globArray)
  end if  ! Master / slave



end subroutine MPIbcOut
