!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Additional snow-induced mortality function

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  Mort
!
!  Purpose:
!
!  Remarks:  contains the subroutine
!
!            Mort (<LocalForestDynamics<DoLocalWork<DistrLocalDynThisTime<
!                    TimeLoop<TreeMig)
!
!==============================================================================


SUBROUTINE Mort(spp, height, light, srvinhl, numb,   disturbanceMort, thiscell)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY:  spc, spec, CurrStateInCell      ! Use species parameters

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: spp,    &   ! number of species                                      [=spec]
                       height, &   ! height class                                         [=height]
                       light       ! lightclass                                           [=ilight]
REAL, INTENT(IN)    :: numb     ! # of trees                       [=nInH  <- rNoTreesUnderCond]
REAL, INTENT(IN)    :: disturbanceMort  ! mortality in this cell by disturbance
REAL, INTENT(OUT)   :: srvinhl     ! remaining ind. per spec/height/light class         [=survNInH]

!<Local variables>---------------------------------------------------------------------------------
REAL :: gfc,    &               ! Stress factor due to growth stress               [=rStessDepDiam]
        efc,    &               ! Stress factor due to env. stress               [=rStressDepOther]
        cmp,    &               ! Comparison of growth- and enviro.-stress factors       [=compare]
        cmplow, &               ! This is "cmp" of the lower light class              [=compareOld]
        vmort,  &               ! Stress related mortality                           [=rStresvmort]
        tmort                   ! Total mortality (constant & stress based)            [=totalMort]

real :: sMort                   ! Snow-induced mortality

TYPE(CurrStateInCell):: thiscell
!<Here the subroutine starts>**********************************************************************
  !----- determine, whether growth dependent vitality is bigger than threshold (0.0003)
  gfc = spc(spp)%growthVitality(height,light) / 0.0003

  !-----determine, whether environment dependent vitality is bigger than threshold (0.1)
  efc = spc(spp)%envVitality(height,light) / 0.1

  !----- get the more severe of them (minimum)
  cmp = MIN (gfc, efc)

  IF (cmp < 1.0) THEN !----- if lower vitality  below threshold, set variable mortality to 1
    vmort = 1.0
  ELSE ! -----determine mortalities in next lower light class
    IF (light > 1) THEN !----- more light than in first light class
      IF (cmp == efc) THEN
        cmplow = spc(spp)%envVitality(height,light - 1) / 0.1
      ELSE
        cmplow = spc(spp)%growthVitality(height,light - 1) / 0.0003
      END IF
    ELSE !----- first light class, verrrry dark!
      cmplow = 0.0
    END IF

    !----- interpolate if next lower light class is below threshold
    IF (cmplow < 1.0) THEN
      vmort = (1.0 - cmplow) / (cmp - cmplow)
    ELSE
      vmort = 0.0
    END IF
  END IF
  !----- determine total mortality
 ! IF (height >= 0) THEN
  IF (height > 0) THEN
     tmort = MIN (spec(spp)%cmort + disturbanceMort + vmort * 0.184, 1.0) ! Mortality is assumed to act only after 3 subseqent bad years, but distributed over the years
     !tmort = MIN (spec(spp)%cmort + disturbanceMort + vmort, 1.0)
  ELSE
  ! For lowest HC, get more severe mortality term between vitality-related mortality
  ! and snow-induced mortality. This only applies to Larix decidua, Pinus cembra and
  ! P. montana. For all other species, snow-induced mortality is set to zero.
  ! MS, after Zurbriggen et al. (2014)
    sMort = spc(spp)%snowMortality(light)
    if(sMort .gt. vmort) vmort = sMort
    tmort = MIN (spec(spp)%cmort + disturbanceMort + vmort, 1.0)         ! Mortality is assumed to act directly in sapling heigth class
   !   tmort = MIN (spec(spp)%cmort + disturbanceMort + vmort* 0.184, 1.0)    !
  END IF
  IF (thiscell%chestnutcult.and.spp.ne.16) tmort=1.0
  !----- determine survivors
  srvinhl = (1.0 - tmort) * numb
  !----- update state variable
  thiscell%sp(spp)%numin(height) = thiscell%sp(spp)%numin(height) - (tmort * numb)
  IF (thiscell%sp(spp)%numin(height) < 0) THEN
  !  WRITE(1070,*) 'neg.trees in mort ', height, thiscell%sp(spp)%numin(height), numb, tmort
    thiscell%sp(spp)%numin(height) = 0.0
  END IF

END SUBROUTINE Mort
!****<Here the subroutine ends>********************************************************************
