!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------



!==============================================================================
!
!  Name   :  PrepareOutput
!
!  Remarks:  contains the subroutines
!
!            PrepareOutput (<TreeMig)
!            PrepareMyOutput (<TreeMigSlaveWork)
!
!--------------------------------------------------------
!
! LIST OF FILES AND UNITS
!  INPUT:
! 10   List of control files
! 11   Control Parameters
! 12   Species parameters
! 13   Immigration
! 14   Bioclimate
! 15   Constant Environment
! 71   statefile
! 73   2nd statefile
!  OUTPUT:
! 70    : logfile
! 71    : statefile
! 72    : old statefile
! 75    : resultsfile
! 20-42 : simulation output data (see above)
! 51:   Constant Environment
! 52:   Species parameters
! 53:   Immigration
! 54:   Bioclimate
! 55:   Constant Environment
!
!==============================================================================
! design          : H. Lischke, N. Zimmermann
! author(s)       : H. Lischke, N. Zimmermann
! implementation  : H. Lischke, N. Zimmermann
! cleaner         : T.J. Loeffler
! copyright       : (c) 1999, 03 by H. Lischke
!==============================================================================

SUBROUTINE PrepareOutput(tabsepOutput, experimentID, nspc, Iam)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: writeOutput, pathletter2
use prevahglobals, ONLY: diroutput
use forhycsglobals, only: tpswitch,meant,sump

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)       :: nspc, Iam              ! site index
LOGICAL, INTENT(IN)       :: tabsepOutput
CHARACTER(12), INTENT(IN) :: experimentID

!<Local variables>---------------------------------------------------------------------------------
INTEGER      :: iunit, tmpos
CHARACTER*30 :: biomassname, numbername, seedname, antaname, pollenname, &   ! output file names
                ingrowthname, biodivname, &
                leafareaname !FORHYCS
CHARACTER*30 :: hstructname(16)
CHARACTER*256 :: pathfn

!<Here the subroutine starts>**********************************************************************
  biomassname     = experimentID//'_T_'//'.txt'
  numbername      = experimentID//'_N_'//'.txt'
  seedname        = experimentID//'_S_'//'.txt'
  antaname        = experimentID//'_A_'//'.txt'
  pollenname      = experimentID//'_P_'//'.txt'
  ingrowthname    = experimentID//'_I_'//'.txt'
  biodivname      = experimentID//'_D_'//'.txt'
  leafareaname =experimentID//'_L_'//'.txt' !FORHYCS
  hstructname(1)  = experimentID//'_H0__' //'.txt'
  hstructname(2)  = experimentID//'_H1__' //'.txt'
  hstructname(3)  = experimentID//'_H2__'//'.txt'
  hstructname(4)  = experimentID//'_H3__'//'.txt'
  hstructname(5)  = experimentID//'_H4__'//'.txt'
  hstructname(6)  = experimentID//'_H5__'//'.txt'
  hstructname(7)  = experimentID//'_H6__'//'.txt'
  hstructname(8)  = experimentID//'_H7__'//'.txt'
  hstructname(9)  = experimentID//'_H8__'//'.txt'
  hstructname(10) = experimentID//'_H9__'//'.txt'
  hstructname(11) = experimentID//'_H10_'//'.txt'
  hstructname(12) = experimentID//'_H11_'//'.txt'
  hstructname(13) = experimentID//'_H12_'//'.txt'
  hstructname(14) = experimentID//'_H13_'//'.txt'
  hstructname(15) = experimentID//'_H14_'//'.txt'
  hstructname(16) = experimentID//'_H15_'//'.txt'

  !----- open file for biomass output
  IF (writeOutput%biomass)   CALL OpenAndWriteHeader (1020, biomassname, tabsepOutput, nspc)
  !----- open file for seedbed output - only works if the seeds are stored in the seedbed variable
  IF (writeOutput%number) CALL OpenAndWriteHeader (1021, numbername, tabsepOutput, nspc)
  !----- open file for seedbed output - only works if the seeds are stored in the seedbed variable
  IF (writeOutput%seeds) CALL OpenAndWriteHeader (1022, seedname, tabsepOutput, nspc)
  !----- open file for antagonist output
  IF (writeOutput%antagonists) CALL OpenAndWriteHeader (1023, antaname, tabsepOutput, nspc)
  !----- open file for pollen output
  IF (writeOutput%pollen) CALL OpenAndWriteHeader (1024, pollenname, tabsepOutput, nspc)
  !----- open file for ingrowth output
  IF (writeOutput%ingrowth) CALL OpenAndWriteHeader (1025, ingrowthname, tabsepOutput, nspc)
  !----- open file for biodiversity output
  IF (writeOutput%leafarea) CALL OpenAndWriteHeader (1026, leafareaname, tabsepOutput, nspc)
  !----- open file for leaf area output FORHYCS
  IF (writeOutput%biodiv) CALL OpenAndWriteHeader (1042, biodivname, tabsepOutput, nspc)
  !---- open file for height structure output
  IF (writeOutput%hstruct) THEN
    DO iunit=0,15
      CALL OpenAndWriteHeader (iunit+1166, hstructname(iunit +1), tabsepOutput, nspc)
    END DO
  END IF
  CALL OpenAndWriteHeader (1244, experimentID//"_Av0.txt      ", tabsepOutput, nspc)
  CALL OpenAndWriteHeader (1241, experimentID//"_Av1.txt      ", tabsepOutput, nspc)
  CALL OpenAndWriteHeader (1242, experimentID//"_Av2.txt      ", tabsepOutput, nspc)
  CALL OpenAndWriteHeader (1243, experimentID//"_Av3.txt      ", tabsepOutput, nspc)

 !FORHYCS

  CALL POSNAM (diroutput,1,tmpos)
  pathfn =  diroutput(1:tmpos)//'treemig'//pathletter2//'R'//pathletter2//experimentID//'_'//tpswitch//"_BCO.txt      "
  OPEN(UNIT=1237, FILE=trim(pathfn), STATUS='unknown')
  WRITE(1237,100)  "x,y,year,DD,WiT,Dr,Dr2,EBJS,EPJS,EIJS"
  100 FORMAT(1(A37,A1))

  CALL POSNAM (diroutput,1,tmpos)
  pathfn =  diroutput(1:tmpos)//'treemig'//pathletter2//'R'//pathletter2//experimentID//'_'//tpswitch//"_CO.txt      "
  OPEN(UNIT=1238, FILE=trim(pathfn), STATUS='unknown')
  WRITE(1238,101)  "x,y,year,slp,asp,MeanT(1),MeanT(2),MeanT(3),MeanT(4),MeanT(5),MeanT(6),MeanT(7),MeanT(8),MeanT(9),MeanT(10),MeanT(11),MeanT(12),SumP(1),SumP(2),SumP(3),SumP(4),SumP(5),SumP(6),SumP(7),SumP(8),SumP(9),SumP(10),SumP(11),SumP(12)"
  101 FORMAT(1(A226,A1))

END SUBROUTINE  PrepareOutput
!****<Here the subroutine ends>********************************************************************

SUBROUTINE PrepareMyOutput(tabsepOutput, experimentID, nspc, Iam)
!  For the parallel version
!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: writeOutput

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN)       :: nspc , Iam             ! site index
LOGICAL, INTENT(IN)       :: tabsepOutput
CHARACTER(12), INTENT(IN) :: experimentID

!<Local variables>---------------------------------------------------------------------------------
INTEGER      :: iunit
CHARACTER(30) :: biomassname, numbername, seedname, antaname, pollenname, &   ! output file names
                ingrowthname, biodivname,&
                leafareaname !FORHYCS
CHARACTER(30) :: hstructname(16)
CHARACTER(3)    :: IamTxt
!<Here the subroutine starts>**********************************************************************

!  IamTxt =CHAR(48+Iam/10)//CHAR(48+Iam-(Iam/10)*10)
  CALL IntToChar(3,Iam,IamTxt)
  biomassname     = experimentID//IamTxt//'_T_'//'.txt'
  numbername      = experimentID//IamTxt//'_N_'//'.txt'
  seedname        = experimentID//IamTxt//'_S_'//'.txt'
  antaname        = experimentID//IamTxt//'_A_'//'.txt'
  pollenname      = experimentID//IamTxt//'_P_'//'.txt'
  ingrowthname    = experimentID//IamTxt//'_I_'//'.txt'
  biodivname      = experimentID//IamTxt//'_D_'//'.txt'
  leafareaname      = experimentID//IamTxt//'_L_'//'.txt' !FORHYCS
  hstructname(1)  = experimentID//IamTxt//'_H0__' //'.txt'
  hstructname(2)  = experimentID//IamTxt//'_H1__' //'.txt'
  hstructname(3)  = experimentID//IamTxt//'_H2__'//'.txt'
  hstructname(4)  = experimentID//IamTxt//'_H3__'//'.txt'
  hstructname(5)  = experimentID//IamTxt//'_H4__'//'.txt'
  hstructname(6)  = experimentID//IamTxt//'_H5__'//'.txt'
  hstructname(7)  = experimentID//IamTxt//'_H6__'//'.txt'
  hstructname(8)  = experimentID//IamTxt//'_H7__'//'.txt'
  hstructname(9)  = experimentID//IamTxt//'_H8__'//'.txt'
  hstructname(10) = experimentID//IamTxt//'_H9__'//'.txt'
  hstructname(11) = experimentID//IamTxt//'_H10_'//'.txt'
  hstructname(12) = experimentID//IamTxt//'_H11_'//'.txt'
  hstructname(13) = experimentID//IamTxt//'_H12_'//'.txt'
  hstructname(14) = experimentID//IamTxt//'_H13_'//'.txt'
  hstructname(15) = experimentID//IamTxt//'_H14_'//'.txt'
  hstructname(16) = experimentID//IamTxt//'_H15_'//'.txt'

  !----- open file for biomass output
  IF (writeOutput%biomass) &
     CALL OpenAndWriteHeader  (1020, biomassname, tabsepOutput, nspc)
  !----- open file for seedbed output - only works if the seeds are stored in the seedbed variable
  IF (writeOutput%number) CALL OpenAndWriteHeader (1021, numbername, tabsepOutput, nspc)
  !----- open file for seedbed output - only works if the seeds are stored in the seedbed variable
  IF (writeOutput%seeds) CALL OpenAndWriteHeader (1022, seedname, tabsepOutput, nspc)
  !----- open file for antagonist output
  IF (writeOutput%antagonists) CALL OpenAndWriteHeader (1023, antaname, tabsepOutput, nspc)
  !----- open file for pollen output
  IF (writeOutput%pollen) CALL OpenAndWriteHeader (1024, pollenname, tabsepOutput, nspc)
  !----- open file for ingrowth output
  IF (writeOutput%ingrowth) CALL OpenAndWriteHeader (1025, ingrowthname, tabsepOutput, nspc)
  !----- open file for biodiversity output
    IF (writeOutput%leafarea) CALL OpenAndWriteHeader (1026, leafareaname, tabsepOutput, nspc)
  !----- open file for leaf area output !FORHYCS
  IF (writeOutput%biodiv) CALL OpenAndWriteHeader (1042, biodivname, tabsepOutput, nspc)
  !---- open file for height structure output
  IF (writeOutput%hstruct) THEN
    DO iunit=0,15
      CALL OpenAndWriteHeader (iunit+1166, hstructname(iunit + 1), tabsepOutput, nspc)
    END DO
  END IF
  CALL OpenAndWriteHeader (1241, experimentID//IamTxt//"_Av1.txt", tabsepOutput, nspc)
  CALL OpenAndWriteHeader (1242, experimentID//IamTxt//"_Av2.txt", tabsepOutput, nspc)
  CALL OpenAndWriteHeader (1243, experimentID//IamTxt//"_Av3.txt", tabsepOutput, nspc)
  CALL OpenAndWriteHeader (1244, experimentID//IamTxt//"_Av0.txt", tabsepOutput, nspc)


END SUBROUTINE PrepareMyOutput
