!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation to FORHYCS MPI structure

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------



!==============================================================================
!
!  Name   :  ReadNewBioClim
!
!  Purpose:  It calculates the spatial forest dynamics on a grid with or
!            without seed dispersal
!
!  Remarks:  contains the subroutine
!
!            ReadNewBioClim (<PrepareBioclimInput<TreeMig AND/OR
!                             <ReadNewBioclimIfNecessary<TimeLoop<TreeMig)
!
!==============================================================================

SUBROUTINE ReadNewBioClim(spinupYN)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: newBioClim, maxlat, maxlon, latRealStart, &
                   lonRealStart, latRealEnd, lonRealEnd, cellSideLength, &
                   unitOfSpatialData
use forhycsglobals, only : PRtoTM, compBioClim
use MPIVars, only: firstRow, lastRow, rank
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
logical :: spinupYN

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: err, ilat, ilon, oldcctime, isite, ngridcells
REAL    :: x, y, xr, yr, muDD, stdDD, muWiT, stdWiT, prop0DrStr, muDrStr, stdDrStr, newtime, smday
CHARACTER*100:: textline

!<Here the subroutine starts>**********************************************************************
  err                = 0
  oldcctime          = 0
  newBioClim%bc%muDD = -999.0

  ngridcells         = INT (((latRealEnd - latRealStart)*unitOfSpatialData / cellSideLength + 1) &
                          * ((lonRealEnd - lonRealStart)*unitOfSpatialData / cellSideLength + 1))
  DO isite = 1, ngridcells
!     READ(1014,*, IOSTAT=err) xr, yr, newtime, muDD, stdDD, &
 !                           muWiT, stdWiT, prop0DrStr, muDrStr, stdDrStr
     READ(1014,*, IOSTAT=err) xr, yr, newtime, muDD,     muWiT,  muDrStr, smday
     !if(err .ne. 0) print*, "err", err, isite, spinupYN, rank
  !if (isite==1)  print *,'inreadnewbioclim..', xr, yr, newtime, muDD,   muWiT,  muDrStr, smday, oldcctime
  !if (isite==80) print *,'isite80', xr, yr, newtime, muDD,  muWiT,newtime, oldcctime
     IF ((isite > 1) .AND. (newtime .ne. oldcctime)) THEN
        !print *, ' is ready ', xr, yr, newtime, muDD,   muWiT, newtime, oldcctime !this year of bioclim
        !pause
       BACKSPACE(1014)
       EXIT
     END IF
  ! Coordinates are rounded to 0.1 m
     xr=ANINT(xr*10.)/10.
     yr=ANINT(yr*10.)/10.
  !----- spatial coordinates are transformed to m
    x = xr * unitOfSpatialData
    y = yr * unitOfSpatialData
  !----- Standard calculation scale are km s
 !  ilat =  1 + (INT (y) - latRealStart* unitOfSpatialData) / cellSideLength
 !   ilon = 1 + (INT (x) - lonRealStart* unitOfSpatialData) / cellSideLength
   ilat =  ANINT(1. + (y - latRealStart* unitOfSpatialData) / cellSideLength )
   ilon = ANINT(1. + (x - lonRealStart* unitOfSpatialData) / cellSideLength )

   IF((ilat > maxlat) .or. (ilon > maxlon) )THEN
     print *, 'in readnewbioclim, coordinates outside domain',  yr,latRealStart, ilat, maxlat,xr, lonRealStart, ilon, maxlon, &
ngridcells, unitOfSpatialData, cellSideLength
      STOP
   END IF

!if((ilat .ge. firstRow) .and. (ilat .le. lastRow)) htfsjrks   ! Matthias

! If PRtoTM == 0 (i.e. no transfer of information from PREVAH to TreeMig), then
! the BioClim file is read and its information stored in newBioClim (as in stand-alone TM).
! If PRtoTM == 1, the Bioclim file is still read in, but stored in a variable called
! compBioClim. Only for comparing the two models;
! compBioClim is not used for calculations.

   if ((PRtoTM == 0) .or. (spinupYN == .true.)) then
     newBioClim%cctime                   = newtime
     newBioClim%bc(ilat,ilon)%muDD       = muDD
  ! newBioClim%bc(ilat,ilon)%stdDD      = stdDD
     newBioClim%bc(ilat,ilon)%muWiT      = muWiT
  ! newBioClim%bc(ilat,ilon)%stdWiT     = stdWiT
  ! newBioClim%bc(ilat,ilon)%prop0DrStr = prop0DrStr
     newBioClim%bc(ilat,ilon)%muDrStr    = muDrStr
     newBioClim%bc(ilat,ilon)%smday   = smday
  ! newBioClim%bc(ilat,ilon)%stdDrStr   = stdDrStr

  else if((PRtoTM == 1) .and. (spinupYN == .false.)) then
     compBioClim(ilon,ilat,4) = smday
     compBioClim(ilon,ilat,3) = muDD
     compBioClim(ilon,ilat,2) = muWiT
     compBioClim(ilon,ilat,1) = muDrStr
  end if

   IF (err .ne. 0) THEN
     print *, 'reading of bioclim with error or ready, ilat, ilon, newtime ', err, ilat, ilon, newtime
     EXIT
   END IF

   oldcctime = newBioClim%cctime

   IF (err .ne. 0) EXIT

  END DO  ! loop over cells
END SUBROUTINE ReadNewBioClim
! ****<Here the function ends>*********************************************************************
