!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation for the different configurations of the FORHYCS
! case study (no coupling, one-way or two-way)

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! This subroutine defines the bioclimatic indices to be used in the next
! simulation year. Depending on the chosen model configuration and simulation
! stage (spin-up or future simulation), these indices are either read from a
! file or calculated endogeneously.
!-------------------------------------------------------------------------------


SUBROUTINE ReadNewBioClimIfNecessary(year,trueyear)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: simustartyear, newBioClim, oldBioClim, envFromFile,&
          climatestartyear, & !ThirtyYearBioClim,
          spi_sample_yrs, maxlat,maxlon, stockability, &
          BioClimSU1, BioClimSU2, BioClimSU3, BioClimSU4, BioClimSU5, &
          BioClimSU6, BioClimSU7, BioClimSU8, BioClimSU9, BioClimSU10, &
          BioClimSU11, BioClimSU12, BioClimSU13, BioClimSU14, BioClimSU15, &
          BioClimSU16, BioClimSU17, BioClimSU18, BioClimSU19, BioClimSU20, &
          BioClimSU21, BioClimSU22, BioClimSU23, BioClimSU24, BioClimSU25, &
          BioClimSU26, BioClimSU27, BioClimSU28, BioClimSU29, BioClimSU30

use forhycsglobals, ONLY: ddegsum5old,muwint,drstrold,PRTOTM,prevahBC,&
      phenBBold,phenMaxOld,phenSenOld,phenFallOld,fhla,snowoffOld, &
      annprecold, mmotold,spinupYearsFH,dstr30,gdd30
use prevahglobals, only : nxll,nyll,ngridsize,fsad_old,ncol,nrow,landin,rnodat
use MPIVars, only: rank,root,firstRow,lastRow,ierr,nrowLocal,numslaves,nrowLocalArray, &
      status, ierr
use mpi

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT (IN) :: year              ! from TimeLoop (or ReadFirst30Years)
LOGICAL, INTENT (IN) :: trueyear

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: actualyear, irand, ilat, ilon
!integer, dimension(171) :: fixRandYears  ! Keep same randomly selected years over different runs (for dev only)
!integer, dimension(471) :: fixRandYears
integer, dimension(471) :: fixRandYears  ! should be 471, change later
REAL   :: randomvalue
logical :: spinupYN
logical :: bcFile, tpfile
integer :: actX, actY  ! Actual coordinates of each grid cell, for output
real, allocatable :: bcTableLocal(:,:), bcTableGlobal(:,:)
real, allocatable :: tpTableLocal(:,:), tpTableGlobal(:,:)
!integer :: j, count  -  Careful: variable name COUNT conflicts with intrinsic function
real:: avgdstr
real, dimension(3) :: dstr3
integer :: inan

!<Here the subroutine starts>

  fixRandYears = spinupYearsFH-1970

  actualyear = year + simustartyear

!----------------------

  ! Update grid containing 30 year values of drought stress
  if((rank.ne.root).and.(trueyear==.true.)) then
    do ilat = lastRow, firstRow, -1
    do ilon = 1,maxlon

      ! Eliminate NaNs (may appear when previously unstockable cells
      ! become forested)
      !
      ! x.ne.x returns TRUE only if x is NaN

      do inan=1,30
        if(dstr30(inan,ilon,ilat).ne.dstr30(inan,ilon,ilat)) dstr30(inan,ilon,ilat) = rnodat
        if(gdd30(inan,ilon,ilat).ne.gdd30(inan,ilon,ilat)) gdd30(inan,ilon,ilat) = rnodat
      end do


      if(stockability(ilat,ilon).lt.1) cycle
      dstr30(2:30,ilon,ilat) = dstr30(1:29,ilon,ilat)
      dstr30(1,ilon,ilat) = drstrold(ilon,ilat)

      gdd30(2:30,ilon,ilat) = gdd30(1:29,ilon,ilat)
      gdd30(1,ilon,ilat) = ddegsum5old(ilon,ilat)
    end do
    end do

  end if

  !Reading in sample years or spin-up?
  if(trueyear==.false.) then
    !print*, "I should be here in initialization"
    spinupYN = .true.
    CALL ReadNewBioClim(spinupYN)
    oldBioClim = newBioClim
  else ! trueyear==TRUE - not reading in sample years
    ! Bioclim from PREVAH
    if((PRtoTM == 1) .and. ((actualyear.ge.climatestartyear).or.(prevahBC==1))) then
  !      print*, "I should not be here until end of TreeMig spin-up"
    if(rank .ne. root) then
      !allocate(bcTableLocal(13,ncol*nrowLocal))
      oldBioClim = newBioClim
      !count = 1
      DO ilat=firstRow,lastRow    !1, maxlat
      DO ilon=1, maxlon
      ! Average drought index from the last 3 years, excluding those years for which there is no value (at the beginning of simulation)
      ! MS, Sep 2017


      dstr3 = dstr30(1:3,ilon,ilat)
      avgdstr = sum(dstr3,mask=dstr3.ne.rnodat) / count(dstr3.ne.rnodat)


        newBioClim%cctime                   = actualyear
        newBioClim%bc(ilat,ilon)%muDD       = ddegsum5old(ilon,ilat)
        newBioClim%bc(ilat,ilon)%muWiT      = muwint(ilon,ilat,4)
        !newBioClim%bc(ilat,ilon)%muDrStr    = drstrold(ilon,ilat)
        newBioClim%bc(ilat,ilon)%muDrStr    = avgdstr
        newBioClim%bc(ilat,ilon)%smday      = snowoffOld(ilon,ilat)

      end do
      end do

    end if  ! Master / Slaves


    else  ! Bioclim not from PREVAH
      if(actualyear.lt.climatestartyear) then  ! Bioclim from bootstrapped years - read in during initialization
     ! print*, "I should be here during TreeMig spinup"
        irand = fixRandYears(year+1)
        !newBioClim=ThirtyYearBioClim(irand)
        select case(irand)
          case(1)
            newBioClim=BioClimSU1
          case(2)
            newBioClim=BioClimSU2
          case(3)
            newBioClim=BioClimSU3
          case(4)
            newBioClim=BioClimSU4
          case(5)
            newBioClim=BioClimSU5
          case(6)
            newBioClim=BioClimSU6
          case(7)
            newBioClim=BioClimSU7
          case(8)
            newBioClim=BioClimSU8
          case(9)
            newBioClim=BioClimSU9
          case(10)
            newBioClim=BioClimSU10
          case(11)
            newBioClim=BioClimSU11
          case(12)
            newBioClim=BioClimSU12
          case(13)
            newBioClim=BioClimSU13
          case(14)
            newBioClim=BioClimSU14
          case(15)
            newBioClim=BioClimSU15
          case(16)
            newBioClim=BioClimSU16
          case(17)
            newBioClim=BioClimSU17
          case(18)
            newBioClim=BioClimSU18
          case(19)
            newBioClim=BioClimSU19
          case(20)
            newBioClim=BioClimSU20
          case(21)
            newBioClim=BioClimSU21
          case(22)
            newBioClim=BioClimSU22
          case(23)
            newBioClim=BioClimSU23
          case(24)
            newBioClim=BioClimSU24
          case(25)
            newBioClim=BioClimSU25
          case(26)
            newBioClim=BioClimSU26
          case(27)
            newBioClim=BioClimSU27
          case(28)
            newBioClim=BioClimSU28
          case(29)
            newBioClim=BioClimSU29
          case(30)
            newBioClim=BioClimSU30
        end select
      else  ! Bioclim from BC file
   !       print*, "I should not be here in coupled runs"
        spinupYN = .false.
        oldBioClim = newBioClim
        CALL ReadNewBioClim(spinupYN)
      end if
    end if ! Bioclim from PREVAH or not
  end if    ! Trueyear


  end SUBROUTINE ReadNewBioClimIfNecessary
