!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  Regen
!
!  Purpose:  "Regen" determines the germination of seeds
!            "ContinuousLightDepGerm" determines a continuous light
!            dependence of germination
!
!  Remarks:  contains the subroutines
!
!            Regen (<LocalForestDynamics<DoLocalWork<DistrLocalDynThisTime<
!                     TimeLoop<TreeMig)
!            ContinuousLightDepGerm (not used at the moment)
!
!==============================================================================


SUBROUTINE Regen(year,  nspc, stock, idsp,thiscell)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: maxlc,   pltsiz,                                  &
                   spec,       &
                   ltcum, seedBank, ltfrq, ltval,                         &
                   seedlcrowndiamInCm,                                    &
                   contLightDepGerm,   stockability, &
                   CurrStateInCell
                                                          ! Use control, species and stand  parameters
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER,INTENT(IN) :: nspc, year          ! number of species                                     [=spec]
REAL,INTENT(IN)    :: stock         ! number of species                                     [=spec]
 Logical,INTENT(IN) :: idsp

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: ilstart,                    &    ! Integer val. of inlt                        [=ilstart]
           i, ltcl,                    &    !  looping variable for species and light
           threshold
REAL    :: newborn,                              &  ! # newborn trees in htcl=1       [=birthTo0]
           inlt,                                 &  ! Index of light        [=RealIndOfLight(..)]
           lidep, ld,                            &  ! light dependent growth             [=lidep]
           lih, allgerms, alldeps, ndeps,        &
           relNewB(1:31),                        &
           maxseedlnr, estabFrac, seedlcrownarea
TYPE(CurrStateInCell):: thiscell

!<Here the subroutine starts>**********************************************************************
  allgerms = 0.0
  alldeps  = 0.0
  ndeps    = 0.0

  !----- seedlcrowndiamInCm = 10
  estabFrac      = stock
  seedlcrownarea = 3.141592654 *(( seedlcrowndiamInCm / 2.0) * 0.01) ** 2   ! convert to area (m2)

  BySpecies: DO i=1,nspc
     IF (thiscell%sp(i)%there ) THEN
      !----- Determine in which light class the critical threshold for establishment of this species falls
      inlt    = (SQRT ( spec(i)%lthres   * REAL (maxlc * (maxlc + 1)) + 0.25) - 0.5)
      ilstart = INT (inlt)

      !----- Determine the proportion of the stand in which light is equal or brighter than this threshold
      lidep  = ltcum (ilstart + 1)

      !----- If the seeds stem from dispersal, the newborn seedlings germinate from the seedbed, depending
      !----- on light and other environment
      IF (idsp) THEN
        !----- Here a test for a continuous light dependence function of germination
        IF (contLightDepGerm) THEN
          lidep = 0.0
          DO ltcl=1,maxlc
            lih       = ltval(ltcl)
            threshold = spec(i)%klighs
            CALL ContLightDep (lih, threshold, ld)
            lidep = lidep + ld * ltfrq(ltcl)
          END DO
        END IF !------ contLightDepGerm

        !----- If light dependence works on 0th height class, light dep. of germination is 1
        thiscell%sp(i)%sb  = thiscell%sp(i)%seedbed

         !-----  if wished seeds stay in the seedbank depending on mortality and germination rates
        IF (seedBank) THEN
          newborn            = thiscell%sp(i)%seedbed * lidep * spec(i)%enveta * spec(i)%seedGerm
          thiscell%sp(i)%seedbed = (thiscell%sp(i)%seedbed - newborn) * spec(i)%seedSurv * (1.0 - spec(i)%seedLoss)
        ELSE
          newborn            = thiscell%sp(i)%seedbed * lidep * spec(i)%enveta * spec(i)%seedGerm
          thiscell%sp(i)%seedbed = 0.0
        END IF
         !----- newborns enter  0th height class (0-1.37 m)
        !-----  nur fuer freq test raus
!----->     thiscell%sp(i)numin(0,lat,lon)=thiscell%sp(i)numin(0,lat,lon)+newborn
        relNewB(i) = newborn
        allgerms   = allgerms + newborn
        alldeps    = alldeps + lidep * spec(i)%enveta
        ndeps      = ndeps + 1.0
      ELSE !----- not idisp, i.e. constant seed inflow
        !----- Otherwise, the newborn saplings germinate from a constant sapling pool (birth),
        !----- depending on light and other environment...
        newborn = spec(i)%birth * pltsiz * lidep * spec(i)%enveta / 833.0

        !----- ... and enter first height class (> 1.37 m)
        thiscell%sp(i)%numin(1) = thiscell%sp(i)%numin(1) + newborn
        thiscell%sp(i)%ingrowth  = newborn
       END IF !----- idsp
	!    print *, 'in regen,idsp,enveta,birth, pltsiz , lidep , numin1', idsp, spec(i)%enveta, thiscell%sp(i)%numin(1)
    END IF !----- there

    ! TEST - Matthias Oct 17: Exclude certain species
    !if((i==9).or.(i==18)) then ! Acer platanoides & Fagus sylvatica
    !  thiscell%sp(i)%numin(1) = 0.0
    !  thiscell%sp(i)%seedbed = 0.0
    !end if
  END DO BySpecies

  !----- Test fuer Frequenzabh�ngigkeit
  IF (idsp) THEN
    IF (allgerms > 0.0) THEN
      maxseedlnr = pltsiz * estabFrac / seedlcrownarea
      DO i=1,nspc
        IF (thiscell%sp(i)%numin(0 ) < 0) THEN
          WRITE(1070,*) "Achtung, negative keimlinge!"
        END IF
!----->     thiscell%sp(i)%sb = allgerms * maxseedlnr / (allgerms + maxseedlnr) * (relNewB(i) / allgerms)  ! only for output
        thiscell%sp(i)%numin(0) = thiscell%sp(i)%numin(0) + &
                                  allgerms * maxseedlnr / (allgerms + maxseedlnr) * (relNewB(i) / allgerms)
!      print*, "numin(0)", thiscell%lat,thiscell%lon,i,thiscell%sp(i)%numin(0)
!----->    thiscell%sp(i)%sb  = thiscell%sp(i)%numin(0 )                                                     ! only for output
!----->    thiscell%sp(i)%sb = relNewB(i)                                                                  ! only for output
     END DO !----- i=1,nspc
    END IF !----- (allgerms > 0.0)
  END IF !----- (idsp)

END SUBROUTINE Regen
!****<Here the subroutine ends>********************************************************************



!****<Here the subroutine ContinuousLightDepGerm starts>*******************************************
!SUBROUTINE ContinuousLightDepGerm(threshold, lidep)

!<CALL modules for type definitions>---------------------------------------------------------------
!USE All_par, ONLY: ltval, ltfrq, maxlc

!IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
!REAL, INTENT(IN)  :: threshold          ! species specific threshold of light dependence
!REAL, INTENT(OUT) :: lidep              ! light dependence value (summed over all light classes)

!<Local variables>---------------------------------------------------------------------------------
!INTEGER :: ltcl
!REAL    ::  ld,lih      ! light dependent growth                               [=lidep]
!
!<Here the subroutine starts>**********************************************************************
!  lidep = 0.0
!  DO ltcl=1,maxlc
!    lih = ltval(ltcl)
!    CALL ContLightDep (lih, threshold, ld)
!    lidep = lidep + ld * ltfrq(ltcl)
!  END DO
!
! END SUBROUTINE ContinuousLightDepGerm
!****<Here the subroutine ends>********************************************************************
