!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!==============================================================================
!
!  Name   :  Seedprod
!
!  Purpose:  Calculates the seeds, which are produced by all trees of one
!            species in one height class and in a certain light class and
!            sums them over all light classes. For each height class the
!            seeds are recorded.
!
!  Remarks:  contains the subroutines
!
!            Seedprod (<LocalForestDynamics<DoLocalWork<DistrLocalDynThisTime<
!                        TimeLoop<TreeMig)
!            SumSpecsSeedprodsOverHeights (<LocalForestDynamics<DoLocalWork<
!                                            DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================
! design          : H. Lischke, N. Zimmermann
! author(s)       : H. Lischke, N. Zimmermann
! implementation  : H. Lischke, N. Zimmermann
! cleaner         : T.J. Loeffler
! copyright       : (c) 1999, 03 by H. Lischke
!==============================================================================

SUBROUTINE Seedprod(ispec, htcl, ntinh, year,lat,lon,Iam , thiscell)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: spc, spec, hcwidth, &          ! Use species parameters
                   mastSeeding,CurrStateInCell

IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER,INTENT(IN) :: ispec,year, &      ! species
                      htcl,lat,lon,Iam          ! height class
 REAL, INTENT(IN) :: ntinh         ! number of trees in this height-light class

!<Local variables>---------------------------------------------------------------------------------
REAL :: maxSeedsPerTree, &          ! actual number of seeds per tree
        newSeeds,        &          ! actual number of seeds per tree species in this height class
        fractionInLowHC             ! fraction of seeds in lowest seed producing height class

TYPE(CurrStateInCell):: thiscell
!<Here the subroutine starts>**********************************************************************
  !----- Determine how much seeds are produced by a tree of this species  at maximum
  maxSeedsPerTree = spec(ispec)%maxseed     !----- Repro: here also a light or density dependence of seed production could be included
  !----- IF mast seeding desired, apply a sine-wave
  IF (mastSeeding) maxSeedsPerTree = 0.5 * maxSeedsPerTree * &
                                     (1.0 + SIN (6.2831853 * REAL ( year) / spec(ispec)%period))
  !----- Calculate seeds produced by all trees of this species in this height class
  newSeeds = ntinh * spc(ispec)%mxsdht(htcl)* maxSeedsPerTree
  !----- If actual height class above lowest seed production height, no problem, just add the produced seeds
  IF (htcl .ge. CEILING (spec(ispec)%mathcp)) &
    thiscell%sp(ispec)%seeds(htcl ) = thiscell%sp(ispec)%seeds(htcl ) + newSeeds
  !----- If actual height class just below lowest seed production height,   add only proportion of the produced seeds
  fractionInLowHC = (REAL (CEILING (spec(ispec)%mathcp)) - spec(ispec)%mathcp) / hcwidth
  IF (htcl .eq. FLOOR (spec(ispec)%mathcp)) &
    thiscell%sp(ispec)%seeds(htcl )= thiscell%sp(ispec)%seeds(htcl ) + newSeeds * fractionInLowHC
 END SUBROUTINE Seedprod
!****<Here the subroutine ends>********************************************************************  !==============================================================================

SUBROUTINE SumSpecsSeedprodsOverHeights(nspc,thiscell)
USE All_par, ONLY:   spec, CurrStateInCell
IMPLICIT NONE
!<Passed variables>--------------------------------------------------------------------------------
INTEGER,INTENT(IN) ::  nspc     ! species
TYPE(CurrStateInCell):: thiscell
!<Local variables>---------------------------------------------------------------------------------
 INTEGER::   htcl,isp
!<Here the subroutine starts>**********************************************************************
      DO isp=1, nspc
         thiscell%sp(isp)%seedsum = 0.0
        DO htcl=FLOOR(spec(isp)%mathcp),spec(isp)%maxhtc
          thiscell%sp(isp)%seedsum = thiscell%sp(isp)%seedsum + thiscell%sp(isp)%seeds(htcl)
        END DO
     END DO
 END SUBROUTINE SumSpecsSeedprodsOverHeights
!****<Here the subroutine ends>********************************************************************
