!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1999 - 2003 H. Lischke, N. Zimmermann, T.J. Loeffler

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


!==============================================================================
!
!  Name   :  SelfRegulation
!
!  Purpose:  determines the dynamics of the seed antagonists and their grazing
!
!  Remarks:  contains the subroutine
!
!            SelfRegulation (<LocalForestDynamics<DoLocalWork<DistrLocalDynThisTime<
!                              TimeLoop<TreeMig)
!
!==============================================================================
! design          : H. Lischke, N. Zimmermann
! author(s)       : H. Lischke, N. Zimmermann
! implementation  : H. Lischke, N. Zimmermannactualyear
! cleaner         : T.J. Loeffler
! copyright       : (c) 1999, 03 by H. Lischke
!==============================================================================
SUBROUTINE SelfRegulation(nspc,   idsp, thiscell)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: pltsiz, &
                   withSeedAntagonists,                &
                   seedAntaMort, seedAntaRain,         &
                   seedAntaGraz, seedAntaEff,          &
                   withSeedCarrCap, seedCarrCap, CurrStateInCell,sumseeds3
                                   ! Use control, species and stand parameters
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: nspc     ! number of species                                     [=spec]
 Logical, INTENT(IN) :: idsp

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: i                      ! looping variable for species and light
REAL    :: seedbedUngrazed, grazing, grazersOld, sharePerGrazer, amountPerGrazer, &
           ratio, exporatio

TYPE(CurrStateInCell):: thiscell

!<Here the subroutine starts>**********************************************************************
  BySpecies: DO i=1,nspc
    IF (thiscell%sp(i)%there ) THEN
      IF (idsp) THEN
        !----- if wished antagonists eat seeds from seedbed and grow
        IF (withSeedAntagonists) THEN
          grazersOld      = thiscell%sp(i)%antagonist
          seedbedUngrazed = thiscell%sp(i)%seedbed + thiscell%sp(i)%newSeeds
          sharePerGrazer  = 1 / (grazersOld + 0.5 / seedAntaGraz)
          amountPerGrazer = seedbedUngrazed * sharePerGrazer
          grazing         = grazersOld * amountPerGrazer

          !----- they can eat only as much as is there
          thiscell%sp(i)%seedbed         = seedbedUngrazed - grazing
          thiscell%sp(i)%antagonist  = (grazersOld + grazing * seedAntaEff) * (1. - seedAntaMort) + &
                                        seedAntaRain * pltsiz / 833.0
        END IF !----- withseedAntagonists

        !----- more abstract: the antagonist dynamics is formulated by a simple carrying capacity
        !----- This is assumed to work continuously during each year. That means the dynamics is a solved logistic equation
        IF (withSeedCarrCap) THEN
          ratio              = MIN (thiscell%sp(i)%newSeeds / seedCarrCap, 20.0) !it is 0 anyway with values smaller exp(-20)
          exporatio          = EXP (-ratio)
          thiscell%sp(i)%seedbed = seedCarrCap + (thiscell%sp(i)%seedbed - seedCarrCap) * exporatio
           !----- technical trick to avoid 0 trees with very little influx
          IF ((ratio > 0.0) .AND. (thiscell%sp(i)%seedbed .eq. 0.0)) thiscell%sp(i)%seedbed = thiscell%sp(i)%newSeeds
        END IF !----- withSeedCarrCap

        IF ((.not. withSeedCarrCap) .and. (.not. withSeedAntagonists)) THEN !----- this case happens only in reprotest
          thiscell%sp(i)%seedbed = thiscell%sp(i)%seedbed + thiscell%sp(i)%newSeeds
        END IF

        thiscell%sp(i)%sb = thiscell%sp(i)%seedbed !----- thiscell%sp(i)%newSeeds
        sumseeds3 = sumseeds3 + thiscell%sp(i)%seedbed
        thiscell%sp(i)%newSeeds = 0.0;
      END IF !----- idsp
    END IF !----- there
  END DO BySpecies

END SUBROUTINE SelfRegulation
!****<Here the subroutine ends>********************************************************************
