!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Transient update of glacier extent and mass after Huss et al. (2010)
!-------------------------------------------------------------------------------

!=======================================================================

module GlacGlob

  use forhycsglobals, only: dblP

  integer :: numGlac  ! Number of individual glaciers to be read
  character(100), allocatable :: pathsGlac(:)

  type glacier
    integer :: id
    integer :: class  ! Glacier type / class after Huss et al., depending on size
    integer, dimension(4) :: bbox  ! Bounding box of glacier (minX, minY, maxX, maxY)
    real :: area
    real(kind=dblP) :: waterEq  ! Water currently stored in the glacier
    real :: minH, maxH  ! minimum and maximum elevation
    real :: ela         ! Equilibrium-line altitude
  end type glacier

  type(glacier), allocatable :: glacList(:)

  ! Density of ice
  real, parameter :: rhoIce = 0.9

  ! Minimum height of ice sheet to cover the whole cell (totally arbitrary value).
  ! Cells with an ice cover below this value have an ice fraction and a rock
  ! or bare soil fraction.
  real, parameter :: critIce = 5.0
  real, dimension(10) :: dH  ! Normalized elevation bands

  ! Minimal snow cover during melting season. This determines for each firn
  ! cell (land==13) how much of the snow cover is considered firn / old snow
  ! and accounted for in the glacier mass balance each year. At the beginning
  ! of the melting season, this is set to the current snow cover value.
  real, allocatable :: minSnowCover(:,:)

  ! MPI version only: arrays for sharing relevant information across
  ! processes (see subroutine glacReduce)
  real, allocatable :: oldIce(:,:), curIce(:,:), curSnow(:,:)
  integer, allocatable :: newLand(:,:)
  logical, allocatable :: glacDone(:,:)

  type :: glacUpdate
!    sequence
    real :: area
    integer :: class
    real(kind=dblP) :: waterEq
    real :: minH
    real :: maxH
    real :: ela
  end type glacUpdate

end module GlacGlob

!=======================================================================

subroutine glacInit

  !use GlacGlob, only: glacier, glacList, numGlac, critIce, rhoIce
  use GlacGlob
  use prevahglobals, only: ncol, nrow, rnodat, glacID, bedrock, SGlac, landin, fhoehe,&
                      icefrac, jahr
  use help  ! for median
  use MPIVars, only: rank, root

  implicit none

  integer :: i, xcoor, ycoor, rloop
  integer :: uwas
  real, dimension(ncol,nrow) :: glacIn
  real:: curMinH, curMaxH, curArea
  real(kind=dblP) :: curWEq
  real, dimension(6) :: dumStats  ! Placeholder variable for stats in input grids
  integer :: minX, minY, maxX, maxY
  real, allocatable :: altVect(:)  ! Vector of elevation values of each cell
  integer :: vectLen  ! Length of altVect (= number of glacier cells)
  integer :: j
  real :: med  ! Median elevation of glacier
  logical :: exFile  ! for writing output

  real :: rcol, rrow,rgridsize, rxll, ryll
! Load spatial information on individual glaciers

  SGlac = 0.0
  glacIn = rnodat
  glacID = int(rnodat)

  allocate(glacList(numGlac))

  do i = 1,numGlac

    curMinH = 5000.0
    curMaxH = 0.0
    curArea = 0.0
    curWEq = 0.0

    minX = ncol
    minY = nrow
    maxX = 1
    maxY = 1

! Read spatial grid of the corresponding glacier
    !print*, pathsGlac(i)
    uwas = 556
    open(uwas,file=trim(pathsGlac(i)),status='old',form='binary')

    read (uwas) rcol
    read (uwas) rrow
    read (uwas) rxll
    read (uwas) ryll
    read (uwas) rgridsize
    read (uwas) rnodat

!    ncol=int(rcol)
!    nrow=int(rrow)
!    nxll=int(rxll)
!    nyll=int(ryll)
!
!    ngridsize=int(rgridsize)
!    nodat=int(rnodat)

    do rloop=1,6
      read (uwas) dumStats(rloop)
    end do

    do ycoor=nrow,1,-1
      read (uwas) (glacIn(xcoor,ycoor),xcoor=1,ncol)
    end do
    close (uwas)

! Prepare vector containing altitude values
    vectLen = count((int(glacIn) .ne. int(rnodat)) .and. (glacIn .ne.0))
    allocate(altVect(vectLen))
    j = 1

! Loop over cells

    do ycoor = nrow, 1, -1
    do xcoor = 1,ncol

      if((int(glacIn(xcoor,ycoor)) == int(rnodat)) .or. (int(glacIn(xcoor,ycoor)) == 0.)) cycle

      if(xcoor .lt. minX) minX = xcoor
      if(xcoor .gt. maxX) maxX = xcoor
      if(ycoor .lt. minY) minY = ycoor
      if(ycoor .gt. maxY) maxY = ycoor

      glacID(xcoor,ycoor) = i

! Get rid of isolated glacier cells (they will be treated using the standard
! glacier treatment in PREVAH).

  ! TODO if necessary (depends on pre-processing)

! Get ice fraction of current cell

      if(glacIn(xcoor,ycoor) .ge. critIce) then
        iceFrac(xcoor,ycoor) = 1.0
      else
        iceFrac(xcoor,ycoor) = glacIn(xcoor,ycoor)/critIce
      end if

! Get elevation of bedrock by subtracting height of ice sheet from DHM cell
      bedrock(xcoor,ycoor) =  fhoehe(xcoor,ycoor) - glacIn(xcoor,ycoor)

! Get water equivalent of ice initially stored in the current cell
! Input given in m ice thickness, converting to mm water equivalent
      SGlac(xcoor,ycoor) = glacIn(xcoor,ycoor) * rhoIce * 1000

      print*, "xcoor, ycoor, glacIn, fhoehe, bedrock", xcoor, ycoor, glacIn(xcoor,ycoor), fhoehe(xcoor,ycoor), bedrock(xcoor,ycoor)

! Get statistics for initialization
      if(fhoehe(xcoor,ycoor) .lt. curMinH) curMinH = fhoehe(xcoor,ycoor)
      if(fhoehe(xcoor,ycoor) .gt. curMaxH) curMaxH = fhoehe(xcoor,ycoor)
      altVect(j) = fhoehe(xcoor,ycoor)
      curWEq = curWEq + (sGlac(xcoor,ycoor) * rgridsize * rgridsize)
      curArea = curArea + (rgridsize*rgridsize*icefrac(xcoor,ycoor)/1000000.)

      j = j+1
    end do
    end do

! Get median of elevation values (as an estimate for ELA)
    med = median(altVect)  ! Function MEDIAN is defined in module HELP.


! Initialize status of the current glacier
    glacList(i)%area = curArea
    glacList(i)%ela = med

    if(curArea .lt. 5.0) then
      glacList(i)%class = 1
    else if(curArea .lt. 20) then
      glacList(i)%class = 2
    else
      glacList(i)%class = 3
    end if

    glacList(i)%bbox = (/minX, minY, maxX, maxY/)
    glacList(i)%waterEq = curWEq
    glacList(i)%minH = curMinH
    glacList(i)%maxH = curMaxH

    !print*, "glacier number ", i, "bbox: ", glacList(i)%bbox

! Set land cover class (ice or firn) of glacier cells according to ELA

    do ycoor = maxY, minY, -1
    do xcoor = minX, maxX
      if((int(glacIn(xcoor,ycoor)) == int(rnodat)) .or. (int(glacIn(xcoor,ycoor)) == 0.)) cycle
      if(fhoehe(xcoor,ycoor) .le. glacList(i)%ela) then
      ! Cell is below ELA : ice
        landin(xcoor,ycoor) = 14
      else
      ! Cell is above ELA: firn
        landin(xcoor,ycoor) = 13
      end if
    end do
    end do

    deallocate(altVect)

  end do

! MPI Version only

  allocate(curSnow(ncol,nrow))
  allocate(curIce(ncol,nrow))
  allocate(newLand(ncol,nrow))
  allocate(glacDone(ncol,nrow))

  if(rank==root) then
      inquire(file="glacStats.txt",exist=exFile)
      if(exFile) then
       open(665,file="glacStats.txt",status="old", position="append")
       else
         open(665,file="glacStats.txt",status="new")
      end if
72     format(I5,I3,F8.3,I3,F20.3,3F15.3)
      do i = 1,numglac
      write(665,72) jahr, i, glacList(i)%area, glacList(i)%class, glacList(i)%waterEq,&
                    glacList(i)%minH, glacList(i)%maxH, glacList(i)%ela
     end do
      close(9257)

  end if


end subroutine glacInit


!=======================================================================

subroutine glacReduce(opt)

! MPI version only: this subroutine makes sure that all processes have all
! the relevant information, i.e. snow and ice content of all glacier cells,
! before running glacDeltaH.
! The extent of the glaciers does not match the subarray structure of the
! rest of the model. The distribution of glaciers among slaves is not known
! a priori, so all slaves must see the whole grid.
! To avoid transferring the whole grid, the arrays transfer a list of
! coordinates with the corresponding snow and ice info.
! This subroutine is also called at the end of glacDeltaH to update status
! variables (land cover class, snow, ice, glacier fraction).

  use glacGlob
  use mpi
  use MPIvars, only: rank, root, mpitag, status, ierr, firstRow, lastRow, numslaves
  use prevahglobals, only: rnodat, glacID, ncol, nrow, ssotf, sglac, iceFrac, fhoehe,&
                           landin, ssnowtf, sliqtf

  implicit none

  integer :: xcoor, ycoor
  integer :: opt  ! 1 if this subroutine is called before glacDeltaH, and 2 if it is called at the end of glacDeltaH
  integer :: listLen, listLenTot  ! Length of list for each array, and of total list
  integer, dimension(numslaves) :: listLenArray
  real, allocatable :: sendArr(:,:), recvArr(:,:), sendArr2(:,:), recvArr2(:,:), recvSlice(:,:), recvSlice2(:,:)
  integer :: i
  integer :: listCount

  ! COMMON PART:

!  allocate(listLenArray(numslaves))
  if (opt == 1) then
    curIce = rnodat
    curSnow = rnodat
  end if

  ! MASTER PART:

  if(rank == root) then
  ! Master receives array size from each slave, sends total length back
  ! to all slaves and initializes array
    do i = 1,numslaves
      print*, "listLenArray before RECV", ListLenArray(i), i
      call MPI_RECV(listLen,1,MPI_INTEGER,i,mpiTag,MPI_COMM_WORLD,status,ierr)
      listLenArray(i) = listLen
      print*, "received ListLenArray", ListLenArray(i), i
    end do

    listLenTot = sum(listLenArray)
    call MPI_BCAST(listLenTot,1,MPI_INTEGER,root,MPI_COMM_WORLD,ierr)

    print*, "listLenTot", listLenTot
    print*, "listLenArray", listLenArray

    allocate(recvArr(5,listLenTot))
    if(opt==2) allocate(recvArr2(5,listLenTot))

  ! Sends complete list to all slaves

    listCount = 1
    do i = 1,numslaves
      if(listCount <= (listLenTot + 1)) then
        allocate(recvSlice(5, listLenArray(i)))
        !call MPI_RECV(recvArr(:,listCount:listCount+listLenArray(i)-1),5*listLenArray(i),MPI_REAL,&
        !          i, mpiTag,MPI_COMM_WORLD,status,ierr)
        call MPI_RECV(recvSlice,5*listLenArray(i),MPI_REAL,i, 22,MPI_COMM_WORLD,status,ierr)
        recvArr(:,listCount:listCount+listLenArray(i)-1) = recvSlice
        listCount = listCount + listLenArray(i)
        print*, "i", i, "listcount", listcount
        !print*, "recvArr", recvArr(:,1)

        deallocate(recvSlice)
      end if
    end do
    call MPI_BCAST(recvArr,5*listLenTot,MPI_REAL,root,MPI_COMM_WORLD,ierr)

    if(opt==2) then
    listCount=1
      do i = 1,numslaves
        if(listCount <= (listLenTot + 1)) then
          allocate(recvSlice2(5, listLenArray(i)))
          !call MPI_RECV(recvArr2(:,listCount:listCount+listLenArray(i)-1),5*listLenArray(i),MPI_REAL,&
          !        i, mpiTag,MPI_COMM_WORLD,status,ierr)
          call MPI_RECV(recvSlice2,5*listLenArray(i),MPI_REAL,i, 23,MPI_COMM_WORLD,status,ierr)
          recvArr2(:,listCount:listCount+listLenArray(i)-1) = recvSlice2
          listCount = listCount + listLenArray(i)
          print*, "i", i, "listcount2", listcount
          deallocate(recvSlice2)
        end if
      end do
      call MPI_BCAST(recvArr2,5*listLenTot,MPI_REAL,root,MPI_COMM_WORLD,ierr)
    end if

  else
  ! SLAVES PART:


  ! Each slave counts the number of glacier cells in its subdomain
    if(opt==1) then
      listLen = count(glacID(:,firstRow:lastRow) .ne. int(rnodat))
    else
      listLen = count(glacDone)
      print*, "listLen", listLen, rank
    end if

  ! Each slave creates a (4,X) array with the coordinates of the glacier
  ! cells together with the ice and snow contents.
    allocate(sendArr(5,listLen))
    if(opt==2) allocate(sendArr2(5,listLen))

    if(opt == 1) then
      i = 1
      do ycoor = lastRow, firstRow, -1
      do xcoor = 1,ncol
        if (glacID(xcoor, ycoor) == int(rnodat)) cycle
        sendArr(1,i) = real(xcoor)
        sendArr(2,i) = real(ycoor)
!        sendArr(3,i) = ssotf(xcoor,ycoor)
        sendArr(3,i) = minSnowCover(xcoor,ycoor)
        sendArr(4,i) = sglac(xcoor,ycoor)
        sendArr(5,i) = real(glacID(xcoor,ycoor))
      !  print*, "sending: xcoor, ycoor, snow, ice", xcoor, ycoor, ssotf(xcoor,ycoor), sglac(xcoor,ycoor)
        i = i+1
      end do
      end do

    else if(opt == 2) then
      i=1
      do ycoor = nrow, 1, -1
      do xcoor = 1,ncol
        !if ((glacID(xcoor, ycoor) == int(rnodat)) .and. (newLand(xcoor,ycoor) == int(rnodat))) cycle
        if(glacDone(xcoor,ycoor) == .FALSE.) cycle
        sendArr(1,i) = real(xcoor)
        sendArr(2,i) = real(ycoor)
        sendArr(3,i) = curSnow(xcoor,ycoor)
        sendArr(4,i) = curIce(xcoor,ycoor)
        sendArr(5,i) = real(glacID(xcoor,ycoor))
        sendArr2(1,i) = real(xcoor)
        sendArr2(2,i) = real(ycoor)
        sendArr2(3,i) = real(newLand(xcoor,ycoor))
        sendArr2(4,i) = iceFrac(xcoor,ycoor)
        sendArr2(5,i) = fhoehe(xcoor,ycoor)
        i = i+1
      end do
      end do

    end if

  ! Reduction operation to get total number of glacier cells. Slaves also
  ! communicate their respective number of cells to master.

    print*, "Listlen before sending", listLen, rank
    call MPI_SEND(listLen,1,MPI_INTEGER,root,mpiTag,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(listLenTot,1,MPI_INTEGER,root,MPI_COMM_WORLD,ierr)

  ! All slaves send their array to master

    call MPI_SEND(sendArr,5*listLen,MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
    if(opt==2) call MPI_SEND(sendArr2,5*listLen,MPI_REAL,root,23,MPI_COMM_WORLD,ierr)

  ! Slaves get whole array from master
    allocate(recvArr(5,listLenTot))
    if(opt==2) allocate(recvArr2(5,listLenTot))
    call MPI_BCAST(recvArr,5*listLenTot,MPI_REAL,root,MPI_COMM_WORLD,ierr)
    if(opt==2) call MPI_BCAST(recvArr2,5*listLenTot,MPI_REAL,root,MPI_COMM_WORLD,ierr)

  ! Slaves update their ice and snow arrays
    do i = 1,listLenTot
      if(opt==1) then
        xcoor = int(recvArr(1,i))
        ycoor = int(recvArr(2,i))
        curSnow(xcoor,ycoor) = recvArr(3,i)
        curIce(xcoor,ycoor) = recvArr(4,i)
        glacID(xcoor,ycoor) = int(recvArr(5,i))
       ! print*, "unpacking: xcoor, ycoor, snow, ice", xcoor, ycoor, curSnow(xcoor,ycoor), curIce(xcoor,ycoor)

      else
        xcoor = int(recvArr(1,i))
        ycoor = int(recvArr(2,i))
        ! For the moment: assume that all snow gets transformed to ice instantaneously
        ! for fully covered glacier cells, and all snow is set to be solid snow in
        ! partly covered glacier cells. NOT TRUE ANYMORE

!        ssotf(xcoor,ycoor) = recvArr(3,i)
!        ssnowtf(xcoor,ycoor) = ssotf(xcoor,ycoor)
!        sliqtf(xcoor,ycoor) = 0.0

        ! Update: snow cover gets updated only for firn cells. The new snow cover is
        ! the snow cover before glacier update minus what has been added to the glacier
        ! mass balance (i.e. snow that did not melt during summer)

                ! NEW VERSION: Old snow converted to glacier also for ice cells

        !if(landin(xcoor,ycoor)==13) then
          ssnowtf(xcoor,ycoor) = ssnowtf(xcoor,ycoor)-minSnowCover(xcoor,ycoor)
          ssotf(xcoor,ycoor) = ssnowtf(xcoor,ycoor) + sliqtf(xcoor,ycoor)
        !end if

        sglac(xcoor,ycoor) = recvArr(4,i)
        glacID(xcoor,ycoor) = int(recvArr(5,i))
        if(int(recvArr2(3,i)) .ne. int(rnodat)) landin(xcoor,ycoor) = int(recvArr2(3,i))
        icefrac(xcoor,ycoor) = recvArr2(4,i)
        fhoehe(xcoor,ycoor) = recvArr2(5,i)
      end if
    end do

  end if  ! Master / Slaves

  if(allocated(sendArr)) deallocate(sendArr)
  if(allocated(recvArr)) deallocate(recvArr)
  if(allocated(sendArr2)) deallocate(sendArr2)
  if(allocated(recvArr2)) deallocate(recvArr2)
!  if(allocated(listLenArray)) deallocate(listLenArray)


end subroutine glacReduce

!=======================================================================

subroutine glacDeltaH

! Core subroutine of the deltaH approach proposed by Huss et al. 2010.
! Also contains a simple procedure for estimating ELA (assumed to equal
! median glacier altitude, see Braithwaite and Raper 2009), to distinguish
! between ice and firn landcover types, required by the subroutine glacmoTrans.

  use GlacGlob
  use mpi
  use MPIvars, only: rank, root, numslaves, ierr, status, MPIGlacType, MPIdbltype
  use prevahglobals, only: ncol, nrow, rnodat, glacID, sGlac, ssotf, fhoehe,&
                           rgridsize, iceFrac, bedrock, landin, jahr
  use forhycsglobals, only: dblP
  use help

  implicit none

  integer :: xcoor, ycoor
  integer :: i, j, k
  integer :: oldClass
  real :: oldArea, oldMinH, oldMaxH  ! Last year's status
  real(kind=dblP) :: oldWEq
  real:: curMinH, curMaxH, curArea  ! Updated status
  real(kind=dblP) :: curWEq  ! Updated status
  real(kind=dblP) :: mb  ! Mass balance
  real :: scaleFac ! Scaling factor for ice thickness update
  real :: a, b, c, gamma  ! Parameters for deltaH equation
  real, dimension(10) :: hr
  real, dimension(10) :: elevArea  ! Area of the glacier in each (normalized) elevation band
  real, dimension(10) :: elevLimits  ! Lower limits of the elevation bands
  real :: hDiff
  real :: curBand, curElev  ! Helper variables for iteration over elevation bands
  integer, allocatable :: tempBand(:,:)  ! Helper variable containing elevation classes
  integer :: minX, minY, maxX, maxY
  real, allocatable :: altVect(:)  ! Vector of elevation values of each cell
  integer :: vectLen  ! Length of altVect (= number of glacier cells)
  real :: med  ! Median elevation of glacier
  logical :: exFile  ! For writing output table

! MPI Version only
  integer :: numsent
  integer :: mess  ! dummy message
  integer :: lastSlave  ! Contains number of the last slave who finished a job
  integer :: curGlac  ! Contains ID of glacier whose status is currently being sent by slave
  integer :: work  ! Sent by master to slaves: number of a glacier, or 0 if there is no more work
  real, dimension(4) :: glacStat  ! updated glacier status, to be sent by slaves to master
  real(kind=dblP) :: tempWeq
  integer :: tempClass


!  type(glacUpdate) :: sendUpdate, recvUpdate, bcUpdate

  ! Debug
  integer :: typeSize, mpiSize, dblsize, mpiDblsize
  integer, dimension(6) :: sizes

  integer :: sizeDH ! TEMP

  allocate(tempBand(ncol,nrow))

! MPI version only: slaves share all relevant information from their subdomains
  newLand = int(rnodat)
  call glacReduce(1)

! MPI version only: dispatching of glacier objects between slaves.

! MASTER PART:
! Send a glacier number to each process

  if(rank==root) then
    numsent = 0

    do i = 1,numslaves !1,min(numslaves, numGlac)
    print*, "Master sending work", i, numsent+1
      if(numsent.lt.numGlac) then
        call MPI_SEND(numsent+1,1,MPI_INTEGER,i,2,MPI_COMM_WORLD,ierr)
      else
        call MPI_SEND(0,1,MPI_INTEGER,i,2,MPI_COMM_WORLD,ierr)
      end if
      numsent = numsent + 1
    end do

    print*, "Master before loop"

! Receive message that processing of a glacier is completed; send another
! number if there are more left
  do i = 1,max(numGlac,numslaves)
  print*, "Master receiving from process", i
    call MPI_RECV(mess,1,MPI_INTEGER,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,status,ierr)
    print*, "Master received from process", i, "(1)"
    lastSlave = status(MPI_SOURCE)
    curGlac = mess !status(MPI_TAG)
    print*, "mess, lastSlave, curGlac", mess, lastSlave, curGlac
    !call MPI_RECV(glacStat,6,MPI_REAL,lastSlave,MPI_ANY_TAG,MPI_COMM_WORLD,status,ierr)
!    call MPI_RECV(recvUpdate,1,MPIglacType,lastSlave,MPI_ANY_TAG,MPI_COMM_WORLD,status,ierr)
    if(curGlac.ne.0) then
      call MPI_RECV(glacStat,4,MPI_REAL,lastSlave,MPI_ANY_TAG,MPI_COMM_WORLD,status,ierr)
      print*, "Master received from process", i, "(2)"
      call MPI_RECV(tempWEq,1,MPIDbltype,lastSlave,MPI_ANY_TAG,MPI_COMM_WORLD,status,ierr)
      print*, "Master received from process", i, "(3)"
      call MPI_RECV(tempClass,1,MPI_INTEGER,lastSlave,MPI_ANY_TAG,MPI_COMM_WORLD,status,ierr)
      print*, "Master received from process", i, "(4)"
    end if
    if((numsent .lt. numGlac).and.(numGlac.gt.numslaves) ) then  ! Send another glacier ID
      print*, "Master sending", i
      call MPI_SEND(numsent+1,1,MPI_INTEGER,lastSlave,2,MPI_COMM_WORLD,ierr)
      print*, "Master sent", i
      numsent = numsent+1
    else! Tell slaves that there is no more work
      print*, "Master sent ZERO", i
      call MPI_SEND(0,1,MPI_INTEGER,lastSlave,2,MPI_COMM_WORLD,ierr)
    end if
! Update current glacier status
!    glacList(curGlac)%area = recvUpdate%area
!    glacList(curGlac)%class = recvUpdate%class
!    glacList(curGlac)%waterEq = recvUpdate%waterEq
!    glacList(curGlac)%minH = recvUpdate%minH
!    glacList(curGlac)%maxH = recvUpdate%maxH
!    glacList(curGlac)%ela = recvUpdate%ela
!    typeSize = sizeof(recvUpdate)
!    call MPI_TYPE_SIZE(MPIGlacType,mpiSize,ierr)
!    dblSize = sizeof(glacList(curGlac)%waterEq)
!    call MPI_TYPE_SIZE(MPIdbltype,mpiDblsize,ierr)
!    print*, "sizeof(recvUpdate), MPI_TYPE_SIZE(MPIGlacSize)", typeSize, mpiSize
!    print*, "sizeof(dblP), MPI_TYPE_SIZE(MPIdbltype)", dblsize, mpiDblSize
!    print*, "master", curGlac, glacList(curGlac)%minH,glacList(curGlac)%maxH,glacList(curGlac)%ela
    if(curGlac.ne.0) then
      glacList(curGlac)%area = glacStat(1)
      glacList(curGlac)%class = tempClass
      glacList(curGlac)%waterEq = tempWEq
      glacList(curGlac)%minH = glacStat(2)
      glacList(curGlac)%maxH = glacStat(3)
      glacList(curGlac)%ela = glacStat(4)
    end if
  end do

! SLAVES PART:
  else

    glacDone = .FALSE.

  !do i = 1,numGlac  Iterating over glaciers (deactivated in MPI version as dispatching is done by master)
  !if(rank .le. numGlac) then  ! If there are more processes than there are glaciers, some processes won't need to work.

  call MPI_RECV(work,1,MPI_INTEGER,root,2,MPI_COMM_WORLD,status,ierr)
    print*, "Before DO loop", rank, work

  if(work.eq.0) then
    print*, "Sending ZERO", rank
    mess = 0
    call MPI_SEND(mess,1,MPI_INTEGER,root,work,MPI_COMM_WORLD,ierr)
    print*, "count glacdone", count(glacDone), rank
  end if

    do while (work .gt. 0)
    i = work
    curMinH = 5000.0
    curMaxH = 0.0
    curArea = 0.0
    curWEq = 0.0

  ! Save last year's status
    oldClass = glacList(i)%class
    oldWEq = glacList(i)%waterEq
    oldArea = glacList(i)%area
    oldMinH = glacList(i)%minH
    oldMaxH = glacList(i)%maxH

! Set parameters of deltaH equation based on glacier size class

    select case(oldClass)
      case(1)
        a = -0.3
        b = 0.6
        c = 0.09
        gamma = 2.0

      case(2)
        a = -0.05
        b = 0.19
        c = 0.01
        gamma = 4.0

      case(3)
        a = -0.02
        b = 0.12
        c = 0.0
        gamma = 6.0
    end select


! Apply deltaH function for each normalized elevation band
!    sizeDH = 10
!    print*, "size(dH)", size(dH)
    hr = (/(i, i=1, 10, 1)/)
    hr = hr/10.
      print*, "hr", hr
    do j = 1,10
!      hr = (1/sizeDH) * j
      dH(j) = ((hr(j)+a)**gamma) + b*(hr(j)+a) + c
    end do
    print*, "glacier", i, "dH", dH

! Define absolute limits of the elevation bands

    hDiff = oldMaxH-OldMinH
    do j = 10,1,-1
      elevLimits(j) = oldMaxH - ((j-1)*hDiff/10)
    end do
    elevArea = 0.0
    print*, "elevLimits", elevLimits

    tempBand = int(rnodat)

    minX = glacList(i)%bbox(1)
    minY = glacList(i)%bbox(2)
    maxX = glacList(i)%bbox(3)
    maxY = glacList(i)%bbox(4)

    vectLen = count(glacID(minX:maxX,minY:maxY) == i)
    print*, "process", rank, "glacier", i, "vectLen", vectLen
    allocate(altVect(vectLen))
    k=1

    do ycoor = maxY, minY, -1
    do xcoor = minX, maxX
    !print*, "i,ice,snow,ID",&
    !xcoor,ycoor,i,curIce(xcoor,ycoor),curSnow(xcoor,ycoor),glacID(xcoor,ycoor)
      if(glacID(xcoor,ycoor) .ne. i) cycle
! Water currently stored in the glacier: all ice and solid snow.
! At the end of the update, all solid snow will be added to the ice.
!      curWEq = curWEq + SGlac(xcoor,ycoor) + ssotf(xcoor,ycoor)
! Need to use curIce and curSnow in MPI version

!      curWEq = curWEq + ((curIce(xcoor,ycoor) + curSnow(xcoor,ycoor)) * rgridsize * rgridsize)

! Update: water stored in glacier is only ice for bare ice cells, and ice + snow that did not
! melt during summer (defined as minimum snow cover during summer) for firn cells.



      if(landin(xcoor,ycoor) == 14) then
!      print*, "curWEq,curIce,curSnow,glacier",curWEq,curIce(xcoor,ycoor),curSnow(xcoor,ycoor),i
        curWEq = curWEq + ((curIce(xcoor,ycoor) + minSnowCover(xcoor,ycoor)) * rgridsize * rgridsize)
      else if (landin(xcoor,ycoor) == 13) then
        curWEq = curWEq + ((curIce(xcoor,ycoor) + minSnowCover(xcoor,ycoor)) * rgridsize * rgridsize)
      end if


! Find out elevation band of the current cell and add cell area to
! glacier area in the corresponding elevation band.
      curBand = 0
      curElev = 10000
      do while (curElev .gt. fhoehe(xcoor,ycoor))
        curBand = curBand + 1
        curElev = elevLimits(curBand)
        if(curBand==10) exit
        print*, "curBand,curElev,fhoehe",curBand,curElev,fhoehe(xcoor,ycoor)
      end do
      elevArea(curBand) = elevArea(curBand) + (rgridsize*rgridsize*icefrac(xcoor,ycoor))
      tempBand(xcoor,ycoor) = curBand
      !print*, "elevArea", elevArea
    end do
    end do

! Get mass balance in kg
!    mb = (curWEq - oldWEq) * rgridsize * rgridsize
    mb = curWEq-oldWEq
    print*,  i, "curWEq", curWEq
    print*,  i, "oldWEq", oldWEq
    print*,  i, "MB", mb


! Multiply deltaH with area for each normalized elevation band; obtain scaling factor
    scaleFac = (mb/(rhoIce * sum(elevArea * dH)))/1000.
    print*, "glacier", i, "elevArea", elevArea
    print*, "glacier", i, "sum(elevArea * dH)", sum(elevArea * dH)

    print*, "glacier", i, "scaleFac", scaleFac


! Apply elevation change for each elevation band using scaleFac and the
! deltaH parameter for the respective elevation band

    do ycoor = maxY, minY, -1
    do xcoor = minX, maxX
      if (tempBand(xcoor,ycoor) == int(rnodat)) cycle
!      if(icefrac(xcoor,ycoor) .lt. 1.0) cycle  ! DeltaH is not applied at the edge of the glacier
      fhoehe(xcoor,ycoor) = fhoehe(xcoor,ycoor) + scaleFac * dH(tempBand(xcoor,ycoor))
!      SGlac(xcoor,ycoor) = (fhoehe(xcoor,ycoor)-bedrock(xcoor,ycoor)) * rhoIce * iceFrac(xcoor,ycoor)
      curIce(xcoor,ycoor) = (fhoehe(xcoor,ycoor)-bedrock(xcoor,ycoor)) * rhoIce * iceFrac(xcoor,ycoor) * 1000.0
!      curSnow(xcoor,ycoor) = 0.0
      !if(landin(xcoor,ycoor)==13) curSnow(xcoor,ycoor) = curSnow(xcoor,ycoor) - minSnowCover(xcoor,ycoor)
      curSnow(xcoor,ycoor) = curSnow(xcoor,ycoor) - minSnowCover(xcoor,ycoor)

! Decide whether the ice cover in the current cell is complete or only a fraction
      if((fhoehe(xcoor,ycoor)-bedrock(xcoor,ycoor)) .ge. critIce) then
        iceFrac(xcoor,ycoor) = 1.0
      else
        iceFrac(xcoor,ycoor) = (fhoehe(xcoor,ycoor)-bedrock(xcoor,ycoor))/critIce
      end if

! Decide when a cell is no longer a glacier cell
! Update land use type if glacier disappeared from this cell
! Totally arbitrary thresholds
      if((iceFrac(xcoor,ycoor) .lt. 0.3) .or. (fhoehe(xcoor,ycoor)-bedrock(xcoor,ycoor)) .lt. 2.0) then
!        if(landin(xcoor,ycoor) == 13) landin(xcoor,ycoor) = 15  ! Former firn cell becomes rock
!        if(landin(xcoor,ycoor) == 14) landin(xcoor,ycoor) = 26  ! Former ice cell becomes bare soil
        if(landin(xcoor,ycoor) == 13) newLand(xcoor,ycoor) = 15  ! Former firn cell becomes rock
        if(landin(xcoor,ycoor) == 14) newLand(xcoor,ycoor) = 26  ! Former ice cell becomes bare soil
        print*, "cell", xcoor,ycoor,"from glacier", i,": sGlac,iceFrac,fhoehe,bedrock",&
        curIce(xcoor,ycoor),iceFrac(xcoor,ycoor),fhoehe(xcoor,ycoor),bedrock(xcoor,ycoor)
        fhoehe(xcoor,ycoor) = bedrock(xcoor,ycoor)
        glacID(xcoor,ycoor) = int(rnodat)
      end if

! Get updated statistics
      altVect(k) = fhoehe(xcoor,ycoor)
      if(glacID(xcoor,ycoor) == i) then
        if(fhoehe(xcoor,ycoor) .lt. curMinH) curMinH = fhoehe(xcoor,ycoor)
        if(fhoehe(xcoor,ycoor) .gt. curMaxH) curMaxH = fhoehe(xcoor,ycoor)
        curWEq = curWEq + SGlac(xcoor,ycoor)
        curArea = curArea + (rgridsize*rgridsize*icefrac(xcoor,ycoor)/1000000.)
      end if

      glacDone(xcoor,ycoor) = .TRUE.
      k = k+1

    end do
    end do

    print*, "Cell loop done", rank

! Update glacier status

    if(curArea .lt. 5.0) then
      tempClass = 1
    else if(curArea .lt. 20) then
      tempClass = 2
    else
      tempClass = 3
    end if
    med = median(altVect)
    glacStat = (/curArea,curMinH,curMaxH,med/)

    print*, "After status update", rank

!    sendUpdate%area = curArea
!
!    med = median(altVect)
!    sendUpdate%ela = med
!
!    if(curArea .lt. 5.0) then
!      sendUpdate%class = 1
!    else if(curArea .lt. 20) then
!      sendUpdate%class = 2
!    else
!      sendUpdate%class = 3
!    end if
!
!    sendUpdate%waterEq = curWEq
!    sendUpdate%minH = curMinH
!    sendUpdate%maxH = curMaxH
!
!      print*, "1 process", rank, i, sendUpdate%minH, sendUpdate%maxH, sendUpdate%ela
!
!    typeSize = sizeof(sendUpdate)
!    call MPI_TYPE_SIZE(MPIGlacType,mpiSize,ierr)
!    dblSize = sizeof(sendUpdate%waterEq)
!    call MPI_TYPE_SIZE(MPIdbltype,mpiDblsize,ierr)
!    print*, "sizeof(sendUpdate), MPI_TYPE_SIZE(MPIGlacSize)", typeSize, mpiSize
!    print*, "sizeof(dblP), MPI_TYPE_SIZE(MPIdbltype)", dblsize, mpiDblSize
!    sizes(1) = sizeof(sendUpdate%area)
!    sizes(2) = sizeof(sendUpdate%class)
!    sizes(3) = sizeof(sendUpdate%waterEq)
!    sizes(4) = sizeof(sendUpdate%minH)
!    sizes(5) = sizeof(sendUpdate%maxH)
!    sizes(6) = sizeof(sendUpdate%ela)
!    print*, "sizes", sizes


!    glacStat = (/glacList(i)%area,real(glacList(i)%class),glacList(i)%waterEq,&
!      glacList(i)%minH,glacList(i)%maxH,glacList(i)%ela/)

! Slave first sends a message to master to tell that it is ready, then
! sends updated glacier status
    mess = i
    call MPI_SEND(mess,1,MPI_INTEGER,root,i,MPI_COMM_WORLD,ierr)
    print*, "After Send 1", rank
    !call MPI_SEND(glacStat,6,MPI_REAL,root,i,MPI_COMM_WORLD,ierr)
!    call MPI_SEND(sendUpdate,1,MPIglacType,root,i,MPI_COMM_WORLD,ierr)
    if(i.ne.0) then
      call MPI_SEND(glacStat,4,MPI_REAL,root,i,MPI_COMM_WORLD,ierr)
      print*, "After Send 2", rank
      call MPI_SEND(curWEq,1,MPIdbltype,root,i,MPI_COMM_WORLD,ierr)
      print*, "After Send 3", rank
      call MPI_SEND(tempClass,1,MPI_INTEGER,root,i,MPI_COMM_WORLD,ierr)
      print*, "After Send 4", rank
      call MPI_RECV(work,1,MPI_INTEGER,root,2,MPI_COMM_WORLD,status,ierr)
    end if

    print*, "After Send/Recv", rank

! Set land cover class (ice or firn) of glacier cells according to ELA

    do ycoor = maxY, minY, -1
    do xcoor = minX, maxX
      if((glacDone(xcoor,ycoor) == .FALSE.) .or. (newLand(xcoor,ycoor) .ne. int(rnodat))) cycle
      if(fhoehe(xcoor,ycoor) .le. glacList(i)%ela) then
      ! Cell is below ELA : ice
        newLand(xcoor,ycoor) = 14
      else
      ! Cell is above ELA: firn
        newLand(xcoor,ycoor) = 13
      end if
    end do
    end do

    print*, "After land cover loop", rank

  !end do  Iterating over glaciers (deactivated in MPI version as dispatching is done by master)

    deallocate(altVect)

  end do  ! (while work .gt. 0)



  !end if  ! (rank .le. numGlac)

  end if  ! Master / Slaves

  if(allocated(tempBand)) deallocate(tempBand)

! Master broadcasts updated glacier status

  do i = 1,numGlac
    if(rank == root) then
!      bcUpdate%area = glacList(i)%area
!      bcUpdate%class = glacList(i)%class
!      bcUpdate%waterEq = glacList(i)%waterEq
!      bcUpdate%minH = glacList(i)%minH
!      bcUpdate%maxH = glacList(i)%maxH
!      bcUpdate%ela = glacList(i)%ela

      glacStat(1) = glacList(i)%area
      glacStat(2) = glacList(i)%minH
      glacStat(3) = glacList(i)%maxH
      glacStat(4) = glacList(i)%ela
      tempWEq = glacList(i)%waterEq
      tempClass = glacList(i)%class


    end if
    !call MPI_BCAST(glacStat,6,MPI_REAL,root,MPI_COMM_WORLD,ierr)
!    call MPI_BCAST(bcUpdate,1,MPIGlacType,root,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(glacStat,4,MPI_REAL,root,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(tempWEq,1,MPIdbltype,root,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(tempClass,1,MPI_INTEGER,root,MPI_COMM_WORLD,ierr)

    if(rank .ne. root) then
      glacList(i)%area = glacStat(1)
      glacList(i)%class = tempClass
      glacList(i)%waterEq = tempWEq
      glacList(i)%minH = glacStat(2)
      glacList(i)%maxH = glacStat(3)
      glacList(i)%ela = glacStat(4)
!      print*, "2 process", rank, i, glacList(i)%minH, glacList(i)%maxH, glacList(i)%ela
    end if
  end do

! All slaves return the updated status of the grid cells they have been
! working on

  call glacReduce(2)

! Output - Master
  if(rank==root) then
      inquire(file="glacStats.txt",exist=exFile)
      if(exFile) then
       open(665,file="glacStats.txt",status="old", position="append")
       else
         open(665,file="glacStats.txt",status="new")
      end if
72     format(I5,I3,F8.3,I3,F20.3,3F15.3)
      do i = 1,numglac
      write(665,72) jahr, i, glacList(i)%area, glacList(i)%class, glacList(i)%waterEq,&
                    glacList(i)%minH, glacList(i)%maxH, glacList(i)%ela
     end do
      close(9257)

  end if



end subroutine glacDeltaH

!=======================================================================

subroutine glacmoTrans(xcoor,ycoor,prl,teck,rdmk,cicemf,caice,erice,kiceh,&
kicsh,kicfh,ticeh,ticsh,ticfh,wice,wics,wicf,rice,rics,ricf,sso,sice,sics,sicf,hfice,hfics,hficf)

! New version of icemelt subroutine Glacmo (Zappa, Badoux & Gurtz 1999) with
! transient update of ice storage.

  use prevahglobals, only: sicetf, sicstf, sicftf, htfice, htfics, htficf, &
                     landin, sGlac
  use glacGlob, only:

  implicit none

! Passed variables:

  integer :: xcoor, ycoor
  real,intent(in) :: prl,&  ! Liquid precipitation for current time step
                     teck,& ! Adjusted temperature
                     rdmk,& ! Locally adjusted potential incoming radiation in W/M2
                     cicemf,& ! constant temperature melt factor for ice
                     caice,&  ! constant radiation melt factor for ice
                     erice,&  ! Evaporation/sublimation from ice
                     kiceh,&  ! Coefficient for runoff generation from ice storage in hours
                     kicsh,&  ! Coefficient for runoff generation from snow storage in hours
                     kicfh,&  ! Coefficient for runoff generation from firn storage in hours
                     sso        ! Snow cover of current cell
  real,intent(inout) :: sice, sics, sicf
  real, intent(inout), dimension(24) :: hfice, hfics, hficf

  integer,intent(in) :: ticeh,&  ! Translation time for icemelt
                        ticsh,&  ! Translation time for snowmelt
                        ticfh    ! Translation time for firn melt
  real,intent(out) :: wice,&  ! Icemelt from a glaciated HRU below HELA
                      wicf,&
                      wics,&
                      rice,&
                      rics,&
                      ricf

! Internal variables




!     Initialization
  WICE=0.0
  !GWICE=0.0
  WICF=0.0
  WICS=0.0
  RICE=0.0
  RICS=0.0
  RICF=0.0

  if((xcoor == 153) .and. (ycoor == 156)) then
    print*, "in GLACMO"
    print*, "sGlac before", sGlac(xcoor, ycoor)

  end if

!   sice = sicetf(xcoor,ycoor)
!   sicf = sicftf(xcoor,ycoor)
!   sics = sicstf(xcoor,ycoor)

! If the current cell belongs to the class "Firn" (above ELA of its glacier):

  if(landin(xcoor,ycoor) == 13) then
! Liquid precipitation and snowmelt gets distributed equally between the
! snow and firn runoff generation storages
    wicf = prl*0.5
    wics = prl*0.5

!     hficf(1:24) = htficf(xcoor,ycoor,1:24)
!     hfics(1:24) = htfics(xcoor,ycoor,1:24)
    CALL LINTRA (SICF,WICF,KICFH,TICFH,hficf,RICF)
    CALL LINTRA (SICS,WICS,KICSH,TICSH,hfics,RICS)
!     htficf(xcoor,ycoor,1:24) = hficf(1:24)
!     htfics(xcoor,ycoor,1:24) = hfics(1:24)
  else if(landin(xcoor,ycoor) == 14) then
! For bare ice cells (below ELA)
    if(sso .ge. 0.1) then
! If cell is still snow covered: only snowmelt gets transferred to runoff
! generation.
      wice = 0.0
     ! erice = 0.0
      wics = prl
    else
! Without snow cover: ice melts if temperature is positive and if there
! is some ice left on this cell.
      if((teck .gt. 0.0) .and. (sGlac(xcoor, ycoor) .gt. 0.0)) then
! Ice melt occurs only if temperature is greater than zero
        WICE=(((1.0/24.0)*CICEMF)+(CAICE*RDMK))*TECK
      else
        wice = 0.0
      end if
    wice = max(wice,0.0)
! The current cell's ice reservoir gets depleted directly by icemelt
! (and evaporation/sublimation, if these are considered)
    sGlac(xcoor,ycoor) = max(sGlac(xcoor,ycoor) - wice, 0.0)
! Evaporation from ice surfaces neglected for now

! Icemelt and liquid precipitation / snowmelt are input to runoff generation storage
!     hfice(1:24) = htfice(xcoor,ycoor,1:24)
!     hfics(1:24) = htfics(xcoor,ycoor,1:24)
    CALL LINTRA (SICE,WICE+prl,KICEH,TICEH,hfice,RICE)
    CALL LINTRA (SICS,WICS,KICSH,TICSH,hfics,RICS)
!     htfice(xcoor,ycoor,1:24) = hfice(1:24)
!     htfics(xcoor,ycoor,1:24) = hfics(1:24)
    end if

  end if


!     sicetf(xcoor,ycoor) = sice
!     sicstf(xcoor,ycoor) = sics
!     sicftf(xcoor,ycoor) = sicf




end subroutine glacmoTrans
