!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.
! Modifications Copyright (C) 2018 Matthias Speich
! 2014-2018: Adapt the subroutine to the disaggregated soil evaporation and
! transpiration calculation.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

! ------------------------------------------------------------------------------
!    Soil moisture and runoff-generation module
!
!    Literature
!    Bergström, S. (1976): Development and Application of a Conceptual Runoff Model
!     for Scandinavian Catchments. Bulletin Series A, No. 52, University of Lund.
!    Bergström, S. (1992): The HBV model - its structure and applications. SMHI, RH
!     No. 4, Norrköping, Sd.
!    Gurtz, J., Zappa, M., Jasper, K., Lang, H., Verbunt, M., Badoux, A. and
!     Vitvar, T. (2003): A Comparative Study in Modelling Runoff and its Components
!     in Two Mountainous Catchments. Hydrol. Processes, 17, 297-311.
!    Schwarze, R., Droege, W. and Opherden, K. (1999): Regional analysis and modelling
!     of groundwater runoff components from catchments in hard rock areas.
!     In: Diekkr�ger B, Kirkby MJ, Schr�der U (Editors): Regionalisation in hydrology,
!     IAHS Publication No. 254, 221-232.
!    Zappa, M. and Gurtz, J. (2003): Plot-scale simulation of soil moisture and
!     evapotranspiration during the 1999 MAP-Riviera campaign.
!     Hydrology and Earth System Sciences. Accepted.
! ------------------------------------------------------------------------------

      subroutine abfgpatch(prl, rice, rics, ricf, aeSoil, aTrans,
     * peInt, asec, sm, sfc,btopi,
     * topmin, topmit, kwert, kwminl, kwmitl, sgrlp, cperc,cbeta,sgrluz,
     * k0,k1, k3, cg1h, cg2h, slz1max,  ssz, suz,slz, slz1, slz2, slz3,
     * rl,rges, r0, r1, r2, r3, rg1s, rg2s, rg3s, eSoil, eTrans,
     * eWater, perc, dsuz, iart, land, aelhg,xcoor,ycoor,kwp)



      use transglob

      implicit none

! Passed variables
      integer, intent(in) :: iart, land

      integer, intent(in) :: xcoor,ycoor  ! DEBUG ONLY

      real::     prl  ! Should be intent IN - can't because of line 332. TODO
      real, intent(in) ::     aeSoil,  ! Actual soil evaporation (with exponential reduction, but might still get reduced slightly if not enough water available)
     *                        aTrans,   ! Actual transpiration (with Jarvis resistance, but might still get reduced slightly if not enough water available)
     *                        peInt,  ! "Interception" PET, representing here open water evaporation in water & wetland cells. Not used here for other land cover types.
     *                        sgrluz,
     *                        cbeta,
     *                        cperc,
     *                        sfc,
     *                        aelhg,
     *                        rice, rics, ricf,
     *                        kwert, kwminl, kwmitl,
     *                        sgrlp,   ! Threshold soil moisture for evaporation (used only for special land cover types)
     *                        btopi, topmin, topmit,
     *                        k0, k1, k3,
     *                        cg1h, cg2h,
     *                        slz1max,
     *                        asec
      real, intent(out) :: kwp

      real, intent(inout) ::  slz1, slz2, slz3,
     *                        suz, slz, ssz,
     *                        sm


      real, intent(out) ::    rl,               ! Total runoff
     *                        rges, r0, r1, r2, r3, rg1s, rg2s, rg3s,
     *                        eSoil, eTrans, eWater,
     *                        perc,
     *                        dsuz


! Internal variables
      integer ::  i,                ! Loop variable for iteration
     *            im               ! Number of iterations
      real ::     cperca, sgrluza,
     *            tpk0,
     *            cg3h,
     *            dssz,
     *            rg1, rg2, rg3,
     *            topsz, kwper, top,
     *            q01, q0, q1, q2, q3,
     *            ts,
     *            awet, prs, prw,
     *            ssme, ssze, slze, suze, slz1e, slz2e, slz3e,
     *            qges,
     *            diff,
     *            rim,
     *            esIter, etIter, ewIter,
     *            percadd,
     *            dsuzIter,
     *            ftransssm,ftranspi,ftranssuz,
     *            crsz,
     *            crsz2,
     *            ewet,
     *            sme

!-------------------------------------------------------------------------------
! Modifications to input/output variables - MS 2014 - 2018:

! DELETED (not used) : ier, hkk(passed), sgrlp,  MINF,NINF,FCINF,AINF,DTINF,PINF, topi,
! qice, qics, rhg, atfc, etra, etr, er

! ADDED : aeSoil, aTrans, peSoil, pTrans, eSoil, eTrans

! DSUZS changed to DSUZ, and DSUZ to DSUZITER (so that the variable name matches that in prevahcore)
! CPERC and CPERCA replaced in the code so that CPERC is intent(in).
!-------------------------------------------------------------------------------

! ------------------------------------------------------------------------------
! Begin
! ------------------------------------------------------------------------------
!    Initialization of the storages and of the variables

      !SSME=SSM
      sme = sm
      SSZE=SSZ
      SUZE=SUZ
      SLZE=SLZ
      SLZ1E=SLZ1
      SLZ2E=SLZ2
      SLZ3E=SLZ3
C	dsuzs=0.0
      DSUZ=0.0
      dsuzIter = 0.0
C      ATFC=1.0
      QGES=0.0
      Q0=0.0
      Q01=0.0
      Q1=0.0
      Q2=0.0
      Q3=0.0
C      QICE=0.0
C      QICS=0.0
      RGES=0.0
	RG1s=0.0
      RG2s=0.0
      RG3s=0.0
      R0=0.0
      R1=0.0
      R2=0.0
      RG1=0.0
      RG2=0.0
      RG3=0.0
      R3=0.0
C      EA=0.0
C      ER=0.0
C      ETRA=0.0
C      ETR=0.0
      eWater = 0.0
      esIter = 0.0
      etIter = 0.0
      ewIter = 0.0
      eSoil = 0.0
      eTrans = 0.0
      ewet = 0.0
      PERC=0.0
	PERCADD=0.0
C      IER=0
      SGRLUZA=SGRLUZ
C	K0A=K0
C	K1A=K1
      CPERCA=CPERC
      TOP=BTOPI
      TOPSZ=1.0


! Topographic index
         TOPSZ=(TOP-TOPMIN)/(TOPMIT-TOPMIN)
         IF (TOPSZ .LT. 0.1) TOPSZ=0.1
         IF (TOPSZ .GT. 0.9) TOPSZ=0.9


      KWPER=1.0
      IF ((KWMITL .GT. KWMINL) .and. (KWMITL .GT. 0.0)) THEN
         !KWPER=(ALOG(KWERT)-KWMINL)/(KWMITL-KWMINL)
         KWPER=exp((alog(KWERT)-KWMINL)/(KWMITL-KWMINL))
         IF (KWPER .LT. 0.05) KWPER=0.05
      ENDIF

      kwp = kwper
      kwper = kwert


! ------------------------------------------------------------------------------
!    PI determines the number of integration steps

      IM=MAX(3,INT(((PRL*10.0)/30.0)+1.5)) ! 6
      TS=1.0/IM
      RIM=IM*1.0


!    After Schwarze CG3H is 1/9 of CG2H
      CG3H=0.11111*CG2H

! Water and land fraction for some land cover types ()
      AWET=1.0-ASEC

!    In case of water surfaces all the water available for runoff-generation is primarly
!    used to fill the storage reservoir SSM. Evaporation for open water surfaces depletes
!    SSM.
      IF (LAND .EQ. 1) THEN
          IF (sme.LT.2500) THEN
               sme=sme+PRL
               PRL=0
          ENDIF
      END IF

! ------------------------------------------------------------------------------
!    Niederschlagskorrektur erfolgt schon im Hauptprogramm
      PRS=PRL*ASEC*TS
      PRW=PRL*AWET*TS

! ------------------------------------------------------------------------------
!    Core of the runoff-generation module - Integration within DT
      DO 200 I=1,IM

!    Updating the soil moisture content by computing and subtracting actual evapotranspiration

      if((land == 1) .or. (land==2) .or. (land == 15)) then
        esIter = aeSoil * asec * ts
        if(sme .lt. sgrlp) esIter = esIter*(sme/sgrlp)
        etIter = 0.0
      else if ((land == 13) .or. (land==14)) then
        perc = ricf * ts
        esIter = 0.0
        etIter = 0.0
      else
        esIter = aeSoil * asec * ts
        etIter = aTrans * asec * ts
      end if

! ------------------------------------------------------------------------------
!    Separating the available water into soil-moisture SSM and water for runoff
!    generation DSUZ

      if((land .ne. 13) .and. (land .ne. 14)) then

        dsuzIter = prs*((sme/sfc)**cBeta)


        if(sme .gt. sfc) then
        ! If plant-available soil moisture is full, all incoming precipitation
        ! goes to runoff generation. A possible overfill of SSM is also subtracted
        ! here (sme-sfc), to keep SSM within its limit (SFC)
          dsuzIter = prs + (sme-sfc)
          sme = sfc
        end if

        !if(sme .ge. sfc) dsuzIter = prs

        sme = sme - etIter - esIter + (prs-dsuzIter)


        if(sme .lt. 0.0) then
          esIter = etIter + esIter + sme
          eTIter = 0.0
          sme = 0.0
        end if

!===================================
C8      CONTINUE


       !IF (SSME .GE. SFC) DSUZ=PRS
       !IF (SSME .GE. SFC) DSUZIter=PRS
!     Change in soil moisture content
       !SSME=SSME+(PRS-DSUZ-EA)

!     Computation of surface runoff r0, new CRSZ definition for small catchments

	 IF ((PRL .lt. 2) .or. (aelhg .gt. 100)) then
	 crsz=0
	 else
C	 crsz=(1+log(prl/aelhg**0.5))*0.1*topsz*(SSME/SFC)
	 crsz=(1+log(prl/aelhg**0.5))*0.1*topsz*(sme/sfc)
	if (crsz .lt. 0.0) crsz=0.0

	 crsz2=0.5*(log(prl/(min(50.0,max(0.01,aelhg))))-log(2./50.))/
     *       (log(50./0.01)-log(2./50.))
!     	 if (hkk .eq. 1) write (9123,*) crsz,crsz2,prl,topsz,(SSME/SFC)
	 end if

       DSSZ=0.
	 Q01=0.
!	crsz=0.
       IF (CRSZ .GT. 0.0)  THEN
          !DSSZ=DSUZ*CRSZ
          DSSZ=dsuzIter*CRSZ
          IF (DSUZ .LT. 0.0) DSSZ=0.0
          !DSUZ=DSUZ-DSSZ
          DSUZIter=DSUZIter-DSSZ
!     Surface runoff
       END IF

       SSZ=SSZE

       IF (SSZ .GT. 0.0) THEN
          Q01=k0*10.*SSZ
!          SSZE=SSZ-(Q01*TS)
       IF (SSZE .LT. 0.0) THEN
           Q01=(Q01+SSZE)/TS
           SSZE=0.0
       ENDIF
       ENDIF

!   Actualization of SSZE
          SSZE=SSZE+DSSZ-(Q01*TS)
          IF (SSZE .LT. 0.0) THEN
             Q01=(Q01+SSZE)/TS
             SSZE=0.0
          ENDIF

!	if (hkk .eq. 1) write (9123,*) crsz,dssz,ssz,q01
! ------------------------------------------------------------------------------
!         Computaiton of surface runoff r0 basing on the threshold parameter SGRLUZ
!         if CRSZ < 0.0
          IF ( SUZE .GT. SGRLUZ) THEN
             Q0=K0*(SUZE-SGRLUZ)
          ELSE
             Q0=0.0
          ENDIF
          IF ( Q0 .LE. 0.0) Q0=0.0
!        ENDIF

! ------------------------------------------------------------------------------
!       Computation of interflow Q1 and, eventuallym karstflow Q3.
!       Q1 and Q3 generates from the storage of gravitative water in the
!       unsaturated zone SUZ
        IF (SUZE .GT. 0.0) THEN
          Q1=K1*SUZE
          IF (K3 .GT. 0.0) Q3=K3*SUZE
        ELSE
          Q1=0.0
          Q3=0.0
       ENDIF

! ------------------------------------------------------------------------------
!      If the initial deep infiltration rate CPERCA < 0.0 then a constant percolation rate
!      is used
       IF (CPERCA .GT. 0.0) THEN
          GOTO 9
       ENDIF
!      If SGRLP>=SFC, and CPERCA<0.0 then a constant deep infiltration rate
!       CPERC=CPERCA*KWPER is used
       !CPERC=-CPERCA
       cperca = -cperc
       !IF (SGRLP .GE. SFC) THEN

          !CPERC=CPERCA*KWPER
C          cperca = cperc*kwper
C          GOTO 9
C       ENDIF

!      CPERC might be reduced as function of the plant available moisture content SSM
!                SSM < SFC and SSM > SGRLP - linear reduction of CPERC
!                SSM <= SGRLP - then CPERC = 0.0
C       IF ((SSME .GT. SGRLP) .and. (SSME .LT. SFC)) THEN
C          !CPERC=CPERCA*KWPER*((SSME-SGRLP)/(SFC-SGRLP))
C          cperca = cperc*kwper*((SSME-SGRLP)/(SFC-SGRLP))
C       ENDIF
C       !IF (SSME .LE. SGRLP) CPERC=0.0
C       IF (SSME .LE. SGRLP) cperca=0.0

! Percolation

      if((sme .gt. (sfc*0.75)) .and. (sme .le. sfc)) then
        cperca = cperc*kwper*((sme-(sfc*0.75))/(sfc-(sfc*0.75)))
      end if
      if(sme .le. (sfc*0.75)) cperca = 0.0

9      CONTINUE
       PERC=CPERCA*TS
       IF (PERC .GT. SUZE) PERC=SUZE


!      Actualization of the storage of gravitative water in the
!      unsaturated zone SUZ
!       IF (CRSZ .GT. 0.0) THEN
!          SUZE=SUZE+DSUZ-(PERC+(Q1*TS)+(Q3*TS))
!        ELSE


         !SUZE=SUZE+DSUZ-(PERC+(Q0*TS)+(Q1*TS)+(Q3*TS))
         SUZE=SUZE+dsuzIter-(PERC+(Q0*TS)+(Q1*TS)+(Q3*TS))


!       ENDIF



       IF (SUZE .LT. 0.0) THEN
            IF (Q3 .LT. 0.0) THEN
               Q1=Q1+Q3
               Q3=0.0
            ENDIF
            IF (Q1 .LT. 0.0) THEN
               Q0=Q0+Q1
               Q1=0.0
            ENDIF
            SUZE=0.0
       ENDIF


       end if  ! (land .ne. 13 or 14)

! ------------------------------------------------------------------------------
!     Ausfluss aus den Speichern der LOWER ZONE (Grundwasserspeicher)
!        Verwendung der Modellvorstellung SLOWCOMP (IHM-TUD)
!        Unterteilung der LOWER ZONE SLZ in 3 Teilspeicher
!        1. Teilspeicher SLZ1 ist der f�r den schnellen GW-Abfluss RG1.
!                        Er ist begrenzt durch SLZ1MAX mit Speicherzeit CG1H
!                        (5-30 Tage) und kann nur so schnell gefuellt werden,
!                        wie er bei Fuellung auslaufen wuerde.
!        Der 2. und 3. Teilspeicher SLZ2 und SLZ3 bilden zusammen den
!            langsamen Basis- oder GW-Abfluss RG3+RG2 mit den
!            Speicherzeiten CG2H und CG3H, wobei CG3H=(1/9)*CG2H und
!            CG2H liegt im Bereich 250 bis 1000 Tage.
!            Der Zufluss zu SLZ3 ist immer 1/9 des Zuflusses zu SLZ2

12    CONTINUE
      percadd=percadd+perc
      IF ((SLZ1MAX .GT. 0.0) .and. (CG1H .GT. 0.0)) THEN
         SLZ1=SLZ1E*EXP(-TS/CG1H)
         IF (SLZ1 .GE. SLZ1MAX) THEN
            DIFF=PERC+PRW
            GOTO 10
         ENDIF
         DIFF=(SLZ1MAX-SLZ1)/(CG1H*RIM)
         IF (DIFF .GT. (PERC+PRW)) THEN
            SLZ1=SLZ1+((1.0-EXP(-TS/CG1H))*(PERC+PRW)*(CG1H*RIM))
            DIFF=0.0
           ELSE
            SLZ1=SLZ1+((1.0-EXP(-TS/CG1H))*(DIFF*CG1H*RIM))
            DIFF=(PERC+PRW)-DIFF
         ENDIF
       ELSE
         DIFF=PERC+PRW
         SLZ1=0.0
         RG1=0.0
      ENDIF
10    CONTINUE
      IF (CG2H .GT. 0.0) THEN
         IF (DIFF .LE. 0.0) THEN
!          no infiltration in the slow groundwater component SLZ2+SLZ3
            DIFF=0.0
            SLZ2=SLZ2E*EXP(-TS/CG2H)
            SLZ3=SLZ3E*EXP(-TS/CG3H)
           ELSE
!    infiltration in the slow groundwater component SLZ2+SLZ3
            SLZ2=(SLZ2E*EXP(-TS/CG2H))+((1.0-EXP(-TS/CG2H))*
     *           (DIFF*0.8889*CG2H*RIM))
            SLZ3=(SLZ3E*EXP(-TS/CG3H))+((1.0-EXP(-TS/CG3H))*
     *           (DIFF*0.1111*CG3H*RIM))
         ENDIF
       ELSE
         SLZ2=0.0
         RG2=0.0
         SLZ3=0.0
         RG3=0.0
      ENDIF

      IF (CG1H .GT. 0.0) RG1=SLZ1/(CG1H*RIM)
      IF (CG2H .GT. 0.0) THEN
         RG2=SLZ2/(CG2H*RIM)
         RG3=SLZ3/(CG3H*RIM)
      ENDIF


! ------------------------------------------------------------------------------
!     Storage change in the saturated zone SLZ*.

! Only for cells which have a wet fraction (i.e. water and wetlands)

  !    if((land==1) .or. (land == 22)) then

C       EWET=AWET*EPT*TS
        ewet = awet*aeSoil*ts
C       EWETR=EWET
       ewIter = ewet
C       SLZ1E=SLZ1-EWETR
       slz1e = slz1-ewIter
       SLZ2E=SLZ2
       SLZ3E=SLZ3


       IF (SLZ1E .LT. 0.0) THEN
C          EWETR=EWETR+SLZ1
          ewIter = ewIter+slz1
          SLZ1E=0.0
         ELSE
C          EWETR=0.0
         ewIter = 0.0
       ENDIF



C       IF (EWETR .GT. 0.0) THEN
        if(ewIter .gt. 0.0) then
C          SLZ2E=SLZ2-EWETR
          slz2e = slz2-ewIter
          IF (SLZ2E .LT. 0.0) THEN
C             EWETR=EWETR-SLZ2
             ewIter = ewIter-slz2
             SLZ2E=0.0
            ELSE
C             EWETR=0.0
            ewIter = 0.0
          ENDIF
       ENDIF


C       IF (EWETR .GT. 0.0) THEN
        if(ewIter .gt. 0.0) then
C          SLZ3E=SLZ3-EWETR
           slz3e = slz3 - ewIter
          IF (SLZ3E .LT. 0.0) THEN
C             EWETR=EWETR-SLZ3
             ewIter = ewIter - slz3
             SLZ3E=0.0
            ELSE
C             EWETR=0.0
            ewIter = 0.0
          ENDIF
       ENDIF


C       IF (EWETR .GT. 0.0) THEN
        if(ewIter .gt. 0.0) then
          IF (LAND .EQ. 1) THEN
C             SSME=SSME-EWETR
             !ssme = ssme - ewIter
             sme = sme - ewIter
            ! IF (SSME .LT. 0.0) THEN
            IF (sme .LT. 0.0) THEN
C                EWETR=-SSME
                !ewIter = -ssme
                ewIter = -sme
                !SSME=0.0
                sme = 0.0
               ELSE
C                EWETR=0.0
               ewIter = 0.0
             ENDIF
          ENDIF
       ENDIF


C       EWET=EWET-EWETR
        ewet = ewet - ewIter
   !   end if

! ------------------------------------------------------------------------------
!    Determination of the total runoff
       IF ((LAND .EQ. 14).or.(LAND .EQ. 13)) THEN
          Q0=RICE*TS
          !QICE=Q0
          Q1=RICS*TS
          !QICS=Q1
          GOTO 11
       ENDIF
       Q0=Q0*TS
       Q01=Q01*TS
       Q1=Q1*TS
	 Q0=Q0+Q01 ! New CRSZ
!       Q1=Q1+Q01
11     CONTINUE
       RG1S=RG1S+RG1
       RG2S=RG2S+RG2
       RG3S=RG3S+RG3
       Q2=RG1+RG2+RG3
       Q3=Q3*TS
       QGES=Q0+Q1+Q2
       R0=R0+Q0
       R1=R1+Q1
       R2=R2+Q2
       R3=R3+Q3
       RGES=RGES+QGES
	dsuz=dsuz+dsuzIter



! ------------------------------------------------------------------------------
!     Determination of total evapotranspiration
       eSoil = eSoil + esIter
       eTrans = eTrans + etIter
       eWater = eWater + ewet


200   CONTINUE

! ------------------------------------------------------------------------------
!     Actualization of the state variables
      RL=QGES
      !SSM=SSME
      sm = sme
      SSZ=SSZE
      SUZ=SUZE
      SLZ=SLZ1E+SLZ2E+SLZ3E
      SLZ1=SLZ1E
      SLZ2=SLZ2E
      SLZ3=SLZ3E
      !SGRLUZ=SGRLUZA
C	K0=K0A
C	K1=K1A
      !CPERC=CPERCA
	PERC=PERCADD


300   CONTINUE

! ------------------------------------------------------------------------------
      RETURN
      END
! ------------------------------------------------------------------------------
