!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Calculates above-ground biomass using the stem volume functions of
! Kaufmann (2000) (http://www.lfi.ch/publikationen/publ/Schaftholztarife.pdf)
!-------------------------------------------------------------------------------

subroutine bioKaufmann(xcoor, ycoor, thiscell, prodReg)

  use forhycsglobals, only: biomass2d, vh99, deltaBio
  use all_par, only: birht, maxhc, htmax, spc, CurrStateInCell, a1, a2, c1, spec, &
      pltsiz
  use prevahglobals, only: fhoehe, rnodat

  implicit none

  ! Production region (spatial sub-unit of Switzerland according to the Swiss NFI)
  ! 1 - Jura
  ! 2 - Mittelland
  ! 3 - Prealps
  ! 4 - Alps
  ! 5 - Southern Alps
  integer, intent(in) :: prodReg
  integer, intent(in) :: xcoor, ycoor
  type(CurrStateInCell), intent(in) :: thiscell

  ! Coefficients of the empirical stem volume equation. Vary with
  ! species and production region.
  real, dimension(30) :: vb0, vb1, vb2, vb3, vb4, vb6, vb7
  integer :: i,j, diam

  real, parameter :: areaConv = 12.0048   ! Factor to convert from trees on a TreeMig plot
                                          ! (833 m^2) to trees/ha

  real :: hCan, hDom
  integer :: hcCan
  real :: bioOld, bioNew
  real :: growth
  real :: maxdbh, curdbh
  integer :: maxdbhi
  real :: diasum
  real :: ddom
  real :: stemvol
  real :: b0, b1, b2, b3, b4, b6, b7
  real :: lndiam
  real :: elv
  real :: canopyyn
  real :: ka1, ka2, kc1
  real :: folwgt, stmwgt, bm
  integer :: inan

!=======================================================================

! Set model coefficients for each species, depending on production region
  select case (prodReg)
    case(4) ! Alps
    data(vb0(j), j=1,30) /-13.294965, -10.992584, -11.225599, -10.143015, -10.143015, -10.143964, -10.143015, & ! Conifers
    -10.67774, -10.67774, -10.67774, -7.719545, -7.719545, -7.719545, -7.719545, -7.719545, -6.2690702, -7.719545, &
    -11.036856, -8.661268, -7.719545, -7.719545, -8.9570692, -8.9570692, -8.9570692, -7.719545, -7.719545, -7.719545, &
     -7.719545, -7.719545, -7.719545/

     data(vb1(j), j=1,30) /4.11904301, 3.26436371, 3.43239299, 3.16197313, 3.16197313, 2.77278851, 3.16197313, & ! Conifers
     3.40254883, 3.40254883, 3.40254883, 1.86753005, 1.86753005, 1.86753005, 1.86753005, 1.86753005, 1.66305684, 1.86753005, &
     3.27767482, 2.43793729, 1.86753005, 1.86753005, 2.50307234, 2.50307234, 2.50307234, 1.86753005, 1.66305684, 1.86753005, &
     1.86753005, 1.66305684, 1.86753005/

     data(vb2(j), j=1,30) /-0.00360841, -0.00505475, -0.0058899, -0.0046934, -0.0046934, -0.00357987, -0.0046934, & ! Conifers
     -0.00780443, -0.00780443, -0.00780443, 0.002000041, 0.002000041, 0.002000041, 0.002000041, 0.002000041, 5.79063E-04, 0.002000041, &
     -0.00517713, -0.00162922, 0.002000041, 0.002000041, -0.0020347, -0.0020347, -0.0020347, 0.002000041, 0.002000041, 0.002000041, &
     0.002000041, 0.002000041, 0.002000041/

     data(vb3(j), j=1,30) /3.69E-06, 7.14E-05, 3.39E-05, 0.0, 0.0, 7.62E-05 ,0.0, & ! Conifers
     0.0, 0.0, 0.0, 5.24E-05, 5.24E-05, 5.24E-05, 5.24E-05, 5.24E-05, 0.0, 5.24E-05, &
     3.35E-05, 5.9E-05, 5.24E-05, 5.24E-05, 7.18E-05, 7.18E-05, 7.18E-05, 5.24E-05, 5.24E-05, 5.24E-05, &
     5.24E-05, 5.24E-05, 5.24E-05/

     data(vb4(j), j=1,30) /0.005529859, 0.003074087, 0.005502126, 0.004242866, 0.004242866, 0.013275281, 0.004242866, & ! Conifers
     0.005672662, 0.005672662, 0.005672662, 0.010988475, 0.010988475, 0.010988475, 0.010988475, 0.010988475, 0.0, 0.010988475, &
     0.014177976, 0.008327065, 0.010988475, 0.010988475, 0.007726275, 0.007726275, 0.007726275, 0.010988475, 0.010988475, 0.010988475, &
     0.010988475, 0.010988475, 0.010988475/

     data(vb6(j), j=1,30) /-0.00035237, -0.00012482, -0.00022606, -0.00048265, -0.00048265, 0.0, -0.00048265, & ! Conifers
     -0.00045636, -0.00045636, -0.00045636, 0.0, 0.0, 0.0, 0.0, 0.0, -0.00038093, 0.0, &
     -0.00039528, -0.00034256, 0.0, 0.0, -0.00037562, -0.00037562, -0.00037562, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/

     data(vb7(j), j=1,30) /0.0, 0.0, -0.37261846, -0.38813724, -0.38813724, -0.30989771, -0.38813724, & ! Conifers
     -0.14852764, -0.14852764, -0.14852764, -0.28054224, -0.28054224, -0.28054224, -0.28054224, -0.28054224, -0.3339261, -0.28054224, &
     -0.51779497, -0.17147782, -0.28054224, -0.28054224, 0.0, 0.0, 0.0, -0.28054224, -0.28054224, -0.28054224, &
     -0.28054224, -0.28054224, -0.28054224/
  end select

  ! Old (i.e. currently stored) biomass value in cell
  bioOld = sum(biomass2d(xcoor,ycoor,1:30))

  ! Elevation
  elv = fhoehe(xcoor,ycoor)

  !!! Determine site-specific, dynamic factors that influence the allometric relationship

  !! Growth term

  ! Remove NaNs (may occur if a previously unforested cell becomes forested
  ! (e.g. because of LCDyn control parameter)
  !
  ! X .ne. X only returns TRUE if X is NaN

  do inan = 1,30
    if(deltaBio(xcoor,ycoor,inan).ne.deltaBio(xcoor,ycoor,inan)) deltaBio(xcoor,ycoor,inan) = rnodat

  end do

  growth = sum(deltaBio(xcoor,ycoor,1:30), mask=deltaBio(xcoor,ycoor,1:30).ne.rnodat) / max(1,count(deltaBio(xcoor,ycoor,1:30).ne.rnodat))
  growth = growth * 1000      ! t/ha to kg/ha
  ! Specify a minimum and maximum growth
  growth = max(750.0,min(4500.0, growth))

  !! Diameter of the (100) largest trees (per ha)

  ! Determine maximum possible DBH
  maxdbh = 0.0
  do i = 1,30
  do j = 1, maxhc
    curdbh = spc(i)%diain(j)
    if(curdbh.gt.maxdbh) maxdbh = curdbh
  end do
  end do

  maxdbhi = floor(maxdbh)

  do diam = maxdbhi, 0, -1
    diasum = 0.0
    do i = 1,30
    do j = 0, maxhc
      if(spc(i)%diain(j).ge.real(diam)) then
        diasum = diasum + thiscell%sp(i)%numin(j) * areaConv
      end if
    end do
    end do
    if(diasum.ge.100.0) then
      ddom = max(12.0,real(diam))
      exit
    end if
  end do

  if(diasum.lt.100.0) ddom = 12.0


  !! Determine the height of the upper canopy ("Oberschicht") - taken to be 2/3 of the height of dominant trees
  hDom = vh99(xcoor, ycoor)
  hCan = hDom * 0.666
  hcCan = MIN (FLOOR (REAL (maxhc - 1) * &      ! Adapted from GetStd
                (hCan - birht) / (REAL (htmax) - birht) + 1.0), maxhc)


  !!! Calculate new biomass per species/size.
  ! As in the original TreeMig, biomass of seedlings (height class 0) is set to 1/10 of
  ! the biomass of height class 1 (per individual).
  biomass2d(xcoor,ycoor,:) = 0.0
  do i = 1,30
    b0 = vb0(i)
    b1 = vb1(i)
    b2 = vb2(i)
    b3 = vb3(i)
    b4 = vb4(i)
    b6 = vb6(i)
    b7 = vb7(i)

    ka1 = a1(spec(i)%stn)
    ka2 = a2(spec(i)%stn)
    kc1 = c1(spec(i)%tb)

  do j = 0, maxhc
    lndiam = log(spc(i)%diain(j) * 100)

    ! Seedlings - perform calculations using DBH of height class 1 - scaling
    ! by factor 0.1 will be done later
    if(j==0) lndiam = log(spc(i)%diain(1) * 100)

    if(j.ge.hcCan) then
      canopyyn = 0.0
    else
      canopyyn = 1.0
    end if
    stemvol = exp(b0 + b1*lndiam + b2*(lndiam**4) + b3*growth + b4*ddom + b6*elv + b7*canopyyn)
    stmwgt = 500*stemvol !0.5*stemvol
    folwgt = kc1 * ka1 * EXP (lndiam * ka2)

    ! Add stem and foliage biomass
    bm = stmwgt + folwgt

    ! Special case - seedlings
    if(j==0) bm = bm*0.1

    biomass2d(xcoor,ycoor,i) = biomass2d(xcoor,ycoor,i) + bm * (10 / pltsiz) * thiscell%sp(i)%numin(j)
  end do
  end do


  ! Update biomass change tracker
  bioNew = sum(biomass2d(xcoor,ycoor,1:30))
  deltaBio(xcoor,ycoor,2:30) = deltaBio(xcoor,ycoor,1:29)
  if(bioNew.gt.bioOld) then
    deltaBio(xcoor,ycoor,1) = bioNew-bioOld
  else
    deltaBio(xcoor,ycoor,1) = rnodat
  end if


end subroutine bioKaufmann
