!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Sets up the grid where each cell is assigned an ecoregion/catchment 
! combination (ecoCatchInit), and writes annual outputs (ecoCatchWrite)
!-------------------------------------------------------------------------------


subroutine ecoCatchInit

  use prevahglobals, only: nadgkw, ncol, nrow
  use forhycsglobals, only: ecoc, ecocID, ecocSize
  use MPIvars, only: rank, root, doParallel

  implicit none

  integer :: posi1
  real, allocatable :: dom2d(:,:)
  integer :: necoc, curMin, maxEcoc
  integer :: xcoor, ycoor, count
  integer :: i
  character(len=4) :: checoc
  ! Variables for input handling
  integer :: uwas, nxll, nyll, ngridsize, nodat, rloop
  real :: rcol, rrow, rxll, ryll, rgridsize, rnodat
  real, dimension(6) :: wstats

!======================

  allocate(ecoc(ncol,nrow))
  call posnam(nadgkw,1,posi1)
  uwas = 731
  OPEN (731, file=NADGKW(1:POSI1)//'.ecc', status='old',form='binary')
  INCLUDE 'rwasim.inc'
  INCLUDE 'r2d.inc'
  ecoc = int(dom2d)
  deallocate(dom2d)

! Determine number of ecoregion/catchment combinations
  necoc = 0
  curMin = -1
  maxEcoc = maxval(ecoc)
  do while(curmin .lt. maxEcoc)
    curMin = minval(ecoc, mask = ecoc .gt. curMin)
    necoc = necoc + 1
  end do

  allocate(ecocID(necoc))
  allocate(ecocSize(necoc))
  ecocID(1:necoc) = 0
  ecocSize(1:necoc) = 0

  ! Get numbers (IDs) of all ecoregions
  count = 1
  do ycoor = nrow, 1, -1
  do xcoor = 1,ncol
    if(any(ecocID==ecoc(xcoor,ycoor))) cycle
    !if(stockability(ycoor,xcoor)==0.0) cycle
    !print*, "ecoc", ecoc(xcoor,ycoor), rank
    ecocID(count) = ecoc(xcoor,ycoor)
    count = count + 1
  end do
  end do

  ! Determine number of cells in each ecoregion
  do i = 1,size(ecocID)
    do ycoor = nrow, 1, -1
    do xcoor = 1,ncol
      !if(stockability(ycoor,xcoor)==0.0) cycle
      if(ecoc(xcoor,ycoor)==ecocID(i)) ecocSize(i) = ecocSize(i) + 1
    end do
    end do
  end do

end subroutine ecoCatchInit



!===============================



subroutine ecoCatchWrite(year)
  use mpi
  use MPIvars, only: rank, root, nrowLocal, firstRow, lastRow,ierr,status, numslaves
  use prevahglobals, only: ncol
  use forhycsglobals, only: ecoc, ecocID, ecocSize,TMdir
  use All_par, only: cell,maxhc

  implicit none


  integer, intent(in) :: year
  integer :: i,j
  integer :: xcoor,ycoor
  integer :: spc,height
  real, dimension(30,0:maxhc) :: locavg, globavg, celltemp
  integer :: lncells, gncells
  integer,dimension(numslaves) :: lncellsArray
  real :: prop
  character(len=5) :: checoc
  character(len=4) :: chyear
  integer :: nhcls
  integer, parameter, dimension(16) :: hcinds = (/0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15/)
  character(len=7) :: charspc
  integer, parameter :: nspc=30



!====================

  do i=1,size(ecocID)
    if(rank .ne. root) then
      ! Number of cells of each process corresponding to the current eceCatchment
      lncells = count(ecoc(:,firstRow:lastRow)==ecocID(i))

      ! Send lncells to master
      call MPI_SEND(lncells,1,MPI_INTEGER,root,21,MPI_COMM_WORLD,ierr)

      locavg = 0.0

      ! Loop over cells, calculate average for all cells of the current ecoC
      do ycoor=lastRow,firstRow,-1
      do xcoor=1,ncol
        if(ecoc(xcoor,ycoor).ne.ecocID(i)) cycle
          do spc=1,nspc
          do height=0,maxhc
            locavg(spc,height) = locavg(spc,height) + (cell(ycoor,xcoor)%sp(spc)%numin(height) / real(lncells))
          end do
          end do
      end do
      end do

      ! Send array to master
      call MPI_SEND(locavg,(nspc*(maxhc+1)),MPI_REAL,root,22,MPI_COMM_WORLD,ierr)

    else  ! Master
      lncellsArray = 0
      do j=1,numslaves
        call MPI_RECV(lncells,1,MPI_INTEGER,j,21,MPI_COMM_WORLD,status,ierr)
        lnCellsArray(j) = lncells
      end do

      globavg = 0.0

      do j=1,numslaves
        prop = real(lncellsArray(j))/sum(real(lncellsArray))  ! Proportion of each slave to the cells of the current ecoC
        call MPI_RECV(locavg,(nspc*(maxhc+1)),MPI_REAL,j,22,MPI_COMM_WORLD,status,ierr)
        globavg = globavg + (locavg*prop)
      end do


      ! Write file
      nhcls = maxhc+1
      charspc="Species"
      write(checoc,"(I5.5)") ecocID(i)
      write(chyear,"(I4.4)") year
      open(unit=225,file=trim(TMDir)//'dist/hspc_'//checoc//'_'//chyear//'.txt',status="new")
      write(225,'(A7,X,<nhcls>(I2,X))') charspc,hcinds
      do spc=1,nspc
        write(225,'(I2,X,<nhcls>(f12.5,X))') spc, globavg(spc,:)
      end do

      close(225)

    end if  ! Slaves / Master


  end do



end subroutine ecoCatchWrite
