!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Sets up the grid where each cell is assigned an ecoregion/catchment
! combination (ecoGridInit), and writes annual outputs (ecoGridWrite...)
!-------------------------------------------------------------------------------




subroutine ecoGridInit

! Reads in a grid with ecoregions; prepares output files

  use prevahglobals, only: nadgkw, ncol, nrow
  use forhycsglobals, only: ecor, ecorID, ecorSize, hbecor, hlecor,hsecor,hwecor, hmecor, hnecor
  use All_par, only: stockability
  use MPIvars, only: rank, root, doParallel

  implicit none

  integer :: posi1
  real, allocatable :: dom2d(:,:)
  integer :: necor, curMin, maxEcor
  integer :: xcoor, ycoor, count
  integer :: i
  character(len=4) :: checor
  ! Variables for input handling
  integer :: uwas, nxll, nyll, ngridsize, nodat, rloop
  real :: rcol, rrow, rxll, ryll, rgridsize, rnodat
  real, dimension(6) :: wstats

!======================

  allocate(ecor(ncol,nrow))
! Read in ecoregions grid
  call posnam(nadgkw,1,posi1)
  uwas = 731
  OPEN (731, file=NADGKW(1:POSI1)//'.ecr', status='old',form='binary')
  INCLUDE 'rwasim.inc'
  INCLUDE 'r2d.inc'
  ecor = int(dom2d)
  deallocate(dom2d)

! Determine number of ecoregions
  necor = 0
  curMin = -1
  maxEcor = maxval(ecor)
  do while(curmin .lt. maxEcor)
    curMin = minval(ecor, mask = ecor .gt. curMin)
    necor = necor + 1
  end do

  allocate(ecorID(necor))
  allocate(ecorSize(necor))
  ecorID(1:necor) = 0
  ecorSize(1:necor) = 0

  ! Get numbers (IDs) of all ecoregions
  count = 1
  do ycoor = nrow, 1, -1
  do xcoor = 1,ncol
    if(any(ecorID==ecor(xcoor,ycoor))) cycle
    !if(stockability(ycoor,xcoor)==0.0) cycle
    !print*, "ecor", ecor(xcoor,ycoor), rank
    ecorID(count) = ecor(xcoor,ycoor)
    count = count + 1
  end do
  end do

  ! Determine number of cells in each ecoregion
  do i = 1,size(ecorID)
    do ycoor = nrow, 1, -1
    do xcoor = 1,ncol
      !if(stockability(ycoor,xcoor)==0.0) cycle
      if(ecor(xcoor,ycoor)==ecorID(i)) ecorSize(i) = ecorSize(i) + 1
    end do
    end do
  end do

  ! Initializing files for biomass, LAI and stem number (one file per output variable and ecoregion)
  if((rank == root) .or. (doParallel==.FALSE.)) then
  ! Attribute a file handle to each ecoregion
    allocate(hbecor(necor))
    allocate(hlecor(necor))
    allocate(hnecor(necor))
    do i=1,necor
    ! Biomass files
      hbecor(i) = 229 + i  ! File handles starting at 230 because why not.
      write(checor,"(I4.4)") ecorID(i)
      print*, "ecorID(i)", ecorID(i)
      print*, "checor", checor
      open(hbecor(i),file="bio_ecor_"//checor//".txt",status="new")
      write(hbecor(i),"(32(A2,X))") "yr", "Aa","Ld","Pe","Pc","Pm","Ps","Tb",&
      "Ac","Ap","As","Ag","Ai","Av","Bp","Cb","Cs","Ca","Fs","Fe","Pn","Pt",&
      "Qp","Qu","Qr","Sa","So","Sr","Tc","Tp","Us","S"
      close(hbecor(i))

    ! LAI files
      hlecor(i) = 329 + i  ! File handles starting at 330 because why not.
      write(checor,"(I4.4)") ecorID(i)
      open(hlecor(i),file="lai_ecor_"//checor//".txt",status="new")
      write(hlecor(i),"(32(A2,X))") "yr", "Aa","Ld","Pe","Pc","Pm","Ps","Tb",&
      "Ac","Ap","As","Ag","Ai","Av","Bp","Cb","Cs","Ca","Fs","Fe","Pn","Pt",&
      "Qp","Qu","Qr","Sa","So","Sr","Tc","Tp","Us","S"
      close(hlecor(i))

    ! Stem number files
      hnecor(i) = 529 + i  ! File handles starting at 530 because why not.
      write(checor,"(I4.4)") ecorID(i)
      open(hnecor(i),file="stn_ecor_"//checor//".txt",status="new")
      write(hnecor(i),"(32(A2,X))") "yr", "Aa","Ld","Pe","Pc","Pm","Ps","Tb",&
      "Ac","Ap","As","Ag","Ai","Av","Bp","Cb","Cs","Ca","Fs","Fe","Pn","Pt",&
      "Qp","Qu","Qr","Sa","So","Sr","Tc","Tp","Us","S"
      close(hnecor(i))

    ! Water balance files
!       hwecor(i) = 429 + i  ! File handles starting at 430 because why not.
!       write(checor,"(I4.4)") ecorID(i)
!       open(hwecor(i),file="wb_ecor_"//checor//".txt",status="new")
!       write(hwecor(i),"(7(A2,X))") "yr", "P", "Tpot", "Tact", "Esoil", "EInt", "R"
!       close(hwecor(i))
    end do

  ! Initializing file for soil depth (single file for all ecoregions)
    hsecor = 430 ! Why not
    open(hsecor,file="sd_ecor.txt",status="new")
    write(hsecor,"(A2,X,<necor>(I4,2X))") "yr", ecorID
    close(hsecor)

  ! Initializing file for de Martonne Index (single file for all ecoregions)
    hsecor = 429 ! Why not
    open(hsecor,file="dmi_ecor.txt",status="new")
    write(hsecor,"(A2,X,<necor>(I4,2X))") "yr", ecorID
    close(hsecor)

    allocate(hwecor(6))
    ! Initializing water files
    hwecor(1) = 431
    open(hwecor(1),file="prec_ecor.txt",status="new")
    write(hwecor(1),"(A2,X,<necor>(I4,2X))") "yr", ecorID
    close(hwecor(1))

    hwecor(2) = 432
    open(hwecor(2),file="tpot_ecor.txt",status="new")
    write(hwecor(2),"(A2,X,<necor>(I4,2X))") "yr", ecorID
    close(hwecor(2))

    hwecor(3) = 433
    open(hwecor(3),file="tact_ecor.txt",status="new")
    write(hwecor(3),"(A2,X,<necor>(I4,2X))") "yr", ecorID
    close(hwecor(3))

    hwecor(4) = 434
    open(hwecor(4),file="esoil_ecor.txt",status="new")
    write(hwecor(4),"(A2,X,<necor>(I4,2X))") "yr", ecorID
    close(hwecor(4))

    hwecor(5) = 435
    open(hwecor(5),file="eint_ecor.txt",status="new")
    write(hwecor(5),"(A2,X,<necor>(I4,2X))") "yr", ecorID
    close(hwecor(5))

    hwecor(6) = 436
    open(hwecor(6),file="runoff_ecor.txt",status="new")
    write(hwecor(6),"(A2,X,<necor>(I4,2X))") "yr", ecorID
    close(hwecor(6))

  end if  ! Master


end subroutine ecoGridInit

!===============================

subroutine ecoGridWrite(year)

  use mpi
  use MPIvars, only: rank, root, nrowLocal, firstRow, lastRow,ierr,status, numslaves
  use forhycsglobals, only: ecor, ecorID, ecorSize, hbecor, hlecor, biomass2d, stnum2d, &
   fhla, sfcMax, hsecor, hmecor, hnecor, gsDMI
  use prevahglobals, only: ncol, rgridsize, sfcG
  use All_par, only: stockability

  implicit none

  integer, intent(in) :: year
  integer :: i, j, spc
  integer :: xcoor, ycoor
  real, allocatable :: locRSum(:,:), globRSum(:,:)
  real, allocatable :: locSsum(:), globSsum(:)
  real, allocatable :: rgTemp(:,:,:)
  real, allocatable :: rsTemp(:,:)
  integer, allocatable :: ecorTemp(:,:)
  real :: tempsum, allsum
  character(len=4) :: checor
  real, allocatable :: lncells(:), gncells(:)


!------------------------
! Local and global sums of BIOMASS per species and ecoregion
  allocate(locRSum(30,size(ecorID)))
  locRSum = 0.0

  allocate(lncells(size(ecorID)))
  lncells = 0.0

  if(rank .ne. root) then
    allocate(rgTemp(ncol,nrowLocal,30))
    allocate(ecorTemp(ncol,nrowLocal))
    rgTemp = biomass2d(:,firstRow:lastRow,1:30)
    ecorTemp = ecor(:,firstRow:lastRow)
    !print*, "rank, firstrow, lastRow", rank, firstrow, lastRow
    do i =1,size(ecorID)
    do spc = 1,30
      tempsum = 0.0
      do ycoor = nrowLocal, 1, -1 !lastRow, firstRow, -1
      do xcoor = 1, ncol
        !if(stockability(ycoor,xcoor)==0.0) cycle
        if(rgTemp(xcoor,ycoor,spc) < 0.0) cycle
        if((spc==2).and.(ecorTemp(xcoor,ycoor) == ecorID(i))) lncells(i) = lncells(i) + 1
        if(ecorTemp(xcoor,ycoor) == ecorID(i)) tempsum = tempsum + rgTemp(xcoor,ycoor,spc)
      end do
      end do
      locRSum(spc,i) = tempsum
    end do
    end do
    !print*, "Biomass - lncells", rank, lncells
    call MPI_SEND(lncells,size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
    call MPI_SEND(locRSum,30*size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
  else ! Master
    allocate(globRSum(30,size(ecorID)))
    globRSum = 0.0

    allocate(gncells(size(ecorID)))
    gncells = 0.0
    do j = 1,numslaves
      call MPI_RECV(lncells,size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      gncells = gncells + lncells
    end do
   ! print*, "Biomass - gncells", gncells
    ! Get biomass sums from each slave
    do j = 1,numslaves
      call MPI_RECV(locRSum,30*size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      globRSum = globRSum + locRSum
    end do
    ! Transform sums into average and write to file
    do i = 1,size(ecorID)
      globRSum(:,i) = globRSum(:,i) / gncells(i)
      allsum = sum(globRSum(:,i))
      write(checor,"(I4.4)") ecorID(i)
      open(hbecor(i),file="bio_ecor_"//checor//".txt",status="old",position="append")
      write(hbecor(i),"(I4,X,31(F8.2,X))") year, globRSum(:,i), allsum
      close(hbecor(i))
    end do

  end if

  deallocate(locRSum)
  deallocate(lncells)
  if(allocated(globRSum)) deallocate(globRSum)
  if(allocated(gncells)) deallocate(gncells)
  if(allocated(rgTemp)) deallocate(rgTemp)
  if(allocated(ecorTemp)) deallocate(ecorTemp)
!------------------------
! Local and global averages of LAI per species and ecoregion
  allocate(locRSum(30,size(ecorID)))
  locRSum = 0.0

  allocate(lncells(size(ecorID)))
  lncells = 0.0

  if(rank .ne. root) then
    allocate(rgTemp(ncol,nrowLocal,30))
    allocate(ecorTemp(ncol,nrowLocal))
    rgTemp = fhla(:,firstRow:lastRow,1:30)/833.
    ecorTemp = ecor(:,firstRow:lastRow)
    !print*, "rank, firstrow, lastRow", rank, firstrow, lastRow
    do i =1,size(ecorID)      ! Loop over ecoregions
    do spc = 1,30             ! Loop over species
      tempsum = 0.0
      do ycoor = nrowLocal, 1, -1 !lastRow, firstRow, -1
      do xcoor = 1, ncol
        !if(stockability(ycoor,xcoor)==0.0) cycle
        if(rgTemp(xcoor,ycoor,spc) < 0.0) cycle
        if((spc==2).and.(ecorTemp(xcoor,ycoor) == ecorID(i))) lncells(i) = lncells(i) + 1
        if(ecorTemp(xcoor,ycoor) == ecorID(i)) tempsum = tempsum + rgTemp(xcoor,ycoor,spc)
      end do
      end do
      locRSum(spc,i) = tempsum
    end do
    end do
    !print*, "lncells", rank, lncells
    call MPI_SEND(lncells,size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
    call MPI_SEND(locRSum,30*size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
  else ! Master
    allocate(globRSum(30,size(ecorID)))
    globRSum = 0.0

    allocate(gncells(size(ecorID)))
    gncells = 0.0
    do j = 1,numslaves
      call MPI_RECV(lncells,size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      gncells = gncells + lncells
    end do
    !print*, "gncells", gncells
    ! Get LAI sums from each slave
    do j = 1,numslaves
      call MPI_RECV(locRSum,30*size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      globRSum = globRSum + locRSum
    end do
    ! Transform sums into average and write to file
    do i = 1,size(ecorID)
      globRSum(:,i) = globRSum(:,i) / gncells(i)
      allsum = sum(globRSum(:,i))
      write(checor,"(I4.4)") ecorID(i)
      open(hlecor(i),file="lai_ecor_"//checor//".txt",status="old",position="append")
      write(hlecor(i),"(I4,X,31(F8.2,X))") year, globRSum(:,i), allsum
      close(hlecor(i))
    end do

  end if

  deallocate(locRSum)
  deallocate(lncells)
  if(allocated(globRSum)) deallocate(globRSum)
  if(allocated(gncells)) deallocate(gncells)
  if(allocated(rgTemp)) deallocate(rgTemp)
  if(allocated(ecorTemp)) deallocate(ecorTemp)

!------------------------
! Local and global averages of STEM NUMBER per species and ecoregion
  allocate(locRSum(30,size(ecorID)))
  locRSum = 0.0

  allocate(lncells(size(ecorID)))
  lncells = 0.0

  if(rank .ne. root) then
    allocate(rgTemp(ncol,nrowLocal,30))
    allocate(ecorTemp(ncol,nrowLocal))
    rgTemp = stnum2d(:,firstRow:lastRow,1:30)
    ecorTemp = ecor(:,firstRow:lastRow)
    !print*, "rank, firstrow, lastRow", rank, firstrow, lastRow
    do i =1,size(ecorID)      ! Loop over ecoregions
    do spc = 1,30             ! Loop over species
      tempsum = 0.0
      do ycoor = nrowLocal, 1, -1 !lastRow, firstRow, -1
      do xcoor = 1, ncol
        !if(stockability(ycoor,xcoor)==0.0) cycle
        if(rgTemp(xcoor,ycoor,spc) < 0.0) cycle
        if((spc==2).and.(ecorTemp(xcoor,ycoor) == ecorID(i))) lncells(i) = lncells(i) + 1
        if(ecorTemp(xcoor,ycoor) == ecorID(i)) tempsum = tempsum + rgTemp(xcoor,ycoor,spc)
      end do
      end do
      locRSum(spc,i) = tempsum
    end do
    end do
    !print*, "lncells", rank, lncells
    call MPI_SEND(lncells,size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
    call MPI_SEND(locRSum,30*size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
  else ! Master
    allocate(globRSum(30,size(ecorID)))
    globRSum = 0.0

    allocate(gncells(size(ecorID)))
    gncells = 0.0
    do j = 1,numslaves
      call MPI_RECV(lncells,size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      gncells = gncells + lncells
    end do
    !print*, "gncells", gncells
    ! Get stem number sums from each slave
    do j = 1,numslaves
      call MPI_RECV(locRSum,30*size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      globRSum = globRSum + locRSum
    end do
    ! Transform sums into average and write to file
    do i = 1,size(ecorID)
      globRSum(:,i) = globRSum(:,i) / gncells(i)
      allsum = sum(globRSum(:,i))
      write(checor,"(I4.4)") ecorID(i)
      open(hnecor(i),file="stn_ecor_"//checor//".txt",status="old",position="append")
      write(hnecor(i),"(I4,X,31(F8.2,X))") year, globRSum(:,i), allsum
      close(hnecor(i))
    end do

  end if

  deallocate(locRSum)
  deallocate(lncells)
  if(allocated(globRSum)) deallocate(globRSum)
  if(allocated(gncells)) deallocate(gncells)
  if(allocated(rgTemp)) deallocate(rgTemp)
  if(allocated(ecorTemp)) deallocate(ecorTemp)



! Average (ROOTING DEPTH) SFC in each ecoregion
  allocate(locSsum(size(ecorID)))
  locSSum = 0.0

  allocate(lncells(size(ecorID)))
  lncells = 0.0

  if(rank .ne. root) then
    allocate(rsTemp(ncol,nrowLocal))
    allocate(ecorTemp(ncol,nrowLocal))
    rsTemp = sfcG(:,firstRow:lastRow)!/(1000*sfcMax(:,firstRow:lastRow))
    ecorTemp = ecor(:,firstRow:lastRow)
    do i =1,size(ecorID)      ! Loop over ecoregions
      tempsum = 0.0
      do ycoor = nrowLocal, 1, -1 !lastRow, firstRow, -1
      do xcoor = 1, ncol
        !if(stockability(ycoor,xcoor)==0.0) cycle
        if(rsTemp(xcoor,ycoor) < 0.0) cycle
        if(ecorTemp(xcoor,ycoor) == ecorID(i)) then
          lncells(i) = lncells(i) + 1
          tempsum = tempsum + rsTemp(xcoor,ycoor)
        end if
      end do
      end do
      locSSum(i) = tempsum
    end do
!     print*, rank, "rsTemp", rsTemp(1:10,1:10)
!     print*, rank, "ecorTemp",ecorTemp(1:10,1:10)
!     print*, rank, "sfcG",sfcG(1:10,firstrow:(firstrow+10))
!     print*, rank, "sfcMax", (1000*sfcMax(1:10,firstrow:(firstrow+10)))
!     print*, "locssum", locssum, rank
    call MPI_SEND(lncells,size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
    call MPI_SEND(locSSum,size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)

  else  ! Master
    allocate(globSSum(size(ecorID)))
    globSSum = 0.0

    allocate(gncells(size(ecorID)))
    gncells = 0.0
    do j = 1,numslaves
      call MPI_RECV(lncells,size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      gncells = gncells + lncells
    end do
    !print*, "SFC - gncells", gncells
    ! Get SFCr sums from each slave
    do j = 1,numslaves
      call MPI_RECV(locSSum,size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      globSSum = globSSum + locSSum
    end do
    !print*, "SFC - globssum", globssum
    globSsum = globSsum/ecorSize

    ! Write to file
    open(hsecor,file="sd_ecor.txt",status="old",position="append")
    write(hsecor,"(I4,X,<size(ecorID)>(F8.2,X))") year, globSSum
    close(hsecor)

  end if  ! Master / Slaves


! Average DE MARTONNE INDEX in each ecoregion
  locSSum = 0.0
  lncells = 0.0

  if(rank .ne. root) then
    !allocate(rsTemp(ncol,nrowLocal))
    !allocate(ecorTemp(ncol,nrowLocal))
    rsTemp = gsDMI(:,firstRow:lastRow)
    ecorTemp = ecor(:,firstRow:lastRow)
    do i =1,size(ecorID)      ! Loop over ecoregions
      tempsum = 0.0
      do ycoor = nrowLocal, 1, -1 !lastRow, firstRow, -1
      do xcoor = 1, ncol
        !if(stockability(ycoor,xcoor)==0.0) cycle
        !if(rsTemp(xcoor,ycoor) < 0.0) cycle
        if(ecorTemp(xcoor,ycoor) == ecorID(i)) then
          lncells(i) = lncells(i) + 1
          tempsum = tempsum + rsTemp(xcoor,ycoor)
        end if
      end do
      end do
      locSSum(i) = tempsum
    end do
!     print*, rank, "rsTemp", rsTemp(1:10,1:10)
!     print*, rank, "ecorTemp",ecorTemp(1:10,1:10)
!     print*, "locssum", locssum, rank
    call MPI_SEND(lncells,size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
    call MPI_SEND(locSSum,size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)

  else  ! Master
    !allocate(globSSum(size(ecorID)))
    globSSum = 0.0

    !allocate(gncells(size(ecorID)))
    gncells = 0.0
    do j = 1,numslaves
      call MPI_RECV(lncells,size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      gncells = gncells + lncells
    end do
    !print*, "gncells", gncells
    ! Get SFCr sums from each slave
    do j = 1,numslaves
      call MPI_RECV(locSSum,size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      globSSum = globSSum + locSSum
    end do
    !print*, "globssum", globssum
    globSsum = globSsum/ecorSize

    ! Write to file
    open(hmecor,file="dmi_ecor.txt",status="old",position="append")
    write(hmecor,"(I4,X,<size(ecorID)>(F8.2,X))") year, globSSum
    close(hmecor)

  end if  ! Master / Slaves

end subroutine ecoGridWrite

!=======================================================================

subroutine ecoWBWrite(year)
! Writes annual water balance components for each ecoregion

  use mpi
  use MPIvars, only: rank, root, nrowLocal, firstRow, lastRow,ierr,status, numslaves
  use forhycsglobals, only: ecor, ecorID, ecorSize, hwecor!, checor
  use prevahglobals, only: ncol, rgridsize, pakJS,peTransJS,eTransJS,eSoilJS,eIntJS,eSnowJS,rgeJS
  use All_par, only: stockability

  implicit none

  integer, intent(in) :: year
  integer :: i, j
  integer :: xcoor, ycoor
  real, allocatable :: locRSum(:,:), globRSum(:,:)
  real, allocatable :: locSsum(:), globSsum(:)
  real, allocatable :: rsTemp(:,:)
  integer, allocatable :: ecorTemp(:,:)
  real :: tempsum, allsum
  character(len=4) :: checor
  real, allocatable :: lncells(:), gncells(:)
  integer :: var

!------------------------

  if(rank.ne.root) then
    allocate(locSsum(size(ecorID)))
    locSSum = 0.0
    allocate(lncells(size(ecorID)))
    lncells = 0.0
    allocate(rsTemp(ncol,nrowLocal))
    allocate(ecorTemp(ncol,nrowLocal))
  else
    allocate(locSsum(size(ecorID)))
    locSSum = 0.0
    allocate(lncells(size(ecorID)))
    lncells = 0.0
    allocate(globSSum(size(ecorID)))
    globSSum = 0.0
    allocate(gncells(size(ecorID)))
    gncells = 0.0
  end if

  do var=1,6

  if(rank .ne. root) then

    select case(var)
    case(1)  ! Precipitation
      rsTemp = pakJS(:,firstRow:lastRow)
    case(2)  ! Potential transpiration
      rsTemp = peTransJS(:,firstRow:lastRow)
    case(3)  ! Actual transpiration
      rsTemp = eTransJS(:,firstRow:lastRow)
    case(4)  ! Soil evaporation
      rsTemp = esoilJS(:,firstRow:lastRow)
    case(5)  ! Interception (+ snow sublimation)
      rsTemp = eIntJS(:,firstRow:lastRow) + eSnowJS(:,firstRow:lastRow)
    case(6)  ! Total runoff
      rsTemp = rgeJS(:,firstRow:lastRow)
    end select
    ecorTemp = ecor(:,firstRow:lastRow)
    do i =1,size(ecorID)      ! Loop over ecoregions
      tempsum = 0.0
      do ycoor = nrowLocal, 1, -1 !lastRow, firstRow, -1
      do xcoor = 1, ncol
        !if(stockability(ycoor,xcoor)==0.0) cycle
        if(rsTemp(xcoor,ycoor) < 0.0) cycle
        if(ecorTemp(xcoor,ycoor) == ecorID(i)) then
          lncells(i) = lncells(i) + 1
          tempsum = tempsum + rsTemp(xcoor,ycoor)
        end if
      end do
      end do
      locSSum(i) = tempsum
    end do
    call MPI_SEND(lncells,size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)
    call MPI_SEND(locSSum,size(ecorID),MPI_REAL,root,21,MPI_COMM_WORLD,ierr)

  else  ! Master

    do j = 1,numslaves
      call MPI_RECV(lncells,size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      gncells = gncells + lncells
    end do
    !print*, "gncells", gncells
    ! Get SFCr sums from each slave
    do j = 1,numslaves
      call MPI_RECV(locSSum,size(ecorID),MPI_REAL,j,21,MPI_COMM_WORLD,status,ierr)
      globSSum = globSSum + locSSum
    end do
    !print*, "globssum", globssum
    globSsum = globSsum/ecorSize

    ! Write to file
    select case(var)
    case(1)
      open(hwecor(var),file="prec_ecor.txt",status="old",position="append")
      write(hwecor(var),"(I4,X,<size(ecorID)>(F8.2,X))") year, globSSum
      close(hwecor(var))
    case(2)
      open(hwecor(var),file="tpot_ecor.txt",status="old",position="append")
      write(hwecor(var),"(I4,X,<size(ecorID)>(F8.2,X))") year, globSSum
      close(hwecor(var))
    case(3)
      open(hwecor(var),file="tact_ecor.txt",status="old",position="append")
      write(hwecor(var),"(I4,X,<size(ecorID)>(F8.2,X))") year, globSSum
      close(hwecor(var))
    case(4)
      open(hwecor(var),file="esoil_ecor.txt",status="old",position="append")
      write(hwecor(var),"(I4,X,<size(ecorID)>(F8.2,X))") year, globSSum
      close(hwecor(var))
    case(5)
      open(hwecor(var),file="eint_ecor.txt",status="old",position="append")
      write(hwecor(var),"(I4,X,<size(ecorID)>(F8.2,X))") year, globSSum
      close(hwecor(var))
    case(6)
      open(hwecor(var),file="runoff_ecor.txt",status="old",position="append")
      write(hwecor(var),"(I4,X,<size(ecorID)>(F8.2,X))") year, globSSum
      close(hwecor(var))

    end select

  end if  ! Master / Slaves

  end do

end subroutine ecoWBWrite
