!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Calculates above- and within-canopy resistances, including transformation of wind speed data
! (interpolated from meteo stations) to levels at canopy height.

! Reference:
!
! Zhou et al. (2006): Estimating potential evapotranspiration using Shuttleworth–Wallace model
! and NOAA-AVHRR NDVI data to feed a distributed hydrological model over the Mekong River basin.
! J. Hydrol. 317(1-2): 151-173
! 10.1016/j.jhydrol.2005.11.013
!-------------------------------------------------------------------------------

subroutine eraeroCan(u2,lai,fvc,h,land,raa,rca)

  implicit none

  ! Passed variables
  real, intent(out) :: raa,rca
  real,intent(in) ::    u2, &       ! Wind speed interpolated from measurements (measurement height 2m)
                        lai, &      ! Effective LAI
                        fvc, &      ! Fractional vegetation (tree) cover
                        h           ! Canopy height [m]
  integer, intent(in) :: land

  ! Internal variables
  real ::   z,&           ! Reference height above canopy [m]
            zm,&              ! Vegetation roughness length governing momentum transfer [m]
            zh,&              ! Vegetation roughness length governing transfer of heat and vapor [m]
            d,&               ! Zero plane displacement height [m]
            uz, &             ! Wind speed at height z
            uh,&              ! Wind speed at canopy height
            lw                ! Characteristic leaf width [m]
  real :: raaF,raaZ           ! raa at full and zero vegetation cover

  real, parameter :: zmS = 0.01
  real, parameter :: zhS = 0.01
  real, parameter :: zb = 296.97    ! Height of internal boundary layer, for conversion of wind speeds
  real, parameter :: zow = 0.005    ! Assumed roughness length at weather stations
  real, parameter :: k = 0.4        ! von Kármán's constant
  real, parameter :: b = 0.01       ! Proportionality constant for rca

! ===============================

  ! Set aerodynamic parameters as a function of canopy height
  z = h + 2.0
  zm = 0.1*h
  zh = 0.1*zm
  d = 0.6*h

  ! Characteristic leaf width
  select case (land)
    case(3)
      lw = 0.001

    case(4)
      lw = 0.05

    case(5,8,21,24,25,26)
      lw = 0.01
  end select

  ! Get wind at reference level and canopy level
  ! See Zhou et al. 2006, eqs. 27 and 28
  uz = u2*(log(zb/zow)/log(zb/zm))*(log((z-d)/zm)/log(2./zow))
  uh = u2*(log(zb/zow)/log(zb/zm))*(log((h-d)/zm)/log(2./zow))

  ! Above-canopy aerodynamic resistance
  raaF = (log((z-d)/zm)*log((z-d)/zh))/((k**2)*uz)
  raaZ = ((log(z/zmS)*log(z/zhS))/((k**2)*uz) - ((log(h/zmS)*log(h/zhS))/((k**2)*uz)))
  if(lai .ge. 4) then  ! Interpolation à la Shuttleworth-Wallace
    raa = raaF
  else if (lai == 0) then
    raa = raaZ
  else
    raa = 0.25*lai*raaF + 0.25*(4-lai)*raaZ
  end if

  ! Intra-canopy aerodynamic resistance
  rca = 1/(b*sqrt(uh/lw))

end subroutine eraeroCan
