!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE ESCIMO1 (PRECIP,TEMP,WIND,HUM,RGL,
     *              IR,SWE,PHASE,ENERGY,MELT,DMASLAT,SURFTEMPOLD,
     *              SNOWAGE,A0,ABKLINGPOS,ABKLINGNEG,ALBEDO,ALBADD)

! ESCIMO 2.0 - A snow model developed by U. Strasser 2001 (IHW/ETH Zuerich)
!                            Strasser@ihw.baug.ethz.ch
!                         Version 1.0:  August 1997  (IGGF/LMU Muenchen)
!                         Version 2.0:    July 2000  (CEN/CNRM Grenoble)
!       SR-Version vom 8. 02. 2001 fuer PREVAH by Gurtz, Pos und Zappa
!     ESCIMO computes snow accumulation and snowmelt of snow cover
!                under consideration of its energy balance
! Publ: Zappa, Pos, Strasser et al. Nordic Hydrology Vol 34(3) 2003
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------
      REAL PRECIP,TEMP,WIND,HUM
      REAL RGL,IR,SURFTEMPOLD
      REAL SNOWAGE,ALBEDO,PHASE
      REAL SWE,ENERGY,MELT
      REAL ALBADD, ABKLINGPOS, ABKLINGNEG,A0
      REAL DMASLAT

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------
      REAL AIRTEMP,SHORT,LONG,INTERVALL
      REAL SURFTEMP,PRESAIR,PRESSNOW
      REAL JOULE,MELTENERGY
      REAL SENSFLUX,LATFLUX,ALPHA,EMISSION,RAINEN
      REAL SNOWT, SOILFLUX, EMISSIVITY
      REAL ALBMIN
      REAL SPEWSNOW, SPEWAS, SCHMELWAS, STEFBOLTZ
      REAL DENSITY, THERMCOND, CONSUBSNOW

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

! ---------------------------------------------------------------------
! Variables and Parameters for ESCIMO in PREVAH
! ---------------------------------------------------------------------
! PRECIP                 !Presipitation in mm/h
! PHASE                  !0=Rain, 1=Snow
! TEMP                   !Air temperature in deg C
! AIRTEMP                !Air temperature TEMP+273.16
! WIND                   !Windg speed in m/s
! RGL                    !Global radiation in W/m2
! IR                     !Longwave radiation in W/m2
! HUM                    !Relative Humidity (0.0...1.0)
! SWE                    !Snow water equivalent
! SHORT                  !Short-wave radiation balance
! LONG                   !Lon wave radiation balance
! SURFTEMP               !Temperature of snow surface
! SURFTEMPOLD            !Temperature of snow surface in previous time step
! PRESAIR                !Vapour Pressure at plot height
! PRESSNOW               !Vapour Pressure above snow
! SNOWAGE                !Snow age
! ALBEDO                 !Snow albedo
! JOULE                  !Energy balance of snow cover
! MELTENERGY             !required energy for melting
! SENSFLUX               !sensible heat
! LATFLUX                !latent heat
! ALPHA                  !Heat transfer number
! EMISSION               !long-wave emission of snow
! ENERGY                 !snow energy
! MELT                   !snow melt
! SNOWT                  !snow cover temperature(deg C)
! DMASLAT                !mass change owed to latent heat (mm)
! SOILFLUX   =     2.0   !ground heat flux    (W/m2)
! EMISSIVITY =     1.0   !Emissivity of snow
! ALBMIN     =     0.40  !Minimal Albedo of snow
! ALBADD     =     0.44  !Addition parameter for Albedo
! ABKLINGPOS =     0.10  !ageing parameter for albeto at positive air temperature
! ABKLINGNEG =     0.05  !ageing parameter for albeto at negative air temperature
! SPEWSNOW   =   1840.0  !Spec. heat of snow  (J/kg�K), after Geiger
! SPEWAS     =   4200.0  !Spec. heat of water (J/kg�K)
! SCHMELWAS  = 337000.0  !Melting heat        (J/kg)
! STEFBOLTZ  =  5.67E-8  !Stefan-Boltzmann-constant (W/(m2*�K**4))
! DENSITY    =    800.0  !density of snow surface (for computation of temperature)
! THERMCOND  =    1.676  !Therm. conductivity (for computation of temperature)
! CONSUBSNOW =2836000.0  !Condens./Sublim.-Heat for snow

! ---------------------------------------------------------------------
! Variables and parameters are initialized
! ---------------------------------------------------------------------

      AIRTEMP=TEMP+273.16
	IF (SWE .EQ. 0) SURFTEMPOLD=AIRTEMP

      INTERVALL  =  3600.0

      SOILFLUX   =     2.0
      EMISSIVITY =     1.0
      ALBMIN     =     0.40
!      ALBADD     =     0.50
!      ABKLINGPOS =     0.10
!      ABKLINGNEG =     0.05
      SPEWSNOW   =   1840.0
      SPEWAS     =   4200.0
      SCHMELWAS  = 337000.0
      STEFBOLTZ  =  5.67E-8
      DENSITY    =    500.0
      THERMCOND  =    1.676
      CONSUBSNOW =2838000.0

	RAINEN     = 0.0
      MELT       = 0.0
      SHORT      = 0.0
	ALBEDO     = ALBMIN
	JOULE      = 0.0
      MELTENERGY = 0.0
	ALPHA      = 0.0
	SENSFLUX   = 0.0
	LATFLUX    = 0.0
	PRESAIR    = 0.0
	PRESSNOW   = 0.0

! ---------------------------------------------------------------------
!  Computation of snow surface temperature, combined approach for negative
!  air temperature
      IF(AIRTEMP .LT. 273.16) THEN
!           !Ambach/Vetter!
           SURFTEMP = SURFTEMPOLD
        ELSE    !positive air temperature
           SURFTEMP = 273.16
      ENDIF
! ---------------------------------------------------------------------
! 1.radiation budget of snow cover
!
!     Heat transfer number Alpha: shouldn't turn negative
      IF (WIND.GT.0.1069498) THEN
          ALPHA = 4.4602 * LOG(WIND) + 9.97031
!        !fit taken from literature
      ENDIF

!     Better after Escher-Vetter (1980) from observed wind profiles:
      ALPHA = 5.7 * SQRT(WIND+0.00001)
!        Sensible heat flux:
      SENSFLUX = ALPHA * (AIRTEMP - SURFTEMP)


! ---------------------------------------------------------------------
! Computation of water vapour pressure (mb) at plot height with Magnus-Formel
      PRESAIR  = 6.1078 * EXP((17.08085*( AIRTEMP-273.16))/
     *                   (234.175+(AIRTEMP-273.16))) * (HUM)  !/100.
! Computation of water vapour pressure (mb) at the snow surface
!                (saturated vapour pressure at SURFTEMP)
!      write (7841,*) surftemp
      surftemp=max(100., surftemp)
      PRESSNOW = 6.1078 * EXP((17.08085*(SURFTEMP-273.16))/
     *                   (234.175+(SURFTEMP-273.16)))
! ---------------------------------------------------------------------
! Latent heat flux from condensation/sublimation either after
!     Kuchment/Gelfan, and Kuz'min (J.Hydrol.1996)
      LATFLUX = 32.82 * (0.18 + 0.098*WIND) * (PRESAIR - PRESSNOW)
! ...or after Bloeschl/Kirnbauer/Gutknecht (DGM,31,1987,H.5)
!      LATFLUX = (3.0 + 3.6*WIND**0.7) * ((2831.0*0.623)/ (1.005*1000.0))
!     *                                * (PRESAIR - PRESSNOW)
! Latent fluxes cause a positive mass change (condensation), or
! a negative mass change (sublimation)
      DMASLAT = (LATFLUX*INTERVALL) / CONSUBSNOW

! ---------------------------------------------------------------------
! Albedo of snow cover after Rohrer
!        (original parameterization 0.12 bzw. 0.05)
      IF (SURFTEMP.GE.273.16) THEN
          ALBEDO = ALBMIN + ALBADD * EXP(-ABKLINGPOS * SNOWAGE)
        ELSE
          ALBEDO = ALBMIN + ALBADD * EXP(-ABKLINGNEG * SNOWAGE)
      ENDIF

! ---------------------------------------------------------------------
! short wave radiation budget:
      SHORT    = (1. - ALBEDO) * RGL
! long wave emission of snow:
      EMISSION = - STEFBOLTZ * EMISSIVITY * SURFTEMP**4
      LONG     = IR !+ EMISSION

! ---------------------------------------------------------------------
! total radiation balance for the time step
      JOULE = (SENSFLUX + LATFLUX + SHORT + LONG + SOILFLUX) * INTERVALL
! ---------------------------------------------------------------------

! 2.Phase of precipitation
! parametrized in the main program of PREVAH
! ---------------------------------------------------------------------

! 3.computation of the snow water equivalent
      SWE = SWE + PRECIP + DMASLAT
!      Update of the energy balance through the contribution of condensation/sublimation
      ENERGY = ENERGY + (273.16 * SPEWSNOW * DMASLAT)

! ---------------------------------------------------------------------

! 4.computation of the energy balance = energy of last time step
!     + new energy, according to precipitation and phase of precipitation
!        !no precipitation
      IF (PRECIP.EQ.0.0) THEN
!          !only change in radiation balance is relevant
          ENERGY = ENERGY + JOULE
!          !precipitation
        ELSEIF (PRECIP.NE.0.0.AND.PHASE.EQ.0.0) THEN
	    RAINEN = (273.16*SPEWSNOW+SCHMELWAS+SPEWAS*
     *                               (AIRTEMP-273.16))*PRECIP
          ENERGY = ENERGY + JOULE +RAINEN
!          !snowfall
        ELSEIF (PRECIP.NE.0.0.AND.PHASE.EQ.1.0) THEN
!          !if snow intensity > 0.5 mm / h
          IF ((PRECIP/INTERVALL) .GT. (A0/INTERVALL)) THEN
!            !snow albedo put to maximal value
              ALBEDO  = ALBMIN + ALBADD
              SNOWAGE = 0.0        !new initialization of ageing
 	        SHORT   = (1. - ALBEDO) * RGL      !new computation of short wave radiaiton
       JOULE   = (SENSFLUX + LATFLUX + SHORT + LONG + SOILFLUX)*
     * INTERVALL !new computation of energy balance
      ENDIF
      ENERGY    = ENERGY + JOULE + (273.16 * SPEWSNOW * PRECIP)
      ENDIF

! ---------------------------------------------------------------------
! required energy for the initialization of snowmelt:
       MELTENERGY = 273.16 * SPEWSNOW * SWE


! internal energy connot be negative:

      IF (ENERGY < 0.0) ENERGY = 0.0

! ---------------------------------------------------------------------
! Heat-transfer-based computation of snow surface temperature after Ambach (1955)
! and Escher-Vetter (1980)
!        !enough energy
       IF (ENERGY.GE.MELTENERGY) THEN
!         !0 deg ! after melt event
          SURFTEMPOLD = 273.16
!         !no snow melt
         ELSE
          SURFTEMPOLD = SURFTEMP + 2. * (JOULE / INTERVALL) *
     *          SQRT (INTERVALL / (3.14159*THERMCOND*DENSITY*SPEWSNOW))
          IF (SURFTEMPOLD .GT. 273.16) SURFTEMPOLD = 273.16
       ENDIF


! ---------------------------------------------------------------------
! 5.test if there is enogh energy (Wippe):
!       !Energy is enough:

      IF (ENERGY.GT.MELTENERGY) THEN

! ---------------------------------------------------------------------
! 6.computation of snow melt according to the available energy (in mm)
       MELT = (ENERGY - MELTENERGY) / SCHMELWAS

! ---------------------------------------------------------------------
! 7.Computation of the new snow water equivalent
       SWE = SWE - MELT
!      Mass balance cannot turn negative
       IF (SWE.LT.0.0) THEN
           MELT  = MELT + SWE
           SWE   = 0.0
       ENDIF

! ---------------------------------------------------------------------
! 8.computation of new energy balance
       ENERGY = ENERGY - (SPEWSNOW * 273.16 + SCHMELWAS) * MELT
      ENDIF

! ---------------------------------------------------------------------
! actualization/reinitialization of snow age
      IF (SWE.GT.0.0) THEN
      SNOWAGE  = SNOWAGE + (INTERVALL/86400.)
      !computation of average temperature of snow cover
      SNOWT    = (ENERGY / (SWE * SPEWSNOW))
      ELSE     !...snow cover missing...
      ENERGY        = 0.0
      SNOWAGE       = 0.0
      ENDIF

! ---------------------------------------------------------------------

      RETURN

      END
