!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Calculation of the different evaporation fluxes. The process formulations
! differ for different land cover types. Also, another process formulation is
! used if the cell is covered in snow.
!
! Based in part on the PREVAH subroutines EPEMONDT and EPENRESZ
!
!-------------------------------------------------------------------------------

subroutine evap(r,s,t,f,u,elv,hc,lat,day,mon,year,hour,sso,lai,albedo,si,simax,&
   sm,sfc,qg,kurzwsf,langwsf,vfeuchtf,irgl,tsa,tsu,drydays,slope,aspect,&
  land,srsmin,vFrac,peTrans,peSnow,peSoil,peInt,aTrans,aeSoil,pe,stock,xcoor,ycoor)

  use forhycsglobals, only: eeqrmean, pefrmean, grsmin, rsmin2d

  implicit none

! Passed variables
! Output: Potential soil evaporation and transpiration,
!         Evaporation of intercepted water and snow sublimation,
!         Actual soil evaporation and transpiration

  real, intent(out) :: peTrans,peSnow,peSoil,peInt, aTrans,aeSoil, pe
  integer, intent(in) :: land, &  ! Land cover class
                         day, mon, year, hour, &  ! Duh
                         kurzwsf, langwsf, &   ! Control parameters for the estimation of SW and LW radiation in EPENRESZ
                         vfeuchtf, &              ! Control parameter for the aerodynamic term in Penman (EPENRESZ)
                         irgl, &                ! Control parameter for estimation of incoming radiation
                         tsa, tsu, &               ! Time of sunrise and sunset (hours of day)
                         drydays                      ! Days since last rainfall

  integer, intent(in) :: xcoor, ycoor  ! DEBUG ONLY
  real, intent(in) :: r, &          ! Hourly, adjusted global radiation [W m-2]
                      s, &          ! Relative sunshine duration (daily)
                      t, &          ! Air temperature [°C]
                      f, &          ! Relative humidity (0 ... 1)
                      u, &          ! Wind speed at 2 m [m s-1]
                      elv, &        ! Elevation of the grid cell [m asl]
                      hc, &          ! Canopy height [m]
                      lat, &        ! latitude [°]
                      sso, &        ! Snow water equivalent in the cell
                      lai, &        ! Leaf area index
                      vFrac, &      ! Fractional vegetation (tree) cover
                      srsmin, &     ! Minimum canopy resistance (prescribed for non-forest types)
                      albedo, &     ! eh...
                      slope,&       ! Yes
                      aspect,&      ! Indeed
                      si, &         ! Intercepted water
                      simax, &      ! Maximal amout of water that can be stocked in interception storage
                      sm, &        ! Amount of plant available water in SFC [mm]
                      sfc,&           ! Size of plant available water reservoir [mm]
                      qg            ! Ground heat flux (prescribed for non-forested LC types)
  logical, intent(in) :: stock      ! TreeMig stockability

! Internal variables
  !real :: rc      ! Corrected (topography) global radiation
  real :: qs, ql
  real :: nmdays,nydays
  real :: rs
  real :: peiZero
  !real :: rgmax, rgkdm  ! Outputs from ERGKORZ, used to scale radiation
  !integer :: dumtsa, dumtsu   ! Dummy outputs from ERGKORZ (TSA/TSU are already given by RTKORZ)
  !real, dimension(24) :: dumrg      ! Dummy
  !real :: dums0
  real :: gamma  ! Psychrometric "constant"
  real :: del    ! Slope of saturated vapor pressure curve
  real :: tk     ! Temperature expressed in K
  real :: tr      ! Temperature coefficient
  real :: tp      ! Temperature at 0 m asl
  real :: pt      ! Air pressure at current elevation
  real :: l       ! Latent heat for evaporation of water [J kg-1]
  real :: rhoAir  ! Density of air
  real :: x, dx   ! Helper variables for saturation vapor pressure (Richards 1971)
  integer :: i    ! Loop index
  real :: elx     ! Saturation vapor pressure
  real :: el      ! Actual vapor pressure
  real :: extCoeff  ! Extinction coefficient for solar radiation in the canopy, used in Beer-Lambert
  real :: ra      ! Aerodynamic resistance
  !real :: rs      ! Surface / stomatal resistance
  real :: rsmin      ! Minimal stomatal resistance for dominant land cover type

  real :: hRef    ! Screen height of meteorological "measurements"
  real :: clai    ! Clumped LAI for vegetated surfaces

  real :: wetFrac  ! Wet fraction of foliage
! Internal variables for aerodynamic resistances
  real :: raa, rca
  real :: d, zm, zh  ! See eraeroVeg
  real :: alphaL  ! Aerodynamic term in Penman equation (snow and special LC types only)
  real :: tsso    ! Surface temperature of snowpack
  real :: energy  ! Evaporation terms expressed as energy fluxes
  real :: epen    ! Penman evaporation
  real :: awacsol
  real :: cor     ! Isothermal radiance correction term
  real :: ba      ! Helper variable for cor
  real :: ca      ! Atmospheric CO2 concentration [ppm]
  real :: laic    ! Clumped LAI
  real :: Anet, ACan, ASoil, AInt
  real :: num, denom
  real :: fzhang  ! Function of antecedent precipitation and eq. evaporation, see Morillas et al. (2013)

! Variables needed for old PREVAH routines
  integer :: idebug
  integer :: iart
  real :: daylength  ! 0 or 24, to get daytime or nighttime values from erstom

! Constants
  real, parameter :: cp = 1004.0          ! Specific heat at constant pressure [J/kg/K]
  real, parameter :: at = 0.95            ! Reflectance coefficient of the surface for longwave radiation
  real, parameter :: sigma = 5.6703E-8    ! Stefan-Boltzmann constant [W m-2 K-1]
  real, parameter :: a = 3600000.         ! Unit conversion (m/s -> mm/h)
  real, parameter :: zeroK = 273.15       ! 0°C expressed in K
  real, parameter :: hundredK = 373.15    ! 100°C expressed in K
  real, parameter :: pZero = 1013.25      ! Air pressure at 0 m asl
  real, parameter :: albSoil = 0.15       ! Albedo of bare soil, following Kroes et al. 2009
  real, parameter :: gammaT = 0.005       ! Temperature lapse rate [k m-1]
  real, parameter :: g = 9.81             ! Earth acceleration
  real, parameter :: rAir = 287.04        ! Gas constant for dry air
  real, parameter :: rWat = 461.5         ! Gas constant for water vapor
  real, parameter :: rhoWat = 999.941     ! Density of water
  ! H: Helper variable for saturation vapor pressure (Richards 1971)
  real, dimension(4), parameter :: h = (/13.3185,-1.976,-0.6445,-0.1299/)
  real, parameter :: kappa = 15.     ! Characteristic timescale of soil drying, see Brutsaert 2014

  ! Average heat storage of the soil
  real, dimension(12), parameter :: p = &
    (/-137.0, -75.0,  30.0, 167.0, 236.0, 252.0, &
                  213.0,  69.0, -85.0,-206.0,-256.0,-206.0/)

! ---------------------------------------------------------------------


! Computation of saturated and actual vapor pressure, slope of saturated VPD curve
! (common for all fluxes).
! After Richards, J. M. (1971) : A simple expression for the saturation vapour pressure of
! water in the range −50 to 140°C. J. Phys. D: Appl. Phys. 4 L15
! http://iopscience.iop.org/0022-3727/4/4/101/pdf/0022-3727_4_4_101.pdf
    tk = t+zeroK
    tr = 1.0-(hundredK/tk)
    tp = tk+gammaT*elv/2.0
    pt = pZero * exp(-g*elv/rAir/tp)
    l = 2500800*(zeroK/tk)
    gamma = pt*cp/l/(rAir/rWat)
    rhoAir = 100.0*pt/rAir/tk
    x = 0.0
    dx = 0.0
    do i = 1,4
      x = x + h(i)*(tr**i)
      dx = dx + i * h(i)*(tr**(i-1))
    end do
    elx = pZero * exp(x)
    del = hundredK*elx/tk/tk*dx
    el = elx*f

! Computation of available energy (parameterization of SW and LW radiation)
    qs = (1.0 - albedo) * r
    ql = -SIGMA*(TK**4.)*(0.52-(0.065*SQRT(EL)))* (0.23+(0.77*s))  ! Schulla 1997
    Anet = qs+ql


    ! Penman equation for special LC types
    ! Aerodynamic term of Penman equation, following Penman 1954
    ! This is an empirical formula on a daily basis, therefore divide by 24
    alphaL = gamma*rhoWat*l/(a*24)*0.263*(0.5+0.537*u)
    alphaL = alphaL/24.
    energy = ((del*Anet)+(alphaL*elx*(1.-f)))/(gamma+del)
    epen = energy*a/rhoWat/l
    epen = max(epen,0.0)

    pe = epen     ! Output, used in PREVAHCORE to build growing-season sums of PE for rooting depth module



! Special case: presence of snow. In this case, no transpiration or evaporation
! takes place except snow sublimation (Penman evaporation with modified latent heat
! and VPD).

  if(sso .gt. 0.01) then
!     TSSO=T+(7.5*(((1.01-s)**1.15)-0.93))
!     L=2.5008E6*(zeroK/(TK))
!     IF (TSSO .LT. 0.0) L=2.790E6
!     TK=TSSO+zeroK
!     TR=1.-(hundredK/TK)
!     X=0
!     DX=0
!     DO I=1,4
!       X=X+h(I)*(TR**I)
!       DX=DX+I*h(I)*(TR**(I-1))
!     end do
!     ELX=min(pZero*EXP(X),6.108)
!
!     ! Aerodynamic term of Penman equation, following Penman 1954
!     alphaL = gamma*rhoAir*l/a*0.263*(0.5+0.537*u)
!
!     ! Penman equation
!     energy = (del*Anet*alphaL*elx*(1.-f))/(gamma+del)
!     peSnow = energy*a/rhoWat/l
    peSnow = epen  ! For consistency with standard PREVAH
    peTrans = 0.0
    aTrans = 0.0
    peInt = 0.0
    peSoil = 0.0
    aeSoil = 0.0
  else
    peSnow = 0.0

! General case: no snow cover. Type of evaporation calculation depends on land cover type

    !if(stock == .false.) then
    select case (land)

    ! Open water
      case(1)
        peSoil = epen
        aeSoil = epen
        peTrans = 0.0
        aTrans = 0.0
        peInt = 0.0

    ! Built-up
      case(2)
        peSoil = epen
        aeSoil = epen
        peTrans = 0.0
        aTrans = 0.0
        peInt = 0.0

    ! Ice / Firn
      case(13,14)
        peSoil = epen
        aeSoil = epen
        peTrans = 0.0
        aTrans = 0.0
        peInt = 0.0

    ! Bare rock
      case(15)
        peSoil = epen
        aeSoil = epen
        peTrans = 0.0
        aTrans = 0.0
        peInt = 0.0

    ! Wetlands
      case(22)
        peSoil = epen
        aeSoil = epen
        peTrans = 0.0
        aTrans = 0.0
        peInt = 0.0

    ! Agricultural lands (assumed that no forest growth is possible)
    ! Evaporation as in standard PREVAH (see e.g. Viviroli et al. 2007 (PREVAH Manual Part II),
    ! Gurtz et al. 1997 (in German))
      !case(6,7,18,19,20,28,29,21,23,24,25,26)
      case(6,7,18,19,20,28,29)

      ! Aerodynamic resistance
      idebug = 0
      call eraero (u,hc,ra,idebug,land)

      ! Stomatal resistance
      rsmin = srsmin
            ! The subroutine erstom was originally written for daily calculation of surface resistance.
            ! Here (hourly), the variable daylength is set to 24 if the current hour is during the day
            ! and to zero if it is during nighttime.
      if ((hour .lt. tsa) .or. (hour .gt. tsu)) then
        daylength = 0.
      else
        daylength = 24.
      end if
      iart = 2
      idebug = 0
      awacsol = sfc*0.5
      call erstom01 (lai,rsmin,daylength,t,elx,el,IART,LAND,sfc,awacsol,sm,rs,idebug)

      ! Temperature correction
      BA=4.*AT*SIGMA*(TK**3.)
      cor=1.+BA*RA/rhoAir/CP

      ! Evaporation flux
      num = del*(Anet-(Anet*qg))+rhoAir*cp*(elx-el)*cor/ra
      denom = del+gamma*(1+rs/ra)*cor
      energy = num/denom
      peTrans = energy*a/rhoWat/l
      aTrans = peTrans
      peSoil = 0.0
      aeSoil = 0.0
      peInt = 0.0




      !end select
      !else  ! stock == TRUE
    ! Forests, shrubs and potentially forested areas
      case(3,4,5,8,21,23,24,25,26)

        ! Aerodynamic resistance
        call eraeroCan(u,lai,vFrac,hc,land,raa,rca)
        ra = raa + rca

        ! Temperature correction
        BA=4.*AT*SIGMA*(TK**3.)
        cor=1.+BA*RA/rhoAir/CP

        ! Energy split canopy - soil
        extCoeff = 0.5
        ACan = ANet*(1.0-exp(-extCoeff*lai))
        ASoil = ANet*exp(-extCoeff*lai)

        ! Calculation and subtraction of interception evaporation
        wetFrac = (SI/SIMAX)**0.66
        ! TODO: limit EInt to WetFrac???
        !num = del*(max(0.0,ACan)) + vFrac*rhoAir*cp*(elx-el) / ra
        num = del*ACan + vFrac*rhoAir*cp*(elx-el) / ra
        denom = del + gamma  ! No surface resistance here
        energy = min(num/denom,Anet)
        peiZero = energy * a/rhoWat/l * wetFrac
        peInt = max(0.0,min(si,peiZero))
        ! Calculate energy used for interception evaporation
        AInt = peInt * (rhoWat*l)/a
        ACan = max(0.0, ACan - AInt)  ! Energy intercepted by the canopy and not used for interception evaporation
        !ACan = ACan - AInt  ! Energy intercepted by the canopy and not used for interception evaporation

        ! Canopy surface resistance and transpiration
        !rsmin = 170.
        !rsmin = real(grsmin)
        rsmin = rsmin2d(xcoor,ycoor)
        if(rsmin == 0.0) then
        ! Set a standard RSMIN value if there are no trees yet
        ! (e.g. at the beginning of simulation)
          rsmin = 180.0
        end if

        ! For non-forested land cover types: use standard RSMIN
        if(land.ge.21.and.land.le.26.and.stock.eq..FALSE.) then
          rsmin = srsmin

        end if

        call co2(day, mon, year, 21, ca)
        ! Hack for FORHYCS Val d'Anniviers case study: disable effect of
        ! CO2 on stomatal behavior. For this purpose the parameter GRSMIN
        ! is recycled, as it is no longer used.
        if(grsmin==0) ca = 350.

        !if((xcoor==56).and.(ycoor==97)) then
        !  print*, "rsmin: ", rsmin
        !end if
        laic = lai/vFrac
        if(vFrac==0.0) laic = 0.0
        ! Potential transpiration
        call erstomJarvis(laic,rsmin,r,land,sm,sfc,el,elx,t,ca,rs,1,xcoor,ycoor)
        num = del*ACan + vFrac*rhoAir*cp*(elx-el) / ra
        denom = del + gamma * (1.0 + rs/ra)
        energy = min(num/denom,ACan)
        peTrans = energy * a/rhoWat/l * (1.0-wetFrac)


        ! Actual transpiration
        call erstomJarvis(laic,rsmin,r,land,sm,sfc,el,elx,t,ca,rs,2,xcoor,ycoor)
        !num = del*(max(0.0,ACan)) + vFrac*rhoAir*cp*(elx-el) / ra
        num = del*ACan + vFrac*rhoAir*cp*(elx-el) / ra
        denom = del + gamma * (1.0 + rs/ra)
        energy = min(num/denom,ACan)
        aTrans = energy * a/rhoWat/l * (1.0-wetFrac)

        ! Soil (+ ground veg.) evaporation
        peSoil = (del*(0.8*ASoil)/(del+gamma)) * a/rhoWat/l

        if(drydays==0) then  ! During precipitation days
          if(eeqrmean(xcoor,ycoor).lt.0.001) eeqrmean(xcoor,ycoor) = 0.001
          fzhang = min((pefrmean(xcoor,ycoor)/eeqrmean(xcoor,ycoor)),1.0)
          aeSoil = ((del*(0.8*ASoil)/(del+gamma)) * fzhang) * a/rhoWat/l
        else  ! During dry days
          aeSoil = ((del*(0.8*ASoil)/(del+gamma)) * exp(-drydays/kappa)) * a/rhoWat/l
        end if  ! Precipitation vs. dry days

        peSoil = max(0.0,peSoil)
        aeSoil = max(0.0,aeSoil)

        ! Special case if no trees - drought stress index calculated on the
        ! basis of actual and potential soil/GV evaporation
        if(laic == 0.0) then
          peTrans = peSoil
          aTrans = aeSoil
          peSoil = 0.0
          aeSoil = 0.0
        end if

    end select
    !end if
  end if ! snow or not

  return


  end subroutine evap
