!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

	SUBROUTINE FLOODSTAT(FNAME,INNAME,OUTNAME,mhtml)

! ---------------------------------------------------------------------
* CALCULATE EFFICIENCY OF FLOOD ESTIMATION
*
* # Daniel Viviroli <viviroli@giub.unibe.ch>
* # Hydrology Group / Dept. of Geography, University of Berne
*
* VER 1.3, Build 01.12.2004
*
* following recommendations and formulas in
* - LAMB 1999 [Wat.Res.Res.35(10):3103-3114]
* - BAHRENBERG/GIESE/NIPPER 1990 [Stuttgart: Teubner, p. 204ff.]
*
* INPUT			(subroutine arguments)
* -fname			name [and path] of control file
* -inname			name [and path] of text formatted runoff record
* -outname        name [and path] of text formatted output file
*
* CONTROL FILE KEYS:
* -headrs			number of header rows to ignore
* -obscol			column containing observed runoff
* -simcol			column containing simulated runoff
* -timest 		timestep: "D"=daily, "H"=hourly, "X"=unknown/other
* -waeexp			exponent for WAE
* -potnum			basis on which peaks are chosen for statistics:
*                    x --mode1a--> x peaks
*                   -x --mode1b--> peaks threshold x/1000 (for OBS only, [x]=mm!)
*                 99xx --mode1c--> xx times number of measured years
*                    0 --mode2---> annual peaks
* -potsep			separation of peaks
*                    0 --> no separation
*                    x --> timesteps required at least between two peaks (recommended: 72 hrs / 3 days)
*                   -x --> x times minimal OBSERVED time from valley to peak
*
* -areobs,aresim  area for HQEX output (optional)
*                     x --> area is x [m**3/s], produce HQEX output
*                 -8888 --> prompt for areas and produce HQEX output
*                 -9999 --> (or key missing) do not produce HQEX output
*
* OUTPUT FILE
* -...._fls.dat	text formatted statistics file
*
* ID 81			NSE		Nash-Sutcliffe efficiency of whole record
*                 WAE		Sum og weighted absolute errors (exponent 1.5 recommended)
*                 POT-QT	Peaks-over-threshold, timing and magnitue
*                 POT-SAE	Peaks-over-threshold, standard absolute error
*				SPM_R   Spearman rank-correlation coefficient
*                 SPM_P   Significance level of SPM_R (normal distribution)
*
* <POT_N> x		Peaks record (x peaks selected)
* <POT_T> x		Peaks-over-threshold record (threshold x [float])
* <ANNL>			Annual peaks record
*                 An asterisk (*) marks the record used for the statistics
*
* SKILL Scores from 2x2 Contingency Tables (EG. Zappa (2002), Jasper and Kaufmann (2002, QJRMS)
* - Hi = Hits
* - Mi = Misses
* - Fa = False Alarms
* - Ze = Zeros (if flood bigger than 50% of threshold)
* - ACC  = Accuracy
* - CSI  = Critical success index
* - pod1 = Probability of detection
* - FAR  = False alarm ratio
* - BIAS = Bias score
* - ETS  = Equitability Threat score
* - HSS  = Heidke Skill Score
* Added by massimiliano.zappa@wsl.ch, November 2004
*
! ---------------------------------------------------------------------


	IMPLICIT NONE
	INTEGER					I,J,K,L,M,N,POSF,K1,K2
	CHARACTER*256			TINP,TDUMP(99),BUFF
	LOGICAL					LVAL,RETVAL
	LOGICAL					CHANGED

	CHARACTER*256			FNAME,INNAME,OUTNAME,OUTDIR,GAUGENAME
	CHARACTER				KEY*6
	INTEGER					HEADERS,OBSCOL,SIMCOL,POTNUM,MODE,MHTML
	INTEGER					POTSEP,SEPTIME
	INTEGER					SEPLO,SEPHI
	REAL					THRESHOLD
	CHARACTER				TIMESTEP*1
	INTEGER					TIMEINTERV
	REAL					WAEEXP
	REAL,ALLOCATABLE::		OBS     (:),  SIM     (:)
	INTEGER,ALLOCATABLE::	OBSPEAK (:,:),SIMPEAK (:,:)
	INTEGER					OBSPEAKS(2),  SIMPEAKS(2)
	CHARACTER,ALLOCATABLE::	DATECHR(:)*13
	INTEGER					NUMRECS,MAXRECS
	INTEGER					YY,MM,DD,HH
	INTEGER					YY1,YY2,YEARS,LASTYY
	REAL					NSE,NSE1,NSE2
	REAL					OBSSUM,OBSMEAN
	REAL					WAE
	REAL					TMIN,TMAX,TDIFF,QMIN,QMAX,QDIFF
	REAL					POTQT
	REAL					POTSAE
	REAL					SPM_R,SPM_S,SPM_Z,SPM_P
	INTEGER,ALLOCATABLE::	OBSRANKPOINTER(:),SIMRANKPOINTER(:)
	INTEGER,ALLOCATABLE::	OBSRANK(:),SIMRANK(:)
	REAL					PHI_Z,PHI_Z_LAST,PI,Z,D_Z(0:9999)
	REAL					AREA_OBS,AREA_SIM
	INTEGER					EXISTYEAR(1850:2100)

	integer dmj(3)
	real hi,ze,mi,fa,nn,csi(100),acc(100),bias(100)
	real pod1(100),thresh(100),far(100),cobs,csim,oe,se
      real ets(100),hss(100)

	AREA_OBS=-9999;AREA_SIM=-9999


C PROCESS CONTROL FILE
C ------------------------------------------------------------------------------
	OPEN (10,FILE=FNAME)
	DO
		READ (10,'(A)',END=19) TINP
		READ (TINP(1:6),'(A)') KEY
		SELECT CASE (KEY)
			CASE ("HEADRS")
				READ (TINP(10:256),'(I)') HEADERS
			CASE ("OBSCOL")
				READ (TINP(10:256),'(I)') OBSCOL
			CASE ("SIMCOL")
				READ (TINP(10:256),'(I)') SIMCOL
			CASE ("TIMEST")
				READ (TINP(10:256),'(A)') TIMESTEP
			CASE ("WAEEXP")
				READ (TINP(10:256),'(F)') WAEEXP
			CASE ("POTNUM")
				READ (TINP(10:256),'(I)') POTNUM
			CASE ("POTSEP")
				READ (TINP(10:256),'(I)') POTSEP
			CASE ("AREOBS")
				READ (TINP(10:256),*) AREA_OBS
			CASE ("ARESIM")
				READ (TINP(10:256),*) AREA_SIM
		END SELECT
	ENDDO
19	CLOSE (10)

	IF ((TIMESTEP.EQ."D").OR.(TIMESTEP.EQ."d")) TIMEINTERV=86400 !(secs)
	IF ((TIMESTEP.EQ."H").OR.(TIMESTEP.EQ."h")) TIMEINTERV= 3600 !(secs)

C LOOK FOR TEMPORARY CATCHMENT AREAS FILE AND READ VALUES IF AVAILABLE
C ------------------------------------------------------------------------------
	INQUIRE (FILE='AREA.TMP', EXIST=LVAL)
	IF (LVAL) THEN
		OPEN (721,FILE="AREA.TMP",DISPOSE="DELETE")
		READ (721,*) AREA_OBS
		READ (721,*) AREA_SIM
!		READ (721,'(A256)') OUTDIR !(obsolete)
		READ (721,'(A256)') GAUGENAME
		CLOSE (721)
	ELSE
		call messagerun (1,'FLOODSTAT non-critical error:')
		call messagerun (1,'Temporary catchment area file not found,')
		call messagerun (1,'therefore no HQ-EX output')
	ENDIF


C READ FLOW RECORDS
C ------------------------------------------------------------------------------
	MAXRECS=250*365*24
	ALLOCATE (OBS(MAXRECS),SIM(MAXRECS))
	ALLOCATE (DATECHR(MAXRECS))

	OPEN (20,FILE=TRIM(INNAME))
	DO I=1,HEADERS
		READ (20,*)
	ENDDO

	NUMRECS=0
	EXISTYEAR=0
	YEARS=0
	YY1=0;YY2=0
	DO I=1,MAXRECS
		READ (20,'(A)',END=29) TINP
		READ (TINP,*) (TDUMP(J),J=1,MAX(OBSCOL,SIMCOL))
		READ (TDUMP(OBSCOL),*) OBS(I)
		READ (TDUMP(SIMCOL),*) SIM(I)
		IF ((TIMESTEP.EQ."D").OR.(TIMESTEP.EQ."d")) THEN
			READ (TDUMP(1)     ,*) YY
			READ (TDUMP(2)     ,*) MM
			READ (TDUMP(3)     ,*) DD
			WRITE (DATECHR(I),'(I4.4,2("-",I2.2))') YY,MM,DD
			IF (YY1.EQ.0) YY1=YY
			EXISTYEAR(YY)=1
		ELSE IF ((TIMESTEP.EQ."H").OR.(TIMESTEP.EQ."h")) THEN
			READ (TDUMP(1)     ,*) YY
			READ (TDUMP(2)     ,*) MM
			READ (TDUMP(3)     ,*) DD
			READ (TDUMP(4)     ,*) HH
			WRITE (DATECHR(I),'(I4.4,3("-",I2.2))') YY,MM,DD,HH
			IF (YY1.EQ.0) YY1=YY
			EXISTYEAR(YY)=1
		ENDIF

		IF (LASTYY.NE.YY) THEN
			LASTYY=YY
			IF (YY.EQ.YY1) WRITE (*,'(I4$)') YY
			WRITE (*,'(A$)') "."
		ENDIF

	ENDDO

29	CLOSE (20)
	NUMRECS=I-1

	YY2=YY
!	YEARS=YY2-YY1+1
	DO I=YY1,YY2
		YEARS=YEARS+EXISTYEAR(I)
	ENDDO
	IF (YEARS.LT.YY2-YY1+1) THEN
		call messagerun (1,'FLOODSTAT warning:')
		call messagerun (1,'Runoff data seems to be incomplete')
	ENDIF


C ==============================================================================
C EXTRACT PEAKS -> Pointers to obs/sim array are saved, not the actual peaks!
C ==============================================================================
	IF (POTNUM.GT.0) THEN
		IF (POTNUM.GE.9900) THEN
			POTNUM=YEARS*(POTNUM-9900)
		ENDIF
		THRESHOLD=0
		MODE=1
	ELSE IF (POTNUM.LT.0) THEN
		THRESHOLD=-POTNUM/1000
		MODE=1
	ELSE
		THRESHOLD=0
		MODE=2
	ENDIF
	OBSPEAKS=0
	SIMPEAKS=0
	ALLOCATE (OBSPEAK(NUMRECS,3),SIMPEAK(NUMRECS,3))

C PEAKS-OVER-THRESHOLD
C ------------------------------------------------------------------------------


C (i) Detect change in sign (from + to -) in 1st derivate
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DO I=2,NUMRECS-1
		IF ((OBS(I-1).LT.OBS(I)).AND.(OBS(I+1).LT.OBS(I))
	1		.AND.(OBS(I).GT.THRESHOLD)) THEN
			OBSPEAKS(1)=OBSPEAKS(1)+1
			OBSPEAK(OBSPEAKS(1),1)=I
		ENDIF
		IF ((SIM(I-1).LT.SIM(I)).AND.(SIM(I+1).LT.SIM(I))) THEN
			SIMPEAKS(1)=SIMPEAKS(1)+1
			SIMPEAK(SIMPEAKS(1),1)=I
		ENDIF
	ENDDO

	IF (OBSPEAKS(1).LT.POTNUM) THEN
		call messagerun (1,'TOO FEW OBSERVED PEAKS')
		RETURN
	ENDIF
	IF (SIMPEAKS(1).LT.POTNUM) THEN
		call messagerun (1,'TOO FEW SIMULATED PEAKS')
		RETURN
	ENDIF

C (ii) Determine separation time !![not really functional yet... "event"=???]
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	IF (POTSEP.LT.0) THEN
		SEPLO=0
		SEPTIME=NUMRECS
		DO I=20,NUMRECS-1
			IF (SEPLO.EQ.0) THEN
				IF (OBS(I).LT.(OBS(I-1)).AND.(OBS(I).LT.OBS(I+1)))
	1				THEN
					SEPLO=I
					SEPHI=0
				ENDIF
			ELSE
				IF ((OBS(I-1).LT.OBS(I)).AND.(OBS(I+1).LT.OBS(I)))
	1				THEN
					SEPHI=I
					SEPTIME=MIN(SEPTIME,SEPHI-SEPLO)
					SEPLO=0
				ENDIF
			ENDIF
		ENDDO
	ELSE IF (POTSEP.GT.0) THEN
		SEPTIME=POTSEP
	ELSE
		SEPTIME=0
	ENDIF

C (iii) Keep only larger event within separation time
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	IF (SEPTIME.NE.0) THEN

		DO I=2,OBSPEAKS(1)-1
31			IF (OBSPEAK(I,1)-OBSPEAK(I-1,1).LT.SEPTIME) THEN
				IF (OBS(OBSPEAK(I-1,1)).GT.OBS(OBSPEAK(I,1))) THEN
					K=0
				ELSE
					K=1
				ENDIF
				DO J=I-K,OBSPEAKS(1)-1
					OBSPEAK(J,1)=OBSPEAK(J+1,1)
				ENDDO
				OBSPEAKS(1)=OBSPEAKS(1)-1
				IF (I.LT.OBSPEAKS(1)) GOTO 31
			ENDIF
			IF (I.GT.OBSPEAKS(1)) OBSPEAK(I,1)=0
		ENDDO

		DO I=2,SIMPEAKS(1)-1
32			IF (SIMPEAK(I,1)-SIMPEAK(I-1,1).LT.SEPTIME) THEN
				IF (SIM(SIMPEAK(I-1,1)).GT.SIM(SIMPEAK(I,1))) THEN
					K=0
				ELSE
					K=1
				ENDIF
				DO J=I-K,SIMPEAKS(1)-1
					SIMPEAK(J,1)=SIMPEAK(J+1,1)
				ENDDO
				SIMPEAKS(1)=SIMPEAKS(1)-1
				IF (I.LT.SIMPEAKS(1)) GOTO 32
			ENDIF
			IF (I.GT.SIMPEAKS(1)) SIMPEAK(I,1)=0
		ENDDO

	ENDIF

	IF ((POTNUM.LT.0).AND.(MODE.EQ.1)) POTNUM=OBSPEAKS(1)

C ANNUAL PEAKS -> change in datechr (1:4) determines years!
C ------------------------------------------------------------------------------
	OBSPEAKS(2)=1
	SIMPEAKS(2)=1
	OBSPEAK(1,2)=1
	SIMPEAK(1,2)=1
	DO I=2,NUMRECS-1
		IF (DATECHR(I)(1:4).NE.DATECHR(I-1)(1:4)) THEN
			OBSPEAKS(2)=OBSPEAKS(2)+1;OBSPEAK(OBSPEAKS(2),2)=I
			SIMPEAKS(2)=SIMPEAKS(2)+1;SIMPEAK(SIMPEAKS(2),2)=I
		ENDIF
		IF (OBS(I).GT.OBS(OBSPEAK(OBSPEAKS(2),2))) THEN
			OBSPEAK(OBSPEAKS(2),2)=I
		ENDIF
		IF (SIM(I).GT.SIM(SIMPEAK(SIMPEAKS(2),2))) THEN
			SIMPEAK(SIMPEAKS(2),2)=I
		ENDIF
	ENDDO
	IF (MODE.EQ.2) POTNUM=OBSPEAKS(2)

C SAVE THE UNSORTED ANNUAL PEAK POINTER ARRAY (ARRAY #3)
C ------------------------------------------------------------------------------
	DO I=1,YEARS
		OBSPEAK(I,3)=OBSPEAK(I,2)
		SIMPEAK(I,3)=SIMPEAK(I,2)
	ENDDO


C SORT PEAK-POINTER-ARRAYS IN DESCENDING ORDER (BUBBLESORT METHOD)
C ------------------------------------------------------------------------------
	DO J=1,2

51		CHANGED=.FALSE.
		DO I=1,OBSPEAKS(J)-1
			IF (OBS(OBSPEAK(I,J)).LT.OBS(OBSPEAK(I+1,J))) THEN
				CALL FLIPINT(OBSPEAK(I,J),OBSPEAK(I+1,J))
				CHANGED=.TRUE.
			ENDIF
		ENDDO
		IF (CHANGED) GOTO 51

52		CHANGED=.FALSE.
		DO I=1,SIMPEAKS(J)-1
			IF (SIM(SIMPEAK(I,J)).LT.SIM(SIMPEAK(I+1,J))) THEN
				CALL FLIPINT(SIMPEAK(I,J),SIMPEAK(I+1,J))
				CHANGED=.TRUE.
			ENDIF
		ENDDO
		IF (CHANGED) GOTO 52

	ENDDO

C ==============================================================================
C DETERMINE THE SCORES
C ==============================================================================

C NSE (NASH-SUTCLIFFE)
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	NSE=0
	NSE1=0
	NSE2=0
	OBSSUM=0
	OBSMEAN=0
	DO I=1,NUMRECS
		OBSSUM=OBSSUM+OBS(I)
	ENDDO
	OBSMEAN=OBSSUM/NUMRECS
	DO I=1,NUMRECS
		NSE1=NSE1+(OBS(I)-SIM(I))**2
		NSE2=NSE2+(OBS(I)-OBSMEAN)**2
	ENDDO
	NSE=1-(NSE1/NSE2)


C WAE (SUM OF WEIGHTED ABSOLUTE ERRORS)
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	WAE=0
	DO I=1,NUMRECS
		WAE=WAE+((OBS(I)**WAEEXP)*ABS(OBS(I)-SIM(I)))
	ENDDO

C POT-QT (PEAKS-OVER-THRESHOLD, WEIGHTED FOR TIMING AND MAGNITUDE)
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	TMIN=NUMRECS
	TMAX=0
	DO I=1,POTNUM
		IF (OBSPEAK(I,MODE).LT.TMIN) TMIN=OBSPEAK(I,MODE)
		IF (OBSPEAK(I,MODE).GT.TMAX) TMAX=OBSPEAK(I,MODE)
	ENDDO

	TDIFF=TMAX-TMIN
	QDIFF=OBS(OBSPEAK(1,MODE))-OBS(OBSPEAK(POTNUM,MODE))

	POTQT=0
	DO I=1,POTNUM
		POTQT=POTQT+SQRT(
	1	(    (OBS(OBSPEAK(I,MODE))-SIM(SIMPEAK(I,MODE)))/QDIFF)**2+
	2	(REAL(OBSPEAK(I,MODE)     -SIMPEAK(I,MODE)     )/TDIFF)**2
     9															   )
	ENDDO

C POT-SAE (PEAKS-OVER-THRESHOLD, SUM OF ABSOLUTE ERRORS)
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	POTSAE=0
	DO I=1,POTNUM
		POTSAE=POTSAE+ABS(OBS(OBSPEAK(I,MODE))-SIM(SIMPEAK(I,MODE)))
	ENDDO

C SPM_C and SPM_P (SPEARMAN RANK CORRELATION: COEFFICIENT AND SIGNIFICANCE)
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	SPM_S=0
	ALLOCATE (OBSRANK(POTNUM),SIMRANK(POTNUM))

	K1=1;K2=1
	DO I=1,NUMRECS
		DO J=1,POTNUM
			IF (OBSPEAK(J,MODE).EQ.I) THEN
				OBSRANK(K1)=J
				K1=K1+1
			ENDIF
			IF (SIMPEAK(J,MODE).EQ.I) THEN
				SIMRANK(K2)=J
				K2=K2+1
			ENDIF
		ENDDO
	ENDDO

	DO I=1,POTNUM
		SPM_S=SPM_S+(OBSRANK(I)-SIMRANK(I))**2
	ENDDO
	SPM_R=1-(6*(SPM_S)/(POTNUM*(POTNUM**2-1)))
	SPM_Z=SPM_R*SQRT(REAL(POTNUM)-1) !actually only for N>=30!

	PI=3.14159265358979
	D_Z(0)=0
	PHI_Z=1/SQRT(2*PI)
	DO I=1,500 !Density function for normal distribution Z=(I/100)!
		PHI_Z_LAST=PHI_Z
		PHI_Z=1/SQRT(2*PI)*EXP(-(((REAL(I)/100)**2)/2))
		D_Z(I)=D_Z(I-1)+(PHI_Z+PHI_Z_LAST)*0.01
	ENDDO
	IF(SPM_Z.GT.0) THEN
		SPM_P=D_Z(NINT(SPM_Z*100))
	ELSE
		SPM_P=0
	ENDIF

C ==============================================================================
C WRITE OUTPUT FILE
C ==============================================================================

      call posnam (outname,1,posf)
	BUFF=outname(1:(posf-8))//"_fls.dat"
	OPEN (30,FILE=TRIM(BUFF),status='unknown')
	write(mhtml,'(A)')'Results of flood statistics module<br>'
      write(mhtml,'(A)')'<a href='//trim(BUFF)//
     * '>'//trim(BUFF)//'</a><br>'

101	FORMAT
	1(TR1,A2,TR3,A4,TR3,A7,TR1,A12,TR1,A12,TR1,A12,TR3,A7,TR3,A7)
102	FORMAT
	1(TR1,I2,TR3,I4,TR3,F7.4,TR1,I12,TR1,F12.3,TR1,F12.2,TR3,F7.3,
     2 TR3,F7.3)

	WRITE (30,'(A)') "Objective functions for flood peaks"
	WRITE (30,101)
	1"ID","TYPE","NSE","WAE","P-QT","P-SAE","SPM_R","SPM_P"
	WRITE (30,102)
	191,9999,NSE,NINT(WAE),POTQT,POTSAE,SPM_R,SPM_P

	WRITE (30,'(A)')
	WRITE (30,'(A,I4)') "<ANNUAL>  ",YEARS
111	FORMAT (TR1,2(A10  ,TR1,A13:TR2))
112	FORMAT (TR1,2(F10.4,TR1,A13:TR2))
	WRITE (30,111) "OBS_HQA","DATE","SIM_HQA","DATE"
	DO I=1,OBSPEAKS(2)
		WRITE(30,112) OBS(OBSPEAK(I,2)),TRIM(DATECHR(OBSPEAK(I,2))),
	1				  SIM(SIMPEAK(I,2)),TRIM(DATECHR(SIMPEAK(I,2)))
	ENDDO

	WRITE (30,'(A)')
121	FORMAT (TR1,2(A10  ,TR1,A13:TR1))
122	FORMAT (TR1,2(F10.4,TR1,A13:TR1))
	IF (THRESHOLD.EQ.0) THEN
		IF (MODE.EQ.2) THEN
			WRITE (30,'(A,I4)'  ) "<P-REC:A>  ",POTNUM
			WRITE (30,111) "OBS_HQA","DATE","SIM_HQA","DATE"
			DO I=1,OBSPEAKS(2)
				WRITE(30,112)
	1					OBS(OBSPEAK(I,2)),TRIM(DATECHR(OBSPEAK(I,2))),
	2					SIM(SIMPEAK(I,2)),TRIM(DATECHR(SIMPEAK(I,2)))
			ENDDO
		ELSE
			WRITE (30,'(A,I4)'  ) "<P-REC:N>  ",POTNUM
		WRITE (30,121) "OBS_HQ","DATE","SIM_HQ","DATE"
		DO I=1,POTNUM
			WRITE (30,122)
	1				  OBS(OBSPEAK(I,1)),TRIM(DATECHR(OBSPEAK(I,1))),
	2				  SIM(SIMPEAK(I,1)),TRIM(DATECHR(SIMPEAK(I,1)))
		ENDDO
		ENDIF
	ELSE
		WRITE (30,'(A,F10.4)'  ) "<P-REC:T>  ",THRESHOLD



		WRITE (30,121) "OBS_HQ","DATE","SIM_HQ","DATE"
		DO I=1,OBSPEAKS(1)
			WRITE (30,122)
	1				  OBS(OBSPEAK(I,1)),TRIM(DATECHR(OBSPEAK(I,1))),
	2				  SIM(SIMPEAK(I,1)),TRIM(DATECHR(SIMPEAK(I,1)))
		ENDDO

	ENDIF


!     Skill Scores from 2x2 contingency tables

	WRITE (30,'(A)')
	WRITE (30,'(A)') "<SKILL SCORES FROM CONTINGENCY TABLES>  "
	IF (TIMESTEP.EQ."H") obscol=obscol+1
	IF (TIMESTEP.EQ."H") simcol=simcol+1

	open (50,file=inname)

	write(30,'(A)') 'Threshold Hits Zeros False_alarms Misses'//
     *                      ' Events Accuracy CSI POD1 FAR BIAS ETS HSS'
	do i=1,100
! Define Thresholds
	IF (TIMESTEP.EQ."D") thresh(I)=2.0+float(I-1)/2
	IF (TIMESTEP.EQ."H") thresh(I)=0.5+float(I-1)/8
	nn=0
	hi=0
	ze=0
	fa=0
	mi=0
710    continue
	read(50,*,end=711) (TDUMP(J),J=1,MAX(OBSCOL,SIMCOL))

	READ (TDUMP(OBSCOL),*) COBS
	READ (TDUMP(SIMCOL),*) CSIM

	if (cobs .lt. thresh(I)/2.) goto 710
	nn=nn+1
	oe=0
	se=0
	if (cobs .gt. thresh(I)) oe=1.
	if (csim .gt. thresh(I)) se=1.

	if ((se+oe) .eq. 2.0) hi=hi+1.
	if ((se+oe) .eq. 0.0) ze=ze+1.
	if ((se-oe) .eq. 1.0) fa=fa+1.
	if ((se-oe) .eq. -1.0) mi=mi+1.

      goto 710
711    continue
      rewind (50)

	if ((nn-ze) .lt. 1) exit

	acc(I)=(hi+ze)/nn
	csi(I)=(hi/(hi+fa+mi))
	pod1(I)=(hi/(hi+mi))
	bias(I)=(hi+fa)/(hi+mi)
	far(I)=fa/(hi+fa)
	ets(I)=(((ze*Hi)-(fa*mi))/((fa+Mi)*nn+Ze*Hi-fa*mi))
	hss(I)=(2*((ze*Hi)-(fa*mi))/((Ze+Mi)*(Mi+Hi)+(Ze+Fa)*(Fa+Hi)))

	write(30,'(f8.3,5I9,7f7.3)') thresh(I),int(hi),int(ze),
     *                      int(fa),int(mi),int(nn),acc(I),csi(I),
     *                      pod1(I),far(I),bias(I),ets(I),hss(I)
	end do
      CLOSE (50)
	CLOSE (30)

C ==============================================================================
C PRODUCE HQEX OUTPUT FILE IF DESIRED
C ==============================================================================
	IF ((AREA_OBS.EQ.-9999).AND.(AREA_SIM.EQ.-9999)) RETURN

	IF (AREA_OBS.EQ.-8888) THEN
		WRITE (*,'(a$)') "OBS_AREA="
		READ (*,*) AREA_OBS
	ENDIF
	IF (AREA_SIM.EQ.-8888) THEN
		WRITE (*,'(a$)') "SIM_AREA="
		READ (*,*) AREA_SIM
	ENDIF

C WRITE HQEX OBS MM VALUES
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      CALL POSNAM (OUTNAME,1,POSF)
	BUFF=OUTNAME(1:(POSF-8))//"_HQ-OBS.MM"
	OPEN  (51,FILE=BUFF)
	WRITE (51,'(A)') TRIM(GAUGENAME)//"_OBS_"//TRIM(TIMESTEP)
	WRITE (51,'(A)') TRIM(GAUGENAME)
	WRITE (51,'(I4,A,A)') YY1," mm ","-900"
	DO I=1,YEARS
		IF (I.EQ.YEARS) THEN
			WRITE (51,'(F10.5$)') OBS(OBSPEAK(I,3))
		ELSE
			WRITE (51,'(F10.5 )') OBS(OBSPEAK(I,3))
		ENDIF
	ENDDO
	CLOSE (51)

C WRITE HQEX SIM MM VALUES
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	BUFF=OUTNAME(1:(POSF-8))//"_HQ-SIM.MM"
	OPEN  (52,FILE=BUFF)
	WRITE (52,'(A)') TRIM(GAUGENAME)//"SIM_"//TRIM(TIMESTEP)
	WRITE (52,'(A)') TRIM(GAUGENAME)
	WRITE (52,'(I4,A,A)') YY1," mm ","-900"
	DO I=1,YEARS
		IF (I.EQ.YEARS) THEN
			WRITE (52,'(F10.5$)') SIM(SIMPEAK(I,3))
		ELSE
			WRITE (52,'(F10.5 )') SIM(SIMPEAK(I,3))
		ENDIF
	ENDDO
	CLOSE (52)

C WRITE HQEX OBS CUMS VALUES
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	BUFF=OUTNAME(1:(POSF-8))//"_HQ-OBS.HQX"
	OPEN  (61,FILE=BUFF)
	WRITE (61,'(A)') TRIM(GAUGENAME)//"_OBS_"//TRIM(TIMESTEP)
	WRITE (61,'(A)') TRIM(GAUGENAME)
	WRITE (61,'(I4,A,F10.2)') YY1," m�/s ",AREA_OBS
	DO I=1,YEARS
		IF (I.EQ.YEARS) THEN
			WRITE (61,'(F10.2$)')
	1		 OBS(OBSPEAK(I,3))/TIMEINTERV*AREA_OBS*1000
		ELSE
			WRITE (61,'(F10.2 )')
	1		 OBS(OBSPEAK(I,3))/TIMEINTERV*AREA_OBS*1000
		ENDIF
	ENDDO
	CLOSE (61)

C WRITE HQEX SIM CUMS VALUES
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	BUFF=OUTNAME(1:(POSF-8))//"_HQ-SIM.HQX"
	OPEN  (62,FILE=BUFF)
	WRITE (62,'(A)') TRIM(GAUGENAME)//"_SIM_"//TRIM(TIMESTEP)
	WRITE (62,'(A)') TRIM(GAUGENAME)
	WRITE (62,'(I4,A,F10.2)') YY1," m�/s ",AREA_SIM
	DO I=1,YEARS
		IF (I.EQ.YEARS) THEN
			WRITE (62,'(F10.2$)')
	1		 SIM(SIMPEAK(I,3))/TIMEINTERV*AREA_SIM*1000
		ELSE
			WRITE (62,'(F10.2 )')
	1		 SIM(SIMPEAK(I,3))/TIMEINTERV*AREA_SIM*1000
		ENDIF
	ENDDO
	CLOSE (62)

C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


	END SUBROUTINE

C -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

	SUBROUTINE FLIPINT(A,B)
		INTEGER A,B,C
		C=A
		A=B
		B=C
	END SUBROUTINE
