!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Module FORHYCSGLOBALS contains all variables used by and/or transferred
! between the models PREVAH and TreeMig.
!-------------------------------------------------------------------------------

module forhycsglobals

! Define number of digits necessary where single precision (7 digits) is not enough
! (e.g. for global sums). Variables can be specified with (real(kind=dblP) :: haha)
  integer, parameter :: dblPVal = 16
  integer, parameter :: dblPR = 307
  integer, parameter :: dblP = selected_real_kind(p=dblPVal,r=dblPR)

! Partial Kahan sums (for each process) to be calculated in ADMSHYD. Global Kahan
! sum is calculated in PREVAHENDDAY.
  real(kind=dblP) :: pnksmp, paksmp, wsasmp, epsmp, ersmp, eismp, ebsmp, r0smp,&
r1smp, r2smp, glacssp, rgessmp
! New PET and AET variables
  real(kind=dblP) :: eSoilsmp, eTranssmp, eIntsmp, eSnowsmp, eWatersmp

! Correction terms to be carried for Kahan summation of daily values in
! ADMSHYD
      real(kind=dblP) :: pnksmc, paksmc, wsasmc, epsmc, ersmc, eismc, ebsmc, r0smc,&
r1smc, r2smc, glacssc, rgessmc
! New PET and AET variables
  real(kind=dblP) :: eSoilsmc, eTranssmc, eIntsmc, eSnowsmc, eWatersmc

  INTEGER :: nspc_g						!nspc as global variable
  INTEGER :: counter_g						!counter as global variable
  !REAL :: fhla(1:maxArraylat,1:maxArraylon,maxspc)

  TYPE wrOutp
    LOGICAL :: biomass, number, seeds, antagonists, hstruct, pollen, ingrowth, biodiv, plot
    LOGICAL :: leafarea !FORHYCS
  END TYPE wrOutp

  TYPE(wrOutp) :: writeOutput

!Marc's LAI stuff
  real, allocatable :: forhycs(:,:,:)		! forhycs_output xcoor,ycoor,variable: Var=1: RLAI, VAR=2:Max Inteception
  real, allocatable :: gdd5(:,:)		! growing degree day sum on 5 degree celsius basis
  real, allocatable :: chilldays(:,:)		! chilldays
  real, allocatable :: dtemp31(:,:,:)		! container for the last 31 temperature values (x,y,day)
  real, allocatable :: mtemp(:,:)		! mean temperature of last 31 days
  integer, allocatable :: sen_day_count(:,:,:) 	! count senescence days (x,y,species)
  real, allocatable :: aphen(:,:,:) 		! annual sum of daily fractional leaf cover (x,y,species)
  real, allocatable :: oldphen(:,:,:) 		! phenology of previous day(x,y,species)
  real, allocatable :: spcgdd0(:,:)			! gddramp values  (species, days)
  real, allocatable :: ddegsum5(:,:),	muwint(:,:,:),drstr(:,:)
  real, allocatable :: EBJSout(:,:), EPJSout(:,:), EIJSout(:,:)
  real, allocatable :: meant(:,:,:), sump(:,:,:),drstr2(:,:), meantc(:,:,:)
  real, allocatable :: meantold(:,:,:), sumpold(:,:,:), meantcold(:,:,:)
  real, allocatable :: ddegsum5old(:,:),drstrold(:,:),drstrold2(:,:)
  real, allocatable :: fhla(:,:,:)             !Leaf area (x,y,species)
  real, allocatable :: ebjs2(:,:)
  real, allocatable :: perCC(:,:)  ! Fractional canopy cover - Matthias Jul 2015
  real, allocatable :: vh100(:,:),  vh99(:,:), vh95(:,:)! Vegetation height - Matthias Nov 2015

! Phenodates array
  integer, allocatable :: phenDoyBB(:,:), phenDoyMax(:,:), phenDoySen(:,:), phenDoyFall(:,:), &
                          phenBBold(:,:), phenMaxOld(:,:), phenSenOld(:,:), phenFallOld(:,:)

! TreeMig variables in 2D grid for gridded output
  real, allocatable :: biomass2d(:,:,:), number2d(:,:,:), seeds2d(:,:,:), ingrowth2d(:,:,:), &
                       antagon2d(:,:,:), pollen2d(:,:,:), la2d(:,:,:), stnum2d(:,:,:)
! Temporary storage for TreeMig report tables under parallel execution
  real, allocatable:: biomassTable(:,:), numberTable(:,:), seedsTable(:,:), ingrowthTable(:,:), &
                      antagonTable(:,:), pollenTable(:,:), laTable(:,:)
! Grid variable for output of land cover class change
  real, allocatable :: lcChange(:,:)

! Grids for calculation of snow-off day
  integer, allocatable :: snowoff(:,:), snowoffOld(:,:), snowdays(:,:)

! Array to store BioClim variables when BC indices are being calculated in
! FORHYCS (BioClim for comparison).
  real, allocatable :: compBioClim(:,:,:)

! Determine direction(s) of PREVAH <-> TreeMig coupling
  character*3 ::  tpswitch
  integer :: PRtoTM,TMtoPR,prevahBC

! Path to TreeMig directory
  character*256 :: TMdir

! arrays containing latitude and longitude for TreeMig Report table output
  integer,allocatable:: latTable(:),lonTable(:)

! Whether TM shall output tables and/or spatial grids
  logical :: reptTables, reptGrids

! Threshold value for understory biomass to distinguish between cells with and without understory
  real, parameter :: underBioCrit = 3.0
  real, allocatable :: siuTF(:,:), siumaxTF(:,:)

! Dummy variable to move radiation between subroutines before writing hourly
! file for ET prototype
  real, dimension(2) :: radCell

! Sequence of years for bootstrapped meteo input during coupled spin-up, and for idealized climate runs
  integer, dimension(471) :: spinupYearsFH
  integer, dimension(85) :: futureYearsFH
  integer :: deltaT, deltaP   ! Perturbation for temperature and precipitation, read from control file (.inp)

! Control parameter determining where forest is allowed to grow during
! spin-up and during the actual simulation
  integer :: lcdyn

! New control parameters
  integer :: dohmax,&         ! Limit maximum tree height as a function of drought stress
             dolared,&        ! Reduce leaf area as a function of drought stress
             dosfc,&          ! Dynamic adaptation of rooting zone storage cpacity
             dosmort,&        ! Include snow-induced mortality of Larix and Pinus seedlings
             cdtol,&          ! Choose drought tolerance parameters. 1: Lischke & Zierl 2002; 2: Niinemets & Valladares 2006, mapped to scale of Lischke & Zierl 2002
             doground,&       ! "Groundhog year" simulations (meteorological data always from the same year)
             ghogyear,&         ! Year used in groundhog simulations (if doground==1)
             readFH, writeFH  ! Read or write model state

! RSMIN as an input parameter
  integer :: grsmin

! Switch indicating whether state reading is done or not (used in VegPars)
  logical :: readSwitch

! Variables for biomass output by ecoregion
! These variables get allocated in ecoGridInit (ecoGrid.f90)
  integer, allocatable :: ecor(:,:), ecorID(:), ecorSize(:), hbecor(:), hlecor(:), hwecor(:), hnecor(:)
  integer :: hsecor, hmecor
  integer, allocatable :: ecoc(:,:), ecocID(:), ecocSize(:)

! Grids for annual precipitation and mean May-Oct temperature (output only, for initialization of AWC)
  real, allocatable :: annprec(:,:), mmot(:,:), annprecold(:,:), mmotold(:,:)

! Grids containing (bio)climatic indicators for root zone module, and soil available water content fraction
  real, allocatable :: gslength(:,:,:), gsTpot(:,:,:), gsPrec(:,:,:), gsPrecDays(:,:,:), gsTemp(:,:,:), gsPe(:,:,:)
  real, allocatable :: gslengthCur(:,:), gsTpotCur(:,:), gsPrecCur(:,:), gsPrecDaysCur(:,:), gsTempCur(:,:), gsPeCur(:,:)
  real, allocatable :: gsPes(:,:,:), gsPesCur(:,:)
  integer, allocatable :: prevDayP(:,:)
  real, allocatable :: sfcMax(:,:)
  integer :: rootslag   ! Size in years of moving window used to calculate TPOT and precip for root zone module

! Grid of growing season De Martonne Index
  real, allocatable :: gsDMI(:,:)

! Rolling mean of soil equilibrium evaporation and effective precipitation, used for
! soil evaporation calculation
  real, allocatable :: eeqrmean(:,:), pefrmean(:,:)

! Variables for reduction of Hmax (maximum tree height, species-specific value) due to drought
  real, allocatable :: dstr30(:,:,:), gdd30(:,:,:)    ! Rolling mean of drought stress and GDD
  integer, allocatable :: curhcmax(:,:,:) ! Current maximum height class, for each species/cell

! Vector containing leaf area reduction factor (as a function of environmental stress)
  real, allocatable :: laRed(:,:)

! Grids for alternative phenological module (phenKramer)
  real, allocatable :: Schill(:,:,:), Sforcing(:,:,:), doyBB(:,:,:), doySen(:,:,:), meanT4(:,:)

! Grids for adapted phenology module (LeafPhenology)
  real, allocatable :: ssen3d(:,:,:)

! Variables for output of species/height class distribution in each catchment
 integer, allocatable :: hsstruct(:)

! Grid containing RSMIN (minimum stomatal resistance) in each cell/year
  real, allocatable :: rsmin2d(:,:), rsminStd(:,:)

! Tracking biomass changes (for new biomass output function)
  real, allocatable :: deltaBio(:,:,:)


end module forhycsglobals
