!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Implementation of the Kahan sum algorithm for the summation and averaging of
! cell-level values across MPI stripes.
!-------------------------------------------------------------------------------

module fp_ops

use forhycsglobals, only: dblP

implicit none



!  type esum
!    real(kind=dblP) :: sum
!    real(kind=dblP) :: cor
!  end type esum

contains


real function kahan_sum(in,len,rnodat)
  use mpi
  use MPIvars
  implicit none

  integer, intent(in) :: len
  real, intent(in), dimension(len) :: in
  real, intent(in) :: rnodat
  !type(esum) :: local_kahan_sum, global_kahan_sum
  !external :: MPIkahansum
!  real :: kahan_sum

  !integer, parameter :: dblP = selected_real_kind(16)
  real(kind=dblP) :: sum, cor, newSum, corNext
  real(kind=dblP) :: dblSum
  real(kind=dblP), dimension(numslaves) :: localsums, localcorrs
  integer:: coord, i

  sum = 0.0
  cor = 0.0

  do coord = len, 1, -1
    if(int(in(coord)) .eq. int(rnodat)) cycle
    corNext = in(coord) + cor
    newSum = sum + corNext
    cor = corNext - (newSum-sum)
    sum = newSum
  end do

!  local_kahan_sum%sum = sum
!  local_kahan_sum%cor = cor

!  print*, "sum, cor", sum, cor

  if(sum.ne.sum) THEN
    print*, "NaN in kahan_sum local", rank
  end if


  call MPI_ALLGATHER(sum,1,MPIdbltype,localsums,1,MPIdbltype,slaves,ierr)
  call MPI_ALLGATHER(cor,1,MPIdbltype,localcorrs,1,MPIdbltype,slaves,ierr)

  sum = 0.0
  cor = 0.0

  do i = 1,numslaves
    corNext = localsums(i) + localcorrs(i) + cor
    newSum = sum + corNext
    cor = corNext - (newSum-sum)
    sum = newSum
  end do

  kahan_sum=sum

end function kahan_sum

!====================================================



real function local_kahan_sum(in,len,rnodat)

  implicit none

    integer, intent(in) :: len
  real, intent(in), dimension(len) :: in
  real, intent(in) :: rnodat
  !type(esum) :: kahan_sum

  !integer, parameter :: dblP = selected_real_kind(16)
  real(kind=dblP) :: sum, cor, newSum, corNext
  integer:: coord

  sum = 0.0
  cor = 0.0

  do coord = len, 1, -1
    if(int(in(coord)) .eq. int(rnodat)) cycle
    corNext = in(coord) + cor
    newSum = sum + corNext
    cor = corNext - (newSum-sum)
    sum = newSum
  end do

!  kahan_sum%sum = sum
!  kahan_sum%cor = cor

  local_kahan_sum = sum

end function local_kahan_sum

! =======================================================


      real function kahanReduce(localSum,localCor)
! Reduction operation for partial Kahan sums (and correction terms) calculated
! in ADMSHYD.
        use mpi
        use MPIvars

        real(kind=dblP), intent(in) :: localSum, localCor
        real(kind=dblP) :: sum, cor, newSum, corNext
        real(kind=dblP), dimension(numslaves) :: sums, cors
        integer :: i

!        print*, "localSum", localSum, rank

        call MPI_ALLGATHER(localSum,1,MPIdbltype,sums,1,MPIdbltype,slaves,ierr)
!        print*, "ierr", ierr, rank
        call MPI_ALLGATHER(localCor,1,MPIdbltype,cors,1,MPIdbltype,slaves,ierr)

!        print*, "sums", sums, rank

      sum = 0.0
      cor = 0.0

      do i = 1,numslaves
        corNext = sums(i) + cors(i) + cor
        newSum = sum+corNext
        cor = corNext - (newSum-sum)
        sum = newSum
      end do

!      print*, "sum", sum, rank

      kahanReduce = sum

      end function kahanReduce

end module fp_ops
