!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE GLACMO (LAND,HHOEHE,HELA,DEBUG,MDR,SSO,
     *                    PDT,TECK,RDKM,CICEMF,CAICE,ERICE,
     *                    KICEH,KICSH,KICFH,TICEH,TICSH,TICFH,
     *                    SICE,SICS,SICF,HFICE,HFICS,HFICF,
     *                    WICE,WICS,WICF,RICE,RICS,RICF)

!-------------------------------------------------------------------------------
!    Icemelt module
!
!    Literature
!    Badoux, A. (1999): Untersuchung zur fl�chendifferenzierten Modellierung von
!     Abfluss und Schmelze in teil-vergletscherten Einzugsgebieten. Diploma thesis,
!     ETH Z�rich [available from the Institute for Atmospheric and Climate Science ETH,
!     Winterthurerstr. 190, CH-8057 Z�rich].
!    Hock, R. (1999): Distributed temperature-index ice- and snowmelt model including
!     potential direct solar radiation. J. Glaciol., 45, 101-111.
!    Klok, E.J., Jasper, K., Roelofsma, K.P., Badoux, A. and Gurtz, J. (2001): Distributed
!     hydrological modelling of a glaciated Alpine river basin. Hydrological Sciences
!     Journal, 46, 553-570.
!    Zappa, M., Badoux, A., and Gurtz, J. (2000): The application of a complex distributed
!     hydrological model in an highly glaciated alpine river catchment. In: Horvatic, J.
!     (Editor): Proceedings of the 33rd Conference of International Association for Danube
!     Research, Limnological Reports, 33, 23-28.
! ------------------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Variables
! ---------------------------------------------------------------------

      INTEGER MDR,DEBUG,		! Control the output to the prompt and to the log-files
     *        HELA,				! Can define the equilibrium line altitude of a glacier
     *        TICEH,				! Translation time for icemelt
     *        TICSH,				! Translation time for snowmelt
     *        TICFH				! Translation time for firn melt

      REAL CICEMF,				! constant temperature melt factor for ice
     *     CAICE,				! constant radiation melt factor for ice
     *     KICEH,				! Coefficient for runoff generation from ice storage in hours
     *     KICSH,				! Coefficient for runoff generation from snow storage in hours
     *     KICFH				! Coefficient for runoff generation from firn storage in hours


      INTEGER LAND				! Land use (13=Firn, 14=Glacier below ELA)
      REAL HHOEHE				! Elevation of the HRU
      REAL PDT,					! Input in the runoff generation module (rainfall + snowmelt)
     *     SSO,					! Snow water equivalent
     *     TECK,				! Locally adjusted air temperature
     *     RDKM,				! Locally adjusted potential incoming radiation in W/M2
     *     SICE,				! Icemelt Runoff-generation storages for glaciated HRU below HELA
     *     SICS,				! Snowmelt Runoff-generation storages for glaciated HRU below HELA
     *     SICF,				! Firn melt Runoff-generation storages for glaciated HRU above HELA
     *     RICE,				! Discharge from icemelt storage below HELA
     *     RICS,				! Discharge from snowmelt storage below HELA
     *     RICF,				! Discharge from firn melt storage above HELA to the groundwater storage
     *     WICE,GWICE,		! Icemelt from a glaciated HRU below HELA
     *     WICS,				! Input to the runoff generation module from snow covered HRU below HELA
     *     WICF,				! Input to the runoff generation module from snow/firn covered HRU above HELA
     *     ERICE,				! Evaporation from bare ice HRUs
     *     HFICE(24),HFICS(24),HFICF(24),	! Help fields for the translation of melted water from the three reservoirs
     *     RHG					! Help variable


! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------
!     Initialization
      WICE=0.0
	GWICE=0.0
      WICF=0.0
      WICS=0.0
      RICE=0.0
      RICS=0.0
      RICF=0.0
      IF (HELA .GE. 0.0) THEN
         IF ((LAND .EQ. 13) .or. (LAND .EQ. 14)) GOTO 10
      ENDIF
! Ice melt is not computed if HELA<0.0 or LAND is either 13 nor 14
         SICE=0.0
         SICS=0.0
         SICF=0.0
         GOTO 60

10    CONTINUE
! ------------------------------------------------------------------------------
! Computation of ice melt
! ------------------------------------------------------------------------------
!     Is the HRU above HELA or is he defined as belonging to the accumulation
!     area of the glacier (LAND=13)?
       IF (((HELA.GT.0.0) .and. (HHOEHE.GT.HELA)) .or.
     *     ((HELA.EQ.0.0) .and. (LAND.EQ.13))) THEN
!   Firn storage
          LAND=13
          WICF=PDT*0.5
	    WICS=WICF
          CALL LINTRA (SICF,WICF,KICFH,TICFH,HFICF,RICF)
	    CALL LINTRA (SICS,WICS,KICSH,TICSH,HFICS,RICS)
          GOTO 60
       ENDIF

! ------------------------------------------------------------------------------
!     Is the HRU below HELA or is he defined as belonging to the ablation
!     area of the glacier (LAND=14)?
       LAND=14
       IF (SSO .GE. 0.1) THEN
! If the HRU is still snow covered
          WICE=0.0
          ERICE=0.0
          WICS=PDT
          GOTO 50
       ENDIF

! If snow water equivalent is below 0.1 mm
       IF (SSO .LT. 0.1) THEN
! Bare ice area
          IF (TECK .GT. 0.0) THEN
             WICE=(((1.0/24.0)*CICEMF)+(CAICE*RDKM))*TECK
            ELSE
             WICE=0.0
          ENDIF
       ENDIF
       IF (WICE .LE. 0.0) WICE=0.0
	 GWICE=WICE
       WICE=WICE+PDT-ERICE
       IF (WICE.LT.0.0) THEN
          ERICE=ERICE+WICE
          IF (ERICE.LE.0.0) ERICE=0.0
          WICE=0.0
       ENDIF

50     CONTINUE
! ------------------------------------------------------------------------------
!  Computation of discharge concentration
! ------------------------------------------------------------------------------
        CALL LINTRA (SICE,WICE,KICEH,TICEH,HFICE,RICE)
        CALL LINTRA (SICS,WICS,KICSH,TICSH,HFICS,RICS)

	  WICE=GWICE

! ------------------------------------------------------------------------------
! Code for debugging
! ------------------------------------------------------------------------------
        IF (DEBUG .GT. 1) THEN
22         format (  I9,F6.1,7F6.2,3F6.1,16X,F6.2 )
           rhg=wice-pdt-erice
           write (mdr,22) land,teck,pdt,rice,rics,ricf,wice,wics,wicf,
     *                    sice,sics,sicf,rhg
        ENDIF
60     CONTINUE

! ------------------------------------------------------------------------------
       RETURN
       END
! ------------------------------------------------------------------------------
