!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE KWLW2 (RGL,S,N,T,F,HOEHE,ALBEDO,G0,GT,S0,MONAT,
     *                     LANGWSF,KURZWSF,GAMMA,ZETA,QS,QL,
     *                      ELX,el,RHOL,RHOW,PT,L,IRGL,PRL)

! ---------------------------------------------------------------------
!      Computes long-wave and short-wave radiation
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      REAL RGL					!	Observed global radiation (W/M**2), daily sum
      REAL S					!	Sunshine duration (hours per day)
      REAL N					!	Cloudiness (1-8)
      REAL T					!	Air temperature (deg C)
	REAL F					!	Relative humidity (0..1)
	REAL HOEHE			!	elevation (m asl), slope (deg) and aspect (deg) of the HRU
	REAL ALBEDO				!	albedo
	REAL PRL				! Inflow in the runoff module
	INTEGER MONAT		!	day, month year
	INTEGER KURZWSF,LANGWSF			!	control the computation of the short and long wave radiation
		!     LANGWSF		0:PENMAN 1948 (S)         1:PENMAN 1954 (S)
		!			2:ANGSTROEM 1923 (S)      3:ANGSTROEM 1923 (N)
		!			4:FEUSSNER (S)            5:FEUSSNER (N)
		!			6:CZEPLAK (S)             7:CZEPLAK (N)
		!			8:BRUTSAERT (S)
		!     KURZWSF		0:PENMAN 1948 (S)         1:PENMAN 1954 (S)
		!			2:GOLCHERT 1981 (S)       3:KARSTEN UND CZEPLAK 1980 (N)
		!			4:MORECS 1981 (S)         5:Mit gem. standortkorr. RGL
	INTEGER IRGL			!	controls the computation of incoming radiation
					!	IRGL	= -1 to 3   use obs observed incoming radiation
					!		= 4	without global radiaiton, with sunshine duration
					!	        = 5	withoud sunshine radiaiton, with cloudiness

! ---------------------------------------------------------------------
!     Local variables
! ---------------------------------------------------------------------

      REAL CP					!   specifig heat at constant pressure (J/KG/K)
      REAL AT					!	reflectance coefficient of the surface for longwave radiation
      REAL SIGMA				!	STEFAN BOLTZMANN constant (W/M^2/K)
      REAL A					!	unit transformation (M/S --> MM/D)
      REAL NULLK				!	273.15, zero deg C in Kelvin temperature scale
	REAL HUNDERTK			!	373.15, 100 deg C in Kelvin temperature scale
      REAL P0					!	air pressure at 0 m asl
	REAL PT					!	air pressure at the elevation of the HRU hPa
      REAL GAMMA				!   psychometer constant hPa/K
      REAL ZETA					!	first derivative of saturated vapour pressure curve hPa/K
      REAL QS					!	short wave radiation balance (W/M^2)
      REAL QL					!	long wave radiation balance (W/M^2)
      REAL L					!	latent heat for evaporization for water (J/KG)
      REAL G0					!   potential incoming clear sky dircet radiation (W/M^2)
      REAL S0					!	maximal sunshine duration (hours)
      REAL RHOL					!	density of air (KG/M^3)
      REAL RHOW					!	density of water (KG/M^3)
      REAL EL					!	water vapour pressure (hPa)
      REAL ELX					!	saturation water vapour pressure (hPa)
      REAL RL					!	gas constant for dry air (J/KG/K)
      REAL RW					!	gas constant for water vapour (J/KG/K)

      REAL ACZEPLAK				!	Consant after CZEPLAK
      REAL BCZEPLAK				!	Consant after CZEPLAK (W/M^2)
      REAL EPS0					!	Consant after CZEPLAK (T^2/K^2)
      REAL G					!	earth acceleration (M/S^2)
      REAL GAMMAT				!	gradient elevation/temperature (KELVIN/M)
      REAL H(4)					!	constants for the determination of ELX and ZETA (RICHARDS 1971)

      REAL AGOLCHERT(12) 			!	Consant after Golchert
      REAL BGOLCHERT(12) 			!	Consant after Golchert

      real amorecs 				!	Consant after morecs
      real bmorecs(12) 				!	Consant after morecs
      real cmorecs 				!	Consant after morecs
      real eta 					!	Consant after morecs

      REAL TK	 				!	Air temperature in deg Kelvin
      REAL TR					!	temprerature coefficient
      REAL TP					!	Temperature at sea level
      REAL GT					!	Global radiaiton, computed through cloudiness
      REAL LDN0					!	reflectance of the atmosphere
      REAL X					!	constants for the determination of ELX and ZETA
      REAL DX					!	constants for the determination of ELX and ZETA
      INTEGER I					!	loop variable
      REAL NB					!	cloud coverage
      REAL BOLZ					!	coefficient after Bolz 1949

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

! Setting constants

      DATA H / 13.3185,-1.976,-0.6445,-0.1299 /
      RL=287.04
      RW=461.5
      CP =1004.0
      AT =0.95
      SIGMA =5.6703E-8
      A =8.64E7
      NULLK =273.15
      P0 =1013.25
      HUNDERTK =373.15
      G=9.81
      GAMMAT=0.5E-2

      ACZEPLAK =1.174
      BCZEPLAK =33.0
      EPS0 =9.365E-6

      data AGOLCHERT /0.22,0.23,0.23,0.23,0.24,0.24,
     *                0.24,0.23,0.25,0.21,0.19,0.20/
      data BGOLCHERT /0.62,0.55,0.56,0.53,0.53,0.53,
     *                0.49,0.49,0.51,0.58,0.60,0.59/
      data amorecs /0.24/
      data bmorecs /0.50,0.51,0.52,0.53,0.54,0.55,0.55,
     *              0.54,0.53,0.52,0.51,0.50/
      data cmorecs /0.15/

! ---------------------------------------------------------------------
      N=S/24

!     computing ELX and ZETA
      TK=T+NULLK
      TR=1.-(HUNDERTK/TK)
      TP=TK+GAMMAT*HOEHE/2.
      PT=P0*EXP(-G*HOEHE/RL/TP)
      L=2.5008E6*(NULLK/(TK))
      GAMMA=PT*CP/L/(RL/RW)
      RHOL=100.*PT/RL/TK
      RHOW=999.941
      X=0
      DX=0
      DO 1 I=1,4
        X=X+H(I)*(TR**I)
        DX=DX+I*H(I)*(TR**(I-1))
    1 CONTINUE
      ELX=P0*EXP(X)
      ZETA=HUNDERTK*ELX/TK/TK*DX
      EL=ELX*F

      IF (IRGL .LT. 4) THEN
!     RGL is observed
        QS=(1.0-ALBEDO)*RGL
        IF (KURZWSF .EQ. 5) GOTO 3
      ENDIF

      IF (KURZWSF.EQ.0) THEN
!       PENMAN 1948
	QS=(1.-ALBEDO)*G0*(0.18+0.55*S/S0)
      ENDIF

      IF (KURZWSF.EQ.1) THEN
!       PENMAN 1954
	QS=(1.-ALBEDO)*G0*(0.20+0.48*S/S0)
      ENDIF

      IF (KURZWSF.EQ.2) THEN
!       GOLCHERT 1981
	QS=(1.-ALBEDO)*G0*(AGOLCHERT(MONAT)+BGOLCHERT(MONAT)*S/S0)
      ENDIF

      IF (KURZWSF.EQ.3) THEN
!       KARSTEN UND CZEPLAK 1980 (with cloudiness)
	GT=G0*(1.-0.70*((N)**3.))
        QS=(1.-ALBEDO)*GT*(1-0.55*N-0.25*(N**4.))
      ENDIF

      IF (KURZWSF.EQ.4) THEN
!       MORECS 1981 (with sunshine duration)
	eta=1.
	if(s.eq.0.0) eta=0.
	QS=(1.-ALBEDO)*G0*(eta*(amorecs+bmorecs(monat)*s/s0) +
     +                       cmorecs*(1.-eta))
      ENDIF

      IF (KURZWSF.EQ.6) THEN
!       SCHULLA - 1997
	QS=(1.-ALBEDO)*G0*(0.23+(1.77*S/S0)-
     *   (2.28*((S/S0)**2))+(1.28*((S/S0)**3)))
      ENDIF

	GT=QS/(1.-ALBEDO)

3     CONTINUE

      IF (LANGWSF.EQ.0) THEN
!       PENMAN 1948
	QL=-SIGMA*(TK**4.)*(0.56-0.09*SQRT(0.75*EL))*(0.1+0.9*S/S0)
      ENDIF

      IF (LANGWSF.EQ.1) THEN
!       PENMAN 1954
	QL=-SIGMA*(TK**4.)*(0.47-0.077*SQRT(0.75*EL))*(0.2+0.8*S/S0)
      ENDIF

      IF ((LANGWSF.EQ.2).OR.(LANGWSF.EQ.3)) THEN
!       ANGSTROEM
	LDN0=SIGMA*(TK**4.)*(0.79-0.174*(10.**(-0.055*0.75*EL)))
      ENDIF

      IF ((LANGWSF.EQ.4).OR.(LANGWSF.EQ.5)) THEN
!       FEUSSNER
	LDN0=SIGMA*(TK**4.)*(1.-10.**(-0.424*((0.75*EL)**0.2)))
      ENDIF

      IF ((LANGWSF.EQ.6).OR.(LANGWSF.EQ.7)) THEN
!       CZEPLAK
	LDN0=ACZEPLAK*EPS0*SIGMA*(TK**6.)-BCZEPLAK
      ENDIF

      IF ((LANGWSF.EQ.2).OR.(LANGWSF.EQ.4).OR.(LANGWSF.EQ.6)) THEN
!     with sunshine duration
	QL=AT*(LDN0-SIGMA*(TK**4.))*(0.2+0.8*S/S0)
      ENDIF

      IF ((LANGWSF.EQ.3).OR.(LANGWSF.EQ.5).OR.(LANGWSF.EQ.7)) THEN
!     with cloudiness
      QL=AT*(LDN0*(1.+0.27*((N)**2.))-SIGMA*(TK**4.))
      ENDIF

      IF (LANGWSF.EQ.8) THEN
!       BRUTSAERT
	QL=AT*SIGMA*(TK**4.)*(1.28*(EL/TK)**(1./7.)-1.)*(0.2+0.8*S/S0)
      ENDIF

      IF (LANGWSF.EQ.9) THEN
!       SCHULLA
	QL=-SIGMA*(TK**4.)*(0.52-(0.065*SQRT(EL)))*
     *  (0.23+(0.77*S/S0))
      ENDIF

      IF (LANGWSF.EQ.20) THEN
!       BRUNT (1932) und BOLZ (1949)
	QL = SIGMA * TK**4 * (0.610+0.050*EL**0.5)
	!Nach Brunt (1932), wolkenloser Fall..
	NB=1-S
	NB=MIN(1.0,NB)
	NB=MAX(0.0,NB)

	IF (PRL.EQ.0.0) THEN	!Koeffizienten nach Escher-Vetter (2001)
	        BOLZ = 0.40     !ESCH_V=0.17
		IF (NB.LE.0.2) BOLZ = 0.08     !ESCH_V=0.08
		IF (NB.GE.0.6) BOLZ = 0.35     !ESCH_V=0.20
	 	  ELSE
		    BOLZ = 0.30     !ESCH_V=0.24
		ENDIF
		QL = QL * (1 + BOLZ*NB**2) !Nach Bolz (1949), Modifikation durch Wolken
      ENDIF

! ---------------------------------------------------------------------

      RETURN
      END
