!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! After each TreeMig run (except spin-up), this subroutine updates the land
! use class of each cell, based on biomass (forest/non-forest) and species
! composition (coniferous/broadleaf/mixed forest). Simplified after Trm2PREVAH
! (MSc thesis P. Schattan)
!-------------------------------------------------------------------------------

subroutine lcUpdate

use prevahglobals, only:      ncol,nrow, &
                              landin, &  ! Land use grid
                              rnodat
use all_par, only:      cell, &  ! Grid of TreeMig state variables, of type CurrStateInCell
                        CurrStateInCell, &  ! Derived type
                        maxspc, &            ! Maximum number of species
                        maxhc, &                ! Maximum number of height classes
                        spec, &               ! Array of sets of species-specific parameters
                        DfSpc, &                   ! Derived type
                        stockability
use forhycsglobals, only: biomass2d, lcChange
use MPIVars, only: firstRow, lastRow

implicit none

integer ::  land, &  ! Land use class of current cell
            landNew, &
            xcoor, ycoor, i, j, &
            numCon, numBro    ! Number of conifers and broadleaf trees

real ::     bioCon, bioBro

real :: curBio  ! Total tree biomass in current cell
real, dimension(30) :: bioWrite


real, parameter :: fnfBio = 65.0  ! Parameter value from Schattan 2012

logical :: dbfile  ! For debug output files

!=======================================================================

lcChange = rnodat

! Iterate over grid cells
do ycoor = lastRow, firstRow, -1
do xcoor = 1, ncol
  land = landin(xcoor,ycoor)
! If current cell belongs to an unforestable land cover class, set biomass
! to zero and skip
  select case (land)
    case(1,2,6,7,13,14,15,18,19,20,22,28,29)
      do i = 1,maxspc
      do j = 1,maxhc
      ! Careful: grids in PREVAH are indexed as (x,y) and in TreeMig as (lat,lon) = (y,x)
        cell(ycoor,xcoor)%sp(i)%numin(j) = 0.0
      end do
      end do

      landNew = land

! If current cell belongs to a forest class, first verify it still fulfills criteria.
! Otherwise, assign LC class 8 (= shrub = degraded forest in Schattan 2012).
    case(3,4,5)
      ! biomass2d is PREVAH-indexed
      curBio = sum(biomass2d(xcoor,ycoor,1:30))

      if(curBio .ge. fnfBio) then


        bioCon = sum(biomass2d(xcoor,ycoor,1:7))
        bioBro = sum(biomass2d(xcoor,ycoor,8:30))

        if(bioCon/(bioCon+bioBro) .lt. 0.45) then    ! Cut-offs from Schattan 2012
          landNew = 4
        else if  (bioCon/(bioCon+bioBro) .le. 0.8) then
          landNew = 5
        else
          landNew = 3
        end if
      else  ! Not enough biomass to be considered forest anymore
        landNew = 8
      end if


! Write changes to grid lcChange
      if(land == landNew) then
        lcChange(xcoor,ycoor) = 0.0
      else
        select case(land)
          case(3)
            if(landNew == 8) then
              lcChange(xcoor,ycoor) = 1.0
            else if(landNew == 5) then
              lcChange(xcoor,ycoor) = 10.0
            else if(landNew == 4) then
              lcChange(xcoor,ycoor) = 11.0
            end if

          case(4)
            if(landNew == 8) then
              lcChange(xcoor,ycoor) = 3.0
            else if(landNew == 5) then
              lcChange(xcoor,ycoor) = 14.0
            else if(landNew == 3) then
              lcChange(xcoor,ycoor) = 15.0
            end if

          case(5)
            if(landNew == 8) then
              lcChange(xcoor,ycoor) = 2.0
            else if(landNew == 3) then
              lcChange(xcoor,ycoor) = 12.0
            else if(landNew == 4) then
              lcChange(xcoor,ycoor) = 13.0
            end if
        end select
      end if


! If cell belongs to a potentially forested class, check whether it now fulfills
! forest criteria. If not, continue to next cell.
    case (8,21,23,24,25,26)
      if(stockability(ycoor,xcoor) == 1.0) then
      ! biomass2d is PREVAH-indexed
      curBio = sum(biomass2d(xcoor,ycoor,1:maxspc))

      if(curBio .ge. fnfBio) then
      ! If it does, determine forest type.

        bioCon = sum(biomass2d(xcoor,ycoor,1:7))
        bioBro = sum(biomass2d(xcoor,ycoor,8:30))

        if(bioCon/(bioCon+bioBro) .lt. 0.45) then    ! Cut-offs from Schattan 2012
          landNew = 4
        else if  (bioCon/(bioCon+bioBro) .le. 0.8) then
          landNew = 5
        else
          landNew = 3
        end if
      else  ! Not enough biomass to be considered forest - no LC class change
        landNew = land
      end if
! Write changes to grid lcChange
      if(land .ne. landNew) then
        select case(land)
          case(8)
            if(landNew == 3) then
              lcChange(xcoor,ycoor) = 30.0
            else if(landNew == 5) then
              lcChange(xcoor,ycoor) = 31.0
            else if(landNew == 4) then
              lcChange(xcoor,ycoor) = 32.0
            end if
          case default
            if(landNew == 3) then
              lcChange(xcoor,ycoor) = 20.0
            else if(landNew == 5) then
              lcChange(xcoor,ycoor) = 21.0
            else if(landNew == 4) then
              lcChange(xcoor,ycoor) = 22.0
            end if
        end select
      end if
      else  ! No stockability
        do i = 1,maxspc
        do j = 1,maxhc
      ! Careful: grids in PREVAH are indexed as (x,y) and in TreeMig as (lat,lon) = (y,x)
          cell(ycoor,xcoor)%sp(i)%numin(j) = 0.0
        end do
        end do
        landNew = land
        lcChange(xcoor,ycoor) = 0.0
      end if

      ! Cases for no-data cells
      case(0)
        landnew = 0
      case(-9999)
        landnew = 0

  end select

  ! Update land use grid

  landin(xcoor,ycoor) = landNew

end do
end do






end subroutine lcUpdate
