!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation for parallel execution

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

subroutine MTDinit

! ------------------------------
! REFERENCES
!   Gruber, S. (2007, in press): MTD: a mass-conserving algorithm to parameterize
!   gravitational transport and deposition processes using digital elevation models
!   Water Resources Research.
!
!   Gruber, S. & Peckham, S. (2007, in press): Land surface parameters and objects
!   specfic to hydrlogy. In: Geomorphometry: concepts, software, applications,
!   edited by: Hengl, T. & Reuter, H., JRC, Ispra, Italy.
!
!   Gruber, S., Huggel, C. & Pike, R. (2007, in press): Modeling mass movements and
!   landslide susceptibility using geomorphometry. In: Geomorphometry: concepts,
!   software, applications, edited by: Hengl, T. & Reuter, H., JRC, Ispra, Italy.
!
! MODIFICATION HISTORY:
!   Created by Stephan Gruber (2005-2007)
!
!----------------------------------------------------------------------------
!
!    Fortran M_Zappa, WSL, June 2010
! ------------------------------
!
!    MPI version: M. Speich, WSL, January 2015
!
!    Jan 2018: Modification for cases when the study domain is rectangular
!              instead of just a catchment. Snow accumulating at the edges
!              is subtracted ("leaves" the area). M. Speich
! ------------------------------

use prevahglobals
use MPIvars
use mpi

implicit none

! MTD variables
integer:: shi
character*256 :: afile
integer, dimension(9) :: shiftx,shifty
integer :: lowdhm
real :: dmax,slp,midhm
real, dimension(9) :: dhms

! Variables used for MPI version
integer:: iOrder
real,dimension(ncol,nrow):: tempdhm
integer, dimension(2) :: temploc
!integer :: numNonNA  ! Number of cells that are neither NA nor on the edge of domain - Variable is declared in prevahglobals now

slp_lim = 90.0
dep_lim = 9000.0
gamma = 0.5

allocate (dmax2D(ncol,nrow))
allocate (dir2D(ncol,nrow))
allocate(mtdmask(ncol,nrow))  ! HACK

dmax2D=rnodat
dir2D=rnodat

shiftx(1)=1
shiftx(2)=0
shiftx(3)=-1
shiftx(4)=1
shiftx(5)=0
shiftx(6)=-1
shiftx(7)=1
shiftx(8)=0
shiftx(9)=-1

shifty(1)=1
shifty(2)=1
shifty(3)=1
shifty(4)=0
shifty(5)=0
shifty(6)=0
shifty(7)=-1
shifty(8)=-1
shifty(9)=-1

! --------------------------------
! DMAX: Maximum deposition

do ycoor=nrow,1,-1
do xcoor=1,ncol
  if (int(gezg(xcoor,ycoor)) .ne. int(rnodat)) then
    slp=fslope(xcoor,ycoor)
    dmax = max(0.0,(1.0-(max(1.0,slp)/slp_lim)**gamma))*dep_lim* (1.0-(max(1.0,slp)/slp_lim))+100
    if (landin(xcoor,ycoor) .eq. 1) dmax=0  ! No snow redistribution on water cells, otherwise some cells will have an insane amount of runoff (snow melts immediately)
    dmax2D(xcoor,ycoor)=dmax
  endif
end do
end do

if (.not. allocated(write2d)) ALLOCATE (WRITE2D(ncol,nrow))
write2D=dmax2D

! ------------------------------------
! Write DMAX grid

if((rank==1) .or. (doParallel==.false.)) then

  write (afile,'(A)') trim(NADGKW)//'-dmax.bin'
  print*, 'Writing ..',trim(afile)

  open (78,file=afile,status='unknown',form='BINARY')
  uwas=78
  include 'wstsr.inc'
end if

!---------------------------------------------------
! Lowest neighboring cell (determines direction of snow transport)

!do ycoor=(nrow-1),2,-1
!do xcoor=2,(ncol-1)
 do ycoor = nrow,1,-1
 do xcoor = 1,ncol
  lowdhm=0
  midhm=999999.
  if (int(gezg(xcoor,ycoor)) .ne. int(rnodat)) then
    if((xcoor==1).or.(xcoor==ncol).or.(ycoor==1).or.(ycoor==nrow)) then
    ! Special case: cells at the edge
      lowdhm = 0
    else
      do shi=1,9
        dhms(shi)=fhoehe((xcoor+shiftx(shi)),(ycoor+shifty(shi)))
	  if (dhms(shi) .lt. 0) cycle
	  if (dhms(shi) .lt. midhm) then
	    lowdhm=shi
	    midhm=dhms(shi)
	  end if
      end do
    end if  ! Cells at the edge
    dir2D(xcoor,ycoor)=float(lowdhm)
  endif
end do
end do

if (.not. allocated(write2d)) ALLOCATE (WRITE2D(ncol,nrow))
write2D=dir2D

! ------------------------------------
! Write DIR grid

if((rank==1) .or. (doParallel==.false.)) then
  write (afile,'(A)') trim(NADGKW)//'-dir.bin'
  print*, 'Writing ..',trim(afile)

  open (78,file=afile,status='unknown',form='BINARY')
  uwas=78
  include 'wstsr.inc'
 end if

! ------------------------------------

! Order (sort cells from highest to lowest)

numNonNA = 0
do xcoor = 1,ncol
do ycoor = 1,nrow
 ! if ((int(gezg(xcoor,ycoor)) .ne. int(rnodat)) .and. (xcoor .ne. 1) .and. (xcoor.ne.ncol) &
 ! .and. (ycoor .ne. 1) .and.(ycoor .ne. nrow)) numNonNA = numNonNA + 1
 if(int(gezg(xcoor,ycoor)) .ne. int(rnodat)) numNonNA = numNonNA + 1
end do
end do

!allocate(mtdOrder(3,((ncol*nrow)-2*nrow-2*ncol+4)))  ! Only works if grid is rectangular, without NAs
 allocate(mtdOrder(3,numNonNA))

iOrder = 1
mtdOrder = 0

! Cells at the edge of domain are not considered (no redistribution happens
! from those cells)
tempdhm = fhoehe
!tempdhm(1,:) = rnodat
!tempdhm(ncol,:) = rnodat
!tempdhm(:,1) = rnodat
!tempdhm(:,nrow) = rnodat
!do i = 1,((ncol*nrow)-(2*nrow)-(2*ncol)+4)     ! Only works if grid is rectangular, without NAs
do i = 1,numNonNA
  temploc = maxloc(tempdhm, mask=tempdhm .ne. rnodat)
  mtdOrder(1,iOrder) = temploc(1)
  mtdOrder(2,iOrder) = temploc(2)
  tempdhm(temploc(1),temploc(2)) = rnodat
  iOrder = iOrder + 1
end do

! Arrays to keep track of snow transfer between cells
allocate(transout(4,numNonNA))
allocate(transoutM(4,numNonNA,numslaves))
mtdfh = 1815

! In the parallel version, MTDorder has a 3rd column containing the number of the
! process in which each cell is located

do i = 1,numslaves
  where((mtdOrder(2,:) .ge. firstRowArray(i)) .and. (mtdOrder(2,:) .ge. lastRowArray(i))) &
  mtdOrder(3,:) = numslaves
end do

! ------------------------------------

mtdmask = 0
mtdmask(1,:) = rnodat
mtdmask(ncol,:) = rnodat
mtdmask(:,1) = rnodat
mtdmask(:,nrow) = rnodat


return
! Bye bye
end

! ----------------------------------------------------------------
! ----------------------------------------------------------------


subroutine MTD
! ------------------------------
! REFERENCES
!   Gruber, S. (2007, in press): MTD: a mass-conserving algorithm to parameterize
!   gravitational transport and deposition processes using digital elevation models
!   Water Resources Research.
!
!   Gruber, S. & Peckham, S. (2007, in press): Land surface parameters and objects
!   specfic to hydrlogy. In: Geomorphometry: concepts, software, applications,
!   edited by: Hengl, T. & Reuter, H., JRC, Ispra, Italy.
!
!   Gruber, S., Huggel, C. & Pike, R. (2007, in press): Modeling mass movements and
!   landslide susceptibility using geomorphometry. In: Geomorphometry: concepts,
!   software, applications, edited by: Hengl, T. & Reuter, H., JRC, Ispra, Italy.
!
! MODIFICATION HISTORY:
!   Created by Stephan Gruber (2005-2007)
!
!----------------------------------------------------------------------------
!
!    Fortran M_Zappa, WSL, June 2010
! ------------------------------
!
!    MPI version: M. Speich, WSL, January 2015
!
!    Jan 2018: Modification for cases when the study domain is rectangular
!              instead of just a catchment. Snow accumulating at the edges
!              is subtracted ("leaves" the area). M. Speich
! ------------------------------


use prevahglobals
use MPIvars
use mpi
use fp_ops
use fp_compare

implicit none

integer :: shi,shl
integer, dimension(9) :: shiftx,shifty
real :: dmax
real :: ovalue, lateral
real,dimension(ncol,nrow) :: snowGrid
logical,dimension(ncol,nrow) :: flag  ! Marks cells where snow redistribution has already taken place
real :: nextfak, cinmit, rsum, avg
integer :: len, cnt
real:: tolerance
real, allocatable :: reshVec(:)

integer :: numTrans  ! number of cells to be transferred
integer :: xc, yc
type(xyztype), dimension(3) :: trans
real :: transSnow

! Tolerance for comparing snow cover with maximum deposition, to compensate
! for possible loss of numerical precision
tolerance = 0.001


shiftx(1)=1
shiftx(2)=0
shiftx(3)=-1
shiftx(4)=1
shiftx(5)=0
shiftx(6)=-1
shiftx(7)=1
shiftx(8)=0
shiftx(9)=-1

shifty(1)=1
shifty(2)=1
shifty(3)=1
shifty(4)=0
shifty(5)=0
shifty(6)=0
shifty(7)=-1
shifty(8)=-1
shifty(9)=-1

flag = .false.

snowGrid = ssnowtf

where(int(gezg) .eq. int(rnodat)) snowGrid = rnodat
len = nrowLocal*ncol
allocate(reshVec(len))
reshVec = reshape(snowGrid(:,firstRow:lastRow),(/len/))
rsum = kahan_sum(reshVec, len, rnodat)
if(allocated(reshVec)) deallocate(reshVec)

avg = rsum / cntGlobal
cinmit=avg

!---------------------------------------------------
! REDISTRIBUTION
! First part: lateral redistribution (in the case of sinks, or if there is too much snow accumulated)

!print*, "Starting MTD"

!do i = 1,((ncol*nrow)-(2*nrow)-(2*ncol)+4)
do i = 1,numNonNA
  if(i > 1) then
  if(mtdOrder(2,i-1) == lastRow+1) then
    call MPI_RECV(numTrans,1,MPI_INTEGER,rank+1,33,MPI_COMM_WORLD,status,ierr)
    if(numTrans > 0) then
!print*, rank, "recv", numTrans, "cells from", mtdOrder(1,i-1), mtdOrder(2,i-1), "1st part"
      call MPI_RECV(trans,3,MPIxyztype,rank+1,34,MPI_COMM_WORLD,status,ierr)
    end if

    do j = 1,numTrans
      xc = trans(j)%x
      yc = trans(j)%y
      transSnow = trans(j)%z
!print*, "recv1", xc, yc, transSnow
!print*, "snowGrid", snowGrid(xc,yc), transSnow, snowGrid(xc,yc)+transSnow
      snowGrid(xc,yc) = snowGrid(xc,yc) + transSnow
      trans(j)%x = 0
      trans(j)%y = 0
      trans(j)%z = 0.0
    end do  ! loop over numTrans
  else if (mtdOrder(2,i-1) == firstRow-1) then
    call MPI_RECV(numTrans,1,MPI_INTEGER,rank-1,31,MPI_COMM_WORLD,status,ierr)
    if(numTrans > 0) then
!print*, rank, "recv", numTrans, "cells from", mtdOrder(1,i-1), mtdOrder(2,i-1), "1st part"
      call MPI_RECV(trans,3,MPIxyztype,rank-1,32,MPI_COMM_WORLD,status,ierr)
    end if

    do j = 1,numTrans
      xc = trans(j)%x
      yc = trans(j)%y
      transSnow = trans(j)%z
!print*, "snowGrid", snowGrid(xc,yc), transSnow, snowGrid(xc,yc)+transSnow
      snowGrid(xc,yc) = snowGrid(xc,yc) + transSnow
!print*, "recv1", xc, yc, transSnow
      trans(j)%x = 0
      trans(j)%y = 0
      trans(j)%z = 0.0
    end do  ! loop over numTrans
  end if  ! mtdOrder(2,i-1)
  end if  !(i>1)

  numTrans = 0

  if((mtdOrder(2,i) .lt. firstRow) .or. (mtdOrder(2,i) .gt. lastRow)) cycle  ! Current cell on the list is not part of this subdomain

  xcoor = mtdOrder(1,i)
  ycoor = mtdOrder(2,i)

      transout(1,i) = real(xcoor)
      transout(2,i) = real(ycoor)
      transout(3,i) = 0.0
      transout(4,i) = 0.0

!       if((xcoor==86).and.(ycoor==33)) then
!         print*, "MTD debug"
!         print*, "snowgrid", snowgrid(xcoor, ycoor)
!         print*, "dir2d", dir2d(xcoor, ycoor)
!         print*, "dmax", dmax2D(xcoor,ycoor)
!
!       end if

  if (int(snowGrid(xcoor,ycoor)) .ne. int(rnodat)) then
!    if(flag(xcoor,ycoor)==.true.) cycle



! If current cell is a sink or snow cover exceeds a threshold:
! Snow distributed laterally among all non-NA neighboring cells
      shl=dir2D(xcoor,ycoor)
      ovalue=snowGrid(xcoor,ycoor)
      dmax=dmax2D(xcoor,ycoor)
    if((shl == 5) .or. (fp_gt(ovalue,3000.0,tolerance))) then
      if(fp_gt(ovalue,0.5*dmax,tolerance)) then
        lateral=max(0.1*ovalue,(ovalue-dmax))

        if(shl==0) then
        ! Edge cells: excess snow "leaves" the area
          ovalue = ovalue - lateral
        else

        do shi = 1,9
          if(shi==5) cycle
          if(landin(xcoor+shiftx(shi),ycoor+shifty(shi)) == 1) cycle  ! Avoid redistribution on liquid water

          if(ycoor == firstRow) then
            select case(shi)
            case(7,8,9)
              if(int(gezg(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then
!              if(int(mtdmask(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then  ! HACK
              numTrans = numTrans + 1
              trans(numTrans)%x = xcoor+shiftx(shi)
              trans(numTrans)%y = ycoor+shifty(shi)
              trans(numTrans)%z = lateral/8
              ovalue = ovalue-lateral/8
!              end if  ! HACK
              end if  ! gezg

            case default
              if(int(gezg(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then
!              if(int(mtdmask(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then  ! HACK
              snowGrid((xcoor+shiftx(shi)),(ycoor+shifty(shi)))= &
      snowGrid((xcoor+shiftx(shi)),(ycoor+shifty(shi))) + lateral/8
              ovalue=ovalue-lateral/8
!              end if  ! HACK
              end if  ! gezg

            end select


          else if (ycoor == lastRow) then
            select case(shi)
            case(1,2,3)
              if(int(gezg(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then
!              if(int(mtdmask(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then  ! HACK
              numTrans = numTrans + 1
              trans(numTrans)%x = xcoor+shiftx(shi)
              trans(numTrans)%y = ycoor+shifty(shi)
              trans(numTrans)%z = lateral/8
              ovalue = ovalue-lateral/8



!              end if  ! HACK
              end if  ! gezg

            case default
              if(int(gezg(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then
!              if(int(mtdmask(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then  ! HACK
              snowGrid((xcoor+shiftx(shi)),(ycoor+shifty(shi)))= &
      snowGrid((xcoor+shiftx(shi)),(ycoor+shifty(shi))) + lateral/8
              ovalue=ovalue-lateral/8
!              end if  ! HACK
              end if  ! gezg

            end select



!                     if((xcoor==86).and.(ycoor==33)) then
!         print*, "MTD ex1"
!         print*, "lateral", lateral
!         print*, "ovalue", ovalue
!                   end if


          else  ! Cell is neither on firstRow nor on lastRow
            if(int(gezg(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then
!            if(int(mtdmask(xcoor+shiftx(shi),ycoor+shifty(shi))) .ne. int(rnodat)) then  ! HACK
            snowGrid((xcoor+shiftx(shi)),(ycoor+shifty(shi)))= &
      snowGrid((xcoor+shiftx(shi)),(ycoor+shifty(shi))) + lateral/8
            ovalue=ovalue-lateral/8

!            end if  ! HACK
            end if  ! gezg
          end if  ! firstRow/lastRow/neither
        end do  ! shi

        end if    ! shl==0 or else

      end if  ! (fp_gt(ovalue,0.5*dmax,tolerance))
      flag(xcoor,ycoor)=.true.
    end if  ! (shl == 5) .or. (fp_gt(ovalue,3000.0,tolerance))

!    snowGrid(xcoor,ycoor) = ovalue
!  end if  ! snowGrid(xcoor,ycoor) .ne. rnodat
!
!  if((ycoor == firstRow) .and. (rank .gt. 1)) then
!    call MPI_SEND(numTrans,1,MPI_INTEGER,rank-1,33,MPI_COMM_WORLD,ierr)
!    if(numTrans > 0) then
!print*, mtdOrder(1,i), mtdOrder(2,i), "send", numTrans, "cells to", rank-1, "1st part"
!      call MPI_SEND(trans,3,MPIxyztype,rank-1,34,MPI_COMM_WORLD,ierr)
!print*, "send 1", trans(numTrans)%x,trans(numTrans)%y,trans(numTrans)%z
!    end if
!
!  else if((ycoor == lastRow) .and. (rank .lt. numslaves)) then
!    call MPI_SEND(numTrans,1,MPI_INTEGER,rank+1,31,MPI_COMM_WORLD,ierr)
!    if(numTrans > 0) then
!print*, mtdOrder(1,i), mtdOrder(2,i), "send", numTrans, "cells to", rank-1, "1st part"
!      call MPI_SEND(trans,3,MPIxyztype,rank+1,32,MPI_COMM_WORLD,ierr)
!print*, "send 1", trans(numTrans)%x,trans(numTrans)%y,trans(numTrans)%z
!    end if
!  end if
!
!  do j = 1,numTrans
!    trans(j)%x = 0
!    trans(j)%y = 0
!    trans(j)%z = 0.0
!  end do
!  numTrans = 0
!
!end do

! In the eventuality that snow is being transferred laterally from the last
! cell in the list (i.e. the lowest cell of the whole domain) to another subdomain.
!if((mtdOrder(2,((ncol*nrow)-(2*nrow)-(2*ncol)+4)) == lastRow+1)) then
!  call MPI_RECV(numTrans,1,MPI_INTEGER,rank+1,33,MPI_COMM_WORLD,status,ierr)
!  if(numTrans > 0) then
!print*, rank, "recv", numTrans, "cells from", mtdOrder(1,i-1), mtdOrder(2,i-1), "end 1"
!    call MPI_RECV(trans,3,MPIxyztype,rank+1,34,MPI_COMM_WORLD,status,ierr)
!  end if
!
!  do j = 1,numTrans
!    xc = trans(j)%x
!    yc = trans(j)%y
!    transSnow = trans(j)%z
!    snowGrid(xc,yc) = snowGrid(xc,yc) + transSnow
!print*, "recvlast", xc, yc, transSnow
!
!    trans(j)%x = 0
!    trans(j)%y = 0
!    trans(j)%z = 0.0
!  end do  ! loop over numTrans
!else if((mtdOrder(2,((ncol*nrow)-(2*nrow)-(2*ncol)+4)) == firstRow-1)) then
!  call MPI_RECV(numTrans,1,MPI_INTEGER,rank-1,31,MPI_COMM_WORLD,status,ierr)
!  if(numTrans > 0) then
!print*, rank, "recv", numTrans, "cells from", mtdOrder(1,i-1), mtdOrder(2,i-1), "end 1"
!    call MPI_RECV(trans,3,MPIxyztype,rank-1,32,MPI_COMM_WORLD,status,ierr)
!  end if
!
!  do j = 1,numTrans
!    xc = trans(j)%x
!    yc = trans(j)%y
!    transSnow = trans(j)%z
!    snowGrid(xc,yc) = snowGrid(xc,yc) + transSnow
!print*, "recvlast", xc, yc, transSnow
!
!    trans(j)%x = 0
!    trans(j)%y = 0
!    trans(j)%z = 0.0
!  end do  ! loop over numTrans
!end if

!---------------------------------------------------
! Second part of redistribution (gravitative redistribution)

!do i = 1,((ncol*nrow)-(2*nrow)-(2*ncol)+4)
!  if(i > 1) then
!  if(mtdOrder(2,i-1) == lastRow+1) then
!    call MPI_RECV(numTrans,1,MPI_INTEGER,rank+1,33,MPI_COMM_WORLD,status,ierr)
!    if(numTrans > 0) then
!print*, rank, "recv", numTrans, "cells from", mtdOrder(1,i-1), mtdOrder(2,i-1), "2nd part"
!      call MPI_RECV(trans,3,MPIxyztype,rank+1,34,MPI_COMM_WORLD,status,ierr)
!    end if
!
!    do j = 1,numTrans
!      xc = trans(j)%x
!      yc = trans(j)%y
!      transSnow = trans(j)%z
!      snowGrid(xc,yc) = snowGrid(xc,yc) + transSnow
!print*, "recv2", xc, yc, transSnow
!
!      trans(j)%x = 0
!      trans(j)%y = 0
!      trans(j)%z = 0.0
!    end do  ! loop over numTrans
!  else if (mtdOrder(2,i-1) == firstRow-1) then
!    call MPI_RECV(numTrans,1,MPI_INTEGER,rank-1,31,MPI_COMM_WORLD,status,ierr)
!    if(numTrans > 0) then
!print*, rank, "recv", numTrans, "cells from", mtdOrder(1,i-1), mtdOrder(2,i-1), "2nd part"
!      call MPI_RECV(trans,3,MPIxyztype,rank-1,32,MPI_COMM_WORLD,status,ierr)
!    end if
!
!    do j = 1,numTrans
!      xc = trans(j)%x
!      yc = trans(j)%y
!      transSnow = trans(j)%z
!      snowGrid(xc,yc) = snowGrid(xc,yc) + transSnow
!print*, "recv2", xc, yc, transSnow
!
!      trans(j)%x = 0
!      trans(j)%y = 0
!      trans(j)%z = 0.0
!    end do  ! loop over numTrans
!  end if  ! mtdOrder(2,i-1)
!  end if  !(i>1)

!  numTrans = 0

!  if((mtdOrder(2,i) .lt. firstRow) .or. (mtdOrder(2,i) .gt. lastRow)) cycle  ! Current cell on the list is not part of this subdomain

!  xcoor = mtdOrder(1,i)
!  ycoor = mtdOrder(2,i)
!
!  if ((int(snowGrid(xcoor,ycoor)) .ne. int(rnodat)) .and. (flag(xcoor,ycoor)==.false.)) then



! "Core" of MTD: transfer excess snow to lowest neighboring cell (direction of transfer
! indicated by shl).
    if((fp_gt(ovalue,dmax,tolerance)) .and. (flag(xcoor,ycoor)==.false.).and.(shl.ne.0)) then



! For output
      transout(1,i) = real(xcoor)
      transout(2,i) = real(ycoor)
      !print*, "in MTD:", i, xcoor, ycoor
      transout(3,i) = ovalue-dmax
      transout(4,i) = real(shl)
      if(ycoor == firstRow) then
        select case(shl)
        case(7,8,9)
!          if(int(mtdmask(xcoor+shiftx(shl),ycoor+shifty(shl))) .ne. int(rnodat)) then  ! HACK
              numTrans = numTrans + 1
              trans(numTrans)%x = xcoor+shiftx(shl)
              trans(numTrans)%y = ycoor+shifty(shl)
              trans(numTrans)%z = ovalue-dmax
              ovalue = dmax
!          end if  ! HACK

        case default
!          if(int(mtdmask(xcoor+shiftx(shl),ycoor+shifty(shl))) .ne. int(rnodat)) then  ! HACK
            snowGrid((xcoor+shiftx(shl)),(ycoor+shifty(shl)))= &
   snowGrid((xcoor+shiftx(shl)),(ycoor+shifty(shl)))+ovalue-dmax
            ovalue=dmax
!          end if  ! HACK

        end select

      else if(ycoor == lastRow) then
        select case(shl)
        case(1,2,3)
!          if(int(mtdmask(xcoor+shiftx(shl),ycoor+shifty(shl))) .ne. int(rnodat)) then  ! HACK
              numTrans = numTrans + 1
              trans(numTrans)%x = xcoor+shiftx(shl)
              trans(numTrans)%y = ycoor+shifty(shl)
              trans(numTrans)%z = ovalue-dmax
              ovalue = dmax
!          end if  ! HACK

        case default
!          if(int(mtdmask(xcoor+shiftx(shl),ycoor+shifty(shl))) .ne. int(rnodat)) then  ! HACK
            snowGrid((xcoor+shiftx(shl)),(ycoor+shifty(shl)))= &
   snowGrid((xcoor+shiftx(shl)),(ycoor+shifty(shl)))+ovalue-dmax
            ovalue=dmax
!          end if  ! HACK

        end select

      if((xcoor==86).and.(ycoor==33)) then
        print*, "MTD debug ex 2"
        print*, "ovalue", ovalue
        print*, "dmax", dmax
        print*, "trans(numTrans)%z", trans(numTrans)%z

      end if


        else  ! Cell is neither on firstRow nor on lastRow
!          if(int(mtdmask(xcoor+shiftx(shl),ycoor+shifty(shl))) .ne. int(rnodat)) then  ! HACK
            snowGrid((xcoor+shiftx(shl)),(ycoor+shifty(shl)))= &
snowGrid((xcoor+shiftx(shl)),(ycoor+shifty(shl)))+ovalue-dmax
            ovalue=dmax
!          end if  ! HACK

      end if  ! firstRow/lastRow/neither

    end if  ! (fp_gt(ovalue,dmax,tolerance))

    snowGrid(xcoor,ycoor) = ovalue  ! Assignment valid in both cases (sink or )
  end if  ! (int(snowGrid(xcoor,ycoor)) .ne. int(rnodat))

  if((ycoor == firstRow) .and. (rank .gt. 1)) then
    call MPI_SEND(numTrans,1,MPI_INTEGER,rank-1,33,MPI_COMM_WORLD,ierr)
    if(numTrans > 0) then
!print*, mtdOrder(1,i), mtdOrder(2,i), "send", numTrans, "cells to", rank-1, "2nd part"
      call MPI_SEND(trans,3,MPIxyztype,rank-1,34,MPI_COMM_WORLD,ierr)
!print*, "send 2", trans(numTrans)%x,trans(numTrans)%y,trans(numTrans)%z

    end if

  else if((ycoor == lastRow) .and. (rank .lt. numslaves)) then
    call MPI_SEND(numTrans,1,MPI_INTEGER,rank+1,31,MPI_COMM_WORLD,ierr)
    if(numTrans > 0) then
!print*, mtdOrder(1,i), mtdOrder(2,i), "send", numTrans, "cells to", rank-1, "2nd part"
      call MPI_SEND(trans,3,MPIxyztype,rank+1,32,MPI_COMM_WORLD,ierr)
!print*, "send 2", trans(numTrans)%x,trans(numTrans)%y,trans(numTrans)%z

    end if
  end if

  do j = 1,numTrans
    trans(j)%x = 0
    trans(j)%y = 0
    trans(j)%z = 0.0
  end do
  numTrans = 0
  flag(xcoor,ycoor)=.true.
end do  ! loop over mtdOrder

! In the eventuality that snow needs to be transferred to the last cell on
! the list (i.e. the lowest stripe):

!if((mtdOrder(2,((ncol*nrow)-(2*nrow)-(2*ncol)+4)) == lastRow+1)) then
if((mtdOrder(2,numNonNA) == lastRow+1)) then
  call MPI_RECV(numTrans,1,MPI_INTEGER,rank+1,33,MPI_COMM_WORLD,status,ierr)
  if(numTrans > 0) then
!print*, rank, "recv", numTrans, "cells from", mtdOrder(1,i-1), mtdOrder(2,i-1), "end 2"
    call MPI_RECV(trans,3,MPIxyztype,rank+1,34,MPI_COMM_WORLD,status,ierr)
  end if

  do j = 1,numTrans
    xc = trans(j)%x
    yc = trans(j)%y
    transSnow = trans(j)%z
    snowGrid(xc,yc) = snowGrid(xc,yc) + transSnow
!print*, "recvlast2", xc, yc, transSnow

    trans(j)%x = 0
    trans(j)%y = 0
    trans(j)%z = 0.0
  end do  ! loop over numTrans
!else if((mtdOrder(2,((ncol*nrow)-(2*nrow)-(2*ncol)+4)) == firstRow+1)) then
else if((mtdOrder(2,numNonNA) == firstRow+1)) then
  call MPI_RECV(numTrans,1,MPI_INTEGER,rank-1,31,MPI_COMM_WORLD,status,ierr)
  if(numTrans > 0) then
!print*, rank, "recv", numTrans, "cells from", mtdOrder(1,i-1), mtdOrder(2,i-1), "end2"
    call MPI_RECV(trans,3,MPIxyztype,rank-1,32,MPI_COMM_WORLD,status,ierr)
  end if

  do j = 1,numTrans
    xc = trans(j)%x
    yc = trans(j)%y
    transSnow = trans(j)%z
    snowGrid(xc,yc) = snowGrid(xc,yc) + transSnow
!print*, "recvlast2", xc, yc, transSnow

    trans(j)%x = 0
    trans(j)%y = 0
    trans(j)%z = 0.0
  end do  ! loop over numTrans
end if

!---------------------------------------------------
! End of redistribution

where(int(gezg) .eq. int(rnodat)) snowGrid = rnodat
allocate(reshVec(len))
reshVec = reshape(snowGrid(:,firstRow:lastRow),(/len/))
rsum = kahan_sum(reshVec, len, rnodat)
if(allocated(reshVec)) deallocate(reshVec)

avg = rsum / cntGlobal

! Insert adjustment here if necessary

! ...

nextfak = 1.0

do ycoor=lastRow,firstRow,-1
do xcoor=1,ncol
  if (int(gezg(xcoor,ycoor)) .ne. int(rnodat)) then
    if (int(snowGrid(xcoor,ycoor)) .eq. int(rnodat)) cycle
    SSNOWTF(xcoor,ycoor)=snowGrid(xcoor,ycoor)*nextfak
    ssotf(xcoor,ycoor)=SSNOWTF(xcoor,ycoor)+SLIQTF(xcoor,ycoor)
  else
    SSNOWTF(xcoor,ycoor)=rnodat
  end if
end do
end do

return

! Bye bye
end
