!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2003 Joachim Gurtz, Massimiliano Zappa
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Adaptation to the disaggregated soil evaporation and
! transpiration calculation.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------
! ------------------------------------------------------------------------------
!
! Contacts
! Joachim Gurtz, Dr. rer. nat. habil. 	- gurtz@env.ethz.ch
! Swiss Federal Institute of Technology ETH
! Institute for Atmospheric and Climate Science ETH
! Winterthurerstrasse 190
! CH-8057 Zurich
!
! Tel. ++41 1 635 52 28/27     Fax  ++41 1 362 51 97
! http://www.iac.ethz.ch/
!
!
! Massimiliano Zappa, Dr. sc. nat. ETH 	- massimiliano.zappa@wsl.ch
! Eidgen�ssische Forschungsanstalt WSL
! Abteilung Wasser-, Erd- und Felsbewegungen
! Z�rcherstr. 111
! CH-8903 Birmensdorf
!
! Tel. ++41 1 739 24 33     Fax ++ 41 1 739 24 88
!
! ------------------------------------------------------------------------------
!
! Reviewed literature in international journals
! Gurtz, J., Baltensweiler, A. and Lang, H. (1999): Spatially distributed
!   hydrotope-based modelling of evapotranspiration and runoff in mountainous
!   basins. Hydrol. Process., 13, 2751-2768.
! Gurtz, J., Verbunt, M., Zappa, M., Moesch, M. and U. Moser (2003): Long-term
!   hydrometeorological measurements and model based analyses in the hydrological
!   research catchment Rietholzbach. Journal of Hydrology and Hydromechanics, in print.
! Klok, E.J., Jasper, K., Roelofsma, K.P., Badoux, A. and Gurtz, J. (2001):
!   Distributed hydrological modelling of a glaciated Alpine river basin.
!   Hydrological Sciences Journal, 46, 553-570.
! Oltchev, A., Cermak, J., Gurtz, J., Keily, G., Nadezhdina, N., Tishenko, A.,
!   Zappa, M., Tishenko, N., Lebedeva, N., Vitvar, T., Albertson, J.D., Tatarinov, F.,
!   Tishenko, D., Nadezhdin, V., Kozlov, B., Ibrom, A., Vygodskaya, N. and
!   Gravenhorst, G. (2001): The response of the water flows of the boreal forest region
!   at the Volga's source area to climatic and land-use changes. Phys. and Chem. of the
!   Earth., Vol. 13, 675-690.
! Zappa, M. and Gurtz, J. (2003): Plot-scale simulation of soil moisture and evapotranspiration
!   during the 1999 MAP-Riviera campaign. Hydrology and Earth System Sciences. Accepted.
! Zappa, M., Pos, F., Strasser, U. and J. Gurtz (2003): Seasonal water balance of an alpine
!   catchment as evaluated by different methods for spatially distributed snow melt modelling.
!   Nordic Hydrology, Vol. 34(3), 179-202.
!
! More general documentation
! Gurtz, J., Baltensweiler, A., Lang, H., Menzel, L. and Schulla, J. (1997): Auswirkungen von
!   klimatischen Variationen von Wasserhaushalt und Abfluss im Flussgebiet des Rheins.
!   Schlussbericht NFP 31:�Klima�nderungen und Naturkatastrophen, vdf Hochschulverlag AG ETH
!   Z�rich, 147 pp.
! Gurtz, J., Zappa, M., Verbunt, M. and K. Jasper (2003): Advanced applications of distributed
!   hydrological models in mountainous catchments. In: de Jong, C., Collins, D. and R. Ranzi
!   (Editors): Climate and Hydrology in Mountain Areas. Wiley. Submitted Book Chapter.
! Jasper, K. (2001): Hydrological modelling of alpine river catchments using output variables
!   from atmospheric models. PhD thesis. Swiss Federal Institute of Technology (ETH).
!   Dissertation No. 14�385. [available from the Institute for Atmospheric and Climate Science
!   ETH, Winterthurerstr. 190, CH-8057 Z�rich].
! Schulla, J. and Jasper, K. (2000): Model Description WaSiM-ETH (Water balance Simulation Model
!   ETH). ETH Z�rich [available from the Institute for Atmospheric and Climate Science ETH,
!    Winterthurerstr. 190, CH-8057 Z�rich].
! Schulla, J. (1997): Hydrologische Modellierung von Flussgebieten zur Absch�tzung der Folgen von
!   Klima�nderungen. Z�rcher Geographische Schriften, Heft 69, ETH Z�rich. [available from the
!   Institute for Atmospheric and Climate Science ETH, Winterthurerstr. 190, CH-8057 Z�rich].
! Zappa, M., 2003: Multiple-Response Verification of a Distributed Hydrological Model at Different
!   Spatial Scales. Dissertation No. 14895, ETH Zurich
!   [available online at: http://e-collection.ethbib.ethz.ch/show?type=diss&nr=14895].
!
! ------------------------------------------------------------------------------
!
      SUBROUTINE PREVAHO (itistep,iNAMEIN,iGKNAM,iernam,igken,
     *                    idiroutput,idocali)

! ------------------------------------------------------------------------------
!
! Features:
! Spatially distributed computation of hydrological cycle governed by
! Hydrological Response units (HRUs)
!
! Meteorological input can be differentiated in up to 999 meteorological zones
!
! Model components:
!	- Interface for reading the control files and the spatial information
! 	- Interface for assimilation of meteorological information
!	- Adjustment of rain and snow volumes
!	- Interface for using climate scenarios
!	- Adjustment of Temperature and Radiation as function of Slope and Aspect
!	- Interception module
!	- Evapotranspiration module
!	- Snowmelt module
!	- Icemelt module
!	- Soil moisture and runoff-generation module
!	- Computation of model performance
!	- Internal parameterization for up to 30 land use classes
!
! Credits:  P. Krahe (Evapotranspiration) BfG Koblenz
!           H. Jensen (HBV3ETH)  ETH Zuerich
!           L. Braun (Snow model) ETH Zuerich
!           U. Strasser (Snow model ESCIMO) ETH Zuerich
! ------------------------------------------------------------------------------

	  use prevahglobals
	  use multglob
	  use mpi			! MPI library
	  use MPIVars		! Forhycs-specific variables needed for parallel execution
	  use outvars		! Used for table output in parallelized version
	  use forhycsglobals    ! Variables shared by PREVAH and TreeMig
	  use ifport            ! for functions CHDIR, GETCWD
	  use All_par, only: maxspc, maxhc

	  implicit none

      integer itistep,end_gken,ji,jii

	  real lssm,lread(99)
	  character*256 iernam,igknam,inamein,idiroutput
	  character*50 igken
	  character*4 dowe
	  character*1 idocali
      character*7 trm_string
      integer trmhead,trmpus,trmmod,trmlai,tlandold,tlandnew
      real trmlaifloat
      logical existIceUp,existForUp
      integer istatus    ! Return value of functions CHDIR, GETCWD
!      integer pos        ! Output of subroutine POSNAM
      character*255 curdir   ! Path of current directory
      logical :: existGlacF
      character*4 :: glacyyyy, treeyyyy


    !!!!!forhycs
       INTEGER :: nspc
       CHARACTER(len=255) :: cwd
       integer :: yrind  ! Loop index for root zone bioclimatic variables

	integer myi			! Dr. Thomas "Eddy" Whling - Updating procedure

1033  continue
      ranloop=ranloop+1
      tistep=itistep
	tistep=1
	ernam=iernam
	gknam=igknam
	namein=inamein
	giveuser(1)=inamein
	diroutput=idiroutput
	gken=igken
        call posnam(gken,1,end_gken)       !get length of string, used for name of savefiles
	docali=idocali
	reps=0
	lbulletin=.false.
       upforest=.false.
	if (bulletin .eq. 111) lbulletin=.true.

! ------------------------------------------------------------------------------
! Dr. Thomas "Eddy" W�hling - Updating procedure
! September-October 2005
! Adapted by M.Zappa for use in every basin
! ------------------------------------------------------------------------------
      INQUIRE (file='update.inp',EXIST=douptol)
	if (docali .eq. 't') douptol=.false.
      if (douptol) then
      call posnam(dirstate,1,NN)
	loadnam=dirstate(1:NN)//'updatestate.sav'
	open (9876, file=loadnam, status='unknown', form='binary')
	close (9876, status='delete')
	 if ((rank == root) .or. (doParallel == .false.))
     * call messagerun (1,'Proceeding with updating!')
      include 'updating_opt.inc'
	end if

! -----------------------------------
!     Phase 1 initialization
      call prevahinit

! -----------------------------------
!     Jumping to Routing
	IF ((docali .eq. 'r') .OR. (docali .eq. 'R')
     *    .OR. (docali .eq. 'c') .OR. (docali .eq. 'C')) THEN
	Call messagerun (1,'Jumping to routing!')
	GOTO 157
	END IF

!	IF (mmult) THEN
!	Call messagerun (1,'Jumping to routing!')
!	GOTO 157
!	END IF

! -----------------------------------
!     Phase 2 initialization
      call prevahoutput
      call catchGridInit

!------------------------------------

1023  continue

! ------------------------------------------------------------------------------
!     Begin of the simulation
! ------------------------------------------------------------------------------
!     Definig yesterday -> done for correct assimiation inf LOAD option is chosen
      TAG=TAGA-1
      MON=MONA
      JAHR=JAHRA

      MONUM(2)=28
      IF (JAHR.EQ.(INT(float(JAHR)/4.)*4)) MONUM(2)=29

      IF (TAG .eq. 0) THEN
          MON=MON-1
          IF (MON .eq. 0) THEN
          JAHR=JAHR-1
          MON=12
          TAG=31
          ELSE
          TAG=MONUM(MON)
          END IF
      END IF

      WRITE (DATUMCHAR,'(2I2.2,I4.4)') TAG,MON,JAHR

       TAG=TAGA
       MON=MONA
       JAHR=JAHRA
       DS=0
       NJAHR=0

95     FORMAT ( ' Aufruf von UP DATTAE '   )
       CALL DATTAE (DAT,TAGA,TAGE,MONA,MONE,JAHRA,JAHRE,
     *              DKAL,DSUM)
      if(jahr.le.1971.or.jahr.gt.2015) dat(2) = 28  ! No leap years with bootstrapped meteo data

! ------------------------------------------------------------------------------
!      Happy new year !
! ------------------------------------------------------------------------------
50     CONTINUE
       NJAHR=NJAHR+1
! ------------------------------------------------------------------------------
!      Opening of the files with meteorological information for the
!      current year
! ------------------------------------------------------------------------------

      ERRA=0					! Error flag
      first=0


      if (njahr .eq. 1) then
! Create .stats files (output suppressed)
!      CALL IDATNAM (JAHR,NKZ,IRGL,MDATN,FILENA,FILEN,ERRA,digiext,
!     * rmet,NTG,ntgid)
     	aetg=0
        if((rank.ne.root).or.(doParallel==.false.)) then
	   do ycoor=nrow,1,-1
	     do xcoor=1,ncol
              if (int(gezg(xcoor,ycoor)) .eq. int(rnodat)) cycle

              do i=1,NTG
	         if (ntgid(i) .eq. mezgrid(xcoor,ycoor)) then
	           NRTGFL(xcoor,ycoor)=i
	           AETG(i)=AETG(i)+rgridsize*rgridsize
	           ssotf(xcoor,ycoor)=ssotg(I)			! Initial snow water equivalent
               ssnowtf(xcoor,ycoor)=ssotg(I) 	! Initial snow water equivalent

                  IF (ssotf(xcoor,ycoor) .GT. 0.0) THEN
                    snowtf(xcoor,ycoor)=1	! Snow presence (1=yes)
                    mtf(xcoor,ycoor)=1
                  ENDIF
	           exit
	         end if
	       end do
	     end do
	   end do
        end if
      end if


      if((rank==root).or.(doParallel==.false.)) then
        do i=1,6
          call IDATKLIM (JAHR,NKZ,IRGL,MDATN,FILENA,FILEN,ERRA,
     *                     digiext,rmet,TAGA,MONA,ANFS,NTG,
     *                     NDTD,TAGE,MONE,JAHRE,I)!,reps)
        end do

	IF (rsplit .eq. 'Y') CALL IDAT10M (JAHR,1,ERRA,digiext,dirmeteo)
      IF (ERRA .EQ. 1) GOTO 999 		! If error

      end if  ! Master

! ------------------------------------------------------------------------------
!      Yearly sums are (re-)initialized
! ------------------------------------------------------------------------------

      if((rank.ne.root).or.(doParallel==.false.)) then
        include 'inityy.inc'  ! Slaves only
      end if

! management of the end of the simulation
       ITAG=31
       IMON=12
       IJAHR=JAHR

       IF (JAHR.EQ.JAHRE) THEN
	ITAG=TAGE
	IMON=MONE
       ENDIF

       CALL DATTAE (DAT,TAG,ITAG,MON,IMON,JAHR,IJAHR,
     *              DKAL,NDJAHR)
      if(jahr.le.1971.or.jahr.gt.2015) dat(2) = 28  ! No leap years with bootstrapped meteo data


! ------------------------------------------------------------------------------
!      Beginn eines neuen Monats !
! ------------------------------------------------------------------------------
52     CONTINUE

! ------------------------------------------------------------------------------
!     re-initialization at the begin of a new month
! ------------------------------------------------------------------------------

      if((rank.ne.root).or.(doParallel==.false.)) then
        include 'initmm.inc'  ! Slaves only
      end if

! eventually: update of adjustment factors for rain and snow
      IF (precmokor .eq. 1) THEN
        SNOKOR=snomokor(MON)
	PKORF=remokor(MON)
      END IF


! ------------------------------------------------------------------------------
! re-initialization at the begin of a new day
! ------------------------------------------------------------------------------

54    CONTINUE
      if((rank==root).or.(doParallel==.false.)) then
        Call messagerun (3,' ')
      end if

      IF (MON .EQ. 1) THEN
         CAICE=cagls
         CICEMF=caglt
      ENDIF

      IF ((MON .GE. 9) .and. (SHADST .EQ. 0)) THEN
          CAICE=CAICE*0.97		! from september the melt factors for ice
          CICEMF=CICEMF*0.97		! are reduced every day by 3%
      ENDIF

      if((rank.ne.root).or.(doParallel==.false.)) then
        include 'initdd.inc'  ! All
      end if
! ------------------------------------------------------------------------------
!     Evtl: Sub-Catchment output reset to zero
! ------------------------------------------------------------------------------

	IF ((ZIERG(15:15) .EQ. 'y') .OR. (ZIERG(15:15) .EQ. 'Y')) THEN
	runsubca=0.
	END IF


! ------------------------------------------------------------------------------
!      Reading of the meteorological information
!
!      Literature
!      Jasper, K. (2001): Hydrological modelling of alpine river catchments using
!       output variables from atmospheric models. PhD thesis. Swiss Federal Institute
!       of Technology (ETH). Dissertation No. 14�385. [available from the Institute for
!       Atmospheric and Climate Science ETH, Winterthurerstr. 190, CH-8057 Z�rich].
!      Klok, E.J., Jasper, K., Roelofsma, K.P., Badoux, A. and Gurtz, J. (2001):
!       Distributed hydrological modelling of a glaciated Alpine river basin. Hydrological
!       Sciences Journal, 46, 553-570.
!      Schulla, J. and Jasper, K. (2000): Model Description WaSiM-ETH (Water balance
!       Simulation Model ETH). ETH Z�rich [available from the Institute for Atmospheric
!       and Climate Science ETH, Winterthurerstr. 190, CH-8057 Z�rich].
!      Schulla, J. (1997): Hydrologische Modellierung von Flussgebieten zur Absch�tzung der
!       Folgen von Klima�nderungen. Z�rcher Geographische Schriften, Heft 69, ETH Z�rich.
!       [available from the Institute for Atmospheric and Climate Science ETH,
!       Winterthurerstr. 190, CH-8057 Z�rich]
! ------------------------------------------------------------------------------

      ERRA=0				! flag for error
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)
      call prevahmeteo

! ------------------------------------------------------------------------------
!     Daily loop for all HRUs
! ------------------------------------------------------------------------------
      if((rank==root).or.(doParallel==.false.)) then

      DATUMCHARO=DATUMCHAR
	WRITE (DATUMCHAR,'(2I2.2,I4.4)') TAG,MON,JAHR

!     Determining the day of week
      call DOW (TAG,MON,JAHR,ier,dayofweek)
	if (dayofweek .eq. 0) dayofweek=7

	if (ranuse) loadchar=datumchar

  !---------------------------
  !!! Loading saved model state - this is not used in FORHYCS, therefore
  ! the 'include' statements below are commented out. Saving and loading of
  ! model states is handled by the subroutines in writeFHstate.f90
  !---------------------------
      if ((loadid .eq. 'y') .or. (loadid .eq. 'Y')) THEN
           if (DATUMCHAR .eq. LOADCHAR) THEN
	         call posnam(dirstate,1,NN)
!	call messagerun (1,'Loading ..'//datumcharo)
	if (lscen) then
	if (loadscuse) then
	WRITE (DATUMCHARO,'(2I2.2,I4.4)') loadday,loadmonth,loadyear
	call messagerun (1,'Loading ..'//datumcharo)
	end if
	end if
               loadnam=dirstate(1:NN)//gken(1:end_gken)//'state_'//
     *                 datumcharo//'.sav'
               print*,trim(loadnam)
               call messagerun (1,'Loading ..'//datumcharo)
               !include 'readstate.inc'
	         loadid='N'
	     else
	         goto 1590
           end if
	end if
      if ((loadid .eq. 't') .or. (loadid .eq. 'T')) THEN
		if (tcali .eq. 't') DATUMCHARO=DATUMCHAR
      if (educnr .eq. -99) then
      if (nwpuse) then
       DATUMCHARO=DATUMCHAR
      end if
      end if
	         call posnam(dirstate,1,NN)
	if (lscen) then
	if (loadscuse) then
	WRITE (DATUMCHARO,'(2I2.2,I4.4)') loadday,loadmonth,loadyear
	call messagerun (1,'Loading ..'//datumcharo)
	end if
	end if
               loadnam=dirstate(1:NN)//gken(1:end_gken)//'state_'//
     *                 datumcharo//'.sav'
	call messagerun (1,'Loading ..'//trim(loadnam))
               !include 'readstate.inc'
	         loadid='N'
      end if

      end if  ! End of Master part

      call MPI_BARRIER(MPI_COMM_WORLD,ierr)

	goto 9334

9333  continue
      loadid='N'
9334  continue
	close (9876)
	giveuser(2)=loadnam

!-------JS 2008-04----------------
	IF (isbyn) THEN
	ALLOCATE (qisb(24,numisb)) ! Array containing hourly runoff of internal sub basins defined in *.isb grid-file
	DO ji=1,24
	  DO jii=1,numisb
		qisb(ji,jii)=0
	  ENDDO
	ENDDO
	ENDIF
!-----------------------

!-----------------------
! TreeMig initialization

      if(jahr.eq.jahra.and.mon.eq.1.and.tag.eq.1) then

        call ecoGridInit      ! Output routine initialization
        !call ecoCatchInit     ! Output routine initialization
        call TreeMig_init
        call parstom          ! RSMIN initialization
        include 'allocateForhycs.inc'


        if(readFH==0) then
          readSwitch = .TRUE.
          call LCPreSpinup
          if(prevahBC==0) call TreeMig(jahr)


        else
          call readFHstate
          call LCPostSpinup
          !call ecoGridWrite(jahr)
        end if

      end if
! End TreeMig Init
!-----------------------

!-----------------------
!     CORE of PREVAH, with Grid-loop
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)
      call prevahcore
! Write daily runoff of catchments to table
      call MPIcatchWrite(tag,mon,jahr)
      call MPIcatchPrec(tag,mon,jahr)
      call MPIcatchEvap(tag,mon,jahr)

!
! Alignment of sim runoff to observation at begin of forecast
!      include 'allinf.inc'

! Snow redistribution
      if((rank.ne.root) .or. (doParallel==.false.)) call MTD
! ------------------------------

! ------------------------------
! TEMPORARY (Matthias - Jan 2015): get daily mass balance of glaciers

      if((rank.ne.root) .or. (doParallel==.false.)) then

    	do ycoor=lastRow,firstRow,-1
    	do xcoor=1,ncol
    	  if(glacmask(xcoor,ycoor)==1) then
    	  !Get difference in snow cover of this cell before and after MTD
    	    glacdmtd=glacdmtd +
     * ((ssotf(xcoor,ycoor)-tempsnow(xcoor,ycoor))/countGlac)

! Adjust glacier MB accordingly
    	    glacbily2d(xcoor,ycoor) = glacbily2d(xcoor,ycoor) +
     * (ssotf(xcoor,ycoor)-tempsnow(xcoor,ycoor))

    	  end if
! Add difference to yearly MTD balance
      MTDbil(xcoor,ycoor) = MTDbil(xcoor,ycoor) +
     * (ssotf(xcoor,ycoor)-tempsnow(xcoor,ycoor))
    	end do
    	end do

    	glacbild = glacbild + glacdmtd
    	glacbily = glacbily + glacdmtd

    	end if  ! TEMP

      if(rank == root) then

      inquire(file="glacbil.txt",exist=existGlacF)
      if(existGlacF) then
       open(9357,file="glacbil.txt",status="old", position="append")
       else
         open(9357,file="glacbil.txt",status="new")
      end if


      end if   ! Master

    	allocate(intArgs(3))
    	allocate(writeArgs(10))
    	allocate(writeArgOpts(10))


    	intArgs = (/tag,mon,jahr/)
    	writeArgs = (/glacpd, glaced, glacqd, glacdssd, glacdsid,glacdsmd,
     * glacdsuzd, glacdslzd, glacbild, glacdmtd/)
      writeArgOpts = 1

       call MPIWriteTable(9357,18)

      deallocate(intArgs)
      deallocate(writeArgs)
      deallocate(writeArgOpts)



      if(rank == root) close(9357)

! End glacier mass balance
! ------------------------------

! ------------------------------
      call prevahendday
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)

!-------JS 2008-04----------------
	IF (isbyn) DEALLOCATE (qisb) ! Array containing hourly runoff of internal sub basins
!-----------------------

! ------------------------------------------------------------------------------
!     Switch to next day - checking the progress of the simulation
! ------------------------------------------------------------------------------
      DKAL=DKAL+1
      if ((saveid .eq. 'd') .or. (saveid .eq. 'D')) THEN
      call posnam(dirstate,1,NN)
         savenam=dirstate(1:NN)//gken(1:end_gken)//'state_'//
     *   datumchar//'.sav'
         include 'writestate.inc'
	end if

      if ((saveid .eq. 'y') .or. (saveid .eq. 'Y')) THEN
      call posnam(dirstate,1,NN)
      IF (TAG .EQ. DAT(MON)) THEN
      if ((savemmid .eq. 'y') .or. (savemmid .eq. 'Y')) THEN
         savenam=dirstate(1:NN)//gken(1:end_gken)//'state_'//
     *   datumchar//'.sav'
         include 'writestate.inc'
      end if
	end if

      if ((savemoreid .eq. 'y') .or. (savemoreid .eq. 'Y')) THEN
	DO I=1,SAVEMORENN
         if (DATUMCHAR .eq. SAVECHAR(i)) THEN
         savenam=dirstate(1:NN)//gken(1:end_gken)//'state_'//
     *   datumchar//'.sav'
         include 'writestate.inc'
         end if
      END DO
      end if
	end if
	giveuser(3)=savenam

1590  CONTINUE

      IF ((docali .eq. 'f') .or. (docali .eq. 'F')) THEN
	if (fnext .eq. fordays) goto 138
	end if
      IF ((JAHR.EQ.JAHRE).AND.(MON.EQ.MONE).AND.(TAG.EQ.TAGE)) GOTO 138

      TAG=TAG+1

      IF (TAG .LE. DAT(MON)) GOTO 54			! Daily loop

!     Prompting the progress of the simulation
!     If file here Killing the simulation
      call posnam (diroutput,1,pos)
	open (987,file=diroutput(1:pos)//'leave.pre',status='old',err=345)
	close (987)
	Call messagerun (2,'Stopped')
	Call messagerun (1,'Stopped')
      CLOSE (MDATN(1))
      CLOSE (MDATN(2))
      CLOSE (MDATN(3))
      CLOSE (MDATN(4))
      CLOSE (MDATN(5))
      CLOSE (MDATN(6))
      CLOSE (MDMIT)
	CLOSE (MDR)
      CLOSE (MDTMP)
      CLOSE (MDSTD)
	CLOSE (MDTAG)
	CLOSE (MDHOU)
	CLOSE (MDMON)
	CLOSE (MDQ)
      CLOSE (MIDAT)
	CLOSE (MDBULL)
	goto 994

345   continue

      if((rank==root) .or. (doParallel==.false.)) then
        Call messagerun (2,DATUMCHAR(1:2)//'.'//DATUMCHAR(3:4)//
     *              '.'//DATUMCHAR(5:8)//' completed')
      end if

! ------------------------------------------------------------------------------
!     Writing monthly sums and grids -Setting monthly sums to 0
! ------------------------------------------------------------------------------

138    CONTINUE
140    FORMAT (  I4,I3,I4,I6,17F7.1  )
141    FORMAT (  I3,I5,I4,I6,18F7.1  )
      if(mon==9) then
         fsad_old = fsad_tf
      end if

!	Writing monthly grids
      IF (GRDAKT .EQ. 'Y') THEN
        if(doParallel) then
          include 'MPIMonthGridCentral.inc'
        else
          INCLUDE 'gridmg.inc'
        end if
      ENDIF


      PNKMS=0.0
      PAKMS=0.0
      WSAMS=0.0
      EPMS=0.0
      ERMS=0.0
      EIMS=0.0
      EBMS=0.0
      R0MS=0.0
      R1MS=0.0
      R2MS=0.0
      RGEMS=0.0


      IF ((ZIERG(7:7) .EQ. 'y') .OR. (ZIERG(7:7) .EQ. 'Y')) then

        allocate(intArgs(4))
        allocate(writeArgs(17))
        allocate(writeArgOpts(17))
        intArgs=(/nrgeb,jahr,mon,nugeb/)
        writeArgs=(/pnkmsm,pakmsm,wsamsm,epmsm,ermsm,eimsm,ebmsm,
     *              r0msm,r1msm,r2msm,rgemsm,ssom,sim,ssmm,suzm,
     *              slzm,glacmm/)
        writeArgOpts=1

        call MPIWriteTable(mdr,9)

        deallocate(intArgs)
        deallocate(writeArgs)
        deallocate(writeArgOpts)



      end if

      IF ((JAHR.EQ.JAHRE).AND.(MON.EQ.MONE).AND.(TAG.EQ.TAGE)) GOTO 145
       TAG=1
       MON=MON+1
! ------------------------------------------------------------------------------
!     Updating ICE Areas
! MZ - January 2010
! New version (transient glacier module) - MS - February 2015
! ------------------------------------------------------------------------------

      IF ((mon .eq. 10) .and. (jahr .ge. 1971)) then
      ! Update of glaciated areas - either transient with deltaH method
      ! (if glacSwitch==1), or by loading in modeled glacier extent (as
      ! in the CCHydro project; glacSwitch==0)

      if(glacSwitch==1) then


          if((rank==root).or.(doParallel==.false.)) then
          !call messagerun (1, 'Looking for Updated Ice areas')
            call messagerun (1, 'Running deltaH glacier update')
          end if

          call glacDeltaH

          write(glacyyyy,'(I4.4)') jahr
          include 'MPIGridGlac.inc'


      else
      ! GlacSwitch == 0 => no transient glacier update. Grids with glacier
      ! extent are read in every X simulation years (UZH scenarios)

       if((rank==root).or.(doParallel==.false.)) then
         call messagerun (1, 'Looking for Updated Ice areas')
       end if

        CALL POSNAM (NADGKW,1,POSI1)

         !print*, "deltayy, yyice before", deltayy, yyice

 	 if (deltayy .ne. 1973) then
 	   yyice=jahr-1980+deltayy
 	   if (deltayy .eq. 2021) then
 	     yyice=max(yyice,deltayy-1)
 	   else if (deltayy .eq. 2070) then
 	     yyice=max(yyice,deltayy)
 	   end if
 	 else
 	   yyice=max(jahr,deltayy)
 	 end if

         !print*, "deltayy, yyice after", deltayy, yyice

 	 write (iceyy,'(I4.4)') yyice

         inquire(file=NADGKW(1:POSI1)//'-ela'//iceyy//'.bin',exist=
     *          existIceUp)

         if((rank==root).or.(doParallel==.false.)) then
 	   print*,NADGKW(1:POSI1)//'-ela'//iceyy//'.bin'
        end if

        if(existIceUp==.false.) then			! No ice update file found
          goto 850
        else  						! Update file found

        if (rank.ne. root) then

 	  OPEN(731,file=NADGKW(1:POSI1)//'-ela'//iceyy//'.bin',
     *     status='old',form='binary',err=810)

    	 uwas=731
 	 INCLUDE 'rwasim.inc'
 !        do ycoor=lastRow,firstRow,-1
          do ycoor=nrow,1,-1  ! Need to read whole grid (because of file format)!
           !read (uwas) (GGLM(xcoor,ycoor),xcoor=1,ncol)
           read (uwas) (rGLM(xcoor,ycoor),xcoor=1,ncol)
         end do
         close (uwas)

 	  do ycoor=lastRow,firstRow,-1
 	   do xcoor=1,ncol
             if (int(gezg(xcoor,ycoor)) .eq. int(rnodat)) cycle
        ! Glacier Areas
             !LREAD(2)=gglm(xcoor,ycoor)
             LREAD(2)=rglm(xcoor,ycoor)
 	     LN=LANDIN(xcoor,ycoor)
C	       print*, "Before update", xcoor, ycoor, int(lread(2)),
C    *       landin(xcoor,ycoor)


  ! Glacier pixels determined from g** file
             IF (INT(LREAD(2)) .eq. 2) LANDIN(xcoor,ycoor)=13
             IF (INT(LREAD(2)) .eq. 1) LANDIN(xcoor,ycoor)=14

 ! No more glacier in accumulation area, then ROCK
             IF (LANDIN(xcoor,ycoor) .eq. 13) then
               IF (INT(LREAD(2)) .eq. 0) LANDIN(xcoor,ycoor)=15
             end if
 ! No more glacier in ablation area, then bare soil
             IF (LANDIN(xcoor,ycoor) .eq. 14) then
               IF (INT(LREAD(2)) .eq. 0) LANDIN(xcoor,ycoor)=26
             end if

C	       print*, "After update", xcoor, ycoor, int(lread(2)),
C    *       landin(xcoor,ycoor)

           end do
         end do
 810     continue
         end if  ! End of slaves part
       end if  ! ExistIceUp

         goto 851
 850     continue
         if((rank==root).or.(doParallel==.false.)) then
           call messagerun (1, 'No ice update this year')
         end if
         goto 852
 851     continue
         if((rank==root).or.(doParallel==.false.)) then
           call messagerun (1, iceyy//' - ICE has been updated')
         end if
 852     continue
      end if  ! glacSwitch
      call LCPostSpinup ! Update stockability
      end if  ! Date

! ------------------------------------------------------------------------------
!     Updating FOREST Areas
! old version by MZ - PS - April 2012 (read external file)
! new version by MS 2014 - 2018 (run TreeMig)
! ------------------------------------------------------------------------------

      if(TMtoPR == 1) then

      ! Run TreeMig each March 1st.

        if(jahr.gt.jahra.and.MON.eq.3.and.TAG.eq.1) then
         write(treeyyyy,'(I4.4)') jahr
         CALL TreeMig(JAHR)
         if(rank .ne. root) call lcUpdate
       END IF


       ! Updating rooting zone storage capacity. Done at the beginning of
       ! March, except in the first simulation year, where it is done on the
       ! first day of February (this block does not get executed on January 1st -
       ! not sure why. As a result, SFC is only 30mm for the first month.).
       ! If the simulation starts with bare soil (full spin-up), rooting zone
       ! storage capacity is set to a standard value in the first year, as there
       ! are no climate parameters yet.

       !if(TMtoPR.eq.1.and.PRtoTM.eq.1) then


       if(rank.ne.root) then
         if(jahr==jahra) then
           if(readFH==0) then
             if((mon==2).and.(tag==1))call rzstorage(.true.)
           else
             if((mon==2).and.(tag==1))call rzstorage(.false.)
             !if((mon==1).and.(tag==1))call rzstorage(.true.)
           end if ! readFH
         else   ! After first year
           if((mon==3).and.(tag==1))call rzstorage(.false.)
         end if   ! First year or not
       end if     ! Slaves only

      ! Output routines for ecoregion-aggregated biomass, LAI and rooting depth.

       if((mon==3).and.(tag==1)) then
         call ecoGridWrite(jahr)
       end if

      end if


!==================================


! ------------------------------------------------------------------------------

      IF (MON .LE. 12) GOTO 52				! Monthly loop
! ------------------------------------------------------------------------------
!     Writing yearly sums and grids - Setting yearly sums to 0
! ------------------------------------------------------------------------------

! Output routine for annual ecoregion-aggregated water balance
      call ecoWBWrite(jahr)


145   CONTINUE

143    FORMAT (  I2,I5,I4,I8,11F7.0,6F7.1  )

!	Writing yearly grids
      IF (GRDAKT .EQ. 'Y') THEN
        if(doParallel) then
          include 'MPIYearGridCentral.inc'  ! MPIYearGrid.inc does not work
        else
          INCLUDE 'gridj.inc'
        end if
      ENDIF

! TEMPORARY glacier mass balance



      K=0

! Writing yearly output to .pri table
      IF ((ZIERG(7:7) .EQ. 'y') .OR. (ZIERG(7:7) .EQ. 'Y')) then
        allocate(intArgs(4))
        allocate(writeArgs(17))
        allocate(writeArgOpts(17))
        intArgs=(/nrgeb,jahr,k,nugeb/)
        writeArgs=(/pnkjsm,pakjsm,wsajsm,epjsm,erjsm,eijsm,ebjsm,
     *              r0jsm,r1jsm,r2jsm,rgejsm,ssom,sim,ssmm,suzm,
     *              slzm,glacjj/)
        writeArgOpts=1
        call MPIWriteTable(mdr,14)
        deallocate(intArgs)
        deallocate(writeArgs)
        deallocate(writeArgOpts)

      end if

      GLACTT=GLACTT+GLACJJ



! Write annual temperature and precip output for Bioclim/TreeMig
      !print*, "Calling MPIbcOut", tag, mon, jahr
      if(jahr.ge.1971) call MPIbcOut(jahr)
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)

!	End of simulation reached? If yes -> 999

      IF ((JAHR.EQ.JAHRE).AND.(MON.EQ.MONE).AND.(TAG.EQ.TAGE)) then
	 GOTO 999
      END IF



! ------------------------------------------------------------------------------

      MON=1
      MONA=1
      TAGA=1

! FORHYCS -> save the indices of current year for using them next march first
      !ddegsum5 = (ddegsum5*1.05) + 55.5   ! DDEGS5 values need to be adjusted so that they match BC values
      ddegsum5old=ddegsum5
      ddegsum5=0.0
      phenBBOld = phenDoyBB
      phenMaxOld = phenDoyMax
      phenSenOld = phenDoySen
      phenFallOld = phenDoyFall
      phenDoyBB = rnodat
      phenDoyMax = rnodat
      phenDoySen = rnodat
      phenDoyFall = rnodat
      ! drstrold is the value transferred to in TreeMig (used the following March 1st)
      drstrold=drstr
      drstr=0.0
      drstrold2=drstr2
      drstr2=0.0
      meantold=meant
      meant=0.0
      meantCold = meantC
      meantC = 0.0
      sumpold=sump
      sump=0.0


! Indices for root zone storage module - updated daily
      do yrind = rootslag,2,-1
        gsLength(yrind,:,:) = gsLength((yrind-1),:,:)
        gsTpot(yrind,:,:) = gsTpot((yrind-1),:,:)
        gsPe(yrind,:,:) = gsPe((yrind-1),:,:)
        gsPrec(yrind,:,:) = gsPrec((yrind-1),:,:)
        gsPrecDays(yrind,:,:) = gsPrecDays((yrind-1),:,:)
        gsTemp(yrind,:,:) = gsTemp((yrind-1),:,:)
        gsPes(yrind,:,:) = gsPes((yrind-1),:,:)
      end do
      gsLength(1,:,:) = gsLengthCur(:,:)
      gsTpot(1,:,:) = gsTpotCur(:,:)
      gsPe(1,:,:) = gsPeCur(:,:)
      gsPrec(1,:,:) = gsPrecCur(:,:)
      gsPrecDays(1,:,:) = gsPrecDaysCur(:,:)
      gsTemp(1,:,:) = gsTempCur(:,:)
      gsPes(1,:,:) = gsPesCur(:,:)

! Calculate De Martonne Index
      if(rank .ne. root) call deMartonne

      ! YEAR INCREMENT
      JAHR=JAHR+1


! Grids of annual precipitation and GS temperature for output
      annprecold = annprec
      mmotold = mmot


! Write model state if needed

      if((jahr == 1972).or.(jahr==2014)) then
        if(writeFH==1) call writeFHstate(jahr)
      end if


      if (cleansleep .eq. 'Y') then
        call messagerun (1, 'Scenario sleeping')
        call sleep (30)
      end if
      if (cleangrid .eq. 'Y') then
        call messagerun (1, 'Last scenario sleeping')
        call sleep (30)
      end if

      reps=0
      I=0
196   I=I+1
!     CLOSE (MDATN(I))				! Closing meteo input for actual year
      IF (I .LT. 5) GOTO 196
      IF ((I .EQ. 5) .and. (IRGL .LT. 0)) GOTO 196
      GOTO 50							! Yearly loop

! ------------------------------------------------------------------------------
!    End sums - determining final water balance - Writing outputs
! ------------------------------------------------------------------------------

999   CONTINUE

      call prevahwrite

      CLOSE (MDMIT)
      CLOSE (MDR)
      CLOSE (MDTMP)
      CLOSE (MDSTD)
      CLOSE (MDTAG)
      CLOSE (MDHOU)
      CLOSE (MDMON)
      CLOSE (MDQ)
      CLOSE (MIDAT)
      CLOSE (MDBULL)
      IF (numohru .gt. 0) THEN
        do llhru=1,numohru
	   close (unit)
	 end do
      END IF

! ------------------------------------------------------------------------------
!     Calculating routing!
! ------------------------------------------------------------------------------

157   CONTINUE

      IF ((dorout .eq. 'y') .or. (dorout .eq. 'Y')) THEN
      write(mhtml,'(A)')'<p><p>'
      write(mhtml,'(A)')'Routing module<br>'
	DO I=1,nrouts
	call posnam (routnams(i),1,pos)
	rknam=routnams(i)(1:pos)
      CALL ROUTING (I,rknam,namq,storage(I),translation(I),AELHG,
     *              routeffnam,mhtml,saveid,dirstate,savemmid,
     *              savemoreid,SAVEMORENN,SAVECHAR,loadrout,LOADCHAR)
      END DO
	END IF

! ------------------------------------------------------------------------------
!     Determining the model performance!
!
!     Literature
!     Legates, D.R. and McCabe, G.J. (1999): Evaluating the use of "Goodness-of-Fit"
!      measures in hydrologic and hydroclimatic model validation, Water Resour. Res.,
!      35, 233-241.
!     Nash, J.E. and Sutcliffe, J.V. (1970): River flow forecasting through conceptual
!      models (1), a discussion of principles. J. Hydrol., 10, 282-290.
! ------------------------------------------------------------------------------

      IF ((dorout .eq. 'y') .or. (dorout .eq. 'Y')) then
	 ernam=routeffnam
!	 print*, ernam
      end if

!     Assuring that no efficiency is computed during forecast
      IF ((docali .eq. 'f') .or. (docali .eq. 'F')) doeff='N'

      IF ((doeff .eq. 'y') .or. (doeff .eq. 'Y')) then
        write(mhtml,'(A)')'<p><p>'
        write(mhtml,'(A)')'Efficiency<br>'

        IF ((docali .ne. 'y') .and. (docali .ne. 'Y') .and.
     *    (docali .ne. 'c') .and. (docali .ne. 'C').and.
     *    (docali .ne. 'g') .and. (docali .ne. 'G')) THEN

	   DO I=1,neffis

      write(mhtml,'(A7,2(I2.2,A4))')'Script ',I,' of ',neffis,'<br>'

	     call posnam (effinams(i),1,pos)
	     gknam=effinams(i)(1:pos)

	     write(mhtml,'(A)')'<a href=file:///'//trim(gknam)//
     *      '>'//trim(gknam)//'</a><br>'

            CALL EFFCOEF (gknam,ernam,gken,dirmeteo,diroutput,docali,
     *              dorout,aelhg,ZN,mhtml)
	   END DO

	 ELSE
      CALL EFFCOEF (gknam,ernam,gken,dirmeteo,diroutput,docali,dorout,
     *              aelhg,ZN,mhtml)
	 END IF
      end if

! ------------------------------------------------------------------------------
! Closing output files and deallocating the arrays
! ------------------------------------------------------------------------------
994   CONTINUE

      !INCLUDE 'dealloc.inc'
!     CLOSING HTML REPORT
      write(mhtml,'(A)')'<hr>'
      write(mhtml,'(A)')
     * '<font size="-2">Created by PREVAH - Copyright  2004.</font>'
      write(mhtml,'(A)')'<p>'
      write(mhtml,'(A)')'</body>'
      write(mhtml,'(A)')'</html>'
	close(mhtml)

! ------------------------------------------------------------------------------
!     Closing isb file !JS-2008-04

	IF (isbyn) CLOSE(MISBQ)

! ------------------------------------------------------------------------------
!     Closing xml-file !JS-2008-04

	IF (xmlyn .EQ. 'Y') CALL XMLCLOSE !xmloutput.f90

! ------------------------------------------------------------------------------


	OPEN  (721,FILE="AREA.TMP",status='unknown')
	CLOSE (721,status='delete')

! ------------------------------------------------------------------------------
!     Closing Update Files
! ------------------------------------------------------------------------------

	CLOSE(6545)
	CLOSE(6546)
! ------------------------------------------------------------------------------

	idiroutput=diroutput

      if (ranuse) then
	 ravarar(ranloop)=ravar
	 if (raopt .eq. 3) ravarar(ranloop)=ranloop
	 if (ranloop .lt. 100) then
	   if (raopt .eq. 3) return
	   goto 1033
	 end if
	 write (731,'(I4,1X,I4,1X,100(F7.3,1X))') JAHR,JAHR,
     *  (ravarar(k),k=1,100)
	 do i=1,12
	   do j=1,monum(i)
            write (731,'(I4,1X,I4,1X,100(F7.2,1X))') i,j,
     *      (raerg(i,j,k),k=1,100)
	   end do
	 end do
	 close (731)
	 stop
      end if


! ------------------------------------------------------------------------------
       call posnam (diroutput,1,pos)
       OPEN  (721,FILE=diroutput(1:pos)//"stop.it",status='unknown')
       write (721,'(A8)') "[FINITO]"
	CLOSE (721)

! ------------------------------------------------------------------------------

	RETURN
      END

! ----
