!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.
! Modifications copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa,
! Marc Scherstjanoi, Heike Lischke
! 2014 - 2018: Clarify signification of i/o variables, and correction of
! adjustment factor calculation

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------


      SUBROUTINE RTKORZ (tag,monat,jahr,geobreite,aspect,slope,
     *                  hhoehe,CT,MDR,DEBUG,rgdm,tedti,ssdti,shadti,
     *                  tsa,tsu,ssdmax,shadm,tekdti,tekdm,rgdti,rgkdti,
     *                  rgkdm,rgmdti,rgmdm,rgmkdti,rgmkdm,rdmdti,
     *                  rdmdm,rdmkdti,rdmkdm,cf,xcoor,ycoor)

! ---------------------------------------------------------------------
!      Potential incoming global radiation, the maximal sunshine
!      duration, surise, sunset and make a local adjustemnt of global
!      radiation and air temperature (hourly and daily values)
!      Shadowing can be accounted for.
!
!     For modeling, the main output variables of interest are :
!           - RGKDTI(24)
!           - RGKDM
!           - TEKDTI(24)
!           - TEKDM
!           - TSA, TSU, SSDMAX
! ---------------------------------------------------------------------

      implicit none


! ---------------------------------------------------------------------
!     Passed Variables
! ---------------------------------------------------------------------

      real, intent(in) :: slope, aspect, hhoehe, geobreite
      integer, intent(in) :: tag, monat, jahr
      integer, intent(in) :: debug, mdr               ! Control pars
      real, intent(in) :: rgdm                        ! Daily mean, non-corrected global radiation
      real, intent(in), dimension(24) :: tedti        ! Hourly non-corrected air temperature
      real, dimension(24) :: ssdti                       ! Hourly observed SSD
      real, intent(in), dimension(24) :: shadti       ! Shadowing term (not used for now)

      integer, intent(out) :: tsa, tsu                ! Hour of sunrise and sunset
      real, intent(out) :: ssdmax                     ! Daily maximal sunshine duration
      real, intent(out) :: shadm                      ! Daily shadowing term - not used for now
      real, intent(out), dimension(24) :: tekdti,     ! Hourly adjusted air temperature
     *                                    rgdti,      ! Uncorrected but disaggregated hourly global radiation
     *                                    rgkdti,     ! Corrected hourly global radiation
     *                                    rgmdti,     ! Hourly potential (clear-sky) incoming global radiation
     *                                    rgmkdti,    ! Hourly corrected potential (clear-sky) incoming global radiation
     *                                    rdmdti,     ! Hourly potential (clear-sky) incoming direct radiation
     *                                    rdmkdti     ! Hourly corrected potential (clear-sky) incoming direct radiation

      real, intent(out) :: tekdm,                     ! Mean daily corrected temperature
     *                     rgkdm,                     ! Mean daily corrected global radiation
     *                     rgmdm,                     ! Mean daily potential (clear-sky) incoming global radiation
     *                     rgmkdm,                    ! Mean daily corrected potential (clear-sky) incoming global radiation
     *                     rdmdm,                     ! Mean daily potential (clear-sky) incoming direct radiation
     *                     rdmkdm                     ! Mean daily corrected potential (clear-sky) incoming direct radiation
      real, dimension(24) :: cosZdti                    ! Hourly values of cosZ
      integer, intent(in) :: xcoor,ycoor  ! For debugging

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------
      real, parameter :: pi = 3.14159265359
      real, parameter :: deklination = 23.45          !	Declination of the Equator in Deg
      real, parameter :: jahreslaenge = 365.25        !     Length of the year (Days)
      real, parameter :: kf = 0.27666667              !     Correction parameter
      real, parameter :: rm = 149.5                   !	Average distance earth-sun in millions of km
      real, parameter :: e = 0.01675                  !	excentricity
      real, parameter :: I0 = 1367.0                  !     Solar constant
      real, parameter :: a = 8.64E7                   !	unit transformation from m/s to mm/d
      real, parameter :: d = 24.0                     !     Day length in hours
      real, parameter :: P0 = 1013.0                  !     Air pressure at sea level 1013 hPa

      INTEGER JULDATUM				!	Julian date
      INTEGER SCHALTJAHR			!	variables to identify leap years
      INTEGER NORMDATUM				!	julian date of the 21st July in the previous year
      REAL WINKEL				!	Angle of sun at 21st June
      REAL W0					!	Angle of sun at sunset
      REAL AKTDEKLIN				!	actual declination
      REAL R					!	actual radius of earth ellipse
      INTEGER PERIHELDATUM			!	??????
      REAL PERIHEL				!	PERIHEL
      REAL wsa					!	Angle at sunrise
      REAL wsu					!	angle at sunset
      real ws,stsa,stsu
      REAL zcos					!	Cosinus of zenith angle
      REAL zsin					!	Sinus of Zenith angle
      REAL as					!	Azimuth of sun
      REAL wslope
      integer i,ii,k,std			!	varaibles for loops
      real sumg,sumgk,sumd,sumdk,wink		!	Help values
      REAL KORFAK,RGKDS,RGKOR			!	variables for adjusting global radiation
      real, dimension(24),intent(out) :: cf           ! Vector of hourly correction factors
      REAL CT					!	minimal positive temperature adjustment in deg Celsius

!	Help variables
      REAL RHG, KFAK				!	Help values
      REAL FAKTOR(24)				!	Help array for temperature adjustment
      REAL PHOEHE				!	Air pressure at curret elevation
      REAL RHG1,SHAD,HFAK			!	Help values

      real :: fracSR, fracSS, hFrac       ! Fraction of daylight in first/last hour of the day
      integer :: lnrg
      real, dimension(24) :: ssdPot

! ---------------------------------------------------------------------



! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------


      CALL EJULIANZ (JAHR,MONAT,TAG,JULDATUM)
      SCHALTJAHR=JAHR-MOD(JAHR,4)
      CALL EJULIANZ (SCHALTJAHR,6,21,NORMDATUM)
      WINKEL=360.*(JULDATUM-NORMDATUM)/JAHRESLAENGE+360.  ! Super informative variable name...

      do while(winkel .ge. 360)
        winkel = winkel-360
      end do

!     Computation of the actual declination
      AKTDEKLIN=DEKLINATION*COS(WINKEL*PI/180.)

! computation of the angle of sun above the horizon
      RHG=-TAN(AKTDEKLIN*PI/180.0)
      W0=180.*ACOS(-TAN(AKTDEKLIN*PI/180.)*TAN(GEOBREITE*PI/180.))/PI

! computaiton of the maximal sunshine duration
      ssdmax=24.*W0/180.+KF

! Angle at 2nd of juli
      CALL EJULIANZ (JAHR,7,2,PERIHELDATUM)
      PERIHEL=360.*(JULDATUM-PERIHELDATUM)/JAHRESLAENGE+360.

! Distance between earth and sun
      R=RM*(1.-E*COS(PERIHEL*PI/180.))

! computation of sunrise and sunset
      wsu=w0
      stsu=(D/2.0*wsu/180.0)+12.0
      !stsa=stsu-ssdmax
      stsa=12.-(stsu-12.)
      wsa=(stsa-12.0)*2/D*180.0
      tsa=int(stsa+0.25)
      tsu=int(stsu+0.75)   ! Why +0.75?



! Copied from PREVAHCORE - start
      do LNRG=1,(tsa-1)
             ssdti(LNRG)=0.0
		end do
		do LNRG=tsa,tsu
		   ssdti(LNRG)=ssdti(LNRG)*24./real(tsu-tsa)
		end do
	    do LNRG=(tsu+1),24
             ssdti(LNRG)=0.0
	end do

	ssdPot(1:(tsa-1)) = 0.0
	ssdPot(tsa:tsu) = 1.0
	ssdPot((tsu+1):24) = 0.0



C     DO LNRG=1,24
C        RGDTI(LNRG)=RGMDTI(LNRG)*RGdm/RGMDM
C     END DO
! Copied from PREVAHCORE - end


! ---------------------------------------------------------------------

! Determination of hourly values between midnight and sunrise
      RGKDS=0.0  ! Initialization of daily sums
      !TEKDS=0.0
      do i=1,tsa
        rgdti(i) = 0.0
        RGKDTI(i)=0.0
        RGKDS=RGKDS+RGKDTI(i)
        TEKDTI(i)=TEDTI(i)
        RGMDTI(i)=0.0
        rgmkdti(i)=0.0
        RDMDTI(i)=0.0
        rdmkdti(i)=0.0
        cosZdti(i) = 0.0
        cf(i) = 0  ! "NA"
      end do


! integration to hourly steps
      ii=-1
      sumg=0.0
      sumgk=0.0
      sumd=0.0
      sumdk=0.0
      !shads=0.0

      do i = tsa,(tsu-1)
        ii = ii+1
        k=i



! Hourly angle
        WS=(real(12-(tsa+ii+0.5)))*15.0


! Cosinus of zenith-angle
        ZCOS=
     *       SIN(AKTDEKLIN*PI/180.0)*
     *       SIN(GEOBREITE*PI/180.0)+
     *       COS(Ws*PI/180.0)*
     *       COS(AKTDEKLIN*PI/180.0)*
     *       COS(GEOBREITE*PI/180.0)


      cosZdti(k) = zcos

! Sinus of zenith-angle
      WINK=acos(zcos)
	ZSIN=sin(WINK)


	 as=((SIN (AKTDEKLIN*PI/180.0)*COS (GEOBREITE*PI/180.0))-
     *        (COS (AKTDEKLIN*PI/180.0)*SIN (GEOBREITE*PI/180.0)*
     *         COS (WS*pi/180)))/zsin
         AS=ACOS(as)*180.0/PI
         IF (I .GE. 12) AS=360.0-AS

! cosinus of the sun angle and the angle normal to the slope
	 wslope=
     *       cos(slope*PI/180.0)*
     *       ZCOS+
     *       ZSIN*
     *       sin(slope*PI/180.0)*
     *       cos((as-aspect)*PI/180.0)

!      if((xcoor==98) .and. (ycoor==64)) print*, "wslope", wslope


! Computation of pressure
         PHOEHE=1013*EXP(-HHOEHE/(7991+29.33*TEDTI(K)))

! slope correction
         HFAK=WSLOPE
         IF ((WSLOPE .LE. 0.001) .or. (ZCOS .LE. 0.001)) THEN
             HFAK=0.0
             rgmkdti(k)=0.0
             rdmkdti(k)=0.0
         else
         rgmkdti(k)=HFAK*(I0*RM*RM/R/R)*(1.0-SHADTI(k))
         rdmkdti(k)=HFAK*(I0*RM*RM/R/R)*(0.88**((PHOEHE/P0)/ZCOS))
         rdmkdti(k)=rdmkdti(k)*(1.0-SHADTI(k))
        end if

         sumgk=sumgk+rgmkdti(k)
         sumdk=sumdk+rdmkdti(k)

         HFAK=ZCOS
         IF (ZCOS .LE. 0.001) THEN
             HFAK=0.0
             RGMDTI(k)=0.0
             RDMDTI(k)=0.0
         else
           RGMDTI(k)=HFAK*(I0*RM*RM/R/R)
          RDMDTI(k)=HFAK*(I0*RM*RM/R/R)*(0.88**((PHOEHE/P0)/ZCOS))
         end if
         sumg=sumg+RGMDTI(k)
         sumd=sumd+RDMDTI(k)

         ! Default case (exceptions below)
         korfak = wslope/zcos
!      if((xcoor==98) .and. (ycoor==64)) print*, "korfak1", korfak

         ! Special case: flat surface
         IF (wslope .LE. 0.001) THEN
            korfak = 0.0
         end if

         ! Special case: cloudy day
         IF (SSDTI(k) .LE. 0.1) THEN
            korfak = 1.0
         end if

         ! Special case: low sun
C        IF (ZCOS .LE. 0.033) THEN
C           korfak = 1.0
C         else if (ZCOS .LE. 0.208) then
C           korfak = max(0.0,min(5.0,korfak))
C        end if

        if(zcos .le. 0.34) then
          korfak = wslope/0.34
        end if

        cf(k) = korfak

! Help varaibles for accounting shadowing (not used for now)
         SHAD=1.0-SHADTI(k)
         IF (KORFAK .LE. 0.0) SHAD=0.0
         IF ((KORFAK.GT.0.0) .and. (KORFAK.LT.1.0)) SHAD=SHADTI(k)

      end do  ! End of first loop (daytime part)

! After sunset
      do i = tsu,24
        rgdti(i) = 0.0
        RGKDTI(i)=0.0
        RGKDS=RGKDS+RGKDTI(i)
        TEKDTI(i)=TEDTI(i)
        RGMDTI(i)=0.0
        rgmkdti(i)=0.0
        RDMDTI(i)=0.0
        rdmkdti(i)=0.0
        cosZdti(i) = 0.0
      end do

      rgmdm = sumg/24.

! Help variable for temperature correction
           do std=TSA,12
             faktor(std)=Float(std-TSA)/float(12-TSA)
           enddo

           do std=12,Tsu+1
             faktor(std)=1-float(std-12)/float(TSU+1-12)
             if (faktor(std) .lt. 0) then
               faktor(std)=0
             endif
           enddo

! Second loop for radiation correction and temperature coorrection
      do i = tsa,(tsu-1)
        k = i !(or i+1?)
      ! Get hourly, uncorrected global radiation by scaling hourly potential
      ! radiation with that day's actual-to-potential Rg ratio
        rgdti(k) = rgmdti(k) * (rgdm/rgmdm)

      ! Get hourly corrected global radiation
        korfak = cf(k)
        rgkdti(k) = rgdti(k)  * (1+0.85*ssdti(k)*(korfak-1))  ! Add shadowing term here if needed
        !rgkdti(k) = rgdti(k)  * (0.15+(0.85*ssdti(k)*(korfak)))
        rgkds = rgkds+rgkdti(k)


        if ((korfak .lt. 0.2) .and. (korfak .gt. 0.0)) then
             tekdti(k)=tedti(k)-(CT*0.6)*ssdti(k)*faktor(k)
     *                 *(1.0+(shadti(k)*0.5))
           IF ((k.EQ.TSA+1) .or. (k.EQ.TSA+2)) THEN
              tekdti(k)=tedti(k)-(CT*0.6)*(float(k-TSA)/3.0)*ssdti(k)
     *                  *faktor(k)*(1.0+(shadti(k)*0.5))
           ENDIF
           IF ((k.EQ.TSU-1) .or. (k.EQ.TSU)) THEN
              tekdti(k)=tedti(k)-(CT*0.6)*(float(TSU+1-k)/3.0)
     *          *ssdti(k)*faktor(k)*(1.0+(shadti(k)*0.5))
           ENDIF
        endif

        if ((korfak  .ge. 0.2) .and. (korfak .lt. 5.0)) then
             tekdti(k)=tedti(k)-(CT*0.6)*ssdti(k)*faktor(k)
     *       *(1.0-((korfak-0.2)/0.8))*(1.0+(shadti(k)*0.5))
           IF ((k.EQ.TSA+1) .or. (k.EQ.TSA+2)) THEN
              tekdti(k)=tedti(k)-(CT*0.6)*ssdti(k)*faktor(k)
     *                  *(float(k-TSA)/3.0)*(1.0-((korfak-0.2)/0.8))
     *                  *(1.0+(shadti(k)*0.5))
           ENDIF
           IF ((k.EQ.TSU-1) .or. (k.EQ.TSU)) THEN
              tekdti(k)=tedti(k)-(CT*0.6)*ssdti(k)*faktor(k)
     *                  *(float(TSU+1-k)/3.0)*(1.0-((korfak-0.2)/0.8))
     *                  *(1.0+(shadti(k)*0.5))
           ENDIF
        endif

        if ( korfak .ge. 5.0) then
             tekdti(k)=tedti(k)+CT*ssdti(k)*faktor(k)
     *                 *((korfak-1.0)/4.0)*(1.0-SHADTI(k))
           IF ((k.EQ.TSA+1) .or. (k.EQ.TSA+2)) THEN
              tekdti(k)=tedti(k)+CT*ssdti(k)*faktor(k)
     *                  *(float(k-TSA)/3.0)*((korfak-1.0)/4.0)
     *                  *(1.0-SHADTI(k))
           ENDIF
           IF ((k.EQ.TSU-1) .or. (k.EQ.TSU)) THEN
              tekdti(k)=tedti(k)+CT*ssdti(k)*faktor(k)
     *                  *(float(TSU+1-k)/3.0)*((korfak-1.0)/4.0)
     *                  *(1.0-SHADTI(k))
           ENDIF
        endif

        if ((shadti(k) .ge. 0.9) .or. (korfak .le. 0.0)) then
             tekdti(k)=tedti(k)-(CT*0.87)*ssdti(k)*faktor(k)
           IF ((k.EQ.TSA+1) .or. (k.EQ.TSA+2)) THEN
              tekdti(k)=tedti(k)-(CT*0.87)*(float(k-TSA)/3.0)*ssdti(k)
     *                  *faktor(k)
           ENDIF
           IF ((k.EQ.TSU-1) .or. (k.EQ.TSU)) THEN
              tekdti(k)=tedti(k)-(CT*0.87)*(float(TSU+1-k)/3.0)
     *                  *ssdti(k)*faktor(k)
           ENDIF
        endif


! ---------------------------------------------------------------------
! Code for debugging
C
        IF (DEBUG .GT. 1) THEN
         if (ii.eq.0) then
          WRITE(MDR,'(1X,''ii k ZCOS  ZSIN   AS   '',
     * ''  WS    WSLOPE   SUMG  SUMGK   RGKDTI   RGKDS SSD KORF'')')
         endif
         Write(MDR,'(  2i3,2f5.2,2f8.2,f5.2,4f8.2,2f5.2)' )
     *                ii,k,ZCOS,ZSIN,AS,WS,WSLOPE,
     *                sumg,sumgk,RGKOR,RGKDS,SSDTI(k),KORFAK
        ENDIF

      end do


      tekdm = sum(tekdti)/24.
      rgkdm = sum(rgkdti) /24.
      rgmkdm = sum(rgmkdti)/24.
      rdmdm = sum(rdmdti)/24.
      rdmkdm = sum(rdmkdti)/24.

      shadm = 0.0

      end subroutine rtkorz
