!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE SNOEVAP1 (TEC,RLF,WIV,EPDT,T0,SNOEST,
     *                     ESNODT,ESKDT,EIDT,SSO,SSNOW,SLIQ)

!     This subroutine computes evaporation/condensation from the snow cover
!     if the control parameter SNOEST is higher than 4
!      SNOEST
!      Methods on daily basis:
!      1 -  after PENMAN accounting the ageing of snow cover
!      2 -  after PENMAN-MONTEITH accounting the ageing of snow cover
!      Methods on hourlybasis:
!      3 -  after PENMAN accounting the ageing of snow cover
!      4 -  after PENMAN-MONTEITH accounting the ageing of snow cover
!      Methods 1-4 are already resolved in other routines
!      5 -  Routine Sublimation condensation after KUZMIN
!      6 -  Routine Sublimation condensation after ANDERSON
!--------------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Common Variables
! ---------------------------------------------------------------------

      REAL TEC,					! Air temeprature
     *     RLF,					! Relative humidity
     *     WIV,					! Wind speed
     *     EPDT,				! Potential ET in timestep
     *     ESNODT,				! Snow evaporation in timestep
     *     EIDT,				! Interception evaporation in timestep
     *     SSO,					! Snow water equivalent
     *     SSNOW,				! Solid phase of SSO
     *     SLIQ,				! Liquid phase of SSO
     *     T0,					! Threshold temperature for snowmelt
     *     ESKDT				! Help value for ESNODT
      INTEGER SNOEST

! ---------------------------------------------------------------------
!     Internal Variables
! ---------------------------------------------------------------------

      REAL SSOA,				! SSO at timestep begin
     *     RLFP,				! RLF in percent
     *     VKOR,				! correction facto for wind in Kuzmin
     *     VAP,					! water vapour pressure
     *     VAPS,				! saturated water vapour pressure
     *     CSWU1,				! constant for Kuzmin
     *     CSWA2,				! constant for Kuzmin
     *     CLSV					! CLSV=Ls/Lv  Ls=340 J/g
						! Lv for melting snow = 2498 J/g
						! Lv for ice = 2790 J/g

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

      ESNODT=0.0
      ESKDT=0.0

      CLSV=340.0/2498.0

      IF (TEC .LT. T0) CLSV=340.0/2790.0

      SSOA=SSO

      IF ( SSO .LT. 0.01 ) GOTO 10

      IF ( SNOEST .LE. 4 ) THEN
      IF (EPDT .LT. 0.0) EPDT=0.0
      IF ( EPDT .LE. 0.0 ) GOTO 10
         ESNODT=EPDT
      ENDIF

! Kuzmin
      IF ( SNOEST .EQ. 5 ) THEN
         VKOR=1.2
         CSWU1=0.18
         CSWA2=0.098
         IF ( TEC .GT. 0.0 ) THEN
            VAPS=6.11
         ELSE
            RLFP=RLF*100.0
            CALL VAPOUR (TEC,RLFP,VAP)
            VAPS=VAP/RLF
         ENDIF
         ESNODT=(CSWU1+(CSWA2*(WIV*VKOR))*(VAPS-VAP))/24.0
      ENDIF

! Anderson
      IF ( SNOEST .EQ. 6 ) THEN
         CSWU1=0.0
         CSWA2=0.5
         IF ( TEC .GT. 0.0 ) THEN
            VAPS=6.11
         ELSE
            RLFP=RLF*100.0
            CALL VAPOUR (TEC,RLFP,VAP)
            VAPS=VAP/RLF
         ENDIF
	   ESNODT=(3.0 + 3.6*WIV**0.7) * ((2831.0*0.623)/ (1.005*1000.0))
     *                                * (VAPS - VAP)
	   ESNODT=ESNODT*3600./2836000.0
      ENDIF

! Update of SSO and its liquid and solid phase
      SSO=SSO-ESNODT
      IF ( SSO .LE. 0.0 ) THEN
         ESNODT=SSOA
         SSO=0.0
         SSNOW=0.0
         SLIQ=0.0
         GOTO 8
      ENDIF

      SSNOW=SSNOW*(SSO/SSOA)
      SLIQ=SLIQ*(SSO/SSOA)

8     IF (ESNODT .GE. 0.0) THEN
         EIDT=EIDT+ESNODT
         IF (EIDT .GT. EPDT) EPDT=EIDT
      ELSE
         ESKDT=ESNODT
         ESNODT=0.0
      ENDIF

! ---------------------------------------------------------------------
10    CONTINUE
      RETURN
      END
