!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE SNOMELT2 (TAG,MON,JAHR,LAND,ISNOGL,TACC,TAIR,RDKM,
     * RHUMIT,WIV,PDT,NDTD,CREDV,TGR,TTRANS,T0,M,CRMFMAX,CRMFMIN,CWU1,
     * CWA2,CRFR,CWH,CEXLIQ,PRECMIN,CSNOMF,CASNO,SNOW,SSO,SSNOW,SLIQ,
     * WSA,WSB,IDAT,IDKAL,MDR,DEBUG,kk)
! ------------------------------------------------------------------------------

! ------------------------------------------------------------------------------
!    Snowmelt module
!
!    Literature:
!    Anderson, E.A. (1973): National Weather Service River Forecast System-Snow
!     Accumulation and Ablation Model. NOAA Tech. Memo. NWS. U.S. Dep. of Commerce,
!     Silver Spring.
!    Braun, L. (1985): Simulation of Snowmelt-Runoff in Lowland and Lower Alpine
!     Regions of Switzerland. Z�rcher Geographische Schriften, Heft 21, ETH Z�rich
!     [available from the Institute for Atmospheric and Climate Science ETH,
!     Winterthurerstr. 190, CH-8057 Z�rich].
!    Hock, R. (1999): Distributed temperature-index ice- and snowmelt model including
!     potential direct solar radiation. J. Glaciol., 45, 101-111.
!    Pos, F. (2001): Spatially distributed modelling of snow accumulation and melt in
!     pre-alpine and alpine catchments. Diploma Thesis, Wageningen University. [available
!     from the Institute for Atmospheric and Climate Science ETH, Winterthurerstr.
!     190, CH-8057 Z�rich].
!    Strasser, U. and Mauser, W. (2001): Modelling the spatial and temporal variations
!     of the water balance for the Weser catchment 1965-1994. J. Hydrol., 254, 199-214.
!    Strasser, U., Etchevers, P. and Lejeune, Y. (2002): Intercomparision of two snow
!     models with different complexity using data from an alpine site. Nordic Hydrol.,
!     33, 15-26.
!    Zappa, M., Pos, F., Strasser, U. and J. Gurtz (2003): Seasonal water balance of
!     an alpine catchment as evaluated by different methods for spatially distributed
!     snow melt modelling. Nordic Hydrology, Vol. 34(3), 179-202.
!
!    Control variable ISNOGL
!    ISNOGL = 0 computation after positive degree day index of combination method
!				  = 1 Hock on glaciated areas with constant melt factor
!				  = 2 Hock on glaciated areas with varying melt factor
!				  = 3 Hock everywhere with constant melt factor
!				  = 4 Hock everywhere with varying melt factor
!				  = 5 Hock everywhere with constant melt factor, only if sunny day (no precipitation)
!				  = 6 Hock everywhere with varying melt factor, only if sunny day (no precipitation)
!				  = 10 ESCIMO
!    Last change: 30.06.2000
! ---------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------
!     Variables
! ---------------------------------------------------------------------
      INTEGER SNOW,KK,				! Flag to indicate presence of snow cover
     *        NDTD,				! Number of time steps per day
     *        M,					! Snow age in days
     *        DEBUG,				! Variable for partial output
     *        IT,					! Help variable for computing seasonal melt factors
     *        MDR,				! Control the output to the prompt and to the log-files
     *        ISNOGL,				! Variable to control the snowmelt computation
     *        LAND,				! Land use category
     *        TAG,ITAG,MON,IMON,JAHR,		! Managing the date
     *        IDAT(12),IDKAL			! Managing the date

      REAL TAIR,TACC,				! Air temperature - Air Temperature for Accumulation (not corrected for slope and aspect)
     *     RHUMIT,				! relative humidity
     *     WIV,					! Wind speed
     *     PRECMIN,				! Treshold precipitation for generatig rain on snow events
     *     SSO,					! snow water equivalent
     *     SSNOW,					! solid part of snow water equivalent
     *     SLIQ,					! liquid part of snow water equivalent
     *     WSA,WSB,				! water release from the snow cover
     *     RDKM,					! corrected radiation
     *     RIM					! help variable for indegrating hourly snowmelt values

      CHARACTER SITU*1

      REAL PSNOW,                 ! solid precipitation
     *     PLIQ,                  ! liquid precipitation
     *     PREC,PDT,				! precipitation in this time step
     *     WCMELT,				! potential snowmelt / refreezing
     *     WSTAR,					! snowmelt through temperature (and evtl. radiation)
     *     WH,					! snowmelt through sensible heat flux
     *     WR,					! snowmelt through rainfall
     *     WE,					! snowmelt through latent heat flux
     *     SLIQA,					! liquid content of snow cover at begin of current time step
     *     SLIQMAX,				! maximal liquid content of snow cover
     *     RMFAMP,				! amplitude of seasonal variaiton of melt coefficients
     *     RMF,					! melt coefficient
     *     VAP,					! water vapour pressur
     *     PSY,					! psychometic constant
     *     CONST,CONSTR,			! help variables
     *     ATSLIQ,				! portion of liquid water content that can be filled
     *     CREDV,				! reduction factor - vegetated - unvegetated areas
     *     TGR,					! treshold temperature snowfall
     *     TTRANS,				! transition temperature snow-rain
     *     T0,					! treshold temperature snowmelt
     *     CRMFMAX,				! maximal temperature-index melting factor
     *     CRMFMIN,				! minimal temperature-index melting factor
     *     RMFMEAN,				! seasonal varying temperature-index melting factor
     *     CWU1,				! melting factor by wind 1
     *     CWA2,				! melting factor by wind 2
     *     CWH,					! retention factor for melting snow
     *     CEXLIQ,				! exponent for retention of melting snow
     *     CRFR,				! coefficien for re-freezin if Air temperature < 0 �C
     *     CSNOMF,				! constant temperature melt factor for snow
     *     CASNO				! constant radiation melt factor for snow

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

!     Initialization
      IMON=MON
      SNOW=0
      SITU='i'
      M=M+1
      WSB=0.0
      PSNOW=0.0
      PREC=PDT
      PLIQ=PREC
      WSA=0.0
      WCMELT=0.0
      WSTAR=0.0
      WH=0.0
      WR=0.0
      WE=0.0
      WSB=0.0
      RIM=1.0*NDTD
      SLIQA=SLIQ

! ---------------------------------------------------------------------
!     This operations are processed only if CREDV>0.001, if snow is
!     or accumulation within the current time step.
!     No accumulation is possible on water surfaces (land = 1)
! ---------------------------------------------------------------------

      IF  (CREDV .LT. 0.001)  GOTO 1230
      IF  ((SSO.LT.0.01) .and.
     *    ((PDT.LT.0.01) .or. (TACC.GT.(TGR+TTRANS))) )  GOTO 1230
      IF (LAND .EQ. 1) GOTO 1230

! ---------------------------------------------------------------------
! Seasonal variation of the t-index and radiation-index melt factors
! ---------------------------------------------------------------------
      ITAG=-273
      IF (IDAT(2) .GT. 28) THEN
         ITAG=-274
      ENDIF
      IF (IMON .LT. 10) THEN
         ITAG=93
      ENDIF
      IT=IDKAL+ITAG
      RMFAMP=(CRMFMAX-CRMFMIN)/2.0
      RMFMEAN=CRMFMIN+RMFAMP
      CONST=360.0/(366.0*57.3)
      RMF=RMFMEAN+RMFAMP*SIN((IT-173)*CONST)
      CONSTR=1.162*0.0107
      PSY=0.65

! ---------------------------------------------------------------------
! Ranifall and Snowall already adjusted in main progamm
! Separation of rainfall-snowfall
! ---------------------------------------------------------------------
      IF ((TACC .LE. (TGR-TTRANS)) .or.
     *    ((TTRANS .LE. 0.01) .and. (TACC  .LE. (TGR+TTRANS)))) THEN
         PSNOW=PREC
         PLIQ=0.0
         GOTO 1202
      ENDIF
      IF ((TACC .GT.(TGR-TTRANS)) .and. (TACC .LT.(TGR+TTRANS))) THEN
         PSNOW=PREC*((TGR+TTRANS-TACC)/(2*TTRANS))
         PLIQ=PREC-PSNOW
         IF (PLIQ .LT. 0.0) THEN
            PSNOW=PREC
            PLIQ=0.0
         ENDIF
      ENDIF
1202  IF (PSNOW .GE. 0.01) THEN
         M=0
      ENDIF

! ---------------------------------------------------------------------
!     Assignment of PSNOW and PLIQ to the solid (SSNOW) and liquid (SLIQ)
!     storages of the snowpack
! ---------------------------------------------------------------------

      SSNOW=SSNOW+PSNOW
      SLIQ=SLIQ+PLIQ

! ---------------------------------------------------------------------
!     Determining if accumulation or ablation
! ---------------------------------------------------------------------
! Accumulation
      IF (TAIR .LE. T0) THEN
         SITU='-'
         IF (SSNOW .LT. 0.001) THEN		! Is there snow?
            SITU='K'				! No!
            IF (SLIQ .GE. 0.001) THEN		! Rainfall or melted snow?
               WSA=SLIQ				! All goes in the runoff generation module
               SLIQ=0.0
               SITU='B'
            ENDIF
         ELSE

!        If snowcover is present then
         SITU='A'
         WSTAR = RMF * (TAIR-T0) / RIM		! (TAIR-T0) < 0!!!
         WCMELT = WSTAR * CRFR

!        Liquid water can freeze, SLIQ is reduced, WCMELT < 0!!!
         SLIQA=SLIQ
         SLIQ=SLIQ+WCMELT

!       Checking if SLIQ is now negative, SLIQ mus be >= 0.0
         IF (SLIQ .LT. 0.0) SLIQ=0.0

!        Freezed water is added to SSNOW
         SSNOW=SSNOW+(SLIQA-SLIQ)
         SLIQMAX=CWH*SSNOW
         IF (SLIQ .GT. SLIQMAX) THEN
            WSA=(SLIQ-SLIQMAX)
            SLIQ=SLIQMAX
         ENDIF
	   ENDIF
         GOTO 500 ! No snowmelt !
      ENDIF

      SITU='+'

! ---------------------------------------------------------------------
! Ablation

      IF (SSNOW .LT. 0.01) THEN
! No snow ... no ablation
         SITU='T'
         WSA=SLIQ
         SLIQ=0.0
         IF (WSA .GT. 0.01) SITU='L'
         GOTO 500
      ENDIF

! ---------------------------------------------------------------------
! There is snow, ablation is possible.

! Is a treshold precipitration intensity reached?

! ---------------------------------------------------------------------
! rain on snow situation


 !     IF (kk .eq. 2999) Write (777,*) isnogl,prec,precmin,tair

      IF ((PREC .GE .PRECMIN) .and. (TAIR .GT. 0.0)) THEN
         SITU='P'
         CALL VAP2 (TAIR,RHUMIT,VAP)
! Snowmelt after Anderson and Braun
         WSTAR = 1.2 * TAIR / RIM
         WH = (CWU1 + CWA2*WIV) * TAIR / RIM
         WE = (CWU1 + CWA2*WIV) * (VAP - 6.11) / (PSY*RIM)

! If CWU1 and CWA2 <=0.0 then only positive degree-day approach is used
         IF ((CWU1 .LE. 0.0) .and. (CWA2 .LE. 0.0)) THEN
            WSTAR= RMF * (TAIR-T0) / RIM
            WH=0.0
            WE=0.0
         ENDIF

! Snowmelt after Hock, eventually only on glaciated areas, with constant snowmelt factors
         IF ((ISNOGL .EQ. 3) .OR.
     *     ((ISNOGL .EQ. 1) .and. ((LAND .EQ. 13) .or. (LAND .EQ. 14))))
     *      THEN
            WSTAR=((CSNOMF/RIM)+(CASNO*RDKM))*(TAIR-T0)
            WH=0.0
            WE=0.0
            GOTO 125
         ENDIF

! Snowmelt after Hock, eventually only on glaciated areas, with varying snowmelt factors
         IF ((ISNOGL .EQ. 4) .OR.
     *     ((ISNOGL .EQ. 2) .and. ((LAND .EQ. 13) .or. (LAND .EQ. 14))))
     *      THEN
            WSTAR=((RMF/RIM)+(CASNO*RDKM))*(TAIR-T0)
            WH=0.0
            WE=0.0
            GOTO 125
         ENDIF

125      WR = PREC * TAIR * CONSTR
         WCMELT = WSTAR + WH + WE + WR
         IF (WCMELT.LT.(-SLIQ)) WCMELT=-SLIQ
      ELSE

! ---------------------------------------------------------------------
!        Radiation melt situation

! Positive degree-day-index
         SITU='M'
         WSTAR = RMF * (TAIR-T0) / RIM

! Snowmelt after Hock, eventually only on glaciated areas, with constant snowmelt factors
         IF ((ISNOGL .EQ. 3) .OR. (ISNOGL .EQ. 5) .OR.
     *      ((ISNOGL .EQ. 1) .and. ((LAND .EQ. 13).or.(LAND .EQ. 14))))
     *      THEN
            WSTAR=((CSNOMF/RIM)+(CASNO*RDKM))*(TAIR-T0)
!	      IF (kk .eq. 2999) Write (777,'(3I4,8f8.3)') TAG,MON,KK,TAIR,
!     * RDKM,CASNO,CSNOMF,RIM,WSTAR
            GOTO 135
         ENDIF

! Snowmelt after Hock, eventually only on glaciated areas, with varying snowmelt factors
         IF ((ISNOGL .EQ. 4) .OR. (ISNOGL .EQ. 6) .OR.
     *      ((ISNOGL .EQ. 2) .and. ((LAND .EQ. 13).or.(LAND .EQ. 14))))
     *      THEN
            WSTAR=((RMF/RIM)+(CASNO*RDKM))*(TAIR-T0)
            GOTO 135
         ENDIF
135      WCMELT = WSTAR
      ENDIF
      WSB=WCMELT

! ---------------------------------------------------------------------
!     Checking if potential snowmelt is higher than currently available snowcover
      IF (SSNOW .LE. WCMELT) THEN
         WSA=SSNOW+SLIQ
         SSNOW=0.0
         SLIQ=0.0
         SITU='S'
         GOTO 500
      ENDIF

!     There is still snow after this time step
      SLIQA=SLIQ
      SSNOW=SSNOW-WCMELT
      SLIQMAX=CWH*SSNOW

! ---------------------------------------------------------------------
! Filling and depletion of the liquid content of snow cover
! ---------------------------------------------------------------------
      IF ((SLIQMAX .GT. 0.0) .and. (CEXLIQ .GT. 0.0) .and.
     *     (WCMELT .GT. 0.0))  THEN
         ATSLIQ=(SLIQ/SLIQMAX)**CEXLIQ
         IF (ATSLIQ .GE. 1.0) ATSLIQ=1.0
         IF (ATSLIQ .LE. 0.0) ATSLIQ=0.0
         SLIQ=SLIQ+((1.0-ATSLIQ)*WCMELT)
         WSA=ATSLIQ*WCMELT
      ELSE
! The total melt rate is added to the liquid content of the snow cover, until SLIQ is full
         SLIQ=SLIQ+WCMELT
         WSA=0.0
      ENDIF
      SLIQMAX=CWH*SSNOW
      SITU='C'

!     Is SLIQ full?
      IF (SLIQ .GT. SLIQMAX) THEN
!      Yes -> spillway added to melted water
         WSA=WSA+(SLIQ-SLIQMAX)
         SLIQ=SLIQMAX
         SITU='X'
      ENDIF

! ---------------------------------------------------------------------
500   CONTINUE
!     Determination of the snow water equivalent
      SSO=SSNOW+SLIQ
      IF (WSA .LT. 0.0) WSA=0.0

      PDT=WSA


! ---------------------------------------------------------------------
! Code for debugging

! ---------------------------------------------------------------------
1225  FORMAT (  ' Datum: ',I2,'.',I2,'.',I4 )
1226  FORMAT (' SNOMELZ',6X,'  Niederschlag (Schneeschmelze unberueck',
     *        'sich.): ',F8.2,/,
     *        14X, '  Gefallener Schnee (Tageswert)',15X,': ',F8.2,/,
     *14X,'  Schneespeicherung',27X,': ',F8.2,/,14X,'  Fluessiger ',
     * 'Wasseranteil in der Schneedecke   : ',F8.2,/,14X,
     * '  Pot. Schmelzwassermenge',21X,': ',
     *F8.2,/,14X,'  Schmelzwasserabgabe',25X,': ',F8.2)
      IF (DEBUG .LE. 0) GOTO 1230
      WRITE (MDR,1225) TAG,MON,JAHR
      WRITE (MDR,1226) PREC,PSNOW,SSNOW,SLIQ,WSB,WSA

! ---------------------------------------------------------------------
1230  CONTINUE

!     Flag SNOW, to indicate if snow cover is present (1) or not (0)
!     Needed for updating the age of snow
      IF (SSO .GE. 0.5)  SNOW=1

! ---------------------------------------------------------------------
      RETURN
      END
! ---------------------------------------------------------------------
