!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 1995-2004  J. Gurtz, M. Zappa et al.

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

      SUBROUTINE SNOWFLUX (SNOEST,PRECIP,TEMP,WIND,HUM,RGL,
     *              LONG,SWE,PHASE,DMASLAT,SURFTEMPOLD,JOULE,
     *              SNOWAGE,ALBEDO,pressnow,presair)

! ---------------------------------------------------------------------------
! SNOWFLUX - Based on the parameterization of the turbulent fluxex above snow
!            available in ESCIMO 2.0 - A snow model by U. Strasser 2001
!                             Strasser@ihw.baug.ethz.ch
!            Can be activated in PREVAH if SNOEST 5 or 7. SNOWFLUX computes
!            Sublimation and Condensation                    M.Zappa Okt 2001
! ---------------------------------------------------------------------------

      implicit none

! ---------------------------------------------------------------------------
! Variables
! ---------------------------------------------------------------------------

      INTEGER SNOEST
      REAL PRECIP,TEMP,AIRTEMP,WIND,HUM
      REAL RGL,SHORT,LONG,INTERVALL
      REAL SURFTEMP,SURFTEMPOLD
      REAL PRESAIR,PRESSNOW
      REAL SNOWAGE,ALBEDO,PHASE
      REAL JOULE,MELTENERGY
      REAL SENSFLUX,LATFLUX,ALPHA
      REAL SWE,MELT
      REAL SOILFLUX, EMISSIVITY
      REAL ALBMIN, ALBADD, ABKLINGPOS, ABKLINGNEG,A0
	REAL ABKLINGACT,INDEX
      REAL SPEWSNOW, SPEWAS, SCHMELWAS, STEFBOLTZ
      REAL DENSITY, THERMCOND, CONSUBSNOW
      REAL DMASLAT

! ---------------------------------------------------------------------
!     Begin
! ---------------------------------------------------------------------

! ---------------------------------------------------------------------
! Variables and Parameters
! ---------------------------------------------------------------------
! PRECIP                 !Presipitation in mm/h
! PHASE                  !0=Rain, 1=Snow
! TEMP                   !Air temperature in deg C
! AIRTEMP                !Air temperature TEMP+273.16
! WIND                   !Windg speed in m/s
! RGL                    !Global radiation in W/m2
! HUM                    !Relative Humidity (0.0...1.0)
! SWE                    !Snow water equivalent
! SHORT                  !Short-wave radiation balance
! LONG                   !Lon wave radiation balance
! SURFTEMP               !Temperature of snow surface
! SURFTEMPOLD            !Temperature of snow surface in previous time step
! PRESAIR                !Vapour Pressure at plot height
! PRESSNOW               !Vapour Pressure above snow
! SNOWAGE                !Snow age
! ALBEDO                 !Snow albedo
! JOULE                  !Energy balance of snow cover
! MELTENERGY             !required energy for melting
! SENSFLUX               !sensible heat
! LATFLUX                !latent heat
! ALPHA                  !Heat transfer number
! MELT                   !snow melt
! DMASLAT                !mass change owed to latent heat (mm)
! SOILFLUX   =     2.0   !ground heat flux    (W/m2)
! EMISSIVITY =     1.0   !Emissivity of snow
! ALBMIN     =     0.40  !Minimal Albedo of snow
! ALBADD     =     0.44  !Addition parameter for Albedo
! ABKLINGPOS =     0.10  !ageing parameter for albeto at positive air temperature
! ABKLINGNEG =     0.05  !ageing parameter for albeto at negative air temperature
! SPEWSNOW   =   1840.0  !Spec. heat of snow  (J/kg�K), after Geiger
! SPEWAS     =   4200.0  !Spec. heat of water (J/kg�K)
! SCHMELWAS  = 337000.0  !Melting heat        (J/kg)
! STEFBOLTZ  =  5.67E-8  !Stefan-Boltzmann-constant (W/(m2*�K**4))
! DENSITY    =    800.0  !density of snow surface (for computation of temperature)
! THERMCOND  =    1.676  !Therm. conductivity (for computation of temperature)
! CONSUBSNOW =2836000.0  !Condens./Sublim.-Heat for snow

! ---------------------------------------------------------------------
! Variables and parameters are initialized
! ---------------------------------------------------------------------

      AIRTEMP=TEMP+273.16
      INTERVALL  =  3600.0

      SOILFLUX   =     2.0
      EMISSIVITY =     1.0
	A0		   =     0.1
      ALBMIN     =     0.40
      ALBADD     =     0.50
      ABKLINGPOS =     0.08
      ABKLINGNEG =     0.03
      SPEWSNOW   =   1840.0
      SPEWAS     =   4200.0
      SCHMELWAS  = 337000.0
      STEFBOLTZ  =  5.67E-8
      DENSITY    =    800.0
      THERMCOND  =    1.676
      CONSUBSNOW =2836000.0

      MELT       = 0.0
      SHORT      = 0.0
c	LONG       = 0.0
	ALBEDO     = ALBMIN
	JOULE      = 0.0
      MELTENERGY = 0.0
	ALPHA      = 0.0
	SENSFLUX   = 0.0
	LATFLUX    = 0.0
	PRESAIR    = 0.0
	PRESSNOW   = 0.0

! ---------------------------------------------------------------------
!  Berechnung der Schneeoberflaechentemperatur, kombinierter Ansatz
!             fuer negative Lufttemperaturen
      IF(AIRTEMP .LT. 273.16) THEN
!           !Ambach/Vetter, siehe unten!
           SURFTEMP = SURFTEMPOLD
!           !fuer positive Lufttemperaturen
        ELSE
           SURFTEMP = 273.16
      ENDIF
! ---------------------------------------------------------------------
! 1.Strahlungsbilanz fuer die Schneedecke

!     Waermeuebergangszahl Alpha:
!        !damit Alpha nicht negativ wird
      IF (WIND.GT.0.1069498) THEN
          ALPHA = 4.4602 * LOG(WIND) + 9.97031
!        !ist gefittet durch Literaturwerte
      ENDIF

!     Besser nach Escher-Vetter (1980) aus ihren Messungen von Windprofilen:
      ALPHA = 5.7 * SQRT(WIND+0.00001)
!        Fuehlbarer Waermestrom:
      SENSFLUX = ALPHA * (AIRTEMP - SURFTEMP)


! ---------------------------------------------------------------------
! Berechnung des Dampfdruckes (mb) im Messniveau mit der Magnus-Formel
      PRESAIR  = 6.1078 * EXP((17.08085*( AIRTEMP-273.16))/
     *                   (234.175+(AIRTEMP-273.16))) * (HUM)  !/100.
! Berechnung des Dampfdruckes an der Schneeoberflaeche
!                (Saettigungsdampfdruck zu SURFTEMP)
      PRESSNOW = 6.1078 * EXP((17.08085*(SURFTEMP-273.16))/
     *                   (234.175+(SURFTEMP-273.16)))
! ---------------------------------------------------------------------
! Latenter Energiestrom aus Kondensation + Verdunstung
!     entweder nach Kuchment/Gelfan, diese nach Kuz'min (J.Hydrol.1996)
      IF (SNOEST .EQ. 7)
     *    LATFLUX = 32.82 * (0.18 + 0.098*WIND) * (PRESAIR - PRESSNOW)

C...oder nach Bloeschl/Kirnbauer/Gutknecht (DGM,31,1987,H.5)
      IF (SNOEST .EQ. 8)
     *    LATFLUX = (3.0 + 3.6*WIND**0.7) * ((2831.0*0.623)/
     *              (1.005*1000.0)) * (PRESAIR - PRESSNOW)

! Latente Fluesse haben einen Massengewinn (Kondensation) bzw. -verlust
!                                          (Sublimation) zur Folge
      DMASLAT = (LATFLUX*INTERVALL) / CONSUBSNOW

! ---------------------------------------------------------------------
! Albedo der Schneedecke nach Rohrer
!        (Original-Abklingparameter 0.12 bzw. 0.05)

      INDEX=ABS(TEMP/10.0)
	IF (INDEX .GT. 1.0) INDEX=1.0
	INDEX=INDEX**0.25
	IF (TEMP .LT. 0.0) INDEX=INDEX*(-1)
	ABKLINGACT=ABKLINGNEG+(1+INDEX)*(ABKLINGPOS-
     *           ABKLINGNEG)/2

	ALBEDO = ALBMIN + ALBADD * EXP(-ABKLINGACT * SNOWAGE)

      SHORT    = (1. - ALBEDO) * RGL

! ---------------------------------------------------------------------
! Gesamtstrahlungsbilanz der Schneeoberflaeche fuer den Zeitschritt
      JOULE = (SENSFLUX + LATFLUX + SHORT + LONG + SOILFLUX) * INTERVALL

! ---------------------------------------------------------------------

! 3.Berechnung des Wasseraequivalents in Millimeter mit Massenaenderung durch
!      latenten Flux (Sublimation:Verlust/Kondensation:Gewinn)
      SWE = SWE + DMASLAT

! ---------------------------------------------------------------------

      IF (PRECIP.NE.0.0.AND.PHASE.EQ.1.0) THEN
!          !Bei mehr als 0.5 mm / h Niederschlag
          IF ((PRECIP/INTERVALL) .GT. (A0/INTERVALL)) THEN
!            !Albedo auf Startwert setzen
              ALBEDO  = ALBMIN + ALBADD
              SNOWAGE = 0.0                                           !Alter reinitialisieren
 	        SHORT   = (1. - ALBEDO) * RGL                  !Neuberechnung der kurzw. Str.
       JOULE   = (SENSFLUX + LATFLUX + SHORT + LONG + SOILFLUX)*
     * INTERVALL !und der Strahlungsb.
      ENDIF
	ENDIF

! ---------------------------------------------------------------------
! Heat-transfer-based Berechnung der Schneeoberflaechentemperatur nach Ambach (1955)
! und Escher-Vetter (1980), siehe CD der Kommission fuer Glaziologie...
!        !Energie reicht aus

          SURFTEMPOLD = SURFTEMP + 2. * (JOULE / INTERVALL) *
     *          SQRT (INTERVALL / (3.14159*THERMCOND*DENSITY*SPEWSNOW))
          IF (SURFTEMPOLD .GT. 273.16) SURFTEMPOLD = 273.16


! ---------------------------------------------------------------------
! Alter hochzaehlen oder reinitialisieren
      IF (SWE.GT.0.0) THEN
!           !Alter der Schneedecke hoeher zaehlen:
          SNOWAGE  = SNOWAGE + (INTERVALL/86400.)
!         !...keine Schneedecke mehr...
        ELSE
          SNOWAGE  = 0.0
      ENDIF

	joule=joule/intervall

! ---------------------------------------------------------------------
      RETURN

      END
