!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Interface module to get vegetation properties (max LAI, max FVC, canopy height
! and RSMIN) from TreeMig and share them with PREVAH.
! Previously in LocalForestDynamics (Marc)
!-------------------------------------------------------------------------------


! Jun 2017 - MS:
! LA adapted at species level as a function of drought and temperature stress

subroutine vegPars(lon,lat,thiscell,nspc)

  use forhycsglobals, only: fhla, perCC, vh100, vh99, vh95, laRed,readSwitch, rsmin2d, rsminStd
  use all_par, only: CurrStateInCell, spc, maxspc, maxhc, spec
  use prevahglobals, only: jahr     ! DEBUG ONLY

  implicit none

  ! Passed variables
  integer, intent(in) :: lon, lat, nspc
  type(CurrStateInCell), intent(in) :: thiscell

  ! Internal variables
  real :: caSum
  real, dimension(0:maxhc) :: laByHt, laByHtRel
  real, dimension(0:maxhc) :: nTrees, nTreesRel
  real, dimension(30,0:maxhc) :: laPerCohort, rsPerCohort
  real :: nTreesTot, nTreesCumu, zVeg
  integer :: class100, class99, class95
  real :: laTot, laCumu
  integer :: i
  integer :: height
  real :: numinSqm
  real :: laRedFac



 !---------FORHYCS - leaf area and crown area
           perCC(lon,lat) = 0.0
           caSum = 0.0
           nTrees = 0.0
           laByHt = 0.0
           laPerCohort = 0.0

           fhla(lon,lat,32) = 0.0
           BySpc: DO i=1,nspc
             fhla(lon,lat,i)=0
           ! Calculate C allocation coefficients
           DO height=0,spec(i)%maxhtc
             if(height == 0) then
               laRedFac = laRed(i,1)
             else
               laRedFac = laRed(i,height)
             end if
             ! If state is being read - No LAI reduction at the beginning (for now)
             if(readSwitch==.FALSE.) then
               laRedFac = 1.0
             end if
              fhla(lon,lat,i) =fhla(lon,lat,i)+ thiscell%sp(i)%numin(height )* spc(i)%lain(height) * laRedFac /2.0 !FORHYCS
              if(i .le. maxspc-1) then  ! Exclude ground vegetation from total LAI calculations
                laByHt(height) = laByHt(height)+ thiscell%sp(i)%numin(height )* spc(i)%lain(height) * laRedFac /2.0
                ! Save LAI by cohort (species/size) for calculation of RSMIN
                laPerCohort(i, height) = thiscell%sp(i)%numin(height )* spc(i)%lain(height) * laRedFac /2.0 !FORHYCS
              end if
              numinSqm = thiscell%sp(i)%numin(height) / 833.0
              caSum = caSum + numinSqm * spc(i)%crArea(height)  ! Number of trees per square meter
              ntrees(height) = ntrees(height) + thiscell%sp(i)%numin(height ) ! Counts trees (all species together) in each height class
           END DO

              if(i .le. maxspc-1) then  ! Exclude ground vegetation from total LAI calculations
                fhla(lon,lat,32) = fhla(lon,lat,32) + fhla(lon,lat,i)  ! Temp Matthias
              end if

          END DO BySpc

! Percent crown coverage for this cell (Diss Zurbriggen p. 64)

  perCC(lon,lat) = max(0.0, min(1.0, 1.0 - exp(-1.0*caSum)))

! Minimum stomatal resistance calculation
  laPerCohort(:,:) = laPerCohort(:,:) / fhla(lon,lat,32)    ! Relative proportion of LAI for each cohort
  rsPerCohort(:,:) = laPerCohort(:,:) * rsminStd(:,:)
  rsmin2d(lon,lat) = sum(rsPerCohort)



!============================================
! Trying to determine canopy height through number of trees in each class
! => class that covers a cumulative 90% of trees determines canopy height.
! BUT: this returns very low values (1st or 2nd class for most cells) -> unrealistic
! Trying with 95th and 99th percentile


! Relative number of trees in each height class
  nTreesTot = sum(nTrees)
  nTreesRel = nTrees(0:maxhc)/nTreesTot
  if(nTreesTot==0) nTreesRel(0:maxhc) = 0


! Maximum
  nTreescumu=0.0
  do i = 0,maxhc
    nTreesCumu = nTreesCumu + nTreesRel(i)
! Sometimes the ratios don't add up to 1 because of numerical error, causing the model to crash
! - therefore 0.999 instead of 1
    if(nTreesCumu .ge. 0.999) then
      class100 = i
      exit
    end if
  end do
  if(nTreesTot==0) class100 = 0
  zVeg = 1.37 + real(class100)*4.2
  vh100(lon,lat) = zVeg


! 99th percentile
  nTreescumu=0.0
! Determine the height class containing a cumulative 99% of all trees in
! the current cell (vh99)
  do i = 0,maxhc
    nTreesCumu = nTreesCumu + nTreesRel(i)
    if(nTreesCumu .ge. 0.99) then
      class99 = i
      exit
    end if
  end do
  if(nTreesTot==0) class99 = 0
  zVeg = 1.37 + real(class99)*4.2
  vh99(lon,lat) = zVeg

  ! 95th percentile
  nTreescumu=0.0
  do i = 0,maxhc
    nTreesCumu = nTreesCumu + nTreesRel(i)
    if(nTreesCumu .ge. 0.95) then
      class95 = i
      exit
    end if
  end do
  if(nTreesTot==0) class95 = 0
  zVeg = 1.37 + real(class95)*4.2
  vh95(lon,lat) = zVeg


end subroutine vegPars
