!-------------------------------------------------------------------------------
! FORHYCS: A spatially distributed model combining hydrology and forest dynamics
! Copyright (C) 2014-2018  Matthias Speich, Massimiliano Zappa, Marc
! Scherstjanoi, Heike Lischke

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Subroutines to write the model state to a file at the end of a simulations
! (writeFHstate) and read the initial model state from a previous simulation
! (readFHstate).
!-------------------------------------------------------------------------------

subroutine writeFHstate(year)

! Writes the state of the system to 2 files:
! - TreeMig state: text file with ID x, y, species
! - PREVAH state: tet file with ID x,y
!
! UPDATE August 2018: due to modifications in the subroutine RZstorage, the variables gsTpot and
! gsPes are no longer needed. In this version gsTpot was replaced by gsPe. gsPes is still written/read,
! but is no longer needed and may be replaced by another variable if necessary.

  use prevahglobals, only: ncol, nrow, ssotf, ssnowtf, sliqtf, sitf, ssztf, slz1tf, &
          slz2tf, slz3tf, ssmtf, suztf, landin, sfcG, diroutput, sGlac, icefrac, rnodat, catchAbbr
  use all_par, only: cell, maxspc, maxhc
  use forhycsglobals, only: gslength, gstpot, gsPrec, gsPrecDays, gsTemp, gsPes, dstr30, Schill, Sforcing, &
      ddegsum5old,muwint, snowoffOld, gdd30, deltaBio, gsPe, TMDir
  use MPIvars, only: rank, root, nrowLocal, firstRow, lastRow, nrowLocalArray, numslaves, mpiTag, ierr, status
  use mpi

  implicit none

  real, allocatable :: locTable(:,:), globTable(:,:)
  integer :: cols
  integer :: xcoor, ycoor, cnt, sp, hc
  integer :: tableSize
  integer, dimension(numslaves) :: tableSizes, rowNums
  integer ::  cntRows
  integer :: i

  integer, intent(in) :: year
  character(4) :: chyear
  character*256 :: statefile
  integer :: tmpos
  integer :: iunit
! Variables for MPI
  !integer :: ierr
  !integer :: status
  integer :: proc  ! Loop index for master to loop over slaves


!=======================================================================
  write(chyear,'(I4)') year

! TreeMig part (formerly in writeInitState.f90)

  cols = 3 + maxhc+5  ! X, Y, species, each height class (+1 for lowest HC), seedbed, antagonists, pheno Chill and pheno Forcing

    if(rank .ne. root) then  ! Slaves part
    tableSize = nrowLocal*ncol*maxspc*cols
  ! allocate an array corresponding to the size of each stripe
    allocate(locTable(cols,nrowLocal*ncol*maxspc))
  ! loop over cells of subdomain to retrieve state
      cnt = 1
      do ycoor = lastRow, firstRow, -1
      do xcoor = 1, ncol
          locTable(1,cnt:cnt+maxspc-1) = real(xcoor)
          locTable(2,cnt:cnt+maxspc-1) = real(ycoor)
        do sp = 1,maxspc
          locTable(3,cnt) = real(sp)
          do hc = 0,maxhc
            locTable(3+hc+1,cnt) = cell(ycoor,xcoor)%sp(sp)%numin(hc)
!             if((xcoor==54).and.(ycoor==133)) then
!               print*, "coco sp:", sp, " hc:", hc, "numin:", cell(ycoor,xcoor)%sp(sp)%numin(hc)
!             end if
          end do
          locTable(cols-3,cnt) = cell(ycoor,xcoor)%sp(sp)%seedbed
          locTable(cols-2,cnt) = cell(ycoor,xcoor)%sp(sp)%antagonist
          locTable(cols-1,cnt) = Schill(xcoor,ycoor,sp)
          locTable(cols,cnt) = Sforcing(xcoor,ycoor,sp)
          cnt = cnt + 1
        end do

!         if((xcoor==54).and.(ycoor==133)) then
!               print*, "locTable mimi", locTable(:,(cnt-(maxspc-1)):cnt)
!         end if

      end do
      end do

  ! Send array to Master
!     print*, 'Process ', rank, 'sending array of size ', tablesize, 'with tag', rank
!     print*, 'cols ', cols, 'nrowLocal*ncol*maxspc ', nrowLocal*ncol*maxspc
!     print*, rank, 'locTable', locTable(:,1)
!       print*, rank, 'shape locTable', shape(locTable)
    call MPI_SEND(locTable,tableSize,MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
!     print*, 'ierr', rank, ierr
!     print*, 'Process ', rank, 'sent array of size ', tablesize, 'with tag', rank

  else  ! Master part
  ! Allocate an array corresponding to the size of whole domain
    allocate(globTable(cols,nrow*ncol*maxspc))
    globTable = -9999.0
  ! Calculate size of incoming array from each slave
    tableSizes = nrowLocalArray*ncol*maxspc*cols
    rowNums = nrowLocalArray*ncol*maxspc
    cntRows = 1
    do proc = 1,numslaves
!       print*, 'Master receiving array of size ', tableSizes(proc), 'from process ', proc
!       print*, 'cntRows ', cntRows, 'cntRows+rowNums(proc)-1) ', cntRows+rowNums(proc)-1
      allocate(locTable(cols,rowNums(proc)))
      call MPI_RECV(locTable,tableSizes(proc),MPI_REAL,proc,22,MPI_COMM_WORLD,status,ierr)
!       print*, 'ierr Master', ierr
!       print*, 'Master received array of size ', size(locTable), 'from process ', proc, 'with tag', mpiTag
!
!       print*, "cntRows, rowNums(proc)", cntRows, rowNums(proc), proc
!       print*, rank, 'shape locTable', shape(locTable)
!       print*, rank, 'lbound locTable', lbound(locTable)
!       print*, rank, 'ubound locTable', ubound(locTable)
!       print*, rank, 'shape globTable', shape(globTable)
!       print*, rank, 'locTable', locTable(1:cols,1)

      globTable(1:cols,cntRows:cntRows+rowNums(proc)-1) = locTable
      deallocate(locTable)
      cntRows = cntRows + rowNums(proc)
    end do

    ! Write state file

      CALL POSNAM (diroutput,1,tmpos)
      !statefile =  diroutput(1:tmpos)//'states/treestate_'//chyear//'.txt'
      statefile = trim(TMdir)//'E/treestate'//trim(catchAbbr)//'_'//chyear//'.txt'

      iunit = 1347
      OPEN(UNIT=iunit, FILE=trim(statefile), STATUS='unknown')

      do i = 1,nrow*ncol*maxspc
        write(iunit,"(I4,X,I4,X,I4,X,20(F12.7,X))") int(globTable(1,i)), int(globTable(2,i)), int(globTable(3,i)), globTable(4:cols,i)
      end do

      close(iunit)

  end if ! Master / slave


  if(allocated(locTable)) deallocate(locTable)
  if(allocated(globTable)) deallocate(globTable)

! PREVAH part

  ! Define number of columns
  cols = 28 !26

  if(rank.ne.root) then

    tableSize = ncol*nrowLocal*cols
    allocate(locTable(cols,ncol*nrowLocal))
    cnt = 1

  ! Loop over cells of subdomain
    do ycoor = lastRow, firstRow, -1
    do xcoor = 1, ncol
      locTable(1,cnt) = real(xcoor)
      locTable(2,cnt) = real(ycoor)
      locTable(3,cnt) = sitf(xcoor,ycoor)
      locTable(4,cnt) = ssotf(xcoor,ycoor)
      locTable(5,cnt) = ssnowtf(xcoor,ycoor)
      locTable(6,cnt) = sliqtf(xcoor,ycoor)
      locTable(7,cnt) = ssmtf(xcoor,ycoor)
      locTable(8,cnt) = suztf(xcoor,ycoor)
      locTable(9,cnt) = ssztf(xcoor,ycoor)
      locTable(10,cnt) = slz1tf(xcoor,ycoor)
      locTable(11,cnt) = slz2tf(xcoor,ycoor)
      locTable(12,cnt) = slz3tf(xcoor,ycoor)
      locTable(13,cnt) = sGlac(xcoor,ycoor)
      locTable(14,cnt) = icefrac(xcoor,ycoor)
      locTable(15,cnt) = real(landin(xcoor,ycoor))
      locTable(16,cnt) = sum(gsLength(:,xcoor,ycoor),mask=gsLength(:,xcoor,ycoor).ne.rnodat) / max(1,count(gsLength(:,xcoor,ycoor).ne.rnodat))
      !locTable(17,cnt) = sum(gsTpot(:,xcoor,ycoor),mask=gsTpot(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsTpot(:,xcoor,ycoor).ne.rnodat))
      locTable(17,cnt) = sum(gsPe(:,xcoor,ycoor),mask=gsPe(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsPe(:,xcoor,ycoor).ne.rnodat))
      locTable(18,cnt) = sum(gsPes(:,xcoor,ycoor),mask=gsPes(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsPes(:,xcoor,ycoor).ne.rnodat))
      locTable(19,cnt) = sum(gsPrec(:,xcoor,ycoor),mask=gsPrec(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsPrec(:,xcoor,ycoor).ne.rnodat))
      locTable(20,cnt) = sum(gsPrecDays(:,xcoor,ycoor),mask=gsPrecDays(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsPrecDays(:,xcoor,ycoor).ne.rnodat))
      locTable(21,cnt) = sum(gsTemp(:,xcoor,ycoor),mask=gsTemp(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsTemp(:,xcoor,ycoor).ne.rnodat))
      locTable(22,cnt) = sum(dstr30(1:3,xcoor,ycoor),mask=dstr30(1:3,xcoor,ycoor).ne.rnodat) / count(dstr30(1:3,xcoor,ycoor).ne.rnodat)
      locTable(23,cnt) = sfcG(xcoor,ycoor)
      locTable(24,cnt) = ddegsum5old(xcoor,ycoor)
      locTable(25,cnt) = muwint(xcoor,ycoor,4)
      locTable(26,cnt) = snowoffOld(xcoor,ycoor)
      locTable(27,cnt) = sum(gdd30(1:30,xcoor,ycoor),mask=gdd30(1:30,xcoor,ycoor).ne.rnodat) / count(gdd30(1:30,xcoor,ycoor).ne.rnodat)
      locTable(28,cnt) = sum(deltaBio(xcoor,ycoor,1:30),mask=deltaBio(xcoor,ycoor,1:30).ne.rnodat) / count(deltaBio(xcoor,ycoor,1:30).ne.rnodat)
      cnt = cnt+1
    end do
    end do

    ! Send array to Master
    print*, "HYDROLOGY STATE"
!     print*, 'Process ', rank, 'sending array of size ', tablesize, 'with tag', rank
!     print*, 'cols ', cols, 'nrowLocal*ncol ', nrowLocal*ncol
!     print*, rank, 'locTable', locTable(:,1)
!       print*, rank, 'shape locTable', shape(locTable)
    call MPI_SEND(locTable,tableSize,MPI_REAL,root,22,MPI_COMM_WORLD,ierr)
!     print*, 'ierr', rank, ierr
!     print*, 'Process ', rank, 'sent array of size ', tablesize, 'with tag', rank

  else ! Master
  ! Allocate array that will contain info from all cells
    allocate(globTable(cols,nrow*ncol))
    globTable = -9999.0

  ! Calculate size of incoming array from each slave
    tableSizes = nrowLocalArray*ncol*cols
    rowNums = nrowLocalArray*ncol
    cntRows = 1
    do proc = 1,numslaves
      print*, 'Master receiving array of size ', tableSizes(proc), 'from process ', proc
      print*, 'cntRows ', cntRows, 'cntRows+rowNums(proc)-1) ', cntRows+rowNums(proc)-1
      allocate(locTable(cols,rowNums(proc)))
      call MPI_RECV(locTable,tableSizes(proc),MPI_REAL,proc,22,MPI_COMM_WORLD,status,ierr)
      print*, 'ierr Master', ierr
      print*, 'Master received array of size ', size(locTable), 'from process ', proc, 'with tag', mpiTag

      print*, "cntRows, rowNums(proc)", cntRows, rowNums(proc), proc
      print*, rank, 'shape locTable', shape(locTable)
      print*, rank, 'lbound locTable', lbound(locTable)
      print*, rank, 'ubound locTable', ubound(locTable)
      print*, rank, 'shape globTable', shape(globTable)
      print*, rank, 'locTable', locTable(1:cols,1)

      globTable(1:cols,cntRows:cntRows+rowNums(proc)-1) = locTable
      deallocate(locTable)
      cntRows = cntRows + rowNums(proc)
    end do

    ! Write state file
    CALL POSNAM (diroutput,1,tmpos)
    !statefile =  diroutput(1:tmpos)//'states/hydrostate_'//chyear//'.txt'
    statefile = trim(TMdir)//'E/hydrostate'//trim(catchAbbr)//'.txt'
    statefile = trim(TMdir)//'E/hydrostate'//trim(catchAbbr)//'_'//chyear//'.txt'

    iunit = 1348
    OPEN(UNIT=iunit, FILE=trim(statefile), STATUS='unknown')

    do i = 1,nrow*ncol
      write(iunit,"(I4,X,I4,X,26(F12.5,X))") int(globTable(1,i)), int(globTable(2,i)), globTable(3:cols,i)
    end do

    close(iunit)

  end if ! Master/slave

end subroutine writeFHstate


!=======================================================================
!=======================================================================
!=======================================================================

subroutine readFHstate

!
! UPDATE August 2018: due to modifications in the subroutine RZstorage, the variables gsTpot and
! gsPes are no longer needed. In this version gsTpot was replaced by gsPe. gsPes is still written/read,
! but is no longer needed and may be replaced by another variable if necessary.

  use prevahglobals, only: ncol, nrow, ssotf, ssnowtf, sliqtf, sitf, ssztf, slz1tf, &
          slz2tf, slz3tf, ssmtf, suztf, landin, sfcG, diroutput, sGlac, icefrac, rnodat, catchAbbr
  use all_par, only: cell, maxspc, maxhc,CurrStateInCell,spc
  use forhycsglobals, only: gslength, gstpot, gsPrec, gsPrecDays, gsTemp, gsPes, dstr30, Schill, Sforcing, &
      ddegsum5old, muwint, snowoffOld, nspc_g, readSwitch, biomass2d, gdd30, deltaBio, gsPe, TMDir
  use MPIvars, only: rank, root, nrowLocal, firstRow, lastRow, nrowLocalArray, numslaves, mpiTag
  use mpi

  implicit none

  integer :: iunit
  integer :: tmpos
  real, dimension(23) :: inline      ! Adjust dimension if number of height classes is not 15
  real, dimension(28) :: hline
  integer :: domsize
  integer :: i
  integer :: xcoor, ycoor,sp, hc
  type(CurrStateInCell) :: tempcell
  real:: sumbio     ! Sums biomass per species in a cell - gets stored in biomass2d
  character*256 :: statefile
  character*4 :: chyear

  !===================================

  chyear = "1972" ! Year from which to read previously saved statefile

  ! TreeMig part
  domsize = ncol*nrow*maxspc
  readSwitch = .FALSE.

  call posnam(diroutput,1,tmpos)
  iunit = 1349
  !statefile = trim(TMdir)//'E/treestate'//trim(catchAbbr)//'.txt'
  statefile = trim(TMdir)//'E/treestate'//trim(catchAbbr)//'_'//chyear//'.txt'
  !open(unit=iunit, file=diroutput(1:tmpos)//"treemig/E/treestate.txt", status='old', action='read')  ! Hard-coded for now because lazy
  open(unit=iunit, file=trim(statefile), status='old', action='read')
  ! Loop over lines in state file, assign values to corresponding coordinates in array CELL

  do i = 1,domsize
    read(iunit,*) inline
    if((i .ge. 1) .and. (i .le. 40)) then
    !print*, "inline", inline
    !print*, cell(ycoor,xcoor)%sp(sp)%numin
    end if
    xcoor = int(inline(1))
    ycoor = int(inline(2))
    sp = int(inline(3))
    !print*, "inline(1,2,3)", inline(1),inline(2),inline(3)
    cell(ycoor,xcoor)%sp(sp)%numin(0:maxhc) = inline(4:maxhc+1+3)
    cell(ycoor,xcoor)%sp(sp)%seedbed = inline(20)
    cell(ycoor,xcoor)%sp(sp)%antagonist = inline(21)
    Schill(xcoor,ycoor,sp) = inline(22)
    Sforcing(xcoor,ycoor,sp) = inline(23)

  end do

  ! Calculate canopy properties (LAI, FCC) based on species-size distribution
      if(rank .ne. root) then
       do xcoor = 1,ncol
       do ycoor = firstRow,lastRow
         do sp = 1,30
           biomass2d(xcoor,ycoor,sp) = 0.0
           sumbio = 0.0
           do hc = 1,maxhc
             sumbio = sumbio + (cell(ycoor,xcoor)%sp(sp)%numin(hc)) * (spc(sp)%bioin(hc))
           end do
           biomass2d(xcoor,ycoor,sp) = sumbio
         end do
         tempcell = cell(ycoor,xcoor)
         call vegPars(xcoor,ycoor,tempcell,nspc_g)
       end do
       end do
       end if  ! Slaves only

  close(iunit)

  readSwitch = .TRUE.


  ! PREVAH part
  domsize = ncol*nrow
  iunit = 1350
  !statefile = trim(TMdir)//'E/hydrostate'//trim(catchAbbr)//'.txt'
  statefile = trim(TMdir)//'E/hydrostate'//trim(catchAbbr)//'_'//chyear//'.txt'
  open(unit=iunit, file=trim(statefile), status='old', action='read')
  !open(unit=iunit, file=diroutput(1:tmpos)//"treemig/E/hydrostate.txt", status='old', action='read')  ! Hard-coded for now because lazy

  do i=1,domsize
    read(iunit,*) hline
    xcoor=int(hline(1))
    ycoor=int(hline(2))
    sitf(xcoor,ycoor) = hline(3)
    !ssotf(xcoor,ycoor) = hline(4)
    !ssnowtf(xcoor,ycoor) = hline(5)
    !sliqtf(xcoor,ycoor) = hline(6)
    ! Using snow storage from spin-up simulations introduces large
    ! water reservoirs in high elevations, which melt as temperature
    ! rises under climate scenarios. This hinders the comparison with
    ! e.g. PREVAH runs. Therefore, snow reservoirs are set to zero at
    ! the beginning of a simulation (typically in 1971)
    ssotf(xcoor,ycoor) = 0.0
    ssnowtf(xcoor,ycoor) = 0.0
    sliqtf(xcoor,ycoor) = 0.0
    ssmtf(xcoor,ycoor) = hline(7)
    !suztf(xcoor,ycoor) = hline(8)
    !ssztf(xcoor,ycoor) = hline(9)
    !slz1tf(xcoor,ycoor) = hline(10)
    !slz2tf(xcoor,ycoor) = hline(11)
    !slz3tf(xcoor,ycoor) = hline(12)

    !sGlac(xcoor,ycoor) = hline(13)
    !icefrac(xcoor,ycoor) = hline(14)
    ! MS March 2019: sGlac and icefrac get initialized based on external glacier outputs

    !landin(xcoor,ycoor) = int(hline(15))
    gsLength(:,xcoor,ycoor) = hline(16)
    !gsTpot(:,xcoor,ycoor) = hline(17)
    gsPe(:,xcoor,ycoor) = hline(17)
    gsPes(:,xcoor,ycoor) = hline(18)
    gsPrec(:,xcoor,ycoor) = hline(19)
    gsPrecDays(:,xcoor,ycoor) = hline(20)
    gsTemp(:,xcoor,ycoor) = hline(21)
    dstr30(:,xcoor,ycoor) = hline(22)
    sfcG(xcoor,ycoor) = hline(23)
    ddegsum5old(xcoor,ycoor) = hline(24)
    muwint(xcoor,ycoor,4) = hline(25)
    snowoffOld(xcoor,ycoor) = hline(26)
    gdd30(:,xcoor,ycoor) = hline(27)
    deltaBio(xcoor,ycoor,:) = hline(28)

  end do

  close(iunit)

end subroutine readFHstate
